package ext {

[Uses("ext.Base")]
[Native("Ext.Action", require)]
/**
 * An Action encapsulates a shareable, reusable set of properties which define a "clickable"
 * UI component such as a <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a> or <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu item</a>,
 * or <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">panel header tool</a>, or an <a href="#!/api/Ext.grid.column.Action" rel="Ext.grid.column.Action" class="docClass">ActionColumn item</a>
 *
 * <p>Actions let you share handlers, configuration options and UI updates across any components
 * which were created using the Action.</p>
 *
 * <p>You do not have to create Action instances. They can be configured into Views
 * using the <a href="#!/api/Ext.container.Container-cfg-actions" rel="Ext.container.Container-cfg-actions" class="docClass">actions</a> config.</p>
 *
 * <p>Use a reference to an Action as the config object for any number of UI Components which share the same configuration. The
 * Action not only supplies the configuration, but allows all Components based upon it to have a common set of methods
 * called at once through a single call to the Action.</p>
 *
 * <p>Any Component that is to be configured with an Action may support
 * the following methods:</p>
 *
 * <ul>
 * <li>setText(String)</li>
 * <li>setIconCls(String)</li>
 * <li>setGlyph(String)</li>
 * <li>setDisabled(Boolean)</li>
 * <li>setVisible(Boolean)</li>
 * <li>setHandler(String)</li>
 * </ul>
 *
 *
 * <p>This allows the Action to control its associated Components. Use a
 * <a href="#!/api/Ext.container.Container-method-getAction" rel="Ext.container.Container-method-getAction" class="docClass">reference to an Action</a> to control
 * all components created from that action.</p>
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('ActionsExampleController', {
 *     extend: '<a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a>',
 *     alias: 'controller.actionsexample',
 *
 *     onOperationClick: function() {
 *         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Click', 'Perform the operation');
 *     },
 *
 *     onOperationToggle: function(btn, pressed) {
 *         // The action controls all UI components created from it.
 *         this.view.getAction('operation').setDisabled(pressed);
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('ActionsPanel', {
 *     extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
 *     controller: 'actionsexample',
 *
 *     title: 'Actions',
 *     width: 500,
 *     height: 300,
 *
 *     // Define the shared Action.  Each Component created from these will
 *     // have the same display text, icon and tooltip, and will invoke the
 *     // same controller method on click.
 *     actions: {
 *         operation: {
 *             text: 'Do operation',
 *              handler: 'onOperationClick',
 *             glyph: 'xf005&#64;FontAwesome',
 *             tooltip: 'Perform the operation'
 *         },
 *         disableOperation: {
 *             text: 'Disable operation',
 *             enableToggle: true,
 *             toggleHandler: 'onOperationToggle',
 *             tooltip: 'Disable the operation'
 *         }
 *     },
 *
 *     // Actions are interpreted as Buttons by this view.
 *     // Other descendants such as Menus and Toolbars have
 *     // their own defaults.
 *     defaultActionType: 'button',
 *
 *     tools: [
 *         '&#64;operation'
 *     ],
 *
 *     tbar: [
 *         // Add the Action directly to a toolbar as a menu button
 *         '&#64;operation',
 *         {
 *             text: 'Action Menu',
 *             menu: [
 *                 // Add the Action to a menu as a text item
 *                 '&#64;operation'
 *             ]
 *         }, '&#64;disableOperation'
 *     ],
 *
 *     bodyPadding: 10,
 *     layout: {
 *         type: 'vbox',
 *         align: 'stretch'
 *     },
 *     items: [
 *         // Add the Action to the panel body.
 *         // defaultActionType will ensure it is converted to a Button.
 *         '&#64;operation'
 *     ]
 * });
 *
 * <a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.QuickTips.init</a>();
 * new ActionsPanel({
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class Action extends ext.Base {
  /**
   * Creates new Action.
   * @param config Config object.
   */
  public function Action(config:ext.Action = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated Action, or subclass thereof.
   * @default true
   */
  public native function get isAction():Boolean;

  /**
   * @private
   */
  public native function set isAction(value:Boolean):void;

  [ExtConfig]
  /**
   * The key to use for this action in a container's <code>actions</code> map
   * when this action is added to the map through the container's <code>actionList</code>
   * config.
   *
   * This should usually be specified through a constant defined in the container,
   * but it also falls back to a constant with the magic name <code>ACTION_ID</code>
   * defined in this action class.
   *
   * @see ext.container.Container#actions
   * @see ext.container.Container#actionList
   */
  public native function get actionId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set actionId(value:String):void;

  [ExtConfig]
  /**
   * True to disable all components configured by this Action, false to enable them.
   * @default false
   * @see #setDisabled()
   */
  public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabled(value:Boolean):void;

  [ExtConfig]
  /**
   * A numeric unicode character code to use as the icon.  The default font-family
   * for glyphs can be set globally using
   * <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a> application
   * config or the <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a> method.
   * It is initially set to <code>'Pictos'</code>.
   *
   * <p>The following shows how to set the glyph using the font icons provided in the
   * SDK (assuming the font-family has been configured globally):</p>
   *
   * <pre><code>// assumes the glyphFontFamily is "Pictos"
   * glyph: 'x48'       // the "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 72          // The "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 'H'         // the "home" icon
   * </code></pre>
   *
   * <p>Alternatively, this config option accepts a string with the charCode and
   * font-family separated by the <code>&#64;</code> symbol.</p>
   *
   * <pre><code>// using Font Awesome
   * glyph: 'xf015&#64;FontAwesome'     // the "home" icon
   *
   * // using Pictos
   * glyph: 'H&#64;Pictos'              // the "home" icon
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @since 6.2.0
   * @see #setGlyph()
   */
  public native function get glyph():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  /**
   * The function that will be invoked by each component tied to this Action
   * when the component's primary event is triggered.
   * @see #setHandler()
   */
  public native function get handler():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set handler(value:Function):void;

  [ExtConfig]
  /**
   * True to hide all components configured by this Action, false to show them.
   * @default false
   * @see #setHidden()
   */
  public native function get hidden():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set hidden(value:Boolean):void;

  [ExtConfig]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @default ''
   * @see #setIconCls()
   */
  public native function get iconCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  /**
   * See <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.<a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">itemId</a>.
   */
  public native function get itemId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemId(value:String):void;

  [ExtConfig]
  /**
   * The scope (this reference) in which the <a href="#!/api/Ext.Action-cfg-handler" rel="Ext.Action-cfg-handler" class="docClass">handler</a> is executed
   * if specified as a function instead of a named Controller method.
   * Defaults to the browser window.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The text to set for all components configured by this Action.
   * @default ''
   * @see #getText()
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  /**
   * The tooltip for the action - can be a string to be used as innerHTML (html tags are accepted) or
   * QuickTips config object.
   * @see #setTooltip()
   */
  public native function get tooltip():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tooltip(value:*):void;

  /**
   * This Action's initial configuration specification.
   */
  public native function get initialConfig():Object;

  /**
   * @private
   */
  public native function addComponent(comp:ext.Component):void;

  /**
   * Disables all components configured by this Action.
   */
  public native function disable():void;

  /**
   * Executes the specified function once for each Component currently tied to this Action.  The function passed
   * in should accept a single argument that will be an object that supports the basic Action config/method interface.
   * @param fn The function to execute for each component
   * @param scope The scope (this reference) in which the function is executed.
   * Defaults to the Component.
   */
  public native function each(fn:Function, scope:Object = null):void;

  /**
   * Enables all components configured by this Action.
   */
  public native function enable():void;

  /**
   * Executes this Action manually using the handler function specified in the original config object
   * or the handler function set with <a href="#!/api/Ext.Action-method-setHandler" rel="Ext.Action-method-setHandler" class="docClass">setHandler</a>.  Any arguments passed to this
   * function will be passed on to the handler function.
   * @param args Variable number of arguments passed to the handler function
   */
  public native function execute(...args):void;

  /**
   * Gets the icon CSS class currently used by all components configured by this Action.
   */
  public native function getIconCls():String;

  /**
   * Gets the text currently displayed by all components configured by this Action.
   */
  public native function getText():String;

  /**
   * Gets the tooltip currently displayed by all components configured by this Action.
   */
  public native function getTooltip():String;

  /**
   * Hides all components configured by this Action.
   */
  public native function hide():void;

  /**
   * Returns true if the components using this Action are currently disabled, else returns false.
   */
  public native function isDisabled():Boolean;

  /**
   * Returns true if the components configured by this Action are currently hidden, else returns false.
   */
  public native function isHidden():Boolean;

  /**
   * @private
   */
  public native function removeComponent(comp:ext.Component):void;

  /**
   * Sets the disabled state of all components configured by this Action.  Shortcut method
   * for <a href="#!/api/Ext.Action-method-enable" rel="Ext.Action-method-enable" class="docClass">enable</a> and <a href="#!/api/Ext.Action-method-disable" rel="Ext.Action-method-disable" class="docClass">disable</a>.
   * @param disabled True to disable the component, false to enable it
   */
  public native function setDisabled(disabled:Boolean):void;

  /**
   * Sets the glyph for all components configured by this
   * Action.
   * @param glyph The CSS class supplying the icon image
   */
  public native function setGlyph(glyph:String):void;

  /**
   * Sets the function that will be called by each Component using this action when its
   * primary event (usually a click or tap) is triggered.
   * @param handler The function that will be invoked by the action's components when clicked.
   * See the <code>handler</code> config of the target component for the arguments passed.
   * @param scope The scope (this reference) in which the function is executed. Defaults to an
   * encapsulating <a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Controller</a>, or the Component.
   */
  public native function setHandler(handler:Function, scope:Object = null):void;

  /**
   * Sets the hidden state of all components configured by this Action.  Shortcut method
   * for <code><a href="#!/api/Ext.Action-method-hide" rel="Ext.Action-method-hide" class="docClass">hide</a></code> and <code><a href="#!/api/Ext.Action-method-show" rel="Ext.Action-method-show" class="docClass">show</a></code>.
   * @param hidden True to hide the component, false to show it.
   */
  public native function setHidden(hidden:Boolean):void;

  /**
   * Sets the <a href="#!/api/Ext.Action-cfg-iconCls" rel="Ext.Action-cfg-iconCls" class="docClass">icon CSS class</a> for all components configured by this
   * Action.  The class should supply a background image that will be used as the icon
   * image.
   * @param cls The CSS class supplying the icon image
   */
  public native function setIconCls(cls:String):void;

  /**
   * Sets the text to be displayed by all components configured by this Action.
   * @param text The text to display
   */
  public native function setText(text:String):void;

  /**
   * Sets the tooltip to be displayed by all components configured by this Action.
   * @param tooltip The tooltip to display
   */
  public native function setTooltip(tooltip:String):void;

  /**
   * Shows all components configured by this Action.
   */
  public native function show():void;
}
}