package ext {

[Native("Ext.Base")]
/**
 * The root of all classes created with <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>.
 *
 * <p><a href="#!/api/Ext.Base" rel="Ext.Base" class="docClass">Ext.Base</a> is the building block of all Ext classes. All classes in Ext inherit from <a href="#!/api/Ext.Base" rel="Ext.Base" class="docClass">Ext.Base</a>.
 * All prototype and static members of this class are inherited by all other classes.</p>
 */
public class Base {
  /**
   * Add methods / properties to the prototype of this class.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.awesome.Cat', {
   *     constructor: function() {
   *         ...
   *     }
   * });
   *
   *  My.awesome.Cat.addMembers({
   *      meow: function() {
   *         alert('Meowww...');
   *      }
   *  });
   *
   *  var kitty = new My.awesome.Cat();
   *  kitty.meow();
   * </code></pre>
   * @param members The members to add to this class.
   * @param isStatic Pass <code>true</code> if the members are static.
   * @default false
   * @param privacy Pass <code>true</code> if the members are private. This
   * only has meaning in debug mode and only for methods.
   * @default false
   */
  public static native function addMembers(members:Object, isStatic:Boolean = false, privacy:Boolean = false):void;

  /**
   * Add / override static properties of this class.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.cool.Class', {
   *     ...
   * });
   *
   * My.cool.Class.addStatics({
   *     someProperty: 'someValue',      // My.cool.Class.someProperty = 'someValue'
   *     method1: function() { ... },    // My.cool.Class.method1 = function() { ... };
   *     method2: function() { ... }     // My.cool.Class.method2 = function() { ... };
   * });
   * </code></pre>
   * @return this
   */
  public static native function addStatics(members:Object):ext.Base;

  /**
   * Create a new instance of this Class.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.cool.Class', {
   *     ...
   * });
   *
   * My.cool.Class.create({
   *     someConfig: true
   * });
   * </code></pre>
   *
   * <p>All parameters are passed to the constructor of the class.</p>
   * @return the created instance.
   */
  public static native function create():Object;

  /**
   * Create aliases for existing prototype methods. Example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.cool.Class', {
   *     method1: function() { ... },
   *     method2: function() { ... }
   * });
   *
   * var test = new My.cool.Class();
   *
   * My.cool.Class.createAlias({
   *     method3: 'method1',
   *     method4: 'method2'
   * });
   *
   * test.method3(); // test.method1()
   *
   * My.cool.Class.createAlias('method5', 'method3');
   *
   * test.method5(); // test.method3() -&gt; test.method1()
   * </code></pre>
   * @param alias The new method name, or an object to set multiple aliases. See
   * <a href="#!/api/Ext.Function-method-flexSetter" rel="Ext.Function-method-flexSetter" class="docClass">flexSetter</a>
   * @param origin The original method name
   */
  public static native function createAlias(alias:*, origin:*):void;

  /**
   * Get the current class' name in string format.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.cool.Class', {
   *     constructor: function() {
   *         alert(this.self.getName()); // alerts 'My.cool.Class'
   *     }
   * });
   *
   * My.cool.Class.getName(); // 'My.cool.Class'
   * </code></pre>
   * @return className
   */
  public static native function getName():String;

  /**
   * Override members of this class. Overridden methods can be invoked via
   * <a href="#!/api/Ext.Base-method-callParent" rel="Ext.Base-method-callParent" class="docClass">callParent</a>.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Cat', {
   *     constructor: function() {
   *         alert("I'm a cat!");
   *     }
   * });
   *
   * My.Cat.override({
   *     constructor: function() {
   *         alert("I'm going to be a cat!");
   *
   *         this.callParent(arguments);
   *
   *         alert("Meeeeoooowwww");
   *     }
   * });
   *
   * var kitty = new My.Cat(); // alerts "I'm going to be a cat!"
   *                           // alerts "I'm a cat!"
   *                           // alerts "Meeeeoooowwww"
   * </code></pre>
   *
   * <p>Direct use of this method should be rare. Use <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>
   * instead:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.CatOverride', {
   *     override: 'My.Cat',
   *     constructor: function() {
   *         alert("I'm going to be a cat!");
   *
   *         this.callParent(arguments);
   *
   *         alert("Meeeeoooowwww");
   *     }
   * });
   * </code></pre>
   *
   * <p>The above accomplishes the same result but can be managed by the <a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a>
   * which can properly order the override and its target class and the build process
   * can determine whether the override is needed based on the required state of the
   * target class (My.Cat).</p>
   * @param members The properties to add to this class. This should be
   * specified as an object literal containing one or more properties.
   * @return this class
   */
  public static native function override_(members:Object):ext.Base;

  protected static native function callParent(args:Object):void;

  protected static native function callSuper(args:Object):void;

  /**
   * This property is set to <code>true</code> after the <code>destroy</code> method is called.
   * @default false
   */
  public native function get destroyed():Boolean;

  /**
   * @private
   */
  public native function set destroyed(value:Boolean):void;

  /**
   * This property is set to <code>true</code> during the call to <code>initConfig</code>.
   * @default false
   * @since 5.0.0
   */
  public native function get isConfiguring():Boolean;

  /**
   * This property is set to <code>true</code> if this instance is the first of its class.
   * @default false
   * @since 5.0.0
   */
  public native function get isFirstInstance():Boolean;

  /**
   * This value is <code>true</code> and is used to identify plain objects from instances of
   * a defined class.
   * @default true
   */
  public native function get isInstance():Boolean;

  /**
   * Get the reference to the current class from which this object was instantiated. Unlike <a href="#!/api/Ext.Base-method-statics" rel="Ext.Base-method-statics" class="docClass">statics</a>,
   * <code>this.self</code> is scope-dependent and it's meant to be used for dynamic inheritance. See <a href="#!/api/Ext.Base-method-statics" rel="Ext.Base-method-statics" class="docClass">statics</a>
   * for a detailed comparison
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Cat', {
   *     statics: {
   *         speciesName: 'Cat' // My.Cat.speciesName = 'Cat'
   *     },
   *
   *     constructor: function() {
   *         alert(this.self.speciesName); // dependent on 'this'
   *     },
   *
   *     clone: function() {
   *         return new this.self();
   *     }
   * });
   *
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.SnowLeopard', {
   *     extend: 'My.Cat',
   *     statics: {
   *         speciesName: 'Snow Leopard'         // My.SnowLeopard.speciesName = 'Snow Leopard'
   *     }
   * });
   *
   * var cat = new My.Cat();                     // alerts 'Cat'
   * var snowLeopard = new My.SnowLeopard();     // alerts 'Snow Leopard'
   *
   * var clone = snowLeopard.clone();
   * alert(<a href="#!/api/Ext-method-getClassName" rel="Ext-method-getClassName" class="docClass">Ext.getClassName</a>(clone));             // alerts 'My.SnowLeopard'
   * </code></pre>
   */
  public native function get self():ext.ExtClass;

  /**
   * @private
   */
  public native function set self(value:ext.ExtClass):void;

  [ExtConfig("__mixins__")]
  /**
   * This element contains additional config objects that are mixed into
   * (merged with) this config object.
   * <br/>
   * It is neither recommended to define the same property in the base config object
   * as well as in a mixin nor to define the same property in more than one mixin.
   * However, when a property is defined multiple times, the base config object has
   * precedence over mixins, and a mixin has precedence over following mixins.
   * <br/>
   * It is recommended to use a typed mixin config object in favor of using
   * dynamic attributes.
   */
  public native function set mixins(value:Array):void;

  /**
   * This method is called to cleanup an object and its resources. After calling
   * this method, the object should not be used any further in any way, including
   * access to its methods and properties.
   *
   * <p>To prevent potential memory leaks, all object references will be nulled
   * at the end of destruction sequence, unless <a href="#!/api/Ext.Base-property-clearPropertiesOnDestroy" rel="Ext.Base-property-clearPropertiesOnDestroy" class="docClass">clearPropertiesOnDestroy</a>
   * is set to <code>false</code>.</p>
   */
  public native function destroy(...params):void;

  /**
   * Returns a specified config property value. If the name parameter is not passed,
   * all current configuration options will be returned as key value pairs.
   * @param name The name of the config property to get.
   * @param peek <code>true</code> to peek at the raw value without calling the getter.
   * @default false
   * @return The config property value.
   */
  public native function getConfig(name:String = null, peek:Boolean = false):*;

  /**
   * Returns the initial configuration passed to the constructor when
   * instantiating this class.
   *
   * <p>Given this example <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a> definition and instance:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.Button', {
   *     extend: '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>',
   *     xtype: 'mybutton',
   *
   *     scale: 'large',
   *     enableToggle: true
   * });
   *
   * var btn = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'mybutton',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     text: 'Test Button'
   * });
   * </code></pre>
   *
   * <p>Calling <code>btn.getInitialConfig()</code> would return an object including the config
   * options passed to the <code>create</code> method:</p>
   *
   * <pre><code>xtype: 'mybutton',
   * renderTo: // The document body itself
   * text: 'Test Button'
   * </code></pre>
   *
   * <p>Calling <code>btn.getInitialConfig('text')</code>returns <strong>'Test Button'</strong>.</p>
   * @param name Name of the config option to return.
   * @return The full config object or a single config value
   * when <code>name</code> parameter specified.
   */
  public native function getInitialConfig(name:String = null):*;

  /**
   * Sets a single/multiple configuration options.
   * @param name The name of the property to set, or a set of key value pairs to set.
   * @param value The value to set for the name parameter.
   * @return this
   */
  public native function setConfig(name:*, value:* = null):ext.Base;

  [Deprecated(replacement = "callParent")]
  /**
   * Call the original method that was previously overridden with <a href="#!/api/Ext.Base-static-method-override" rel="Ext.Base-static-method-override" class="docClass">override</a>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Cat', {
   *     constructor: function() {
   *         alert("I'm a cat!");
   *     }
   * });
   *
   * My.Cat.override({
   *     constructor: function() {
   *         alert("I'm going to be a cat!");
   *
   *         this.callOverridden();
   *
   *         alert("Meeeeoooowwww");
   *     }
   * });
   *
   * var kitty = new My.Cat(); // alerts "I'm going to be a cat!"
   *                           // alerts "I'm a cat!"
   *                           // alerts "Meeeeoooowwww"
   * </code></pre>
   * @param args The arguments, either an array or the <code>arguments</code> object
   * from the current method, for example: <code>this.callOverridden(arguments)</code>
   * @return Returns the result of calling the overridden method
   */
  protected native function callOverridden(args:*):Object;

  /**
   * Call the "parent" method of the current method. That is the method previously
   * overridden by derivation or by an override (see <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>).
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Base', {
   *      constructor: function (x) {
   *          this.x = x;
   *      },
   *
   *      statics: {
   *          method: function (x) {
   *              return x;
   *          }
   *      }
   *  });
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Derived', {
   *      extend: 'My.Base',
   *
   *      constructor: function () {
   *          this.callParent([21]);
   *      }
   *  });
   *
   *  var obj = new My.Derived();
   *
   *  alert(obj.x);  // alerts 21
   * </code></pre>
   *
   * <p>This can be used with an override as follows:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.DerivedOverride', {
   *      override: 'My.Derived',
   *
   *      constructor: function (x) {
   *          this.callParent([x*2]); // calls original My.Derived constructor
   *      }
   *  });
   *
   *  var obj = new My.Derived();
   *
   *  alert(obj.x);  // now alerts 42
   * </code></pre>
   *
   * <p>This also works with static and private methods.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Derived2', {
   *      extend: 'My.Base',
   *
   *      // privates: {
   *      statics: {
   *          method: function (x) {
   *              return this.callParent([x*2]); // calls My.Base.method
   *          }
   *      }
   *  });
   *
   *  alert(My.Base.method(10));     // alerts 10
   *  alert(My.Derived2.method(10)); // alerts 20
   * </code></pre>
   *
   * <p>Lastly, it also works with overridden static methods.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Derived2Override', {
   *      override: 'My.Derived2',
   *
   *      // privates: {
   *      statics: {
   *          method: function (x) {
   *              return this.callParent([x*2]); // calls My.Derived2.method
   *          }
   *      }
   *  });
   *
   *  alert(My.Derived2.method(10); // now alerts 40
   * </code></pre>
   *
   * <p>To override a method and replace it and also call the superclass method, use
   * <a href="#!/api/Ext.Base-method-callSuper" rel="Ext.Base-method-callSuper" class="docClass">callSuper</a>. This is often done to patch a method to fix a bug.</p>
   * @param args The arguments, either an array or the <code>arguments</code> object
   * from the current method, for example: <code>this.callParent(arguments)</code>
   * @return Returns the result of calling the parent method
   */
  protected native function callParent(args:*):Object;

  /**
   * This method is used by an <strong>override</strong> to call the superclass method but
   * bypass any overridden method. This is often done to "patch" a method that
   * contains a bug but for whatever reason cannot be fixed directly.
   *
   * <p>Consider:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.some.Class', {
   *      method: function () {
   *          console.log('Good');
   *      }
   *  });
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.some.DerivedClass', {
   *      extend: 'Ext.some.Class',
   *
   *      method: function () {
   *          console.log('Bad');
   *
   *          // ... logic but with a bug ...
   *
   *          this.callParent();
   *      }
   *  });
   * </code></pre>
   *
   * <p>To patch the bug in <code>Ext.some.DerivedClass.method</code>, the typical solution is to create an
   * override:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.patches.DerivedClass', {
   *      override: 'Ext.some.DerivedClass',
   *
   *      method: function () {
   *          console.log('Fixed');
   *
   *          // ... logic but with bug fixed ...
   *
   *          this.callSuper();
   *      }
   *  });
   * </code></pre>
   *
   * <p>The patch method cannot use <a href="#!/api/Ext.Base-method-callParent" rel="Ext.Base-method-callParent" class="docClass">callParent</a> to call the superclass
   * <code>method</code> since that would call the overridden method containing the bug. In
   * other words, the above patch would only produce "Fixed" then "Good" in the
   * console log, whereas, using <code>callParent</code> would produce "Fixed" then "Bad"
   * then "Good".</p>
   * @param args The arguments, either an array or the <code>arguments</code> object
   * from the current method, for example: <code>this.callSuper(arguments)</code>
   * @return Returns the result of calling the superclass method
   */
  protected native function callSuper(args:*):Object;

  /**
   * Initialize configuration for this class. a typical example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.awesome.Class', {
   *     // The default config
   *     config: {
   *         name: 'Awesome',
   *         isAwesome: true
   *     },
   *
   *     constructor: function(config) {
   *         this.initConfig(config);
   *     }
   * });
   *
   * var awesome = new My.awesome.Class({
   *     name: 'Super Awesome'
   * });
   *
   * alert(awesome.getName()); // 'Super Awesome'
   * </code></pre>
   * @return this
   */
  protected native function initConfig(config:Object):ext.Base;

  /**
   * Get the reference to the class from which this object was instantiated. Note that unlike <a href="#!/api/Ext.Base-property-self" rel="Ext.Base-property-self" class="docClass">self</a>,
   * <code>this.statics()</code> is scope-independent and it always returns the class from which it was called, regardless of what
   * <code>this</code> points to during run-time
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Cat', {
   *     statics: {
   *         totalCreated: 0,
   *         speciesName: 'Cat' // My.Cat.speciesName = 'Cat'
   *     },
   *
   *     constructor: function() {
   *         var statics = this.statics();
   *
   *         alert(statics.speciesName);     // always equals to 'Cat' no matter what 'this' refers to
   *                                         // equivalent to: My.Cat.speciesName
   *
   *         alert(this.self.speciesName);   // dependent on 'this'
   *
   *         statics.totalCreated++;
   *     },
   *
   *     clone: function() {
   *         var cloned = new this.self();   // dependent on 'this'
   *
   *         cloned.groupName = this.statics().speciesName;   // equivalent to: My.Cat.speciesName
   *
   *         return cloned;
   *     }
   * });
   *
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.SnowLeopard', {
   *     extend: 'My.Cat',
   *
   *     statics: {
   *         speciesName: 'Snow Leopard'     // My.SnowLeopard.speciesName = 'Snow Leopard'
   *     },
   *
   *     constructor: function() {
   *         this.callParent();
   *     }
   * });
   *
   * var cat = new My.Cat();                 // alerts 'Cat', then alerts 'Cat'
   *
   * var snowLeopard = new My.SnowLeopard(); // alerts 'Cat', then alerts 'Snow Leopard'
   *
   * var clone = snowLeopard.clone();
   * alert(<a href="#!/api/Ext-method-getClassName" rel="Ext-method-getClassName" class="docClass">Ext.getClassName</a>(clone));         // alerts 'My.SnowLeopard'
   * alert(clone.groupName);                 // alerts 'Cat'
   *
   * alert(My.Cat.totalCreated);             // alerts 3
   * </code></pre>
   */
  protected native function statics():ext.ExtClass;
}
}