package ext {

[Uses("ext.ElementLoader")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.ComponentLoader", require)]
/**
 * This class is used to load content via Ajax into a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>. In general
 * this class will not be instanced directly, rather a loader configuration will be passed to the
 * constructor of the <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
 *
 * <h2>HTML Renderer</h2>
 *
 * <p>By default, the content loaded will be processed as raw html. The response text
 * from the request is taken and added to the component. This can be used in
 * conjunction with the <a href="#!/api/Ext.ComponentLoader-cfg-scripts" rel="Ext.ComponentLoader-cfg-scripts" class="docClass">scripts</a> option to execute any inline scripts in
 * the resulting content. Using this renderer has the same effect as passing the
 * <a href="#!/api/Ext.Component-cfg-html" rel="Ext.Component-cfg-html" class="docClass">Ext.Component.html</a> configuration option.</p>
 *
 * <h2>Data Renderer</h2>
 *
 * <p>This renderer allows content to be added by using JSON data and a <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>.
 * The content received from the response is passed to the <a href="#!/api/Ext.Component-method-update" rel="Ext.Component-method-update" class="docClass">Ext.Component.update</a> method.
 * This content is run through the attached <a href="#!/api/Ext.Component-cfg-tpl" rel="Ext.Component-cfg-tpl" class="docClass">Ext.Component.tpl</a> and the data is added to
 * the Component. Using this renderer has the same effect as using the <a href="#!/api/Ext.Component-cfg-data" rel="Ext.Component-cfg-data" class="docClass">Ext.Component.data</a>
 * configuration in conjunction with a <a href="#!/api/Ext.Component-cfg-tpl" rel="Ext.Component-cfg-tpl" class="docClass">Ext.Component.tpl</a>.</p>
 *
 * <h2>Component Renderer</h2>
 *
 * <p>This renderer can only be used with a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> and subclasses. It allows for
 * Components to be loaded remotely into a Container. The response is expected to be a single/series of
 * <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> configuration objects. When the response is received, the data is decoded
 * and then passed to <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">Ext.container.Container.add</a>. Using this renderer has the same effect as specifying
 * the <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">Ext.container.Container.items</a> configuration on a Container.</p>
 *
 * <h2>Custom Renderer</h2>
 *
 * <p>A custom function can be passed to handle any other special case, see the <a href="#!/api/Ext.ComponentLoader-cfg-renderer" rel="Ext.ComponentLoader-cfg-renderer" class="docClass">renderer</a> option.</p>
 *
 * <h2>Example Usage</h2>
 *
 * <pre><code>var cmp = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     tpl: '{firstName} - {lastName}',
 *     loader: {
 *         url: 'myPage.php',
 *         renderer: 'data',
 *         params: {
 *             userId: 1
 *         }
 *     }
 * });
 *
 * // call the loader manually (or use autoLoad:true instead)
 * cmp.getLoader().load();
 * </code></pre>
 */
public class ComponentLoader extends ext.ElementLoader {
  /**
   * @param config @inheritDoc
   */
  public function ComponentLoader(config:ext.ComponentLoader = null) {
    super();
  }

  [ExtConfig]
  /**
   * True or a <a href="#!/api/Ext.LoadMask" rel="Ext.LoadMask" class="docClass">Ext.LoadMask</a> configuration to enable masking during loading.
   * @default false
   */
  override public native function get loadMask():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set loadMask(value:*):void;

  [ExtConfig]
  /**
   * The type of content that is to be loaded into, which can be one of 3 types:
   *
   * <ul>
   * <li><strong>html</strong> : Loads raw html content, see <a href="#!/api/Ext.Component-cfg-html" rel="Ext.Component-cfg-html" class="docClass">Ext.Component.html</a></li>
   * <li><strong>data</strong> : Loads raw html content, see <a href="#!/api/Ext.Component-cfg-data" rel="Ext.Component-cfg-data" class="docClass">Ext.Component.data</a></li>
   * <li><strong>component</strong> : Loads child {<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>} instances. This option is only valid when used with a Container.</li>
   * </ul>
   *
   *
   * <p>Alternatively, you can pass a function which is called with the following parameters.</p>
   *
   * <ul>
   * <li>loader - Loader instance</li>
   * <li>response - The server response</li>
   * <li>active - The active request</li>
   * </ul>
   *
   *
   * <p>The function must return false is loading is not successful. Below is a sample of using a custom renderer:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>({
   *     loader: {
   *         url: 'myPage.php',
   *         renderer: function(loader, response, active) {
   *             var text = response.responseText;
   *             loader.getTarget().setHtml('The response is ' + text);
   *             return true;
   *         }
   *     }
   * });
   * </code></pre>
   * @default 'html'
   */
  override public native function get renderer():Function;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set renderer(value:Function):void;

  [ExtConfig]
  /**
   * The scope (<code>this</code> reference)to execute the <a href="#!/api/Ext.ComponentLoader-cfg-renderer" rel="Ext.ComponentLoader-cfg-renderer" class="docClass">renderer</a> function in. If the <a href="#!/api/Ext.ComponentLoader-cfg-scripts" rel="Ext.ComponentLoader-cfg-scripts" class="docClass">scripts</a> option is
   * <code>true</code>, <em>inline</em> script source (not scripts with a <code>src</code> attribute) is also executed in this scope.
   * Defaults to this Component.
   * @default `this`
   */
  override public native function get rendererScope():Object;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set rendererScope(value:Object):void;

  [ExtConfig]
  /**
   * True to parse any inline script tags in the response. This only used when using the html
   * <a href="#!/api/Ext.ComponentLoader-cfg-renderer" rel="Ext.ComponentLoader-cfg-renderer" class="docClass">renderer</a>. The scripts will be executed with the target Component as the scope (<code>this</code> reference).
   */
  override public native function get scripts():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set scripts(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The target <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> for the loader.
   * If a string is passed it will be looked up via the id.
   * @see #getTarget()
   * @see #setTarget()
   */
  override public native function get target():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set target(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to have the loader make a request when the <a href="#!/api/Ext.ComponentLoader-cfg-target" rel="Ext.ComponentLoader-cfg-target" class="docClass">target</a> is rendered. If the target is
   * already rendered, a load will take place immediately.
   * This argument can also be a set of options that will be passed to <a href="#!/api/Ext.ComponentLoader-method-load" rel="Ext.ComponentLoader-method-load" class="docClass">load</a> when it is called.
   * @default false
   */
  public native function get loadOnRender():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadOnRender(value:*):void;

  /**
   * Set a {<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>} as the target of this loader. Note that if the target is changed,
   * any active requests will be aborted.
   * @param target The component to be the target of this loader. If a string is passed
   * it will be looked up via its id.
   */
  override public native function setTarget(target:*):void;
}
}