package ext {

[Native("Ext.Class", require)]
/**
 * This is a low level factory that is used by <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a> and should not be used
 * directly in application code.
 *
 * <p>The configs of this class are intended to be used in <code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a></code> calls to describe the class you
 * are declaring. For example:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.util.Thing', {
 *     extend: 'App.util.Other',
 *
 *     alias: 'util.thing',
 *
 *     config: {
 *         foo: 42
 *     }
 * });
 * </code></pre>
 *
 * <p><a href="#!/api/Ext.Class" rel="Ext.Class" class="docClass">Ext.Class</a> is the factory and <strong>not</strong> the superclass of everything. For the base class that <strong>all</strong>
 * classes inherit from, see <a href="#!/api/Ext.Base" rel="Ext.Base" class="docClass">Ext.Base</a>.</p>
 */
public class ExtClass {
  /**
   * Create a new anonymous class.
   * @param data An object represent the properties of this class
   * @param onCreated Optional, the callback function to be executed when this class is fully created.
   * Note that the creation process can be asynchronous depending on the pre-processors used.
   * @return The newly created class
   */
  public native function ExtClass(data:Object, onCreated:Function);

  /**
   * The prototype of this class.
   */
  public native function get prototype():Object;

  [ExtConfig]
  /**
   * List of short aliases for class names. An alias consists of a namespace and a name
   * concatenated by a period as &#60;namespace&#62;.&#60;name&#62;
   *
   * <ul>
   * <li><strong>namespace</strong> - The namespace describes what kind of alias this is and must be
   * all lowercase.</li>
   * <li><strong>name</strong> - The name of the alias which allows the lazy-instantiation via the
   * alias. The name shouldn't contain any periods.</li>
   * </ul>
   *
   *
   * <p>A list of namespaces and the usages are:</p>
   *
   * <ul>
   * <li><strong>feature</strong> - <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Grid</a> features</li>
   * <li><strong>plugin</strong> - Plugins</li>
   * <li><strong>store</strong> - <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></li>
   * <li><strong>widget</strong> - Components</li>
   * </ul>
   *
   *
   * <p>Most useful for defining xtypes for widgets:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.CoolPanel', {
   *     extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
   *     alias: ['widget.coolpanel'],
   *     title: 'Yeah!'
   * });
   *
   * // Using <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('widget.coolpanel');
   *
   * // Using the shorthand for defining widgets by xtype
   * <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>('panel', {
   *     items: [
   *         {xtype: 'coolpanel', html: 'Foo'},
   *         {xtype: 'coolpanel', html: 'Bar'}
   *     ]
   * });
   * </code></pre>
   */
  public native function get alias():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set alias(value:*):void;

  [ExtConfig]
  /**
   * Defines alternate names for this class.  For example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Developer', {
   *     alternateClassName: ['Coder', 'Hacker'],
   *     code: function(msg) {
   *         alert('Typing... ' + msg);
   *     }
   * });
   *
   * var joe = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('Developer');
   * joe.code('stackoverflow');
   *
   * var rms = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('Hacker');
   * rms.code('hack hack');
   * </code></pre>
   */
  public native function get alternateClassName():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set alternateClassName(value:*):void;

  [ExtConfig]
  /**
   * This configuration works in a very similar manner to the <a href="#!/api/Ext.Class-cfg-config" rel="Ext.Class-cfg-config" class="docClass">config</a> option.
   * The difference is that the configurations are only ever processed when the first instance
   * of that class is created. The processed value is then stored on the class prototype and
   * will not be processed on subsequent instances of the class. Getters/setters will be generated
   * in exactly the same way as <a href="#!/api/Ext.Class-cfg-config" rel="Ext.Class-cfg-config" class="docClass">config</a>.
   *
   * <p>This option is useful for expensive objects that can be shared across class instances.
   * The class itself ensures that the creation only occurs once.</p>
   */
  public native function get cachedConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cachedConfig(value:Object):void;

  [ExtConfig]
  /**
   * List of configuration options with their default values.
   *
   * <p><strong>Note:</strong> You need to make sure <a href="#!/api/Ext.Base-method-initConfig" rel="Ext.Base-method-initConfig" class="docClass">Ext.Base.initConfig</a> is called from your constructor if you are defining
   * your own class or singleton, unless you are extending a Component. Otherwise the generated getter and setter
   * methods will not be initialized.</p>
   *
   * <p>Each config item will have its own setter and getter method automatically generated inside the class prototype
   * during class creation time, if the class does not have those methods explicitly defined.</p>
   *
   * <p>As an example, let's convert the name property of a Person class to be a config item, then add extra age and
   * gender items.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.sample.Person', {
   *     config: {
   *         name: 'Mr. Unknown',
   *         age: 0,
   *         gender: 'Male'
   *     },
   *
   *     constructor: function(config) {
   *         this.initConfig(config);
   *
   *         return this;
   *     }
   *
   *     // ...
   * });
   * </code></pre>
   *
   * <p>Within the class, this.name still has the default value of "Mr. Unknown". However, it's now publicly accessible
   * without sacrificing encapsulation, via setter and getter methods.</p>
   *
   * <pre><code>var jacky = new Person({
   *     name: "Jacky",
   *     age: 35
   * });
   *
   * alert(jacky.getAge());      // alerts 35
   * alert(jacky.getGender());   // alerts "Male"
   *
   * jacky.walk(10);             // alerts "Jacky is walking 10 steps"
   *
   * jacky.setName("Mr. Nguyen");
   * alert(jacky.getName());     // alerts "Mr. Nguyen"
   *
   * jacky.walk(10);             // alerts "Mr. Nguyen is walking 10 steps"
   * </code></pre>
   *
   * <p>Notice that we changed the class constructor to invoke this.initConfig() and pass in the provided config object.
   * Two key things happened:</p>
   *
   * <ul>
   * <li>The provided config object when the class is instantiated is recursively merged with the default config object.</li>
   * <li>All corresponding setter methods are called with the merged values.</li>
   * </ul>
   *
   *
   * <p>Beside storing the given values, throughout the frameworks, setters generally have two key responsibilities:</p>
   *
   * <ul>
   * <li>Filtering / validation / transformation of the given value before it's actually stored within the instance.</li>
   * <li>Notification (such as firing events) / post-processing after the value has been set, or changed from a
   * previous value.</li>
   * </ul>
   *
   *
   * <p>By standardize this common pattern, the default generated setters provide two extra template methods that you
   * can put your own custom logics into, i.e: an "applyFoo" and "updateFoo" method for a "foo" config item, which are
   * executed before and after the value is actually set, respectively. Back to the example class, let's validate that
   * age must be a valid positive number, and fire an 'agechange' if the value is modified.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.sample.Person', {
   *     config: {
   *         // ...
   *     },
   *
   *     constructor: {
   *         // ...
   *     },
   *
   *     applyAge: function(age) {
   *         if (typeof age !== 'number' || age &lt; 0) {
   *             console.warn("Invalid age, must be a positive number");
   *             return;
   *         }
   *
   *         return age;
   *     },
   *
   *     updateAge: function(newAge, oldAge) {
   *         // age has changed from "oldAge" to "newAge"
   *         this.fireEvent('agechange', this, newAge, oldAge);
   *     }
   *
   *     // ...
   * });
   *
   * var jacky = new Person({
   *     name: "Jacky",
   *     age: 'invalid'
   * });
   *
   * alert(jacky.getAge());      // alerts 0
   *
   * alert(jacky.setAge(-100));  // alerts 0
   * alert(jacky.getAge());      // alerts 0
   *
   * alert(jacky.setAge(35));    // alerts 0
   * alert(jacky.getAge());      // alerts 35
   * </code></pre>
   *
   * <p>In other words, when leveraging the config feature, you mostly never need to define setter and getter methods
   * explicitly. Instead, "apply<em>" and "update</em>" methods should be implemented where necessary. Your code will be
   * consistent throughout and only contain the minimal logic that you actually care about.</p>
   *
   * <p>When it comes to inheritance, the default config of the parent class is automatically, recursively merged with
   * the child's default config. The same applies for mixins.</p>
   */
  public native function get config():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set config(value:Object):void;

  [ExtConfig]
  /**
   * The parent class that this class extends. For example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Person', {
   *     say: function(text) { alert(text); }
   * });
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Developer', {
   *     extend: 'Person',
   *     say: function(text) { this.callParent(["print "+text]); }
   * });
   * </code></pre>
   */
  public native function get extend():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set extend(value:String):void;

  [ExtConfig]
  /**
   * List of inheritable static methods for this class.
   * Otherwise just like <a href="#!/api/Ext.Class-cfg-statics" rel="Ext.Class-cfg-statics" class="docClass">statics</a> but subclasses inherit these methods.
   */
  public native function get inheritableStatics():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inheritableStatics(value:Object):void;

  [ExtConfig]
  /**
   * List of classes to mix into this class. For example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('CanSing', {
   *      sing: function() {
   *          alert("For he's a jolly good fellow...")
   *      }
   * });
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Musician', {
   *      mixins: ['CanSing']
   * })
   * </code></pre>
   *
   * <p>In this case the Musician class will get a <code>sing</code> method from CanSing mixin.</p>
   *
   * <p>But what if the Musician already has a <code>sing</code> method? Or you want to mix
   * in two classes, both of which define <code>sing</code>?  In such a cases it's good
   * to define mixins as an object, where you assign a name to each mixin:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Musician', {
   *      mixins: {
   *          canSing: 'CanSing'
   *      },
   *
   *      sing: function() {
   *          // delegate singing operation to mixin
   *          this.mixins.canSing.sing.call(this);
   *      }
   * })
   * </code></pre>
   *
   * <p>In this case the <code>sing</code> method of Musician will overwrite the
   * mixed in <code>sing</code> method. But you can access the original mixed in method
   * through special <code>mixins</code> property.</p>
   */
  public native function get mixins():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set mixins(value:*):void;

  [ExtConfig("override")]
  /**
   * Overrides members of the specified <code>target</code> class.
   *
   * <p><strong>NOTE:</strong> the overridden class must have been defined using
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a> in order to use the <code>override</code> config.</p>
   *
   * <p>Methods defined on the overriding class will not automatically call the methods of
   * the same name in the ancestor class chain.  To call the parent's method of the
   * same name you must call <a href="#!/api/Ext.Base-method-callParent" rel="Ext.Base-method-callParent" class="docClass">callParent</a>.  To skip the
   * method of the overridden class and call its parent you will instead call
   * <a href="#!/api/Ext.Base-method-callSuper" rel="Ext.Base-method-callSuper" class="docClass">callSuper</a>.</p>
   *
   * <p>See <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a> for additional usage examples.</p>
   */
  public native function get override_():String;

  [ExtConfig("override")]
  /**
   * @private
   */
  public native function set override_(value:String):void;

  [ExtConfig]
  /**
   * Allows setting config values for a class based on specific platforms. The value
   * of this config is an object whose properties are "rules" and whose values are
   * objects containing config values.
   *
   * <p>For example:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.view.Foo', {
   *      extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
   *
   *      platformConfig: {
   *          desktop: {
   *              title: 'Some Rather Descriptive Title'
   *          },
   *
   *          '!desktop': {
   *              title: 'Short Title'
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>In the above, "desktop" and "!desktop" are (mutually exclusive) rules. Whichever
   * evaluates to <code>true</code> will have its configs applied to the class. In this case, only
   * the "title" property, but the object can contain any number of config properties.
   * In this case, the <code>platformConfig</code> is evaluated as part of the class and there is
   * no cost for each instance created.</p>
   *
   * <p>The rules are evaluated expressions in the context of the platform tags contained
   * in <code><a href="#!/api/Ext-property-platformTags" rel="Ext-property-platformTags" class="docClass">Ext.platformTags</a></code>. Any properties of that object are
   * implicitly usable (as shown above).</p>
   *
   * <p>If a <code>platformConfig</code> specifies a config value, it will replace any values declared
   * on the class itself.</p>
   *
   * <p>Use of <code>platformConfig</code> on instances is handled by the config system when classes
   * call <code><a href="#!/api/Ext.Base-method-initConfig" rel="Ext.Base-method-initConfig" class="docClass">initConfig</a></code>. For example:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'panel',
   *
   *      platformConfig: {
   *          desktop: {
   *              title: 'Some Rather Descriptive Title'
   *          },
   *
   *          '!desktop': {
   *              title: 'Short Title'
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>The following is equivalent to the above:</p>
   *
   * <pre><code> if (Ext.platformTags.desktop) {
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *          xtype: 'panel',
   *          title: 'Some Rather Descriptive Title'
   *      });
   *  } else {
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *          xtype: 'panel',
   *          title: 'Short Title'
   *      });
   *  }
   * </code></pre>
   *
   * <p>To adjust configs based on dynamic conditions, see <code><a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a></code>.</p>
   */
  public native function get platformConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set platformConfig(value:Object):void;

  [ExtConfig]
  /**
   * The <code>privates</code> config is a list of methods intended to be used internally by the
   * framework.  Methods are placed in a <code>privates</code> block to prevent developers from
   * accidentally overriding framework methods in custom classes.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Computer', {
   *     privates: {
   *         runFactory: function(brand) {
   *             // internal only processing of brand passed to factory
   *             this.factory(brand);
   *         }
   *     },
   *
   *     factory: function (brand) {}
   * });
   * </code></pre>
   *
   * <p>In order to override a method from a <code>privates</code> block, the overridden method must
   * also be placed in a <code>privates</code> block within the override class.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Override.Computer', {
   *     override: 'Computer',
   *     privates: {
   *         runFactory: function() {
   *             // overriding logic
   *         }
   *     }
   * });
   * </code></pre>
   */
  public native function get privates():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set privates(value:Object):void;

  [ExtConfig]
  /**
   * List of classes that have to be loaded before instantiating this class.
   * For example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Mother', {
   *     requires: ['Child'],
   *     giveBirth: function() {
   *         // we can be sure that child class is available.
   *         return new Child();
   *     }
   * });
   * </code></pre>
   */
  public native function get requires():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set requires(value:Array):void;

  [ExtConfig]
  /**
   * When set to true, the class will be instantiated as singleton.  For example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Logger', {
   *     singleton: true,
   *     log: function(msg) {
   *         console.log(msg);
   *     }
   * });
   *
   * Logger.log('Hello');
   * </code></pre>
   */
  public native function get singleton():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set singleton(value:Boolean):void;

  [ExtConfig]
  /**
   * List of static methods for this class. For example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Computer', {
   *      statics: {
   *          factory: function(brand) {
   *              // 'this' in static methods refer to the class itself
   *              return new this(brand);
   *          }
   *      },
   *
   *      constructor: function() { ... }
   * });
   *
   * var dellComputer = Computer.factory('Dell');
   * </code></pre>
   */
  public native function get statics():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set statics(value:Object):void;

  [ExtConfig]
  /**
   * List of optional classes to load together with this class. These aren't neccessarily loaded before
   * this class is created, but are guaranteed to be available before <a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a> listeners are
   * invoked. For example:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Mother', {
   *     uses: ['Child'],
   *     giveBirth: function() {
   *         // This code might, or might not work:
   *         // return new Child();
   *
   *         // Instead use <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>() to load the class at the spot if not loaded already:
   *         return <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('Child');
   *     }
   * });
   * </code></pre>
   */
  public native function get uses():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set uses(value:Array):void;

  [ExtConfig]
  /**
   * <strong>Note:</strong> Only applies to <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> derived classes when used as
   * a config in <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>.
   *
   * <p>This property provides a shorter alternative to creating objects than using a full
   * class name. Using <code>xtype</code> is the most common way to define component instances,
   * especially in a container. For example, the items in a form containing text fields
   * could be created explicitly like so:</p>
   *
   * <pre><code> items: [
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *          fieldLabel: 'Foo'
   *      }),
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *          fieldLabel: 'Bar'
   *      }),
   *      <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Number" rel="Ext.form.field.Number" class="docClass">Ext.form.field.Number</a>', {
   *          fieldLabel: 'Num'
   *      })
   *  ]
   * </code></pre>
   *
   * <p>But by using <code>xtype</code>, the above becomes:</p>
   *
   * <pre><code> items: [
   *      {
   *          xtype: 'textfield',
   *          fieldLabel: 'Foo'
   *      },
   *      {
   *          xtype: 'textfield',
   *          fieldLabel: 'Bar'
   *      },
   *      {
   *          xtype: 'numberfield',
   *          fieldLabel: 'Num'
   *      }
   *  ]
   * </code></pre>
   *
   * <p>When the <code>xtype</code> is common to many items, <a href="#!/api/Ext.container.Container-cfg-defaultType" rel="Ext.container.Container-cfg-defaultType" class="docClass">Ext.container.Container.defaultType</a>
   * is another way to specify the <code>xtype</code> for all items that don't have an explicit <code>xtype</code>:</p>
   *
   * <pre><code> defaultType: 'textfield',
   *  items: [
   *      { fieldLabel: 'Foo' },
   *      { fieldLabel: 'Bar' },
   *      { fieldLabel: 'Num', xtype: 'numberfield' }
   *  ]
   * </code></pre>
   *
   * <p>Each member of the <code>items</code> array is now just a "configuration object". These objects
   * are used to create and configure component instances. A configuration object can be
   * manually used to instantiate a component using <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>:</p>
   *
   * <pre><code> var text1 = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *      fieldLabel: 'Foo'
   *  });
   *
   *  // or alternatively:
   *
   *  var text1 = <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>({
   *      xtype: 'textfield',
   *      fieldLabel: 'Foo'
   *  });
   * </code></pre>
   *
   * <p>This conversion of configuration objects into instantiated components is done when
   * a container is created as part of its {Ext.container.AbstractContainer#initComponent}
   * process. As part of the same process, the <code>items</code> array is converted from its raw
   * array form into a <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a> instance.</p>
   *
   * <p>You can define your own <code>xtype</code> on a custom <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">component</a> by specifying
   * the <code>xtype</code> property in <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>. For example:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.PressMeButton', {
   *     extend: '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>',
   *     xtype: 'pressmebutton',
   *     text: 'Press Me'
   * });
   * </code></pre>
   *
   * <p>Care should be taken when naming an <code>xtype</code> in a custom component because there is
   * a single, shared scope for all xtypes. Third part components should consider using
   * a prefix to avoid collisions.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.form.CoolButton', {
   *     extend: '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>',
   *     xtype: 'ux-coolbutton',
   *     text: 'Cool!'
   * });
   * </code></pre>
   *
   * <p>See <a href="#!/api/Ext.enums.Widget" rel="Ext.enums.Widget" class="docClass">Ext.enums.Widget</a> for list of all available xtypes.</p>
   */
  public native function get xtype():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set xtype(value:String):void;
}
}