package ext {

[Native("Ext.Error", require)]
/**
 * A helper class for the native JavaScript Error object that adds a few useful capabilities for handling
 * errors in an application. When you use <a href="#!/api/Ext.Error" rel="Ext.Error" class="docClass">Ext.Error</a> to <a href="#!/api/Ext.Error-static-method-raise" rel="Ext.Error-static-method-raise" class="docClass">raise</a> an error from within any class that
 * uses the Class System, the Error class can automatically add the source class and method from which
 * the error was raised. It also includes logic to automatically log the error to the console, if available,
 * with additional metadata about the error. In all cases, the error will always be thrown at the end so that
 * execution will halt.
 *
 * <p><a href="#!/api/Ext.Error" rel="Ext.Error" class="docClass">Ext.Error</a> also offers a global error <a href="#!/api/Ext.Error-static-method-handle" rel="Ext.Error-static-method-handle" class="docClass">handling</a> method that can be overridden in order to
 * handle application-wide errors in a single spot. You can optionally <a href="#!/api/Ext.Error-static-property-ignore" rel="Ext.Error-static-property-ignore" class="docClass">ignore</a> errors altogether,
 * although in a real application it's usually a better idea to override the handling function and perform
 * logging or some other method of reporting the errors in a way that is meaningful to the application.</p>
 *
 * <p>At its simplest you can simply raise an error as a simple string from within any code:</p>
 *
 * <p>Example usage:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-raise" rel="Ext-method-raise" class="docClass">Ext.raise</a>('Something bad happened!');
 * </code></pre>
 *
 * <p>If raised from plain JavaScript code, the error will be logged to the console (if available) and the message
 * displayed. In most cases however you'll be raising errors from within a class, and it may often be useful to add
 * additional metadata about the error being raised.  The <a href="#!/api/Ext.Error-static-method-raise" rel="Ext.Error-static-method-raise" class="docClass">raise</a> method can also take a config object.
 * In this form the <code>msg</code> attribute becomes the error description, and any other data added to the config gets
 * added to the error object and, if the console is available, logged to the console for inspection.</p>
 *
 * <p>Example usage:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.Foo', {
 *     doSomething: function(option){
 *         if (someCondition === false) {
 *             <a href="#!/api/Ext-method-raise" rel="Ext-method-raise" class="docClass">Ext.raise</a>({
 *                 msg: 'You cannot do that!',
 *                 option: option,   // whatever was passed into the method
 *                 'error code': 100 // other arbitrary info
 *             });
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <p>If a console is available (that supports the <code>console.dir</code> function) you'll see console output like:</p>
 *
 * <pre><code>An error was raised with the following data:
 * option:         Object { foo: "bar"}
 *     foo:        "bar"
 * error code:     100
 * msg:            "You cannot do that!"
 * sourceClass:   "Ext.Foo"
 * sourceMethod:  "doSomething"
 *
 * uncaught exception: You cannot do that!
 * </code></pre>
 *
 * <p>As you can see, the error will report exactly where it was raised and will include as much information as the
 * raising code can usefully provide.</p>
 *
 * <p>If you want to handle all application errors globally you can simply override the static <a href="#!/api/Ext.Error-static-method-handle" rel="Ext.Error-static-method-handle" class="docClass">handle</a> method
 * and provide whatever handling logic you need. If the method returns true then the error is considered handled
 * and will not be thrown to the browser. If anything but true is returned then the error will be thrown normally.</p>
 *
 * <p>Example usage:</p>
 *
 * <pre><code><a href="#!/api/Ext.Error-static-method-handle" rel="Ext.Error-static-method-handle" class="docClass">Ext.Error.handle</a> = function(err) {
 *     if (err.someProperty == 'NotReallyAnError') {
 *         // maybe log something to the application here if applicable
 *         return true;
 *     }
 *     // any non-true return value (including none) will cause the error to be thrown
 * }
 * </code></pre>
 */
public class ExtError {
  /**
   * Static flag that can be used to globally disable error reporting to the browser if set to true
   * (defaults to false). Note that if you ignore Ext errors it's likely that some other code may fail
   * and throw a native JavaScript error thereafter, so use with caution. In most cases it will probably
   * be preferable to supply a custom error <a href="#!/api/Ext.Error-static-method-handle" rel="Ext.Error-static-method-handle" class="docClass">handling</a> function instead.
   *
   * <p>Example usage:</p>
   *
   * <pre><code><a href="#!/api/Ext.Error-static-property-ignore" rel="Ext.Error-static-property-ignore" class="docClass">Ext.Error.ignore</a> = true;
   * </code></pre>
   * @default false
   */
  public static native function get ignore():Boolean;

  /**
   * @private
   */
  public static native function set ignore(value:Boolean):void;

  /**
   * Globally handle any Ext errors that may be raised, optionally providing custom logic to
   * handle different errors individually. Return true from the function to bypass throwing the
   * error to the browser, otherwise the error will be thrown and execution will halt.
   *
   * <p>Example usage:</p>
   *
   * <pre><code><a href="#!/api/Ext.Error-static-method-handle" rel="Ext.Error-static-method-handle" class="docClass">Ext.Error.handle</a> = function(err) {
   *     if (err.someProperty == 'NotReallyAnError') {
   *         // maybe log something to the application here if applicable
   *         return true;
   *     }
   *     // any non-true return value (including none) will cause the error to be thrown
   * }
   * </code></pre>
   * @param err The error being raised. It will contain any attributes that were originally
   * raised with it, plus properties about the method and class from which the error originated
   * (if raised from a class that uses the Class System).
   */
  public static native function handle(err:Object):void;

  [Deprecated(replacement = "raise", since = "6.0.0")]
  /**
   * This method is called internally by <a href="#!/api/Ext-method-raise" rel="Ext-method-raise" class="docClass">Ext.raise</a>. Application code should
   * call <a href="#!/api/Ext-method-raise" rel="Ext-method-raise" class="docClass">Ext.raise</a> instead of calling this method directly.
   */
  public static native function raise(err:Object):void;
}
}