package ext {

[Uses("ext.SLoader")]
[Native("Ext.Loader", require)]
/**
 * This class provides dynamic loading support for JavaScript classes. Application code
 * does not typically need to call <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> except perhaps to configure path mappings
 * when not using <a href="http://www.sencha.com/products/sencha-cmd/">Sencha Cmd</a>.
 *
 * <pre><code> <a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">Ext.Loader.setPath</a>('MyApp', 'app');
 * </code></pre>
 *
 * <p>When using Sencha Cmd, this is handled by the "bootstrap" provided by the application
 * build script and such configuration is not necessary.</p>
 *
 * <h1>Typical Usage</h1>
 *
 * <p>The <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> is most often used behind the scenes to satisfy class references in
 * class declarations. Like so:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.Main', {
 *      extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
 *
 *      mixins: [
 *          'MyApp.util.Mixin'
 *      ],
 *
 *      requires: [
 *          '<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>'
 *      ],
 *
 *      uses: [
 *          'MyApp.util.Stuff'
 *      ]
 *  });
 * </code></pre>
 *
 * <p>In all of these cases, <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> is used internally to resolve these class names
 * and ensure that the necessary class files are loaded.</p>
 *
 * <p>During development, these files are loaded individually for optimal debugging. For a
 * production use, <a href="http://www.sencha.com/products/sencha-cmd/">Sencha Cmd</a> will replace
 * all of these strings with the actual resolved class references because it ensures that
 * the classes are all contained in the build in the correct order. In development, these
 * files will not be loaded until the <code>MyApp.view.Main</code> class indicates they are needed
 * as shown above.</p>
 *
 * <h1>Loading Classes</h1>
 *
 * <p>You can also use <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> directly to load classes or files. The simplest form of
 * use is <code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a></code>.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>('MyApp.view.Main', function () {
 *      // On callback, the MyApp.view.Main class is now loaded
 *
 *      var view = new MyApp.view.Main();
 *  });
 * </code></pre>
 *
 * <p>You can alternatively require classes by alias or wildcard.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>('widget.window');
 *
 * <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>(['widget.window', 'layout.border', '<a href="#!/api/Ext.data.Connection" rel="Ext.data.Connection" class="docClass">Ext.data.Connection</a>']);
 *
 * <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>(['widget.*', 'layout.*', 'Ext.data.*']);
 * </code></pre>
 *
 * <p>The callback function is optional.</p>
 *
 * <p><strong>Note</strong> Using <code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a></code> at global scope will cause <code><a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a></code> and
 * <code><a href="#!/api/Ext.app.Application-method-launch" rel="Ext.app.Application-method-launch" class="docClass">Ext.app.Application.launch</a></code> methods to be deferred until the required classes
 * are loaded. It is these cases where the callback function is most often unnecessary.</p>
 *
 * <h2>Using Excludes</h2>
 *
 * <p>Alternatively, you can exclude what you don't need:</p>
 *
 * <pre><code>// Include everything except Ext.tree.*
 * <a href="#!/api/Ext-method-exclude" rel="Ext-method-exclude" class="docClass">Ext.exclude</a>('Ext.tree.*').require('*');
 *
 * // Include all widgets except widget.checkbox* (this will exclude
 * // widget.checkbox, widget.checkboxfield, widget.checkboxgroup, etc.)
 * <a href="#!/api/Ext-method-exclude" rel="Ext-method-exclude" class="docClass">Ext.exclude</a>('widget.checkbox*').require('widget.*');
 * </code></pre>
 *
 * <h1>Dynamic Instantiation</h1>
 *
 * <p>Another feature enabled by <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> is instantiation using class names or aliases.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> var win = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
 *      xtype: 'window',
 *
 *      // or
 *      // xclass: '<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>'
 *
 *      title: 'Hello'
 *  });
 * </code></pre>
 *
 * <p>This form of creation can be useful if the type to create (<code>window</code> in the above) is
 * not known statically. Internally, <code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></code> may need to <em>synchronously</em>
 * load the desired class and its requirements. Doing this will generate a warning in
 * the console:</p>
 *
 * <pre><code> [<a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a>] Synchronously loading '<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>'...
 * </code></pre>
 *
 * <p>If you see these in your debug console, you should add the indicated class(es) to the
 * appropriate <code>requires</code> array (as above) or make an <code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a></code> call.</p>
 *
 * <p><strong>Note</strong> Using <code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></code> has some performance overhead and is best reserved
 * for cases where the target class is not known until run-time.</p>
 */
public const Loader:ext.SLoader;
}