package ext {

[Uses("ext.Base")]
[Native("Ext.Mixin", require)]
/**
 * This class is a base class for mixins. These are classes that extend this class and are
 * designed to be used as a <code>mixin</code> by user code.
 *
 * <p>It provides mixins with the ability to "hook" class methods of the classes in to which
 * they are mixed. For example, consider the <code>destroy</code> method pattern. If a mixin class
 * had cleanup requirements, it would need to be called as part of <code>destroy</code>.</p>
 *
 * <p>Starting with a basic class we might have:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Base', {
 *      destroy: function () {
 *          console.log('B');
 *          // cleanup
 *      }
 *  });
 * </code></pre>
 *
 * <p>A derived class would look like this:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Derived', {
 *      extend: 'Foo.bar.Base',
 *
 *      destroy: function () {
 *          console.log('D');
 *          // more cleanup
 *
 *          this.callParent(); // let Foo.bar.Base cleanup as well
 *      }
 *  });
 * </code></pre>
 *
 * <p>To see how using this class help, start with a "normal" mixin class that also needs to
 * cleanup its resources. These mixins must be called explicitly by the classes that use
 * them. For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Util', {
 *      destroy: function () {
 *          console.log('U');
 *      }
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Derived', {
 *      extend: 'Foo.bar.Base',
 *
 *      mixins: {
 *          util: 'Foo.bar.Util'
 *      },
 *
 *      destroy: function () {
 *          console.log('D');
 *          // more cleanup
 *
 *          this.mixins.util.destroy.call(this);
 *
 *          this.callParent(); // let Foo.bar.Base cleanup as well
 *      }
 *  });
 *
 *  var obj = new Foo.bar.Derived();
 *
 *  obj.destroy();
 *  // logs D then U then B
 * </code></pre>
 *
 * <p>This class is designed to solve the above in simpler and more reliable way.</p>
 *
 * <h2>mixinConfig</h2>
 *
 * <p>Using <code>mixinConfig</code> the mixin class can provide "before" or "after" hooks that do not
 * involve the derived class implementation. This also means the derived class cannot
 * adjust parameters to the hook methods.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Util', {
 *      extend: '<a href="#!/api/Ext.Mixin" rel="Ext.Mixin" class="docClass">Ext.Mixin</a>',
 *
 *      mixinConfig: {
 *          after: {
 *              destroy: 'destroyUtil'
 *          }
 *      },
 *
 *      destroyUtil: function () {
 *          console.log('U');
 *      }
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Class', {
 *      mixins: {
 *          util: 'Foo.bar.Util'
 *      },
 *
 *      destroy: function () {
 *          console.log('D');
 *      }
 *  });
 *
 *  var obj = new Foo.bar.Derived();
 *
 *  obj.destroy();
 *  // logs D then U
 * </code></pre>
 *
 * <p> If the destruction should occur in the other order, you can use <code>before</code>:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Util', {
 *      extend: '<a href="#!/api/Ext.Mixin" rel="Ext.Mixin" class="docClass">Ext.Mixin</a>',
 *
 *      mixinConfig: {
 *          before: {
 *              destroy: 'destroyUtil'
 *          }
 *      },
 *
 *      destroyUtil: function () {
 *          console.log('U');
 *      }
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Class', {
 *      mixins: {
 *          util: 'Foo.bar.Util'
 *      },
 *
 *      destroy: function () {
 *          console.log('D');
 *      }
 *  });
 *
 *  var obj = new Foo.bar.Derived();
 *
 *  obj.destroy();
 *  // logs U then D
 * </code></pre>
 *
 * <h3>Chaining</h3>
 *
 * <p>One way for a mixin to provide methods that act more like normal inherited methods is
 * to use an <code>on</code> declaration. These methods will be injected into the <code>callParent</code> chain
 * between the derived and superclass. For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Util', {
 *      extend: '<a href="#!/api/Ext.Mixin" rel="Ext.Mixin" class="docClass">Ext.Mixin</a>',
 *
 *      mixinConfig: {
 *          on: {
 *              destroy: function () {
 *                  console.log('M');
 *              }
 *          }
 *      }
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Base', {
 *      destroy: function () {
 *          console.log('B');
 *      }
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Derived', {
 *      extend: 'Foo.bar.Base',
 *
 *      mixins: {
 *          util: 'Foo.bar.Util'
 *      },
 *
 *      destroy: function () {
 *          this.callParent();
 *          console.log('D');
 *      }
 *  });
 *
 *  var obj = new Foo.bar.Derived();
 *
 *  obj.destroy();
 *  // logs M then B then D
 * </code></pre>
 *
 * <p>As with <code>before</code> and <code>after</code>, the value of <code>on</code> can be a method name.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Util', {
 *      extend: '<a href="#!/api/Ext.Mixin" rel="Ext.Mixin" class="docClass">Ext.Mixin</a>',
 *
 *      mixinConfig: {
 *          on: {
 *              destroy: 'onDestroy'
 *          }
 *      }
 *
 *      onDestroy: function () {
 *          console.log('M');
 *      }
 *  });
 * </code></pre>
 *
 * <p>Because this technique leverages <code>callParent</code>, the derived class controls the time and
 * parameters for the call to all of its bases (be they <code>extend</code> or <code>mixin</code> flavor).</p>
 *
 * <h3>Derivations</h3>
 *
 * <p>Some mixins need to process class extensions of their target class. To do this you can
 * define an <code>extended</code> method like so:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo.bar.Util', {
 *      extend: '<a href="#!/api/Ext.Mixin" rel="Ext.Mixin" class="docClass">Ext.Mixin</a>',
 *
 *      mixinConfig: {
 *          extended: function (baseClass, derivedClass, classBody) {
 *              // This function is called whenever a new "derivedClass" is created
 *              // that extends a "baseClass" in to which this mixin was mixed.
 *          }
 *      }
 *  });
 * </code></pre>
 */
public class Mixin extends ext.Base implements ext.IMixin {}
}