package ext {

[Uses("ext.Base")]
[Native("Ext.Promise", require)]
/**
 * This class provides an API compatible implementation of the ECMAScript 6 Promises API
 * (providing an implementation as necessary for browsers that do not natively support the
 * <code>Promise</code> class).
 *
 * <p>This class will use the native <code>Promise</code> implementation if one is available. The
 * native implementation, while standard, does not provide all of the features of the
 * Ext JS Promises implementation.</p>
 *
 * <p>To use the Ext JS enhanced Promises implementation, see <code><a href="#!/api/Ext.Deferred" rel="Ext.Deferred" class="docClass">Ext.Deferred</a></code> for
 * creating enhanced promises and additional static utility methods.</p>
 *
 * <p>Typical usage:</p>
 *
 * <pre><code> function getAjax (url) {
 *      // The function passed to <a href="#!/api/Ext.Promise" rel="Ext.Promise" class="docClass">Ext.Promise</a>() is called immediately to start
 *      // the asynchronous action.
 *      //
 *      return new <a href="#!/api/Ext.Promise" rel="Ext.Promise" class="docClass">Ext.Promise</a>(function (resolve, reject) {
 *          <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
 *              url: url,
 *
 *              success: function (response) {
 *                  // Use the provided "resolve" method to deliver the result.
 *                  //
 *                  resolve(response.responseText);
 *              },
 *
 *              failure: function (response) {
 *                  // Use the provided "reject" method to deliver error message.
 *                  //
 *                  reject(response.status);
 *              }
 *          });
 *      });
 *  }
 *
 *  getAjax('http://stuff').then(function (content) {
 *      // content is responseText of ajax response
 *  });
 * </code></pre>
 *
 * <p>To adapt the Ext JS <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">store</a></code> to use a Promise, you might do
 * something like this:</p>
 *
 * <pre><code> loadCompanies: function() {
 *      var companyStore = this.companyStore;
 *
 *      return new <a href="#!/api/Ext.Promise" rel="Ext.Promise" class="docClass">Ext.Promise</a>(function (resolve, reject) {
 *          companyStore.load({
 *              callback: function(records, operation, success) {
 *                  if (success) {
 *                      // Use the provided "resolve" method  to drive the promise:
 *                      resolve(records);
 *                  }
 *                  else {
 *                      // Use the provided "reject" method  to drive the promise:
 *                      reject("Error loading Companies.");
 *                  }
 *              }
 *          });
 *      });
 *  }
 * </code></pre>
 */
public class Promise extends ext.Base {
  /**
   * Returns a new Promise that will only resolve once all the specified
   * <code>promisesOrValues</code> have resolved.
   *
   * <p>The resolution value will be an Array containing the resolution value of each
   * of the <code>promisesOrValues</code>.</p>
   * @param promisesOrValues An Array of values
   * or Promises, or a Promise of an Array of values or Promises.
   * @return A Promise of an Array of the resolved values.
   */
  public static native function all(promisesOrValues:*):ext.Promise;

  /**
   * Convenience method that returns a new Promise rejected with the specified
   * reason.
   * @param reason Rejection reason.
   * @return The rejected Promise.
   */
  public static native function reject(reason:Error):ext.Promise;

  /**
   * Returns a new Promise that either
   *
   * <ul>
   * <li>Resolves immediately for the specified value, or</li>
   * <li>Resolves or rejects when the specified promise (or third-party Promise or
   * then()-able) is resolved or rejected.</li>
   * </ul>
   * @param promiseOrValue A Promise (or third-party Promise or then()-able)
   * or value.
   * @return A Promise of the specified Promise or value.
   */
  public static native function resolve(promiseOrValue:*):ext.Promise;
}
}