package ext {
import ext.container.Container;

[Uses("ext.Base")]
[Native]
/**
 * Provides searching of Components within <a href="#!/api/Ext.ComponentManager" rel="Ext.ComponentManager" class="docClass">Ext.ComponentManager</a> (globally) or a specific
 * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> on the document with a similar syntax to a CSS selector.
 * Returns Array of matching Components, or empty Array.
 *
 * <h2>Basic Component lookup</h2>
 *
 * <p>Components can be retrieved by using their <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">xtype</a>:</p>
 *
 * <ul>
 * <li><code>component</code></li>
 * <li><code>gridpanel</code></li>
 * </ul>
 *
 *
 * <p>Matching by <code>xtype</code> matches inherited types, so in the following code, the previous field
 * <em>of any type which inherits from <code>TextField</code></em> will be found:</p>
 *
 * <pre><code>prevField = myField.previousNode('textfield');
 * </code></pre>
 *
 * <p>To match only the exact type, pass the "shallow" flag by adding <code>(true)</code> to xtype
 * (See Component's <a href="#!/api/Ext.Component-method-isXType" rel="Ext.Component-method-isXType" class="docClass">isXType</a> method):</p>
 *
 * <pre><code>prevTextField = myField.previousNode('textfield(true)');
 * </code></pre>
 *
 * <p>You can search Components by their <code>id</code> or <code>itemId</code> property, prefixed with a #:</p>
 *
 * <pre><code>#myContainer
 * </code></pre>
 *
 * <p>Component <code>xtype</code> and <code>id</code> or <code>itemId</code> can be used together to avoid possible
 * id collisions between Components of different types:</p>
 *
 * <pre><code>panel#myPanel
 * </code></pre>
 *
 * <p>When Component's <code>id</code> or <code>xtype</code> contains dots, you can escape them in your selector:</p>
 *
 * <pre><code>my\.panel#myPanel
 * </code></pre>
 *
 * <p>Keep in mind that JavaScript treats the backslash character in a special way, so you
 * need to escape it, too, in the actual code:</p>
 *
 * <pre><code>var myPanel = <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('my\\.panel#myPanel');
 * </code></pre>
 *
 * <h2>Traversing Component tree</h2>
 *
 * <p>Components can be found by their relation to other Components. There are several
 * relationship operators, mostly taken from CSS selectors:</p>
 *
 * <ul>
 * <li><strong><code>E F</code></strong> All descendant Components of E that match F</li>
 * <li><strong><code>E &gt; F</code></strong> All direct children Components of E that match F</li>
 * <li><strong><code>E ^ F</code></strong> All parent Components of E that match F</li>
 * </ul>
 *
 *
 * <p>Expressions between relationship operators are matched left to right, i.e. leftmost
 * selector is applied first, then if one or more matches are found, relationship operator
 * itself is applied, then next selector expression, etc. It is possible to combine
 * relationship operators in complex selectors:</p>
 *
 * <pre><code>window[title="Input form"] textfield[name=login] ^ form &gt; button[action=submit]
 * </code></pre>
 *
 * <p>That selector can be read this way: Find a window with title "Input form", in that
 * window find a TextField with name "login" at any depth (including subpanels and/or
 * FieldSets), then find an <code><a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a></code> that is a parent of the TextField, and in
 * that form find a direct child that is a button with custom property <code>action</code> set to
 * value "submit".</p>
 *
 * <p>Whitespace on both sides of <code>^</code> and <code>&gt;</code> operators is non-significant, i.e. can be
 * omitted, but usually is used for clarity.</p>
 *
 * <h2>Searching by Component attributes</h2>
 *
 * <p>Components can be searched by their object property values (attributes). To do that,
 * use attribute matching expression in square brackets:</p>
 *
 * <ul>
 * <li><code>component[disabled]</code> - matches any Component that has <code>disabled</code> property with
 * any truthy (non-empty, not <code>false</code>) value.</li>
 * <li><code>panel[title="Test"]</code> - matches any Component that has <code>title</code> property set to
 * "Test". Note that if the value does not contain spaces, the quotes are optional.</li>
 * </ul>
 *
 *
 * <p>Attributes can use any of the following operators to compare values:
 * <code>=</code>, <code>!=</code>, <code>^=</code>, <code>$=</code>, <code>*=</code>, <code>%=</code>, <code>|=</code> and <code>~=</code>.</p>
 *
 * <p>Prefixing the attribute name with an at sign <code>&#64;</code> means that the property must be
 * the object's <code>ownProperty</code>, not a property from the prototype chain.</p>
 *
 * <p>Specifications like <code>[propName]</code> check that the property is a truthy value. To check
 * that the object has an <code>ownProperty</code> of a certain name, regardless of the value use
 * the form <code>[?propName]</code>.</p>
 *
 * <p>The specified value is coerced to match the type of the property found in the
 * candidate Component using <a href="#!/api/Ext-method-coerce" rel="Ext-method-coerce" class="docClass">Ext.coerce</a>.</p>
 *
 * <p>If you need to find Components by their <code>itemId</code> property, use the <code>#id</code> form; it will
 * do the same as <code>[itemId=id]</code> but is easier to read.</p>
 *
 * <p>If you need to include a metacharacter like (, ), [, ], etc., in the query, escape it
 * by prefixing it with a backslash:</p>
 *
 * <pre><code> var component = <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('[myProperty=\\[foo\\]]');
 * </code></pre>
 *
 * <h2>Attribute matching operators</h2>
 *
 * <p>The '=' operator will return the results that <strong>exactly</strong> match the
 * specified object property (attribute):</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel[cls=my-cls]');
 * </code></pre>
 *
 * <p>Will match the following Component:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>', {
 *     cls: 'my-cls'
 * });
 * </code></pre>
 *
 * <p>But will not match the following Component, because 'my-cls' is one value
 * among others:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *      cls: 'foo-cls my-cls bar-cls'
 *  });
 * </code></pre>
 *
 * <p>You can use the '~=' operator instead, it will return Components with
 * the property that <strong>exactly</strong> matches one of the whitespace-separated
 * values. This is also true for properties that only have <em>one</em> value:</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel[cls~=my-cls]');
 * </code></pre>
 *
 * <p>Will match both Components:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     cls: 'foo-cls my-cls bar-cls'
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>', {
 *     cls: 'my-cls'
 * });
 * </code></pre>
 *
 * <p>Generally, '=' operator is more suited for object properties other than
 * CSS classes, while '~=' operator will work best with properties that
 * hold lists of whitespace-separated CSS classes.</p>
 *
 * <p>The '^=' operator will return Components with specified attribute that
 * start with the passed value:</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel[title^=Sales]');
 * </code></pre>
 *
 * <p>Will match the following Component:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     title: 'Sales estimate for Q4'
 * });
 * </code></pre>
 *
 * <p>The '$=' operator will return Components with specified properties that
 * end with the passed value:</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('field[fieldLabel$=name]');
 * </code></pre>
 *
 * <p>Will match the following Component:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
 *     fieldLabel: 'Enter your name'
 * });
 * </code></pre>
 *
 * <p>The '/=' operator will return Components with specified properties that
 * match the passed regular expression:</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('button[action/="edit|save"]');
 * </code></pre>
 *
 * <p>Will match the following Components with a custom <code>action</code> property:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *      action: 'edit'
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.button.Button</a>', {
 *      action: 'save'
 * });
 * </code></pre>
 *
 * <p>When you need to use meta characters like [], (), etc. in your query, make sure
 * to escape them with back slashes:</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel[title="^Sales for Q\\[1-4\\]"]');
 * </code></pre>
 *
 * <p>The following test will find panels with their <code>ownProperty</code> collapsed being equal to
 * <code>false</code>. It will <strong>not</strong> match a collapsed property from the prototype chain.</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel[&#64;collapsed=false]');
 * </code></pre>
 *
 * <p>Member expressions from candidate Components may be tested. If the expression returns
 * a <em>truthy</em> value, the candidate Component will be included in the query:</p>
 *
 * <pre><code>var disabledFields = myFormPanel.query("{isDisabled()}");
 * </code></pre>
 *
 * <p>Such expressions are executed in Component's context, and the above expression is
 * similar to running this snippet for every Component in your application:</p>
 *
 * <pre><code> if (component.isDisabled()) {
 *      matches.push(component);
 *  }
 * </code></pre>
 *
 * <p>It is important to use only methods that are available in <strong>every</strong> Component instance
 * to avoid run time exceptions. If you need to match your Components with a custom
 * condition formula, you can augment <code><a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a></code> to provide custom matcher that
 * will return <code>false</code> by default, and override it in your custom classes:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Component', {
 *      override: '<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>',
 *      myMatcher: function() { return false; }
 *  });
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.Panel', {
 *      extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
 *      requires: ['My.Component'],     // Ensure that Component override is applied
 *      myMatcher: function(selector) {
 *          return selector === 'myPanel';
 *      }
 *  });
 * </code></pre>
 *
 * <p>After that you can use a selector with your custom matcher to find all instances
 * of <code>My.Panel</code>:</p>
 *
 * <pre><code> <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>("{myMatcher('myPanel')}");
 * </code></pre>
 *
 * <p>However if you really need to use a custom matcher, you may find it easier to implement
 * a custom Pseudo class instead (see below).</p>
 *
 * <h2>Conditional matching</h2>
 *
 * <p>Attribute matchers can be combined to select only Components that match <strong>all</strong>
 * conditions (logical AND operator):</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel[cls~=my-cls][floating=true][title$="sales data"]');
 * </code></pre>
 *
 * <p>E.g., the query above will match only a Panel-descended Component that has 'my-cls'
 * CSS class <em>and</em> is floating <em>and</em> with a title that ends with "sales data".</p>
 *
 * <p>Expressions separated with commas will match any Component that satisfies
 * <em>either</em> expression (logical OR operator):</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('field[fieldLabel^=User], field[fieldLabel*=password]');
 * </code></pre>
 *
 * <p>E.g., the query above will match any field with field label starting with "User",
 * <em>or</em> any field that has "password" in its label.</p>
 *
 * <p>If you need to include a comma in an attribute matching expression, escape it with a
 * backslash:</p>
 *
 * <pre><code><a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('field[fieldLabel^="User\\, foo"], field[fieldLabel*=password]');
 * </code></pre>
 *
 * <h2>Pseudo classes</h2>
 *
 * <p>Pseudo classes may be used to filter results in the same way as in
 * <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a>. There are five default pseudo classes:</p>
 *
 * <ul>
 * <li><code>not</code> Negates a selector.</li>
 * <li><code>first</code> Filters out all except the first matching item for a selector.</li>
 * <li><code>last</code> Filters out all except the last matching item for a selector.</li>
 * <li><code>focusable</code> Filters out all except Components which by definition and configuration are
 *    potentially able to recieve focus, regardless of their current state</li>
 * <li><code>canfocus</code> Filters out all except Components which are curently able to recieve focus.
 *    That is, they are defined and configured focusable, and they are also visible and enabled.</li>
 * <li><code>nth-child</code> Filters Components by ordinal position in the selection.</li>
 * <li><code>scrollable</code> Filters out all except Components which are scrollable.</li>
 * <li><code>visible</code> Filters out hidden Components. May test deep visibility using <code>':visible(true)'</code></li>
 * </ul>
 *
 *
 * <p>These pseudo classes can be used with other matchers or without them:</p>
 *
 * <pre><code> // Select first direct child button in any panel
 *  <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel &gt; button:first');
 *
 *  // Select last field in Profile form
 *  <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('form[title=Profile] field:last');
 *
 *  // Find first focusable Component in a panel and focus it
 *  panel.down(':canfocus').focus();
 *
 *  // Select any field that is not hidden in a form
 *  form.query('field:not(hiddenfield)');
 *
 *  // Find last scrollable Component and reset its scroll positions.
 *  tabpanel.down(':scrollable[hideMode=display]:last').getScrollable().scrollTo(0, 0);
 * </code></pre>
 *
 * <p>Pseudo class <code>nth-child</code> can be used to find any child Component by its
 * position relative to its siblings. This class' handler takes one argument
 * that specifies the selection formula as <code>Xn</code> or <code>Xn+Y</code>:</p>
 *
 * <pre><code> // Find every odd field in a form
 *  form.query('field:nth-child(2n+1)'); // or use shortcut: :nth-child(odd)
 *
 *  // Find every even field in a form
 *  form.query('field:nth-child(2n)');   // or use shortcut: :nth-child(even)
 *
 *  // Find every 3rd field in a form
 *  form.query('field:nth-child(3n)');
 * </code></pre>
 *
 * <p>Pseudo classes can be combined to further filter the results, e.g., in the
 * form example above we can modify the query to exclude hidden fields:</p>
 *
 * <pre><code> // Find every 3rd non-hidden field in a form
 *  form.query('field:not(hiddenfield):nth-child(3n)');
 * </code></pre>
 *
 * <p>Note that when combining pseudo classes, whitespace is significant, i.e.
 * there should be no spaces between pseudo classes. This is a common mistake;
 * if you accidentally type a space between <code>field</code> and <code>:not</code>, the query
 * will not return any result because it will mean "find <em>field's children
 * Components</em> that are not hidden fields...".</p>
 *
 * <h2>Custom pseudo classes</h2>
 *
 * <p>It is possible to define your own custom pseudo classes. In fact, a
 * pseudo class is just a property in <code><a href="#!/api/Ext.ComponentQuery-property-pseudos" rel="Ext.ComponentQuery-property-pseudos" class="docClass">Ext.ComponentQuery.pseudos</a></code> object
 * that defines pseudo class name (property name) and pseudo class handler
 * (property value):</p>
 *
 * <pre><code>// Function receives array and returns a filtered array.
 * Ext.ComponentQuery.pseudos.invalid = function(items) {
 *     var i = 0, l = items.length, c, result = [];
 *     for (; i &lt; l; i++) {
 *         if (!(c = items[i]).isValid()) {
 *             result.push(c);
 *         }
 *     }
 *     return result;
 * };
 *
 * var invalidFields = myFormPanel.query('field:invalid');
 * if (invalidFields.length) {
 *     invalidFields[0].getEl().scrollIntoView(myFormPanel.body);
 *     for (var i = 0, l = invalidFields.length; i &lt; l; i++) {
 *         invalidFields[i].getEl().frame("red");
 *     }
 * }
 * </code></pre>
 *
 * <p>Pseudo class handlers can be even more flexible, with a selector
 * argument used to define the logic:</p>
 *
 * <pre><code> // Handler receives array of itmes and selector in parentheses
 *  Ext.ComponentQuery.pseudos.titleRegex = function(components, selector) {
 *      var i = 0, l = components.length, c, result = [], regex = new RegExp(selector);
 *      for (; i &lt; l; i++) {
 *          c = components[i];
 *          if (c.title &amp;&amp; regex.test(c.title)) {
 *              result.push(c);
 *          }
 *      }
 *      return result;
 *  }
 *
 *  var salesTabs = tabPanel.query('panel:titleRegex("sales\\s+for\\s+201[123]")');
 * </code></pre>
 *
 * <p>Be careful when using custom pseudo classes with MVC Controllers: when
 * you use a pseudo class in Controller's <code>control</code> or <code>listen</code> component
 * selectors, the pseudo class' handler function will be called very often
 * and may slow down your application significantly. A good rule of thumb
 * is to always specify Component xtype with the pseudo class so that the
 * handlers are only called on Components that you need, and try to make
 * the condition checks as cheap in terms of execution time as possible.
 * Note how in the example above, handler function checks that Component
 * <em>has</em> a title first, before running regex test on it.</p>
 *
 * <h2>Query examples</h2>
 *
 * <p>Queries return an array of Components. Here are some example queries:</p>
 *
 * <pre><code>// retrieve all Ext.Panels in the document by xtype
 * var panelsArray = <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('panel');
 *
 * // retrieve all Ext.Panels within the container with an id myCt
 * var panelsWithinmyCt = <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('#myCt panel');
 *
 * // retrieve all direct children which are Ext.Panels within myCt
 * var directChildPanel = <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('#myCt &gt; panel');
 *
 * // retrieve all grids or trees
 * var gridsAndTrees = <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>('gridpanel, treepanel');
 *
 * // Focus first Component
 * myFormPanel.child(':canfocus').focus();
 *
 * // Retrieve every odd text field in a form
 * myFormPanel.query('textfield:nth-child(odd)');
 *
 * // Retrieve every even field in a form, excluding hidden fields
 * myFormPanel.query('field:not(hiddenfield):nth-child(even)');
 *
 * // Retrieve every scrollable in a tabpanel
 * tabpanel.query(':scrollable');
 * </code></pre>
 *
 * <p>For easy access to queries based from a particular Container see the
 * <a href="#!/api/Ext.container.Container-method-query" rel="Ext.container.Container-method-query" class="docClass">Ext.container.Container.query</a>, <a href="#!/api/Ext.container.Container-method-down" rel="Ext.container.Container-method-down" class="docClass">Ext.container.Container.down</a> and
 * <a href="#!/api/Ext.container.Container-method-child" rel="Ext.container.Container-method-child" class="docClass">Ext.container.Container.child</a> methods. Also see
 * <a href="#!/api/Ext.Component-method-up" rel="Ext.Component-method-up" class="docClass">Ext.Component.up</a>.</p>
 * <p>Type of singleton ComponentQuery.</p>
 * @see ext.#ComponentQuery ext.ComponentQuery
 */
public class SComponentQuery extends ext.Base {
  /**
   * Tests whether the passed Component matches the selector string.
   * An empty selector will always match.
   * @param component The Component to test
   * @param selector The selector string to test against.
   * @param root The root component.
   * @default null
   * @return True if the Component matches the selector.
   */
  public native function matches(component:ext.Component, selector:String, root:ext.Component = null):Boolean;

  /**
   * Returns an array of matched Components from within the passed root object.
   *
   * <p>This method filters returned Components in a similar way to how CSS selector based DOM
   * queries work using a textual selector string.</p>
   *
   * <p>See class summary for details.</p>
   * @param selector The selector string to filter returned Components.
   * @param root The Container within which to perform the query.
   * If omitted, all Components within the document are included in the search.
   * <p>This parameter may also be an array of Components to filter according to the selector.</p>
   * @return The matched Components.
   */
  public native function query(selector:String, root:ext.container.Container = null):Array;

  /**
   * Traverses the tree rooted at the passed root in post-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>after</strong> being called on its children).
   *
   * <p>For an object to be queryable, it must implement the <code>getRefItems</code> method which returns all
   * immediate child items.</p>
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param root The root queryable object to start from.
   * @param fn The function to call. Return <code>false</code> to abort the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPostOrder(selector:Object, root:String, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Traverses the tree rooted at the passed root in pre-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>before</strong> being called on its children).
   *
   * <p>For an object to be queryable, it must implement the <code>getRefItems</code> method which returns all
   * immediate child items.</p>
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param root The root queryable object to start from.
   * @param fn The function to call. Return <code>false</code> to abort the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPreOrder(selector:Object, root:String, fn:Function, scope:Object = null, extraArgs:Array = null):void;
}
}