package ext {

[Native]
/**
 * This class provides dynamic loading support for JavaScript classes. Application code
 * does not typically need to call <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> except perhaps to configure path mappings
 * when not using <a href="http://www.sencha.com/products/sencha-cmd/">Sencha Cmd</a>.
 *
 * <pre><code> <a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">Ext.Loader.setPath</a>('MyApp', 'app');
 * </code></pre>
 *
 * <p>When using Sencha Cmd, this is handled by the "bootstrap" provided by the application
 * build script and such configuration is not necessary.</p>
 *
 * <h1>Typical Usage</h1>
 *
 * <p>The <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> is most often used behind the scenes to satisfy class references in
 * class declarations. Like so:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.Main', {
 *      extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
 *
 *      mixins: [
 *          'MyApp.util.Mixin'
 *      ],
 *
 *      requires: [
 *          '<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>'
 *      ],
 *
 *      uses: [
 *          'MyApp.util.Stuff'
 *      ]
 *  });
 * </code></pre>
 *
 * <p>In all of these cases, <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> is used internally to resolve these class names
 * and ensure that the necessary class files are loaded.</p>
 *
 * <p>During development, these files are loaded individually for optimal debugging. For a
 * production use, <a href="http://www.sencha.com/products/sencha-cmd/">Sencha Cmd</a> will replace
 * all of these strings with the actual resolved class references because it ensures that
 * the classes are all contained in the build in the correct order. In development, these
 * files will not be loaded until the <code>MyApp.view.Main</code> class indicates they are needed
 * as shown above.</p>
 *
 * <h1>Loading Classes</h1>
 *
 * <p>You can also use <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> directly to load classes or files. The simplest form of
 * use is <code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a></code>.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>('MyApp.view.Main', function () {
 *      // On callback, the MyApp.view.Main class is now loaded
 *
 *      var view = new MyApp.view.Main();
 *  });
 * </code></pre>
 *
 * <p>You can alternatively require classes by alias or wildcard.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>('widget.window');
 *
 * <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>(['widget.window', 'layout.border', '<a href="#!/api/Ext.data.Connection" rel="Ext.data.Connection" class="docClass">Ext.data.Connection</a>']);
 *
 * <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>(['widget.*', 'layout.*', 'Ext.data.*']);
 * </code></pre>
 *
 * <p>The callback function is optional.</p>
 *
 * <p><strong>Note</strong> Using <code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a></code> at global scope will cause <code><a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a></code> and
 * <code><a href="#!/api/Ext.app.Application-method-launch" rel="Ext.app.Application-method-launch" class="docClass">Ext.app.Application.launch</a></code> methods to be deferred until the required classes
 * are loaded. It is these cases where the callback function is most often unnecessary.</p>
 *
 * <h2>Using Excludes</h2>
 *
 * <p>Alternatively, you can exclude what you don't need:</p>
 *
 * <pre><code>// Include everything except Ext.tree.*
 * <a href="#!/api/Ext-method-exclude" rel="Ext-method-exclude" class="docClass">Ext.exclude</a>('Ext.tree.*').require('*');
 *
 * // Include all widgets except widget.checkbox* (this will exclude
 * // widget.checkbox, widget.checkboxfield, widget.checkboxgroup, etc.)
 * <a href="#!/api/Ext-method-exclude" rel="Ext-method-exclude" class="docClass">Ext.exclude</a>('widget.checkbox*').require('widget.*');
 * </code></pre>
 *
 * <h1>Dynamic Instantiation</h1>
 *
 * <p>Another feature enabled by <code><a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a></code> is instantiation using class names or aliases.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> var win = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
 *      xtype: 'window',
 *
 *      // or
 *      // xclass: '<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>'
 *
 *      title: 'Hello'
 *  });
 * </code></pre>
 *
 * <p>This form of creation can be useful if the type to create (<code>window</code> in the above) is
 * not known statically. Internally, <code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></code> may need to <em>synchronously</em>
 * load the desired class and its requirements. Doing this will generate a warning in
 * the console:</p>
 *
 * <pre><code> [<a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a>] Synchronously loading '<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>'...
 * </code></pre>
 *
 * <p>If you see these in your debug console, you should add the indicated class(es) to the
 * appropriate <code>requires</code> array (as above) or make an <code><a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a></code> call.</p>
 *
 * <p><strong>Note</strong> Using <code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a></code> has some performance overhead and is best reserved
 * for cases where the target class is not known until run-time.</p>
 * <p>Type of singleton Loader.</p>
 * @see ext.#Loader ext.Loader
 */
public class SLoader {
  /**
   * An array of class names to keep track of the dependency loading order.
   * This is not guaranteed to be the same everytime due to the asynchronous
   * nature of the Loader.
   */
  public native function get history():Array;

  /**
   * @private
   */
  public native function set history(value:Array):void;

  [ExtConfig]
  /**
   * Appends current timestamp to script files to prevent caching.
   * @default true
   */
  public native function get disableCaching():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disableCaching(value:Boolean):void;

  [ExtConfig]
  /**
   * The get parameter name for the cache buster's timestamp.
   * @default "_dc"
   */
  public native function get disableCachingParam():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disableCachingParam(value:String):void;

  [ExtConfig]
  /**
   * Whether or not to enable the dynamic dependency loading feature.
   * @default true
   */
  public native function get enabled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enabled(value:Boolean):void;

  [ExtConfig]
  /**
   * The mapping from namespaces to file paths
   *
   * <pre><code>{
   *     'Ext': '.', // This is set by default, <a href="#!/api/Ext.layout.container.Container" rel="Ext.layout.container.Container" class="docClass">Ext.layout.container.Container</a> will be
   *                 // loaded from ./layout/Container.js
   *
   *     'My': './src/my_own_folder' // My.layout.Container will be loaded from
   *                                 // ./src/my_own_folder/layout/Container.js
   * }
   * </code></pre>
   *
   * <p>Note that all relative paths are relative to the current HTML document.
   * If not being specified, for example, <code>Other.awesome.Class</code> will simply be
   * loaded from <code>"./Other/awesome/Class.js"</code>.</p>
   */
  public native function get paths():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set paths(value:Object):void;

  [ExtConfig]
  /**
   * <code>false</code> to remove asynchronously loaded scripts, <code>true</code> to retain script
   * element for browser debugger compatibility and improved load performance.
   * @default true
   */
  public native function get preserveScripts():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preserveScripts(value:Boolean):void;

  [ExtConfig]
  /**
   * millisecond delay between asynchronous script injection (prevents stack
   * overflow on some user agents) 'false' disables delay but potentially
   * increases stack load.
   * @default false
   */
  public native function get scriptChainDelay():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scriptChainDelay(value:Boolean):void;

  [ExtConfig]
  /**
   * Optional charset to specify encoding of dynamic script content.
   */
  public native function get scriptCharset():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scriptCharset(value:String):void;

  /**
   * fixes up loader path configs by prepending Ext.Boot#baseUrl to the beginning
   * of the path, then delegates to <a href="#!/api/Ext.Loader-method-addClassPathMappings" rel="Ext.Loader-method-addClassPathMappings" class="docClass">Ext.Loader.addClassPathMappings</a>
   */
  public native function addBaseUrlClassPathMappings(pathConfig:Object):void;

  /**
   * Sets a batch of path entries
   * @param paths a set of className: path mappings
   * @return this
   */
  public native function addClassPathMappings(paths:Object):ext.SLoader;

  /**
   * Get the config value corresponding to the specified name. If no name is given,
   * will return the config object
   * @param name The config property name
   */
  public native function getConfig(name:String):Object;

  /**
   * Translates a className to a file path by adding the
   * the proper prefix and converting the .'s to /'s. For example:
   *
   * <pre><code><a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">Ext.Loader.setPath</a>('My', '/path/to/My');
   *
   * alert(<a href="#!/api/Ext.Loader-method-getPath" rel="Ext.Loader-method-getPath" class="docClass">Ext.Loader.getPath</a>('My.awesome.Class')); // alerts '/path/to/My/awesome/Class.js'
   * </code></pre>
   *
   * <p>Note that the deeper namespace levels, if explicitly set, are always resolved first.
   * For example:</p>
   *
   * <pre><code><a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">Ext.Loader.setPath</a>({
   *     'My': '/path/to/lib',
   *     'My.awesome': '/other/path/for/awesome/stuff',
   *     'My.awesome.more': '/more/awesome/path'
   * });
   *
   * alert(<a href="#!/api/Ext.Loader-method-getPath" rel="Ext.Loader-method-getPath" class="docClass">Ext.Loader.getPath</a>('My.awesome.Class')); // alerts '/other/path/for/awesome/stuff/Class.js'
   *
   * alert(<a href="#!/api/Ext.Loader-method-getPath" rel="Ext.Loader-method-getPath" class="docClass">Ext.Loader.getPath</a>('My.awesome.more.Class')); // alerts '/more/awesome/path/Class.js'
   *
   * alert(<a href="#!/api/Ext.Loader-method-getPath" rel="Ext.Loader-method-getPath" class="docClass">Ext.Loader.getPath</a>('My.cool.Class')); // alerts '/path/to/lib/cool/Class.js'
   *
   * alert(<a href="#!/api/Ext.Loader-method-getPath" rel="Ext.Loader-method-getPath" class="docClass">Ext.Loader.getPath</a>('Unknown.strange.Stuff')); // alerts 'Unknown/strange/Stuff.js'
   * </code></pre>
   * @return path
   */
  public native function getPath(className:String):String;

  /**
   * Loads the specified script URL and calls the supplied callbacks. If this method
   * is called before <a href="#!/api/Ext-property-isReady" rel="Ext-property-isReady" class="docClass">Ext.isReady</a>, the script's load will delay the transition
   * to ready. This can be used to load arbitrary scripts that may contain further
   * <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a> calls.
   * @param options The options object or simply the URL(s) to load.
   * @param {String} options.url The URL from which to load the script.
   * @param {Function} [options.onLoad] The callback to call on successful load.
   * @param {Function} [options.onError] The callback to call on failure to load.
   * @param {Object} [options.scope] The scope (<code>this</code>) for the supplied callbacks.
   */
  public native function loadScript(options:*):void;

  /**
   * Add a new listener to be executed when all required scripts are fully loaded
   * @param fn The function callback to be executed
   * @param scope The execution scope (<code>this</code>) of the callback function.
   * @param withDomReady Pass <code>false</code> to not also wait for document
   * dom ready.
   * @default true
   * @param options Additional callback options.
   * @param {Number} [options.delay] A number of milliseconds to delay.
   * @default 0
   * @param {Number} [options.priority] Relative priority of this callback. Negative
   * numbers are reserved.
   * @default 0
   */
  public native function onReady(fn:Function, scope:Object, withDomReady:Boolean = true, options:Object = null):void;

  /**
   * Set the configuration for the loader. This should be called right after ext-(debug).js
   * is included in the page, and before <a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a>. i.e:
   *
   * <pre><code>&lt;script type="text/javascript" src="ext-core-debug.js"&gt;&lt;/script&gt;
   * &lt;script type="text/javascript"&gt;
   *     <a href="#!/api/Ext.Loader-method-setConfig" rel="Ext.Loader-method-setConfig" class="docClass">Ext.Loader.setConfig</a>({
   *       enabled: true,
   *       paths: {
   *           'My': 'my_own_path'
   *       }
   *     });
   * &lt;/script&gt;
   * &lt;script type="text/javascript"&gt;
   *     <a href="#!/api/Ext-method-require" rel="Ext-method-require" class="docClass">Ext.require</a>(...);
   *
   *     <a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a>(function() {
   *       // application code here
   *     });
   * &lt;/script&gt;
   * </code></pre>
   *
   * <p>Refer to config options of <a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a> for the list of possible properties</p>
   * @param config The config object to override the default values
   * @return this
   */
  public native function setConfig(config:Object):ext.SLoader;

  /**
   * Sets the path of a namespace.
   * For Example:
   *
   * <pre><code><a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">Ext.Loader.setPath</a>('Ext', '.');
   * </code></pre>
   * @param name See <a href="#!/api/Ext.Function-method-flexSetter" rel="Ext.Function-method-flexSetter" class="docClass">flexSetter</a>
   * @param path See <a href="#!/api/Ext.Function-method-flexSetter" rel="Ext.Function-method-flexSetter" class="docClass">flexSetter</a>
   * @return this
   */
  public native function setPath(name:*, path:String = null):ext.SLoader;
}
}