package ext {

[Native("Ext.Version", require)]
/**
 * A utility class that wraps around a version number string and provides convenient methods
 * to perform comparisons. A version number is expressed in the following general format:
 *
 * <pre><code>major[.minor[.patch[.build[release]]]]
 * </code></pre>
 *
 * <p>The <code>Version</code> instance holds various readonly properties that contain the digested form
 * of the version string. The numeric componnets of <code>major</code>, <code>minor</code>, <code>patch</code> and <code>build</code>
 * as well as the textual suffix called <code>release</code>.</p>
 *
 * <p>Not depicted in the above syntax are three possible prefixes used to control partial
 * matching. These are '^' (the default), '>' and '~'. These are discussed below.</p>
 *
 * <p>Examples:</p>
 *
 * <pre><code>var version = new <a href="#!/api/Ext.Version" rel="Ext.Version" class="docClass">Ext.Version</a>('1.0.2beta'); // or maybe "1.0" or "1.2.3.4RC"
 * console.log("Version is " + version); // Version is 1.0.2beta
 *
 * console.log(version.getMajor()); // 1
 * console.log(version.getMinor()); // 0
 * console.log(version.getPatch()); // 2
 * console.log(version.getBuild()); // 0
 * console.log(version.getRelease()); // beta
 * </code></pre>
 *
 * <p>The understood values of <code>release</code> are assigned numberic equivalents for the sake of
 * comparsion. The order of these from smallest to largest is as follows:</p>
 *
 * <ul>
 * <li><code>"dev"</code></li>
 * <li><code>"alpha"</code> or <code>"a"</code></li>
 * <li><code>"beta"</code> or <code>"b"</code></li>
 * <li><code>"RC"</code> or <code>"rc"</code></li>
 * <li><code>"#"</code></li>
 * <li><code>"pl"</code> or <code>"p"</code></li>
 * </ul>
 *
 *
 * <p>Any other (unrecognized) suffix is consider greater than any of these.</p>
 *
 * <h2>Comparisons</h2>
 *
 * <p>There are two forms of comparison that are commonly needed: full and partial. Full
 * comparison is simpler and is also the default.</p>
 *
 * <p>Example:</p>
 *
 * <pre><code>var version = new <a href="#!/api/Ext.Version" rel="Ext.Version" class="docClass">Ext.Version</a>('1.0.2beta');
 *
 * console.log(version.isGreaterThan('1.0.1')); // True
 * console.log(version.isGreaterThan('1.0.2alpha')); // True
 * console.log(version.isGreaterThan('1.0.2RC')); // False
 * console.log(version.isGreaterThan('1.0.2')); // False
 * console.log(version.isLessThan('1.0.2')); // True
 *
 * console.log(version.match(1.0)); // True (using a Number)
 * console.log(version.match('1.0.2')); // True (using a String)
 * </code></pre>
 *
 * <p>These comparisons are ultimately implemented by <a href="#!/api/Ext.Version-method-compareTo" rel="Ext.Version-method-compareTo" class="docClass">compareTo</a>
 * which returns -1, 0 or 1 depending on whether the `Version' instance is less than, equal
 * to, or greater than the given "other" version.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code> var n = version.compareTo('1.0.1');  // == 1  (because 1.0.2beta &gt; 1.0.1)
 *
 *  n = version.compareTo('1.1');  // == -1
 *  n = version.compareTo(version); // == 0
 * </code></pre>
 *
 * <h3>Partial Comparisons</h3>
 *
 * <p>By default, unspecified version number fields are filled with 0. In other words, the
 * version number fields are 0-padded on the right or a "lower bound". This produces the
 * most commonly used forms of comparsion:</p>
 *
 * <pre><code> var ver = new Version('4.2');
 *
 *  n = ver.compareTo('4.2.1'); // == -1  (4.2 promotes to 4.2.0 and is less than 4.2.1)
 * </code></pre>
 *
 * <p>There are two other ways to interpret comparisons of versions of different length. The
 * first of these is to change the padding on the right to be a large number (scuh as
 * Infinity) instead of 0. This has the effect of making the version an upper bound. For
 * example:</p>
 *
 * <pre><code> var ver = new Version('^4.2'); // NOTE: the '^' prefix used
 *
 *  n = ver.compareTo('4.3'); // == -1  (less than 4.3)
 *
 *  n = ver.compareTo('4.2'); // == 1   (greater than all 4.2's)
 *  n = ver.compareTo('4.2.1'); // == 1
 *  n = ver.compareTo('4.2.9'); // == 1
 * </code></pre>
 *
 * <p>The second way to interpret this comparison is to ignore the extra digits, making the
 * match a prefix match. For example:</p>
 *
 * <pre><code> var ver = new Version('~4.2'); // NOTE: the '~' prefix used
 *
 *  n = ver.compareTo('4.3'); // == -1
 *
 *  n = ver.compareTo('4.2'); // == 0
 *  n = ver.compareTo('4.2.1'); // == 0
 * </code></pre>
 *
 * <p>This final form can be useful when version numbers contain more components than are
 * important for certain comparisons. For example, the full version of Ext JS 4.2.1 is
 * "4.2.1.883" where 883 is the <code>build</code> number.</p>
 *
 * <p>This is how to create a "partial" <code>Version</code> and compare versions to it:</p>
 *
 * <pre><code> var version421ish = new Version('~4.2.1');
 *
 *  n = version421ish.compareTo('4.2.1.883'); // == 0
 *  n = version421ish.compareTo('4.2.1.2'); // == 0
 *  n = version421ish.compareTo('4.2.1'); // == 0
 *
 *  n = version421ish.compareTo('4.2'); // == 1
 * </code></pre>
 *
 * <p>In the above example, '4.2.1.2' compares as equal to '4.2.1' because digits beyond the
 * given "4.2.1" are ignored. However, '4.2' is less than the '4.2.1' prefix; its missing
 * digit is filled with 0.</p>
 */
public class Version {
  /**
   * Compare 2 specified versions by ensuring the first parameter is a <code>Version</code>
   * instance and then calling the <code>compareTo</code> method.
   * @param current The current version to compare to
   * @param target The target version to compare to
   * @return Returns -1 if the current version is smaller than the target version, 1 if greater, and 0 if they're equivalent
   */
  public static native function compare(current:String, target:String):Number;

  /**
   * Converts a version component to a comparable value
   * @param value The value to convert
   */
  public static native function getComponentValue(value:Object):Object;

  /**
   * The fourth numeric part of the version number string.
   */
  public native function get build():Number;

  /**
   * The first numeric part of the version number string.
   */
  public native function get major():Number;

  /**
   * The second numeric part of the version number string.
   */
  public native function get minor():Number;

  /**
   * The third numeric part of the version number string.
   */
  public native function get patch():Number;

  /**
   * The release level. The following values are understood:
   *
   * <ul>
   * <li><code>"dev"</code></li>
   * <li><code>"alpha"</code> or <code>"a"</code></li>
   * <li><code>"beta"</code> or <code>"b"</code></li>
   * <li><code>"RC"</code> or <code>"rc"</code></li>
   * <li><code>"#"</code></li>
   * <li><code>"pl"</code> or <code>"p"</code></li>
   * </ul>
   * @default ""
   */
  public native function get release():String;

  /**
   * Compares this version instance to the specified <code>other</code> version.
   * @param other The other version to which to compare.
   * @return -1 if this version is less than the target version, 1 if this
   * version is greater, and 0 if they are equal.
   */
  public native function compareTo(other:*):Number;

  /**
   * Returns whether this version equals to the supplied argument
   * @param target The version to compare with
   * @return <code>true</code> if this version equals to the target, <code>false</code> otherwise
   */
  public native function equals(target:*):Boolean;

  /**
   * Returns the build component value.
   */
  public native function getBuild():Number;

  /**
   * Returns the major component value.
   */
  public native function getMajor():Number;

  /**
   * Returns the minor component value.
   */
  public native function getMinor():Number;

  /**
   * Returns the patch component value.
   */
  public native function getPatch():Number;

  /**
   * Returns the release component text (e.g., "beta").
   */
  public native function getRelease():String;

  /**
   * Returns the release component value for comparison purposes.
   */
  public native function getReleaseValue():*;

  /**
   * Returns shortVersion version without dots and release
   */
  public native function getShortVersion():String;

  /**
   * Convenient alias to <a href="#!/api/Ext.Version-method-isGreaterThan" rel="Ext.Version-method-isGreaterThan" class="docClass">isGreaterThan</a>
   */
  public native function gt(target:*):Boolean;

  /**
   * Convenient alias to <a href="#!/api/Ext.Version-method-isGreaterThanOrEqual" rel="Ext.Version-method-isGreaterThanOrEqual" class="docClass">isGreaterThanOrEqual</a>
   */
  public native function gtEq(target:*):Boolean;

  /**
   * Returns whether this version if greater than the supplied argument
   * @param target The version to compare with
   * @return <code>true</code> if this version if greater than the target, <code>false</code> otherwise
   */
  public native function isGreaterThan(target:*):Boolean;

  /**
   * Returns whether this version if greater than or equal to the supplied argument
   * @param target The version to compare with
   * @return <code>true</code> if this version if greater than or equal to the target, <code>false</code> otherwise
   */
  public native function isGreaterThanOrEqual(target:*):Boolean;

  /**
   * Returns whether this version if smaller than the supplied argument
   * @param target The version to compare with
   * @return <code>true</code> if this version if smaller than the target, <code>false</code> otherwise
   */
  public native function isLessThan(target:*):Boolean;

  /**
   * Returns whether this version if less than or equal to the supplied argument
   * @param target The version to compare with
   * @return <code>true</code> if this version if less than or equal to the target, <code>false</code> otherwise
   */
  public native function isLessThanOrEqual(target:*):Boolean;

  /**
   * Convenient alias to <a href="#!/api/Ext.Version-method-isLessThan" rel="Ext.Version-method-isLessThan" class="docClass">isLessThan</a>
   */
  public native function lt(target:*):Boolean;

  /**
   * Convenient alias to <a href="#!/api/Ext.Version-method-isLessThanOrEqual" rel="Ext.Version-method-isLessThanOrEqual" class="docClass">isLessThanOrEqual</a>
   */
  public native function ltEq(target:*):Boolean;

  /**
   * Returns whether this version matches the supplied argument. Example:
   *
   * <pre><code>var version = new <a href="#!/api/Ext.Version" rel="Ext.Version" class="docClass">Ext.Version</a>('1.0.2beta');
   * console.log(version.match(1)); // true
   * console.log(version.match(1.0)); // true
   * console.log(version.match('1.0.2')); // true
   * console.log(version.match('1.0.2RC')); // false
   * </code></pre>
   * @param target The version to compare with
   * @return <code>true</code> if this version matches the target, <code>false</code> otherwise
   */
  public native function match(target:*):Boolean;

  /**
   * Returns this format: [major, minor, patch, build, release]. Useful for comparison.
   */
  public native function toArray():Array;
}
}