package ext.app {
import ext.container.Container;

[Uses("ext.app.Controller")]
[Uses("ext.app.BaseController")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.app.Application", require)]
/**
 * Represents an Ext JS application, which is typically a single page app using a
 * <a href="#!/api/Ext.container.Viewport" rel="Ext.container.Viewport" class="docClass">Viewport</a>.
 *
 * <p>An application consists of one or more Views. The behavior of a View is managed by its
 * corresponding <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">ViewController</a> and <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">ViewModel</a>.</p>
 *
 * <p>Global activities are coordinated by <a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Controllers</a> which are
 * ultimately instantiated by an instance of this (or a derived) class.</p>
 *
 * <pre><code>Ext.application({
 *     name: 'MyApp',
 *
 *     // An instance of this view is created and set as the Viewport:
 *     autoCreateViewport: 'MyApp.view.Main'
 * });
 * </code></pre>
 *
 * <p>This does several things. First it creates a global variable called 'MyApp' - all of
 * your Application's classes (such as its Models, Views and Controllers) will reside under
 * this single namespace, which drastically lowers the chances of colliding global variables.</p>
 *
 * <p>The MyApp global will also have a getApplication method to get a reference to the current
 * application:</p>
 *
 * <pre><code>var app = MyApp.getApplication();
 * </code></pre>
 *
 * <h1>Telling Application about the rest of the app</h1>
 *
 * <p>Because an <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a> represents an entire app, we should tell it about the other
 * parts of the app - namely the Models, Views and Controllers that are bundled with the application. Let's say we have a blog management app; we
 * might have Models and Controllers for Posts and Comments, and Views for listing, adding and editing Posts and Comments.
 * Here's how we'd tell our Application about all these things:</p>
 *
 * <pre><code>Ext.application({
 *     name: 'Blog',
 *
 *     models: ['Post', 'Comment'],
 *
 *     controllers: ['Posts', 'Comments'],
 *
 *     launch: function() {
 *         ...
 *     }
 * });
 * </code></pre>
 *
 * <p>Note that we didn't actually list the Views directly in the Application itself. This is because Views are managed by
 * Controllers, so it makes sense to keep those dependencies there. The Application will load each of the specified
 * Controllers using the pathing conventions laid out in the <a href="../application_architecture/application_architecture.html">application architecture guide</a> - in this case
 * expecting the controllers to reside in app/controller/Posts.js and app/controller/Comments.js. In turn, each
 * Controller simply needs to list the Views it uses and they will be automatically loaded. Here's how our Posts
 * controller like be defined:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Posts', {
 *     extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
 *     views: ['posts.List', 'posts.Edit'],
 *
 *     //the rest of the Controller here
 * });
 * </code></pre>
 *
 * <p>Because we told our Application about our Models and Controllers, and our Controllers about their Views, Ext JS will
 * automatically load all of our app files for us. This means we don't have to manually add script tags into our html
 * files whenever we add a new class, but more importantly it enables us to create a minimized build of our entire
 * application using Sencha Cmd.</p>
 *
 * <h1>Deriving from <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a></h1>
 *
 * <p>Typically, applications do not derive directly from <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a>. Rather, the
 * configuration passed to <code>Ext.application</code> mimics what you might do in a derived class.
 * In some cases, however, it can be desirable to share logic by using a derived class
 * from <code><a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a></code>.</p>
 *
 * <p>Derivation works as you would expect, but using the derived class should still be the
 * job of the <code>Ext.application</code> method.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.Application', {
 *     extend: '<a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a>',
 *     name: 'MyApp',
 *     ...
 * });
 *
 * Ext.application('MyApp.Application');
 * </code></pre>
 *
 * <p>For more information about writing Ext JS applications, please see the <a href="../../../application_architecture/application_architecture.html">application architecture guide</a>.</p>
 */
public class Application extends ext.app.Controller {
  /**
   * The singleton application instance.
   */
  public static native function get instance():ext.app.Application;

  /**
   * Creates new Application.
   * @param config Config object.
   */
  public function Application(config:ext.app.Application = null) {
    super();
  }

  [ExtConfig]
  /**
   * The path to the directory which contains all application's classes.
   * This path will be registered via <a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">Ext.Loader.setPath</a> for the namespace specified
   * in the <a href="#!/api/Ext.app.Application-cfg-name" rel="Ext.app.Application-cfg-name" class="docClass">name</a> config.
   * @default "app"
   */
  public native function get appFolder():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set appFolder(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of a property to be assigned to the main namespace to gain a reference to
   * this application. Can be set to an empty value to prevent the reference from
   * being created
   *
   * <pre><code>Ext.application({
   *     name: 'MyApp',
   *     appProperty: 'myProp',
   *
   *     launch: function() {
   *         console.log(MyApp.myProp === this);
   *     }
   * });
   * </code></pre>
   * @default 'app'
   * @see #getAppProperty()
   * @see #setAppProperty()
   */
  public native function get appProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set appProperty(value:String):void;

  [Deprecated(replacement = "mainView", since = "5.1")]
  [ExtConfig]
  /**
   * @default false
   */
  public native function get autoCreateViewport():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoCreateViewport(value:*):void;

  [ExtConfig]
  /**
   * Names of <a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">controllers</a> that the app uses.  By default,
   * the framework will look for the controllers in the "controller" folder within the
   * <a href="#!/api/Ext.app.Application-cfg-appFolder" rel="Ext.app.Application-cfg-appFolder" class="docClass">appFolder</a>.  Controller classes should be named using the syntax of
   * "{appName}.controller.{ClassName}" with additional sub-folders under the
   * "controller" folder specified within the class name following "controller.".
   *
   * <pre><code>// by default, the following controller class would be located at:
   * // app/controller/Main.js
   * controllers: '.Main' // or 'MyApp.controller.Main'
   *
   * // while the following would be located at:
   * // app/controller/customer/Main.js
   * controllers: 'customer.Main' // or 'MyApp.controller.customer.Main'
   * </code></pre>
   *
   * <p><strong>Note:</strong> If the controller has a different namespace than that of the
   * application you will need to specify the full class name as well as define a path
   * in the <a href="#!/api/Ext.Loader-cfg-paths" rel="Ext.Loader-cfg-paths" class="docClass">Loader's paths</a> config or
   * <a href="#!/api/Ext.Loader-method-setPath" rel="Ext.Loader-method-setPath" class="docClass">setPath</a> method.</p>
   */
  public native function get controllers():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set controllers(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @see #getCurrentProfile()
   * @see #setCurrentProfile()
   */
  public native function get currentProfile():ext.app.Profile;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set currentProfile(value:ext.app.Profile):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default token to be used at application launch
   * if one is not present. Often this is set to something like <code>'home'</code>.
   * @default null
   * @see #getDefaultToken()
   * @see #setDefaultToken()
   */
  public native function get defaultToken():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultToken(value:String):void;

  [Deprecated(replacement = "quickTips", since = "6.2.0")]
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to automatically set up <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">Ext.tip.QuickTip</a> support.
   * @default true
   * @see #getEnableQuickTips()
   * @see #setEnableQuickTips()
   */
  public native function get enableQuickTips():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set enableQuickTips(value:Boolean):void;

  [ExtConfig]
  /**
   * A class name to use with the <code>Ext.application</code> call. The class must also extend <a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a>.
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.Application', {
   *     extend: '<a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a>',
   *
   *     launch: function() {
   *         <a href="#!/api/Ext.direct.Manager-method-addProvider" rel="Ext.direct.Manager-method-addProvider" class="docClass">Ext.direct.Manager.addProvider</a>(Ext.REMOTING_API);
   *     }
   * });
   *
   * Ext.application({
   *     extend: 'MyApp.Application'
   * });
   * </code></pre>
   */
  public native function get extend():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set extend(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The glyphFontFamily to use for this application.  Used as the default font-family
   * for all components that support a <code>glyph</code> config.
   * @see #getGlyphFontFamily()
   * @see #setGlyphFontFamily()
   */
  public native function get glyphFontFamily():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set glyphFontFamily(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The application class to be used as the main viewport view for the
   * application.  The view will be configured with the
   * <a href="#!/api/Ext.plugin.Viewport" rel="Ext.plugin.Viewport" class="docClass">viewport plugin</a> to ensure the view takes up all
   * available space in the browser viewport.  The main view will be created after
   * the application's <a href="#!/api/Ext.app.Application-method-init" rel="Ext.app.Application-method-init" class="docClass">init</a> method is called and before the
   * <a href="#!/api/Ext.app.Application-method-launch" rel="Ext.app.Application-method-launch" class="docClass">launch</a> method.  The main view should be an application class type and
   * not a class from the framework.
   *
   * <p>The main view value may be:
   *  - string representing the full class name of the main view or the partial class name following "AppName.view." (provided your main view class follows that convention).
   *  - config object for the main view
   *  - main view class instance</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.main.Main', {
   *     extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
   *     xtype: 'mainview',
   *     title: 'Main Viewport View'
   * });
   *
   * Ext.application({
   *     name : 'MyApp',
   *
   *     mainView: 'MyApp.view.main.Main'
   *     // mainView: 'main.Main'
   *     // mainView: new MyApp.view.main.Main()
   *     // mainView: { xtype: 'mainview' }
   * });
   * </code></pre>
   *
   * <p><strong>Note:</strong> You may also call <a href="#!/api/Ext.app.Application-method-setMainView" rel="Ext.app.Application-method-setMainView" class="docClass">setMainView</a> at runtime if you require
   * logic within the application's <a href="#!/api/Ext.app.Application-method-launch" rel="Ext.app.Application-method-launch" class="docClass">launch</a> method to be processed prior to
   * the creation of the main view.</p>
   * @default {$value: null, lazy: true}
   * @see #getMainView()
   * @see #setMainView()
   */
  public native function get mainView():ext.container.Container;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set mainView(value:ext.container.Container):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of your application. This will also be the namespace for your views, controllers
   * models and stores. Don't use spaces or special characters in the name. <strong>Application name
   * is mandatory</strong>.
   * @default ''
   * @see #getName()
   * @see #setName()
   */
  public native function get name():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set name(value:String):void;

  [ExtConfig]
  /**
   * The list of namespace prefixes used in the application to resolve dependencies
   * like Views and Stores:
   *
   * <pre><code> Ext.application({
   *      name: 'MyApp',
   *
   *      namespaces: ['Common.code'],
   *
   *      controllers: [ 'Common.code.controller.Foo', 'Bar' ]
   *  });
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Common.code.controller.Foo', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      models: ['Foo'],    // Loads Common.code.model.Foo
   *      views:  ['Bar']     // Loads Common.code.view.Bar
   *  });
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Bar', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      models: ['Foo'],    // Loads MyApp.model.Foo
   *      views:  ['Bar']     // Loads MyApp.view.Bar
   *  });
   * </code></pre>
   *
   * <p>You don't need to include main namespace (MyApp), it will be added to the list
   * automatically.</p>
   * @default []
   */
  public native function get namespaces():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set namespaces(value:*):void;

  [ExtConfig]
  /**
   * Additional load paths to add to <a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a>.
   * See <a href="#!/api/Ext.Loader-cfg-paths" rel="Ext.Loader-cfg-paths" class="docClass">Ext.Loader.paths</a> config for more details.
   */
  public native function get paths():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set paths(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Names of the profiles that the app uses.
   * @default []
   * @see #getProfiles()
   * @see #setProfiles()
   */
  public native function get profiles():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set profiles(value:*):void;

  [ExtConfig]
  /**
   * True to automatically set up <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">Ext.tip.QuickTip</a> support.
   * @default true
   * @since 6.2.0
   */
  public native function get quickTips():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set quickTips(value:Boolean):void;

  [ExtConfig]
  /**
   * The scope to execute the <a href="#!/api/Ext.app.Application-method-launch" rel="Ext.app.Application-method-launch" class="docClass">launch</a> function in. Defaults to the Application instance.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  /**
   * Get an application's controller based on name or id.  Generally, the controller id will be the same as the name
   * unless otherwise specified.
   */
  override public native function getController(id:String):ext.app.Controller;

  /**
   * Create an instance of a controller by name.
   * @param name The name of the controller. For a controller with the
   * full class name <code>MyApp.controller.Foo</code>, the name parameter should be <code>Foo</code>.
   * If the controller already exists, it will be returned.
   * @return controller
   */
  public native function createController(name:String):ext.app.Controller;

  /**
   * Destroys a controller, any listeners are unbound.
   * @param controller The controller
   */
  public native function destroyController(controller:*):void;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-appProperty" rel="Ext.app.Application-cfg-appProperty" class="docClass">appProperty</a>.
   */
  public native function getAppProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-currentProfile" rel="Ext.app.Application-cfg-currentProfile" class="docClass">currentProfile</a>.
   */
  public native function getCurrentProfile():ext.app.Profile;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-defaultToken" rel="Ext.app.Application-cfg-defaultToken" class="docClass">defaultToken</a>.
   */
  public native function getDefaultToken():String;

  [Deprecated(replacement = "quickTips", since = "6.2.0")]
  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-enableQuickTips" rel="Ext.app.Application-cfg-enableQuickTips" class="docClass">enableQuickTips</a>.
   */
  public native function getEnableQuickTips():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a>.
   */
  public native function getGlyphFontFamily():String;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-mainView" rel="Ext.app.Application-cfg-mainView" class="docClass">mainView</a>.
   */
  public native function getMainView():ext.container.Container;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-name" rel="Ext.app.Application-cfg-name" class="docClass">name</a>.
   */
  public native function getName():String;

  /**
   * Returns the value of <a href="#!/api/Ext.app.Application-cfg-profiles" rel="Ext.app.Application-cfg-profiles" class="docClass">profiles</a>.
   */
  public native function getProfiles():*;

  /**
   * Called automatically when the page has completely loaded. This is an empty function that should be
   * overridden by each application that needs to take action on page load.
   * @param profile The detected application profile
   * @return By default, the Application will dispatch to the configured startup controller and
   * action immediately after running the launch function. Return false to prevent this behavior.
   */
  public native function launch(profile:String):Boolean;

  /**
   * Called automatically when an update to either the Application Cache or the Local Storage Cache is detected.
   * This is mainly used during production builds.
   * @param updateInfo updateInfo Update information object contains properties for checking which cache triggered the update
   */
  public native function onAppUpdate(updateInfo:Object = null):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-appProperty" rel="Ext.app.Application-cfg-appProperty" class="docClass">appProperty</a>.
   * @param appProperty The new value.
   */
  public native function setAppProperty(appProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-currentProfile" rel="Ext.app.Application-cfg-currentProfile" class="docClass">currentProfile</a>.
   * @param currentProfile The new value.
   */
  public native function setCurrentProfile(currentProfile:ext.app.Profile):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-defaultToken" rel="Ext.app.Application-cfg-defaultToken" class="docClass">defaultToken</a>.
   * @param defaultToken The new value.
   */
  public native function setDefaultToken(defaultToken:String):void;

  [Deprecated(replacement = "quickTips", since = "6.2.0")]
  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-enableQuickTips" rel="Ext.app.Application-cfg-enableQuickTips" class="docClass">enableQuickTips</a>.
   * @param enableQuickTips The new value.
   */
  public native function setEnableQuickTips(enableQuickTips:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a>.
   * @param glyphFontFamily The new value.
   */
  public native function setGlyphFontFamily(glyphFontFamily:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-mainView" rel="Ext.app.Application-cfg-mainView" class="docClass">mainView</a>.
   * @param mainView The new value.
   */
  public native function setMainView(mainView:ext.container.Container):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-name" rel="Ext.app.Application-cfg-name" class="docClass">name</a>.
   * @param name The new value.
   */
  public native function setName(name:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.Application-cfg-profiles" rel="Ext.app.Application-cfg-profiles" class="docClass">profiles</a>.
   * @param profiles The new value.
   */
  public native function setProfiles(profiles:*):void;
}
}