package ext.app {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.app.BaseController", require)]
/**
 * Base class for Controllers.
 */
public class BaseController extends ext.mixin.Observable {
  /**
   * Creates new Controller.
   * @param config Configuration object.
   */
  public function BaseController(config:ext.app.BaseController = null) {
    super();
  }

  [ExtConfig("control")]
  /**
   * Adds listeners to components selected via <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a>. Accepts an
   * object containing component paths mapped to a hash of listener functions.<br/>
   * The function value may also be a string matching the name of a method on the
   * controller.
   *
   * <p>In the following example the <code>updateUser</code> function is mapped to to the <code>click</code>
   * event on a button component, which is a child of the <code>useredit</code> component.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Users', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      control: {
   *          'useredit button[action=save]': {
   *              click: 'updateUser'
   *          }
   *      },
   *
   *      updateUser: function(button) {
   *          console.log('clicked the Save button');
   *      }
   *  });
   * </code></pre>
   *
   * <p>The method you pass to the listener will automatically be resolved on the controller.
   * In this case, the <code>updateUser</code> method that will get executed on the <code>button</code> <code>click</code>
   * event will resolve to the <code>updateUser</code> method on the controller,</p>
   *
   * <p>See <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> for more information on component selectors.</p>
   */
  public native function get control_():Object;

  [ExtConfig("control")]
  /**
   * @private
   */
  public native function set control_(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The id of this controller. You can use this id when dispatching.
   *
   * <p>For an example of dispatching, see the examples under the
   * <a href="#!/api/Ext.app.Controller-cfg-listen" rel="Ext.app.Controller-cfg-listen" class="docClass">listen</a> config.</p>
   *
   * <p>If an id is not explicitly set, it will default to the controller's full classname.</p>
   * @see #getId()
   * @see #setId()
   */
  public native function get id():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set id(value:String):void;

  [ExtConfig("listen")]
  /**
   * Adds listeners to different event sources (also called "event domains"). The
   * primary event domain is that of components, but there are also other event domains:
   * <a href="#!/api/Ext.app.domain.Global" rel="Ext.app.domain.Global" class="docClass">Global</a> domain that intercepts events fired from
   * <a href="#!/api/Ext.GlobalEvents" rel="Ext.GlobalEvents" class="docClass">Ext.GlobalEvents</a> Observable instance,
   * <a href="#!/api/Ext.app.domain.Controller" rel="Ext.app.domain.Controller" class="docClass">Controller</a> domain can be used to listen to events
   * fired by other Controllers, <a href="#!/api/Ext.app.domain.Store" rel="Ext.app.domain.Store" class="docClass">Store</a> domain gives access to
   * Store events, and <a href="#!/api/Ext.app.domain.Direct" rel="Ext.app.domain.Direct" class="docClass">Direct</a> domain can be used with
   * Ext Direct Providers to listen to their events.
   *
   * <p>To listen to "bar" events fired by a controller with id="foo":</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      listen: {
   *          controller: {
   *              '#foo': {
   *                  bar: 'onFooBar'
   *              }
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>To listen to "bar" events fired by any controller, and "baz" events
   * fired by Store with storeId="baz":</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      listen: {
   *          controller: {
   *              '*': {
   *                  bar: 'onAnyControllerBar'
   *              }
   *          },
   *          store: {
   *              '#baz': {
   *                  baz: 'onStoreBaz'
   *              }
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>To listen to "idle" events fired by <a href="#!/api/Ext.GlobalEvents" rel="Ext.GlobalEvents" class="docClass">Ext.GlobalEvents</a> when other event
   * processing is complete and Ext JS is about to return control to the browser:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      listen: {
   *          global: {            // Global events are always fired
   *              idle: 'onIdle'   // from the same object, so there
   *          }                    // are no selectors
   *      }
   *  });
   * </code></pre>
   *
   * <p>As this relates to components, the following example:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      listen: {
   *          component: {
   *              'useredit button[action=save]': {
   *                  click: 'updateUser'
   *              }
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>Is equivalent to:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      control: {
   *          'useredit button[action=save]': {
   *              click: 'updateUser'
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>Of course, these can all be combined in a single call and used instead of
   * <code>control</code>, like so:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      extend: '<a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>',
   *
   *      listen: {
   *          global: {
   *              idle: 'onIdle'
   *          },
   *          controller: {
   *              '*': {
   *                  foobar: 'onAnyFooBar'
   *              },
   *              '#foo': {
   *                  bar: 'onFooBar'
   *              }
   *          },
   *          component: {
   *              'useredit button[action=save]': {
   *                  click: 'updateUser'
   *              }
   *          },
   *          store: {
   *              '#qux': {
   *                  load: 'onQuxLoad'
   *              }
   *          }
   *      }
   *  });
   * </code></pre>
   */
  public native function get listen_():Object;

  [ExtConfig("listen")]
  /**
   * @private
   */
  public native function set listen_(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object of routes to handle hash changes. A route can be defined in a simple way:
   *
   * <pre><code>routes : {
   *     'foo/bar'  : 'handleFoo',
   *     'user/:id' : 'showUser'
   * }
   * </code></pre>
   *
   * <p>Where the property is the hash (which can accept a parameter defined by a colon) and the value
   * is the method on the controller to execute. The parameters will get sent in the action method.</p>
   *
   * <p>At the application level, you can define a event that will be executed when no matching
   * routes are found.</p>
   *
   * <pre><code>Ext.application({
   *     name: 'MyApp',
   *     listen: {
   *         controller: {
   *             '#': {
   *                 unmatchedroute: 'onUnmatchedRoute'
   *             }
   *         }
   *     },
   *
   *     onUnmatchedRoute: function(hash) {
   *         console.log('Unmatched', hash);
   *         // Do something...
   *     }
   * });
   * </code></pre>
   *
   * <p>There is also a complex means of defining a route where you can use a before action and even
   * specify your own RegEx for the parameter:</p>
   *
   * <pre><code>routes : {
   *     'foo/bar'  : {
   *         action  : 'handleFoo',
   *         before  : 'beforeHandleFoo'
   *     },
   *     'user/:id' : {
   *         action     : 'showUser',
   *         before     : 'beforeShowUser',
   *         conditions : {
   *             ':id' : '([0-9]+)'
   *         }
   *     }
   * }
   * </code></pre>
   *
   * <p>This will only match if the <code>id</code> parameter is a number.</p>
   *
   * <p>The before action allows you to cancel an action. Every before action will get passed an <code>action</code> argument with
   * a <code>resume</code> and <code>stop</code> methods as the last argument of the method and you <em>MUST</em> execute either method:</p>
   *
   * <pre><code>beforeHandleFoo : function(action) {
   *     //some logic here
   *
   *     //this will allow the handleFoo action to be executed
   *     action.resume();
   * },
   * handleFoo : function() {
   *     //will get executed due to true being passed in callback in beforeHandleFoo
   * },
   * beforeShowUser : function(id, action) {
   *     //allows for async process like an Ajax
   *     <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
   *         url     : 'foo.php',
   *         success : function() {
   *             //will not allow the showUser method to be executed but will continue other queued actions.
   *             action.stop();
   *         },
   *         failure : function() {
   *             //will not allow the showUser method to be executed and will not allow other queued actions to be executed.
   *             action.stop(true);
   *         }
   *     });
   * },
   * showUser : function(id) {
   *     //will not get executed due to false being passed in callback in beforeShowUser
   * }
   * </code></pre>
   *
   * <p>You <em>MUST</em> execute the <code>resume</code> or <code>stop</code> method on the <code>action</code> argument. Executing <code>action.resume();</code> will continue
   * the action, <code>action.stop();</code> will not allow the action to resume but will allow other queued actions to resume,
   * <code>action.stop(true);</code> will not allow the action and any other queued actions to resume.</p>
   *
   * <p>The default RegEx that will be used is <code>([%a-zA-Z0-9\\-\\_\\s,]+)</code> but you can specify any
   * that may suit what you need to accomplish. An example of an advanced condition may be to make
   * a parameter optional and case-insensitive:</p>
   *
   * <pre><code>routes : {
   *     'user:id' : {
   *         action     : 'showUser',
   *         before     : 'beforeShowUser',
   *         conditions : {
   *             ':id' : '(?:(?:\/){1}([%a-z0-9_,\s\-]+))?'
   *         }
   *     }
   * }
   * </code></pre>
   * @see #getRoutes()
   * @see #setRoutes()
   */
  public native function get routes():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set routes(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.app.BaseController-cfg-id" rel="Ext.app.BaseController-cfg-id" class="docClass">id</a>.
   */
  override public native function getId():String;

  /**
   * Sets the value of <a href="#!/api/Ext.app.BaseController-cfg-id" rel="Ext.app.BaseController-cfg-id" class="docClass">id</a>.
   * @param id The new value.
   */
  override public native function setId(id:String):void;

  /**
   * Adds listeners to components selected via <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a>. Accepts an
   * object containing component paths mapped to a hash of listener functions.
   *
   * <p>In the following example the <code>updateUser</code> function is mapped to to the <code>click</code>
   * event on a button component, which is a child of the <code>useredit</code> component.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      init: function() {
   *          this.control({
   *              'useredit button[action=save]': {
   *                  click: this.updateUser
   *              }
   *          });
   *      },
   *
   *      updateUser: function(button) {
   *          console.log('clicked the Save button');
   *      }
   *  });
   * </code></pre>
   *
   * <p>Or alternatively one call <code>control</code> with two arguments:</p>
   *
   * <pre><code> this.control('useredit button[action=save]', {
   *      click: this.updateUser
   *  });
   * </code></pre>
   *
   * <p>See <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> for more information on component selectors.</p>
   * @param selectors If a String, the second argument is used as the
   * listeners, otherwise an object of selectors -> listeners is assumed
   * @param listeners Config for listeners.
   */
  public native function control(selectors:*, listeners:Object = null):void;

  /**
   * Returns the value of <a href="#!/api/Ext.app.BaseController-cfg-control" rel="Ext.app.BaseController-cfg-control" class="docClass">control</a>.
   */
  public native function getControl():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.app.BaseController-cfg-listen" rel="Ext.app.BaseController-cfg-listen" class="docClass">listen</a>.
   */
  public native function getListen():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.app.BaseController-cfg-routes" rel="Ext.app.BaseController-cfg-routes" class="docClass">routes</a>.
   */
  public native function getRoutes():Object;

  /**
   * Adds listeners to different event sources (also called "event domains"). The
   * primary event domain is that of components, but there are also other event domains:
   * <a href="#!/api/Ext.app.domain.Global" rel="Ext.app.domain.Global" class="docClass">Global</a> domain that intercepts events fired from
   * <a href="#!/api/Ext.GlobalEvents" rel="Ext.GlobalEvents" class="docClass">Ext.GlobalEvents</a> Observable instance, <a href="#!/api/Ext.app.domain.Controller" rel="Ext.app.domain.Controller" class="docClass">Controller</a>
   * domain can be used to listen to events fired by other Controllers,
   * <a href="#!/api/Ext.app.domain.Store" rel="Ext.app.domain.Store" class="docClass">Store</a> domain gives access to Store events, and
   * <a href="#!/api/Ext.app.domain.Direct" rel="Ext.app.domain.Direct" class="docClass">Direct</a> domain can be used with Ext Direct Providers
   * to listen to their events.
   *
   * <p>To listen to "bar" events fired by a controller with id="foo":</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      init: function() {
   *          this.listen({
   *              controller: {
   *                  '#foo': {
   *                     bar: this.onFooBar
   *                  }
   *              }
   *          });
   *      },
   *      ...
   *  });
   * </code></pre>
   *
   * <p>To listen to "bar" events fired by any controller, and "baz" events
   * fired by Store with storeId="baz":</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      init: function() {
   *          this.listen({
   *              controller: {
   *                  '*': {
   *                     bar: this.onAnyControllerBar
   *                  }
   *              },
   *              store: {
   *                  '#baz': {
   *                      baz: this.onStoreBaz
   *                  }
   *              }
   *          });
   *      },
   *      ...
   *  });
   * </code></pre>
   *
   * <p>To listen to "idle" events fired by <a href="#!/api/Ext.GlobalEvents" rel="Ext.GlobalEvents" class="docClass">Ext.GlobalEvents</a> when other event
   * processing is complete and Ext JS is about to return control to the browser:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      init: function() {
   *          this.listen({
   *              global: {               // Global events are always fired
   *                  idle: this.onIdle   // from the same object, so there
   *              }                       // are no selectors
   *          });
   *      }
   *  });
   * </code></pre>
   *
   * <p>As this relates to components, the following example:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      init: function() {
   *          this.listen({
   *              component: {
   *                  'useredit button[action=save]': {
   *                     click: this.updateUser
   *                  }
   *              }
   *          });
   *      },
   *      ...
   *  });
   * </code></pre>
   *
   * <p>Is equivalent to:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      init: function() {
   *          this.control({
   *              'useredit button[action=save]': {
   *                 click: this.updateUser
   *              }
   *          });
   *      },
   *      ...
   *  });
   * </code></pre>
   *
   * <p>Of course, these can all be combined in a single call and used instead of
   * <code>control</code>, like so:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('AM.controller.Users', {
   *      init: function() {
   *          this.listen({
   *              global: {
   *                  idle: this.onIdle
   *              },
   *              controller: {
   *                  '*': {
   *                     foobar: this.onAnyFooBar
   *                  },
   *                  '#foo': {
   *                     bar: this.onFooBar
   *                  }
   *              },
   *              component: {
   *                  'useredit button[action=save]': {
   *                     click: this.updateUser
   *                  }
   *              },
   *              store: {
   *                  '#qux': {
   *                      load: this.onQuxLoad
   *                  }
   *              }
   *          });
   *      },
   *      ...
   *  });
   * </code></pre>
   * @param to Config object containing domains, selectors and listeners.
   * @param controller The controller to add the listeners to. Defaults to the current controller.
   */
  public native function listen(to:Object, controller:ext.app.Controller = null):void;

  /**
   * Update the hash. By default, it will not execute the routes if the current token and the
   * token passed are the same.
   * @param token The token to redirect to.  Can be either a String
   * or a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> instance - if a Model instance is passed it will
   * internally be converted into a String token by calling the Model's
   * <a href="#!/api/Ext.data.Model-method-toUrl" rel="Ext.data.Model-method-toUrl" class="docClass">toUrl</a> function.
   * @param force Force the update of the hash regardless of the current token.
   * @return Will return <code>true</code> if the token was updated.
   */
  public native function redirectTo(token:*, force:Boolean):Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.app.BaseController-cfg-control" rel="Ext.app.BaseController-cfg-control" class="docClass">control</a>.
   * @param control The new value.
   */
  public native function setControl(control:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.BaseController-cfg-listen" rel="Ext.app.BaseController-cfg-listen" class="docClass">listen</a>.
   * @param listen The new value.
   */
  public native function setListen(listen:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.app.BaseController-cfg-routes" rel="Ext.app.BaseController-cfg-routes" class="docClass">routes</a>.
   * @param routes The new value.
   */
  public native function setRoutes(routes:Object):void;
}
}