package ext.app {
import ext.Base;
import ext.ExtClass;

[Uses("ext.Base")]
[Native("Ext.app.EventDomain", require)]
/**
 * This class is a base class for an event domain. In the context of MVC, an "event domain"
 * is one or more base classes that fire events to which a Controller wants to listen. A
 * controller listens to events by describing the selectors for events of interest to it.
 *
 * <p>Matching selectors to the firer of an event is one key aspect that defines an event
 * domain. All event domain instances must provide a <code>match</code> method that tests selectors
 * against the event firer.</p>
 *
 * <p>When an event domain instance is created (typically as a <code>singleton</code>), its <code>type</code>
 * property is used to catalog the domain in the
 * <a href="#!/api/Ext.app.EventDomain-static-property-instances" rel="Ext.app.EventDomain-static-property-instances" class="docClass">Ext.app.EventDomain.instances</a> map.</p>
 *
 * <p>There are five event domains provided by default:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.app.domain.Component" rel="Ext.app.domain.Component" class="docClass">Component domain</a>. This is the primary event domain that
 * has been available since Ext JS MVC was introduced. This domain is defined as any class that
 * extends <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>, where the selectors use
 * <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery</a>.</li>
 * <li><a href="#!/api/Ext.app.domain.Global" rel="Ext.app.domain.Global" class="docClass">Global domain</a>. This domain provides Controllers with access
 * to events fired from <a href="#!/api/Ext.GlobalEvents" rel="Ext.GlobalEvents" class="docClass">Ext.GlobalEvents</a> Observable instance. These events represent
 * the state of the application as a whole, and are always anonymous. Because of this, Global
 * domain does not provide selectors at all.</li>
 * <li><a href="#!/api/Ext.app.domain.Controller" rel="Ext.app.domain.Controller" class="docClass">Controller domain</a>. This domain includes all classes
 * that extend <a href="#!/api/Ext.app.Controller" rel="Ext.app.Controller" class="docClass">Ext.app.Controller</a>. Events fired by Controllers will be available
 * within this domain; selectors are either Controller's <a href="#!/api/Ext.app.Controller-cfg-id" rel="Ext.app.Controller-cfg-id" class="docClass">id</a> or
 * '*' wildcard for any Controller.</li>
 * <li><a href="#!/api/Ext.app.domain.Store" rel="Ext.app.domain.Store" class="docClass">Store domain</a>. This domain is for classes extending
 * <a href="#!/api/Ext.data.AbstractStore" rel="Ext.data.AbstractStore" class="docClass">Ext.data.AbstractStore</a>. Selectors are either Store's
 * <a href="#!/api/Ext.data.AbstractStore-cfg-storeId" rel="Ext.data.AbstractStore-cfg-storeId" class="docClass">storeId</a> or '*' wildcard for any Store.</li>
 * <li><a href="#!/api/Ext.app.domain.Direct" rel="Ext.app.domain.Direct" class="docClass">Direct domain</a>. This domain includes all classes that extend
 * <a href="#!/api/Ext.direct.Provider" rel="Ext.direct.Provider" class="docClass">Ext.direct.Provider</a>. Selectors are either Provider's <a href="#!/api/Ext.direct.Provider-cfg-id" rel="Ext.direct.Provider-cfg-id" class="docClass">id</a>
 * or '*' wildcard for any Provider. This domain is optional and will be loaded only if
 * <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a> singleton is required in your application.</li>
 * </ul>
 */
public class EventDomain extends ext.Base {
  /**
   * An object map containing <code><a href="#!/api/Ext.app.EventDomain" rel="Ext.app.EventDomain" class="docClass">Ext.app.EventDomain</a></code> instances keyed by the value
   * of their <code>type</code> property.
   * @default {}
   */
  public static native function get instances():Object;

  /**
   * @private
   */
  public static native function set instances(value:Object):void;

  /**
   * @param config @inheritDoc
   */
  public function EventDomain(config:ext.app.EventDomain = null) {
    super();
  }

  [ExtConfig]
  /**
   * Name of the identifier property for this event domain.
   */
  public native function get idProperty():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set idProperty(value:String):void;

  /**
   * This method matches the firer of the event (the <code>target</code>) to the given <code>selector</code>.
   * Default matching is very simple: a match is true when selector equals target's
   * <a href="#!/api/Ext.app.EventDomain-cfg-idProperty" rel="Ext.app.EventDomain-cfg-idProperty" class="docClass">idProperty</a>, or when selector is '*' wildcard to match any
   * target.
   * @param target The firer of the event.
   * @param selector The selector to which to match the <code>target</code>.
   * @return <code>true</code> if the <code>target</code> matches the <code>selector</code>.
   */
  protected native function match(target:Object, selector:String):Boolean;

  /**
   * This method is called by the derived class to monitor <code>fireEvent</code> calls. Any call
   * to <code>fireEvent</code> on the target Observable will be intercepted and dispatched to any
   * listening Controllers. Assuming the original <code>fireEvent</code> method does not return
   * <code>false</code>, the event is passed to the <code>dispatch</code> method of this object.
   *
   * <p>This is typically called in the <code>constructor</code> of derived classes.</p>
   * @param observable The Observable to monitor for events.
   */
  protected native function monitor(observable:ext.ExtClass):void;
}
}