package ext.app {
import ext.Component;
import ext.data.Session;
import ext.data.Store;
import ext.mixin.IFactoryable;

[Uses("ext.app.BaseController")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Observable")]
[Native("Ext.app.ViewController", require)]
/**
 * A view controller is a controller that can be attached to a specific view
 * instance so it can manage the view and its child components. Each instance of the view
 * will have a new view controller, so the instances are isolated.
 *
 * <p>When a controller is specified on a view, the view automatically becomes a <a href="#!/api/Ext.container.Container-cfg-referenceHolder" rel="Ext.container.Container-cfg-referenceHolder" class="docClass">Ext.container.Container.referenceHolder</a>,
 * so it will receive inline events declared on the view. Sample usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *    extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *    fields: ['name', 'phone']
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserListController', {
 *     extend : '<a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a>',
 *     alias: 'controller.userlist',
 *
 *     init: function(view) {
 *         this.userCount = 0;
 *         var users = [],
 *             i;
 *
 *         for (i = 0; i &lt; 5; ++i) {
 *             users.push(this.getUser());
 *         }
 *         view.getStore().add(users);
 *     },
 *
 *     onAddClick: function() {
 *         this.addUser();
 *     },
 *
 *     onDeleteClick: function() {
 *         var view = this.getView(),
 *             selected = view.getSelectionModel().getSelection()[0],
 *             store = view.getStore();
 *
 *         store.remove(selected);
 *     },
 *
 *     onSelectionChange: function(selModel, selections) {
 *         this.lookupReference('delete').setDisabled(selections.length === 0);
 *     },
 *
 *     getUser: function() {
 *         ++this.userCount;
 *         return {
 *             name: 'User ' + this.userCount,
 *             phone: this.generatePhone()
 *         };
 *     },
 *
 *     addUser: function() {
 *         this.getView().getStore().add(this.getUser());
 *     },
 *
 *     generatePhone: function() {
 *         var num = '',
 *             i;
 *
 *         for (i = 0; i &lt; 7; ++i) {
 *             num += <a href="#!/api/Ext.Number-method-randomInt" rel="Ext.Number-method-randomInt" class="docClass">Ext.Number.randomInt</a>(0, 9);
 *             if (num.length === 3) {
 *                 num += '-';
 *             }
 *         }
 *         return num;
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserList', {
 *     extend: '<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>',
 *     controller: 'userlist',
 *
 *     tbar: [{
 *         text: 'Add',
 *         listeners: {
 *             click: 'onAddClick'
 *         }
 *     }, {
 *         text: 'Delete',
 *         reference: 'delete',
 *         listeners: {
 *             click: 'onDeleteClick'
 *         }
 *     }],
 *     store: {
 *         model: 'User'
 *     },
 *     selModel: {
 *         type: 'rowmodel',
 *         listeners: {
 *             selectionchange: 'onSelectionChange'
 *         }
 *     },
 *     columns: [{
 *         flex: 1,
 *         dataIndex: 'name',
 *         text: 'Name'
 *     }, {
 *         flex: 1,
 *         dataIndex: 'phone',
 *         text: 'Phone'
 *     }]
 * });
 *
 * <a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a>(function() {
 *     new UserList({
 *         renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *         width: 400,
 *         height: 200
 *     });
 * });
 * </code></pre>
 */
public class ViewController extends ext.app.BaseController implements ext.mixin.IFactoryable {
  /**
   * @param config @inheritDoc
   */
  public function ViewController(config:ext.app.ViewController = null) {
    super();
  }

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Destroy the view controller.
   */
  override public native function destroy(...params):void;

  /**
   * Template method called by the owning component's
   * <a href="#!/api/Ext.Component-method-afterRender" rel="Ext.Component-method-afterRender" class="docClass">afterRender</a> method.
   * @param component The owner component attached to the
   * ViewController
   */
  public native function afterRender(component:ext.Component):void;

  /**
   * Template method called by the owning component's
   * <a href="#!/api/Ext.Component-method-beforeRender" rel="Ext.Component-method-beforeRender" class="docClass">beforeRender</a> method.
   * @param component The owner component attached to the
   * ViewController
   */
  public native function beforeRender(component:ext.Component):void;

  /**
   * Template method called by the owning component's
   * <a href="#!/api/Ext.Component-method-onBoxReady" rel="Ext.Component-method-onBoxReady" class="docClass">onBoxReady</a> method.
   * @param component The owner component attached to the
   * ViewController
   */
  public native function boxReady(component:ext.Component):void;

  /**
   * This method closes the associated view. The manner in which this is done (that is,
   * the method called to close the view) is specified by <code>closeViewAction</code>.
   *
   * <p>It is common for views to map one or more events to this method to allow the view
   * to be closed.</p>
   */
  public native function closeView():void;

  /**
   * Returns an object holding the descendants of this view keyed by their
   * <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code>. This object should not be held
   * past the scope of the function calling this method. It will not be valid if items
   * are added or removed from this or any sub-container.
   *
   * <p>The intended usage is shown here (assume there are 3 components with reference
   * values of "foo", "bar" and "baz" at some level below this container):</p>
   *
   * <pre><code> onClick: function () {
   *      var refs = this.getReferences();
   *
   *      // using "refs" we can access any descendant by its "reference"
   *
   *      refs.foo.getValue() + refs.bar.getValue() + refs.baz.getValue();
   *  }
   * </code></pre>
   *
   * <p>If <code>this</code> component has a <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code> assigned
   * to it, that is <strong>not</strong> included in this object. That reference is understood to
   * belong to the ancestor container configured as the <code>referenceHolder</code>.</p>
   * @since 5.0.0
   * @return An object with each child reference. This will be <code>null</code> if this
   * container has no descendants with a <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code>
   * specified.
   */
  public native function getReferences():Object;

  /**
   * Get a <a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Ext.data.Session</a> attached to the view for this controller.
   * See <a href="#!/api/Ext.Component-method-lookupSession" rel="Ext.Component-method-lookupSession" class="docClass">Ext.Component.lookupSession</a>.
   * @since 5.0.0
   * @return The session. <code>null</code> if no session is found.
   */
  public native function getSession():ext.data.Session;

  /**
   * Get a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> attached to the <a href="#!/api/Ext.app.ViewController-method-getViewModel" rel="Ext.app.ViewController-method-getViewModel" class="docClass">ViewModel</a> attached to
   * this controller. See <a href="#!/api/Ext.app.ViewModel-method-getStore" rel="Ext.app.ViewModel-method-getStore" class="docClass">Ext.app.ViewModel.getStore</a>.
   * @since 5.0.0
   * @param name The name of the store.
   * @return The store. <code>null</code> if no store is found, or there is no
   * <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a> attached to the view for this controller.
   */
  public native function getStore(name:String):ext.data.Store;

  /**
   * Get the view for this controller.
   * @return The view.
   */
  public native function getView():ext.Component;

  /**
   * Get a <a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a> attached to the view for this controller.
   * See <a href="#!/api/Ext.Component-method-lookupViewModel" rel="Ext.Component-method-lookupViewModel" class="docClass">Ext.Component.lookupViewModel</a>.
   * @since 5.0.0
   * @return The ViewModel. <code>null</code> if no ViewModel is found.
   */
  public native function getViewModel():ext.app.ViewModel;

  /**
   * Gets a reference to the component with the specified <a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">Ext.Component.reference</a>
   * value.
   *
   * <p>The method is a short-hand for the <a href="#!/api/Ext.app.ViewController-method-lookupReference" rel="Ext.app.ViewController-method-lookupReference" class="docClass">lookupReference</a> method.</p>
   * @since 6.0.1
   * @param key The name of the reference to lookup.
   * @return The component, <code>null</code> if the reference doesn't exist.
   */
  public native function lookup(key:String):ext.Component;

  /**
   * Gets a reference to the component with the specified <a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">Ext.Component.reference</a>
   * value.
   *
   * <p>The <a href="#!/api/Ext.app.ViewController-method-lookup" rel="Ext.app.ViewController-method-lookup" class="docClass">lookup</a> method is a short-hand version of this method.</p>
   * @since 5.0.0
   * @param key The name of the reference to lookup.
   * @return The component, <code>null</code> if the reference doesn't exist.
   */
  public native function lookupReference(key:String):ext.Component;

  /**
   * Called before the view initializes. This is called before the view's
   * initComponent method has been called.
   * @param view The view
   */
  protected native function beforeInit(view:ext.Component):void;

  /**
   * Fires an event on the view. See <a href="#!/api/Ext.Component-method-fireEvent" rel="Ext.Component-method-fireEvent" class="docClass">Ext.Component.fireEvent</a>.
   * @param eventName The name of the event to fire.
   * @param args Variable number of parameters are passed to handlers.
   * @return returns false if any of the handlers return false otherwise it returns true.
   */
  protected native function fireViewEvent(eventName:String, ...args):Boolean;

  /**
   * Called when the view initializes. This is called after the view's initComponent
   * method has been called.
   * @param view The view
   */
  protected native function init(view:ext.Component):void;

  /**
   * Called when the view model instance for an attached view is first created.
   * @param viewModel The ViewModel
   */
  protected native function initViewModel(viewModel:ext.app.ViewModel):void;
}
}