package ext.container {
import ext.Action;
import ext.Component;
import ext.dom.Element;
import ext.layout.container.ContainerLayout;
import ext.mixin.IContainer;
import ext.mixin.IQueryable;
import ext.util.MixedCollection;

[Uses("ext.Component")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Native("Ext.container.Container", require)]
/**
 * Fires after any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> is added or inserted into the container.
 * @since 2.3.0
 * @eventType ext.container.events.Container_component_indexEvent.ADD
 */
[Event(name = 'onAdd', type = 'ext.container.events.Container_component_indexEvent')]
/**
 * Fires when the components in this container are arranged by the associated layout manager.
 * @since 2.3.0
 * @eventType ext.container.events.Container_layoutEvent.AFTER_LAYOUT
 */
[Event(name = 'onAfterLayout', type = 'ext.container.events.Container_layoutEvent')]
/**
 * Fires before any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> is added or inserted into the container.
 * A handler can return false to cancel the add.
 * @since 2.3.0
 * @eventType ext.container.events.Container_component_indexEvent.BEFORE_ADD
 */
[Event(name = 'onBeforeAdd', type = 'ext.container.events.Container_component_indexEvent')]
/**
 * Fires before any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> is removed from the container.  A handler can return
 * false to cancel the remove.
 * @since 2.3.0
 * @eventType ext.container.events.Container_componentEvent.BEFORE_REMOVE
 */
[Event(name = 'onBeforeRemove', type = 'ext.container.events.Container_componentEvent')]
/**
 * Fires after any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> has changed its ordinal position within the container.
 * @eventType ext.container.events.Container_component_prevIndex_newIndexEvent.CHILD_MOVE
 */
[Event(name = 'onChildMove', type = 'ext.container.events.Container_component_prevIndex_newIndexEvent')]
/**
 * Fires after any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> is removed from the container.
 * @since 2.3.0
 * @eventType ext.container.events.Container_componentEvent.REMOVE
 */
[Event(name = 'onRemove', type = 'ext.container.events.Container_componentEvent')]
/**
 * Base class for any <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> that may contain other Components. Containers handle the basic behavior of
 * containing items, namely adding, inserting and removing items.
 *
 * <p>The most commonly used Container classes are <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>, <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a> and
 * <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>. If you do not need the capabilities offered by the aforementioned classes you can create a
 * lightweight Container to be encapsulated by an HTML element to your specifications by using the
 * <a href="#!/api/Ext.Component-cfg-autoEl" rel="Ext.Component-cfg-autoEl" class="docClass">autoEl</a> config option.</p>
 *
 * <p>The code below illustrates how to explicitly create a Container:</p>
 *
 * <pre class='inline-example '><code>// Explicitly create a Container
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>', {
 *     layout: {
 *         type: 'hbox'
 *     },
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     border: 1,
 *     style: {borderColor:'#000000', borderStyle:'solid', borderWidth:'1px'},
 *     defaults: {
 *         labelWidth: 80,
 *         // implicitly create Container by specifying xtype
 *         xtype: 'datefield',
 *         flex: 1,
 *         style: {
 *             padding: '10px'
 *         }
 *     },
 *     items: [{
 *         xtype: 'datefield',
 *         name: 'startDate',
 *         fieldLabel: 'Start date'
 *     },{
 *         xtype: 'datefield',
 *         name: 'endDate',
 *         fieldLabel: 'End date'
 *     }]
 * });
 * </code></pre>
 *
 * <h2>Layout</h2>
 *
 * <p>Container classes delegate the rendering of child Components to a layout manager class which must be configured into
 * the Container using the <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code> configuration property.</p>
 *
 * <p>When either specifying child <code><a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a></code> of a Container, or dynamically <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">adding</a> Components to a
 * Container, remember to consider how you wish the Container to arrange those child elements, and whether those child
 * elements need to be sized using one of Ext's built-in <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code> schemes. By default, Containers use the
 * <a href="#!/api/Ext.layout.container.Auto" rel="Ext.layout.container.Auto" class="docClass">Auto</a> scheme which only renders child components, appending them one after the other
 * inside the Container, and <strong>does not apply any sizing</strong> at all.</p>
 *
 * <p>A common mistake is when a developer neglects to specify a <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code> (e.g. GridPanels or
 * TreePanels are added to Containers for which no <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code> has been specified). If a Container is left to
 * use the default <a href="#!/api/Ext.layout.container.Auto" rel="Ext.layout.container.Auto" class="docClass">Auto</a> scheme, none of its child components will be resized, or changed in
 * any way when the Container is resized.</p>
 *
 * <p>Certain layout managers allow dynamic addition of child components. Those that do include
 * <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a>, <a href="#!/api/Ext.layout.container.Anchor" rel="Ext.layout.container.Anchor" class="docClass">Ext.layout.container.Anchor</a>, <a href="#!/api/Ext.layout.container.VBox" rel="Ext.layout.container.VBox" class="docClass">Ext.layout.container.VBox</a>,
 * <a href="#!/api/Ext.layout.container.HBox" rel="Ext.layout.container.HBox" class="docClass">Ext.layout.container.HBox</a>, and <a href="#!/api/Ext.layout.container.Table" rel="Ext.layout.container.Table" class="docClass">Ext.layout.container.Table</a>. For example:</p>
 *
 * <pre><code>//  Create the GridPanel.
 * var myNewGrid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     store: myStore,
 *     headers: myHeaders,
 *     title: 'Results', // the title becomes the title of the tab
 * });
 *
 * myTabPanel.add(myNewGrid); // <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a> implicitly uses <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a>
 * myTabPanel.setActiveTab.(myNewGrid);
 * </code></pre>
 *
 * <p>The example above adds a newly created GridPanel to a TabPanel. Note that a TabPanel uses <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a> as its layout manager which means all its child items are sized to <a href="#!/api/Ext.layout.container.Fit" rel="Ext.layout.container.Fit" class="docClass">fit</a> exactly into its client area.</p>
 *
 * <p><strong><em>Overnesting is a common problem</em></strong>. An example of overnesting occurs when a GridPanel is added to a TabPanel by
 * wrapping the GridPanel <em>inside</em> a wrapping Panel (that has no <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code> specified) and then add that
 * wrapping Panel to the TabPanel. The point to realize is that a GridPanel <strong>is</strong> a Component which can be added
 * directly to a Container. If the wrapping Panel has no <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code> configuration, then the overnested
 * GridPanel will not be sized as expected.</p>
 *
 * <h2><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">References</a> and <a href="#!/api/Ext.container.Container-cfg-referenceHolder" rel="Ext.container.Container-cfg-referenceHolder" class="docClass">Reference Holders</a></h2>
 *
 * <p>Reference holders are used to keep references to child components inside a hierarchy.</p>
 *
 * <p>This functionality allows the connection of encapsulated references between containers
 * and their child components declaratively. Simple usage:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Login', {
 *     extend: '<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>',
 *
 *     // This config is not compatible with the more common "controller" config
 *     // used to specify a ViewController for the view. When a ViewController is
 *     // specified it effectively acts as the "reference holder" for the view. In
 *     // this example we simply mark this container as the reference holder for
 *     // demonstration purposes.
 *     referenceHolder: true,
 *
 *     title: 'Login',
 *     items: [{
 *         xtype: 'form',
 *         items: [{
 *             xtype: 'textfield',
 *             reference: 'username', // A named reference to be held on the referenceHolder
 *             name: 'username',
 *             fieldLabel: 'Username'
 *         }, {
 *             xtype: 'textfield',
 *             reference: 'password', // A named reference to be held on the referenceHolder
 *             name: 'password',
 *             fieldLabel: 'Password'
 *         }]
 *     }]
 * });
 * var w = new Login();
 * console.log(w.lookupReference('password')); // The password field
 * </code></pre>
 *
 * <p>Reference holders are also encapsulated, so a reference will only be put on the closest
 * reference holder above it in the component hierarchy:</p>
 *
 * <pre><code>var ct = new <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>({
 *     referenceHolder: true,
 *     items: [{
 *         xtype: 'container',
 *         referenceHolder: true,
 *         reference: 'innerCt1',
 *         items: [{
 *             xtype: 'component',
 *             reference: 'a',
 *             id: 'a1'
 *         }, {
 *             xtype: 'component',
 *             reference: 'b',
 *             id: 'b1'
 *         }]
 *     }, {
 *         xtype: 'container',
 *         referenceHolder: true,
 *         reference: 'innerCt2',
 *         items: [{
 *             xtype: 'component',
 *             reference: 'a',
 *             id: 'a2'
 *         }, {
 *             xtype: 'component',
 *             reference: 'b',
 *             id: 'b2'
 *         }]
 *     }]
 * });
 * // The main container will not have references to a/b, each innerCt will
 * console.log(ct.lookupReference('a'), ct.lookupReference('b'));
 * var inner1 = ct.lookupReference('innerCt1');
 * var inner2 = ct.lookupReference('innerCt2');
 *
 * console.log(inner1.lookupReference('a').id, inner1.lookupReference('b').id);
 * console.log(inner2.lookupReference('a').id, inner2.lookupReference('b').id);
 * </code></pre>
 *
 * <p>If the view has a controller attached, it will automatically become a <a href="#!/api/Ext.container.Container-cfg-referenceHolder" rel="Ext.container.Container-cfg-referenceHolder" class="docClass">referenceHolder</a>.
 * References will be available in both the view and the controller:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('ProfileController', {
 *     extend: '<a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a>',
 *     alias: 'controller.profile',
 *
 *     init: function() {
 *         console.log(this.lookupReference('firstName'));
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Profile', {
 *     extend: '<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>',
 *     controller: 'profile',
 *     items: [{
 *         xtype: 'textfield',
 *         reference: 'firstName',
 *         fieldLabel: 'First Name'
 *     }]
 * });
 *
 * new Profile();
 * </code></pre>
 *
 * <h2>Events &amp; <a href="#!/api/Ext.container.Container-cfg-defaultListenerScope" rel="Ext.container.Container-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a></h2>
 *
 * <p>Events can use the default listener scope to determine at runtime the appropriate place
 * to fire. This allows for declarative binding of events in a useful way:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyView', {
 *     extend: '<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>',
 *     defaultListenerScope: true,
 *     referenceHolder: true,
 *     items: [{
 *         xtype: 'textfield',
 *         reference: 'myfield'
 *     }, {
 *         xtype: 'button',
 *         text: 'Set to A',
 *         listeners: {
 *             click: 'onButtonAClick'
 *         }
 *     }, {
 *         xtype: 'button',
 *         text: 'Set to B',
 *         listeners: {
 *             click: 'onButtonBClick'
 *         }
 *     }],
 *
 *     onButtonAClick: function() {
 *         this.lookupReference('myfield').setValue('A');
 *     },
 *
 *     onButtonBClick: function() {
 *         this.lookupReference('myfield').setValue('B');
 *     }
 * });
 * </code></pre>
 *
 * <p>Like <a href="#!/api/Ext.container.Container-cfg-referenceHolder" rel="Ext.container.Container-cfg-referenceHolder" class="docClass">referenceHolder</a>, the <a href="#!/api/Ext.container.Container-cfg-defaultListenerScope" rel="Ext.container.Container-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a> is encapsulated, the scope will
 * be resolved at the closest <a href="#!/api/Ext.container.Container-cfg-defaultListenerScope" rel="Ext.container.Container-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a> above it in the component hierarchy:</p>
 *
 * <pre><code>var ct = new <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>({
 *     defaultListenerScope: true,
 *     onCustomEvent: function() {
 *         console.log('Outer called'); // Will NOT be called
 *     },
 *     items: [{
 *         xtype: 'container',
 *         defaultListenerScope: true,
 *         onCustomEvent: function() {
 *             console.log('Inner called'); // Will be called
 *         },
 *         items: [{
 *             xtype: 'component',
 *             itemId: 'child',
 *             listeners: {
 *                 customevent: 'onCustomEvent'
 *             }
 *         }]
 *     }]
 * });
 * // The main container will not have references to a/b, each innerCt will
 * console.log(ct.lookupReference('a'), ct.lookupReference('b'));
 * var inner1 = ct.lookupReference('innerCt1');
 * var inner2 = ct.lookupReference('innerCt2');
 *
 * console.log(inner1.lookupReference('a').id, inner1.lookupReference('b').id);
 * console.log(inner2.lookupReference('a').id, inner2.lookupReference('b').id);
 * </code></pre>
 *
 * <p>Similar to references, if a <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a> is attached to this view, it becomes
 * the <a href="#!/api/Ext.container.Container-cfg-defaultListenerScope" rel="Ext.container.Container-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>, which means un-scoped, late bound events will be directed to the
 * controller. This is powerful as it allows views to be totally declarative:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.controller.Login', {
 *     extend : '<a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a>',
 *     alias : 'controller.login',
 *
 *     init: function() {
 *         this.sendCount = 0;
 *     },
 *
 *     onLoginClick : function(btn) {
 *         this.login();
 *     },
 *
 *     onFieldSpecialKey : function(field, e) {
 *         if (e.getKey() === e.ENTER) {
 *            this.login();
 *         }
 *     },
 *
 *     login : function() {
 *        var form = this.lookupReference('form');
 *         this.lookupReference('error').hide();
 *         if (form.isValid()) {
 *             console.log('Do the login!');
 *             // Server responded...
 *             if (++this.sendCount % 2 === 0) {
 *                 this.onServerSuccess();
 *             } else {
 *                 this.onServerFailure();
 *             }
 *         }
 *     },
 *
 *     onServerSuccess: function() {
 *         // Proceed
 *         console.log('All good');
 *     },
 *
 *     onServerFailure: function() {
 *         var error = this.lookupReference('error');
 *         error.update('Invalid username/password');
 *         error.show();
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.Login', {
 *     extend : '<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>',
 *     controller : 'login',
 *     referenceHolder: true,
 *
 *     title : 'Login',
 *     width : 400,
 *
 *     items : [{
 *         xtype : 'form',
 *         reference : 'form',
 *         border : false,
 *         bodyPadding : 10,
 *         defaultType : 'textfield',
 *         defaults : {
 *             anchor : '90%',
 *             allowBlank : false,
 *             enableKeyEvents : true
 *         },
 *         items : [{
 *             xtype: 'component',
 *             reference: 'error',
 *             hidden: true,
 *             margin: '0 0 10 0',
 *             style: 'color: red;'
 *         }, {
 *             name : 'username',
 *             fieldLabel : 'Username',
 *             reference : 'username',
 *             listeners : {
 *                 specialkey : 'onFieldSpecialKey'
 *             }
 *         }, {
 *             name : 'password',
 *             fieldLabel : 'Password',
 *             reference : 'password',
 *             inputType : 'password',
 *             listeners : {
 *                 specialkey : 'onFieldSpecialKey'
 *             }
 *         }]
 *     }],
 *     buttons : ['-&gt;', {
 *         text : 'Login',
 *         listeners : {
 *             click : 'onLoginClick'
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <h2>Adding via remote configuration</h2>
 *
 * <p>A server side script can be used to add Components which are generated dynamically on the server. An example of
 * adding a GridPanel to a TabPanel where the GridPanel is generated by the server based on certain parameters:</p>
 *
 * <pre><code>// execute an Ajax request to invoke server side script:
 * <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
 *     url: 'gen-invoice-grid.php',
 *     // send additional parameters to instruct server script
 *     params: {
 *         startDate: <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('start-date').getValue(),
 *         endDate: <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('end-date').getValue()
 *     },
 *     // process the response object to add it to the TabPanel:
 *     success: function(xhr) {
 *         var newComponent = eval(xhr.responseText); // see discussion below
 *         myTabPanel.add(newComponent); // add the component to the TabPanel
 *         myTabPanel.setActiveTab(newComponent);
 *     },
 *     failure: function() {
 *         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>("Grid create failed", "Server communication failure");
 *     }
 * });
 * </code></pre>
 *
 * <p>The server script needs to return a JSON representation of a configuration object, which, when decoded will return a
 * config object with an <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a>. The server might return the following JSON:</p>
 *
 * <pre><code>{
 *     "xtype": 'grid',
 *     "title": 'Invoice Report',
 *     "store": {
 *         "model": 'Invoice',
 *         "proxy": {
 *             "type": 'ajax',
 *             "url": 'get-invoice-data.php',
 *             "reader": {
 *                 "type": 'json'
 *                 "record": 'transaction',
 *                 "idProperty": 'id',
 *                 "totalRecords": 'total'
 *             })
 *         },
 *         "autoLoad": {
 *             "params": {
 *                 "startDate": '01/01/2008',
 *                 "endDate": '01/31/2008'
 *             }
 *         }
 *     },
 *     "headers": [
 *         {"header": "Customer", "width": 250, "dataIndex": 'customer', "sortable": true},
 *         {"header": "Invoice Number", "width": 120, "dataIndex": 'invNo', "sortable": true},
 *         {"header": "Invoice Date", "width": 100, "dataIndex": 'date', "renderer": <a href="#!/api/Ext.util.Format-method-dateRenderer" rel="Ext.util.Format-method-dateRenderer" class="docClass">Ext.util.Format.dateRenderer</a>('M d, y'), "sortable": true},
 *         {"header": "Value", "width": 120, "dataIndex": 'value', "renderer": 'usMoney', "sortable": true}
 *     ]
 * }
 * </code></pre>
 *
 * <p>When the above code fragment is passed through the <code>eval</code> function in the success handler of the Ajax request, the
 * result will be a config object which, when added to a Container, will cause instantiation of a GridPanel. <strong>Be sure
 * that the Container is configured with a layout which sizes and positions the child items to your requirements.</strong></p>
 *
 * <p><strong>Note:</strong> since the code above is <em>generated</em> by a server script, the <code>autoLoad</code> params for the Store, the user's
 * preferred date format, the metadata to allow generation of the Model layout, and the ColumnModel can all be generated
 * into the code since these are all known on the server.</p>
 */
public class Container extends ext.Component implements ext.mixin.IQueryable, ext.mixin.IContainer {
  /**
   * @param config @inheritDoc
   */
  public function Container(config:ext.container.Container = null) {
    super();
  }

  /**
   * The MixedCollection containing all the floating child items of this container.
   * Will be <code>undefined</code> if there are no floating child items.
   * @since 4.1.0
   */
  public native function get floatingItems():ext.util.MixedCollection;

  /**
   * @private
   */
  public native function set floatingItems(value:ext.util.MixedCollection):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Container, or subclass thereof.
   * @default true
   */
  public native function get isContainer():Boolean;

  /**
   * @private
   */
  public native function set isContainer(value:Boolean):void;

  /**
   * The Collection containing all the child items of this container.
   * Alias for <code>items</code> to avoid name-clash with the config option.
   * @since 2.3.0
   */
  public native function get itemCollection():ext.util.MixedCollection;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object containing properties which define named <a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">actions</a>
   * for this container and any descendant components.
   *
   * <p>An Action encapsulates a shareable, reusable set of properties which define a
   * "clickable" UI component such as a <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a> or
   * <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu item</a>, or <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">panel header tool</a>,
   * or an <a href="#!/api/Ext.grid.column.Action" rel="Ext.grid.column.Action" class="docClass">ActionColumn item</a></p>
   *
   * <p>An Action, or more conveniently, the <em>name</em> of an action prefixed with <code>'&#64;'</code>
   * may be used as a config object for creating child components which use a <code>handler</code>
   * config property to reference a Controller method to invoke when the component is
   * clicked.</p>
   *
   * <p>The property name is the action name, which may then be used as a child item
   * configuration in an <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> configuration in
   * any descendant component such as a toolbar or a menu, or in a
   * <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">tools</a> configuration of a Panel.</p>
   *
   * <p>The property value is a configuration object for any clickable component.</p>
   *
   * <p>See the <a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">Ext.Action</a> class for an example of reusable Actions.</p>
   * @since 6.2.0
   * @see #getActions()
   * @see #setActions()
   */
  public native function get actions():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set actions(value:Object):void;

  [Bindable(event = "DUMMY")]
  [ExtConfig(mode = "append")]
  [ArrayElementType("ext.Action")]
  /**
   * An array of named <a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">actions</a>
   * for this container and any descendant components. All actions in this array must have an <code>actionId</code>
   * that is used as the key when the array of actions is converted to the <code>actions</code> map.
   *
   * <p>An Action encapsulates a shareable, reusable set of properties which define a
   * "clickable" UI component such as a <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">button</a> or
   * <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu item</a>, or <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">panel header tool</a>,
   * or an <a href="#!/api/Ext.grid.column.Action" rel="Ext.grid.column.Action" class="docClass">ActionColumn item</a></p>
   *
   * <p>An Action, or more conveniently, the <em>name</em> of an action prefixed with <code>'&#64;'</code>
   * may be used as a config object for creating child components which use a <code>handler</code>
   * config property to reference a Controller method to invoke when the component is
   * clicked.</p>
   *
   * <p>The property name is the action name, which may then be used as a child item
   * configuration in an <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> configuration in
   * any descendant component such as a toolbar or a menu, or in a
   * <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">tools</a> configuration of a Panel.</p>
   *
   * <p>The property value is a configuration object for any clickable component.</p>
   *
   * <p>See the <a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">Ext.Action</a> class for an example of reusable Actions.</p>
   * @since 6.2.0
   */
  public native function get actionList():Array;

  [Bindable(event = "DUMMY")]
  [ExtConfig(mode = "append")]
  [ArrayElementType("ext.Action")]
  /**
   * @private
   */
  public native function set actionList(value:Array):void;

  [ExtConfig]
  /**
   * A string component id or the numeric index of the component that should be
   * initially activated within the container's layout on render.  For example,
   * activeItem: 'item-1' or activeItem: 0 (index 0 = the first item in the
   * container's collection).  activeItem only applies to layout styles that can
   * display items one at a time (like <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a> and
   * <a href="#!/api/Ext.layout.container.Fit" rel="Ext.layout.container.Fit" class="docClass">Ext.layout.container.Fit</a>).
   * @since 2.3.0
   * @see #setActiveItem()
   */
  public native function get activeItem():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set activeItem(value:*):void;

  [ExtConfig]
  /**
   * Defines the anchoring size of container.
   * Either a number to define the width of the container or an object with <code>width</code> and <code>height</code> fields.
   */
  public native function get anchorSize():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set anchorSize(value:*):void;

  [ExtConfig]
  /**
   * If true the container will automatically destroy any contained component that is removed
   * from it, else destruction must be handled manually.
   * @default true
   * @since 2.3.0
   */
  public native function get autoDestroy():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoDestroy(value:Boolean):void;

  [ExtConfig]
  /**
   * An array of events that, when fired, should be bubbled to any parent container.
   * See <a href="#!/api/Ext.util.Observable-method-enableBubble" rel="Ext.util.Observable-method-enableBubble" class="docClass">Ext.util.Observable.enableBubble</a>.
   * @since 3.4.0
   */
  public native function get bubbleEvents():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bubbleEvents(value:Array):void;

  [ExtConfig]
  /**
   * Specifies a child Component to receive focus when this Container's <a href="#!/api/Ext.container.Container-method-focus" rel="Ext.container.Container-method-focus" class="docClass">focus</a>
   * method is called. Should be a valid <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">query</a> selector.
   */
  public native function get defaultFocus():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultFocus(value:String):void;

  [ExtConfig]
  /**
   * The default <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">xtype</a> of child Components to create in this Container when
   * a child item is specified as a raw configuration object, rather than as an instantiated Component.
   * @default "panel"
   * @since 2.3.0
   */
  public native function get defaultType():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultType(value:String):void;

  [ExtConfig]
  /**
   * This option is a means of applying default settings to all added items whether added
   * through the <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> config or via the <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">add</a> or <a href="#!/api/Ext.container.Container-method-insert" rel="Ext.container.Container-method-insert" class="docClass">insert</a> methods.
   *
   * <p>Defaults are applied to both config objects and instantiated components conditionally
   * so as not to override existing properties in the item (see <a href="#!/api/Ext-method-applyIf" rel="Ext-method-applyIf" class="docClass">Ext.applyIf</a>).</p>
   *
   * <p>If the defaults option is specified as a function, then the function will be called
   * using this Container as the scope (<code>this</code> reference) and passing the added item as
   * the first parameter. Any resulting object from that call is then applied to the item
   * as default properties.</p>
   *
   * <p>For example, to automatically apply padding to the body of each of a set of
   * contained <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a> items, you could pass:
   * <code>defaults: {bodyStyle:'padding:15px'}</code>.</p>
   *
   * <p>Usage:</p>
   *
   * <pre><code>defaults: { // defaults are applied to items, not the container
   *     scrollable: true
   * },
   * items: [
   *     // default will not be applied here, panel1 will be scrollable: false
   *     {
   *         xtype: 'panel',
   *         id: 'panel1',
   *         scrollable: false
   *     },
   *     // this component will have scrollable: true
   *     new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *         id: 'panel2'
   *     })
   * ]
   * </code></pre>
   * @since 2.3.0
   */
  public native function get defaults():*;

  [ExtConfig(extractXType = "defaultType")]
  /**
   * @private
   */
  public native function set defaults(value:*):void;

  [ExtConfig]
  /**
   * True to move any component to the <a href="#!/api/Ext-method-getDetachedBody" rel="Ext-method-getDetachedBody" class="docClass">detachedBody</a> when the component is
   * removed from this container. This option is only applicable when the component is not destroyed while
   * being removed, see <a href="#!/api/Ext.container.Container-cfg-autoDestroy" rel="Ext.container.Container-cfg-autoDestroy" class="docClass">autoDestroy</a> and <a href="#!/api/Ext.container.Container-method-remove" rel="Ext.container.Container-method-remove" class="docClass">remove</a>. If this option is set to false, the DOM
   * of the component will remain in the current place until it is explicitly moved.
   * @default true
   */
  public native function get detachOnRemove():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set detachOnRemove(value:Boolean):void;

  [ExtConfig]
  [ArrayElementType("ext.Component")]
  /**
   * A single item, or an array of child Components to be added to this container
   *
   * <p><strong>Unless configured with a <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>, a Container simply renders child
   * Components serially into its encapsulating element and performs no sizing or
   * positioning upon them.</strong></p>
   *
   * <p>Example:</p>
   *
   * <pre><code>// specifying a single item
   * items: {...},
   * layout: 'fit',    // The single items is sized to fit
   *
   * // specifying multiple items
   * items: [{...}, {...}],
   * layout: 'hbox', // The items are arranged horizontally
   * </code></pre>
   *
   * <p>Each item may be:</p>
   *
   * <ul>
   * <li>A <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a></li>
   * <li>A Component configuration object</li>
   * </ul>
   *
   *
   * <p>If a configuration object is specified, the actual type of Component to be
   * instantiated my be indicated by using the <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a> option.</p>
   *
   * <p>Every Component class has its own <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a>.</p>
   *
   * <p>If an <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a> is not explicitly specified, the
   * <a href="#!/api/Ext.container.Container-cfg-defaultType" rel="Ext.container.Container-cfg-defaultType" class="docClass">defaultType</a> for the Container is used, which by default is usually <code>panel</code>.</p>
   *
   * <h1>Notes:</h1>
   *
   * <p>Ext uses lazy rendering. Child Components will only be rendered
   * should it become necessary. Items are automatically laid out when they are first
   * shown (no sizing is done while hidden), or in response to a <a href="#!/api/Ext.container.Container-method-updateLayout" rel="Ext.container.Container-method-updateLayout" class="docClass">updateLayout</a> call.</p>
   *
   * <p>Do not specify <a href="#!/api/Ext.panel.Panel-cfg-contentEl" rel="Ext.panel.Panel-cfg-contentEl" class="docClass">contentEl</a> or
   * <a href="#!/api/Ext.panel.Panel-cfg-html" rel="Ext.panel.Panel-cfg-html" class="docClass">html</a> with <code>items</code>.</p>
   * @since 2.3.0
   */
  public native function get items():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set items(value:Array):void;

  [ExtConfig]
  /**
   * <strong>Important</strong>: In order for child items to be correctly sized and
   * positioned, typically a layout manager <strong>must</strong> be specified through
   * the <code>layout</code> configuration option.
   *
   * <p>The sizing and positioning of child <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> is the responsibility of
   * the Container's layout manager which creates and manages the type of layout
   * you have in mind.  For example:</p>
   *
   * <p>If the layout configuration is not explicitly specified for
   * a general purpose container (e.g. Container or Panel) the
   * <a href="#!/api/Ext.layout.container.Auto" rel="Ext.layout.container.Auto" class="docClass">default layout manager</a> will be used
   * which does nothing but render child components sequentially into the
   * Container (no sizing or positioning will be performed in this situation).</p>
   *
   * <p><strong>layout</strong> may be specified as either as an Object or as a String:</p>
   *
   * <h2>Specify as an Object</h2>
   *
   * <p>Example usage:</p>
   *
   * <pre><code>layout: {
   *     type: 'vbox',
   *     align: 'left'
   * }
   * </code></pre>
   *
   * <ul>
   * <li><p><strong>type</strong></p>
   *
   * <p>The layout type to be used for this container.  If not specified,
   * a default <a href="#!/api/Ext.layout.container.Auto" rel="Ext.layout.container.Auto" class="docClass">Ext.layout.container.Auto</a> will be created and used.</p>
   *
   * <p>Valid layout <code>type</code> values are listed in <a href="#!/api/Ext.enums.Layout" rel="Ext.enums.Layout" class="docClass">Ext.enums.Layout</a>.</p></li>
   * <li><p>Layout specific configuration properties</p>
   *
   * <p>Additional layout specific configuration properties may also be
   * specified. For complete details regarding the valid config options for
   * each layout type, see the layout class corresponding to the <code>type</code>
   * specified.</p></li>
   * </ul>
   *
   *
   * <h2>Specify as a String</h2>
   *
   * <p>Example usage:</p>
   *
   * <pre><code>layout: 'vbox'
   * </code></pre>
   *
   * <ul>
   * <li><p><strong>layout</strong></p>
   *
   * <p>The layout <code>type</code> to be used for this container (see <a href="#!/api/Ext.enums.Layout" rel="Ext.enums.Layout" class="docClass">Ext.enums.Layout</a>
   * for list of valid values).</p>
   *
   * <p>Additional layout specific configuration properties. For complete
   * details regarding the valid config options for each layout type, see the
   * layout class corresponding to the <code>layout</code> specified.</p></li>
   * </ul>
   *
   *
   * <h2>Configuring the default layout type</h2>
   *
   * <p>If a certain Container class has a default layout (For example a <a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Toolbar</a>
   * with a default <code>Box</code> layout), then to simply configure the default layout,
   * use an object, but without the <code>type</code> property:</p>
   *
   * <pre><code>xtype: 'toolbar',
   * layout: {
   *     pack: 'center'
   * }
   * </code></pre>
   * @default 'auto'
   * @since 2.3.0
   * @see #setLayout()
   */
  public native function get layout():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set layout(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If <code>true</code>, this container will be marked as being a point in the hierarchy where
   * references to items with a specified <code>reference</code> config will be held. The container
   * will automatically become a referenceHolder if a controller is specified.
   *
   * <p>See the introductory docs for <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for more information
   * about references &amp; reference holders.</p>
   * @default false
   * @see #getReferenceHolder()
   * @see #setReferenceHolder()
   */
  public native function get referenceHolder():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set referenceHolder(value:Boolean):void;

  [ExtConfig]
  /**
   * If true, suspend calls to updateLayout. Useful when batching multiple adds to a container
   * and not passing them as multiple arguments or an array.
   * @default false
   */
  public native function get suspendLayout():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set suspendLayout(value:Boolean):void;

  [ExtConfig]
  /**
   * When set to true, two elements are added to the container's element. These are the tabGuardBeforeEl and tabGuardAfterEl.
   */
  public native function get tabGuard():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tabGuard(value:Boolean):void;

  /**
   * Disables all child input fields and buttons.
   * @param silent Passing <code>true</code> will suppress the <code>disable</code> event from being fired.
   * @default false
   */
  override public native function disable(silent:Boolean = false):void;

  /**
   * Enables all child input fields and buttons.
   * @param silent Passing <code>true</code> will suppress the <code>enable</code> event from being fired.
   * @default false
   */
  override public native function enable(silent:Boolean = false):void;

  /**
   * Returns the focus holder element associated with this Container.
   * By default, this is the Container's target element; however if <a href="#!/api/Ext.container.Container-cfg-defaultFocus" rel="Ext.container.Container-cfg-defaultFocus" class="docClass">defaultFocus</a>
   * is defined, the child component referenced by that property will be found
   * and returned instead.
   * @return the focus holding element.
   */
  override protected native function getFocusEl():ext.dom.Element;

  /**
   * Called by <code>getInherited</code> to initialize the inheritedState the first time it is
   * requested.
   */
  override protected native function initInheritedState(inheritedState:Object):void;

  [Deprecated(replacement = "moveAfter", since = "5.0")]
  /**
   * Moves a Component within the Container. This method does <strong>not</strong> account for things
   * like splitter components added by a layout. To better handle these situations, it
   * is recommended to use <code><a href="#!/api/Ext.container.Container-method-moveBefore" rel="Ext.container.Container-method-moveBefore" class="docClass">moveBefore</a></code> or <code><a href="#!/api/Ext.container.Container-method-moveAfter" rel="Ext.container.Container-method-moveAfter" class="docClass">moveAfter</a></code> instead.
   * @param direction Possible values are:
   * <ul>
   * <li><code>"l"</code> (or <code>"left"</code>)</li>
   * <li><code>"r"</code> (or <code>"right"</code>)</li>
   * <li><code>"t"</code> (or <code>"top"</code>, or <code>"up"</code>)</li>
   * <li><code>"b"</code> (or <code>"bottom"</code>, or <code>"down"</code>)</li>
   * </ul>
   * @param distance How far to move the element in pixels
   * @param animate true for the default animation or a standard
   * Element animation config object
   * @return component The Component that was moved.
   */
  override public native function move(direction:String, distance:Number, animate:* = undefined):void;

  /**
   * Adds <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>(s) to this Container.
   *
   * <h2>Description:</h2>
   *
   * <ul>
   * <li>Fires the <a href="#!/api/Ext.container.Container-event-beforeadd" rel="Ext.container.Container-event-beforeadd" class="docClass">beforeadd</a> event before adding.</li>
   * <li>The Container's <a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">default config values</a> will be applied
   * accordingly (see <code><a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">defaults</a></code> for details).</li>
   * <li>Fires the <code><a href="#!/api/Ext.container.Container-event-add" rel="Ext.container.Container-event-add" class="docClass">add</a></code> event after the component has been added.</li>
   * </ul>
   *
   *
   * <h2>Notes:</h2>
   *
   * <p>If the Container is <strong>already rendered</strong> when <code>add</code>
   * is called, it will render the newly added Component into its content area.</p>
   *
   * <p><strong>If</strong> the Container was configured with a size-managing <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a> manager,
   * the Container will recalculate its internal layout at this time too.</p>
   *
   * <p>Note that the default layout manager simply renders child Components sequentially
   * into the content area and thereafter performs no sizing.</p>
   *
   * <p>If adding multiple new child Components, pass them as an array to the <code>add</code> method,
   * so that only one layout recalculation is performed.</p>
   *
   * <pre><code>tb = new <a href="#!/api/Ext.toolbar.Toolbar" rel="Ext.toolbar.Toolbar" class="docClass">Ext.toolbar.Toolbar</a>({
   *     renderTo: document.body
   * });  // toolbar is rendered
   * // add multiple items.
   * // default type for Toolbar is 'button')
   * tb.add([{text:'Button 1'}, {text:'Button 2'}]);
   * </code></pre>
   *
   * <p>To inject components between existing ones, use the <a href="#!/api/Ext.container.Container-method-insert" rel="Ext.container.Container-method-insert" class="docClass">insert</a> method.</p>
   *
   * <h2>Warning:</h2>
   *
   * <p>Components directly managed by the BorderLayout layout manager may not be removed
   * or added.  See the Notes for <a href="#!/api/Ext.layout.container.Border" rel="Ext.layout.container.Border" class="docClass">BorderLayout</a> for
   * more details.</p>
   * @since 2.3.0
   * @param component Either one or more Components to add or an Array of Components to add.
   * See <code><a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a></code> for additional information.
   * @return The Components that were added.
   */
  public native function add(...component):*;

  /**
   * Cascades down the component/container heirarchy from this component (passed in
   * the first call), calling the specified function with each component. The scope
   * (this reference) of the function call will be the scope provided or the current
   * component. The arguments to the function will be the args provided or the current
   * component. If the function returns false at any point, the cascade is stopped on
   * that branch.
   * @since 2.3.0
   * @param fn The function to call
   * @param scope The scope of the function (defaults to current component)
   * @param args The args to call the function with. The current component
   * always passed as the last argument.
   * @return this
   */
  public native function cascade(fn:Function, scope:Object = null, args:Array = null):ext.container.Container;

  /**
   * Retrieves the first direct child of this container which matches the passed selector or component.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector. If no selector is
   * specified, the first child will be returned.
   * @return The matching child <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function child(selector:* = undefined):ext.Component;

  /**
   * Determines whether the passed Component is either an immediate child of this Container,
   * or whether it is a descendant.
   * @param comp The Component to test.
   * @param deep Pass <code>true</code> to test for the Component being a descendant at any level.
   * @default false
   * @return <code>true</code> if the passed Component is contained at the specified level.
   */
  public native function contains(comp:ext.Component, deep:Boolean = false):Boolean;

  /**
   * Retrieves the first descendant of this container which matches the passed selector.
   * The passed in selector must comply with an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector, or it can be an actual <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>.
   * @param selector An <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector or <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>. If no selector is
   * specified, the first child will be returned.
   * @return The matching descendant <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> (or <code>null</code> if no match was found).
   */
  public native function down(selector:* = undefined):ext.Component;

  /**
   * Retrieves the named <a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">Action</a> from this view or any ancestor which
   * has that named Action. See <a href="#!/api/Ext.container.Container-cfg-actions" rel="Ext.container.Container-cfg-actions" class="docClass">actions</a>
   */
  public native function getAction(name:String):ext.Action;

  /**
   * Returns the value of <a href="#!/api/Ext.container.Container-cfg-actions" rel="Ext.container.Container-cfg-actions" class="docClass">actions</a>.
   * @since 6.2.0
   */
  public native function getActions():Object;

  /**
   * Return the immediate child Component in which the passed element is located.
   * @param el The element to test (or ID of element).
   * @param deep If <code>true</code>, returns the deepest descendant Component which contains the passed element.
   * @return The child item which contains the passed element.
   */
  public native function getChildByElement(el:*, deep:Boolean):ext.Component;

  /**
   * Examines this container's <a href="#!/api/Ext.container.Container-property-items" rel="Ext.container.Container-property-items" class="docClass">items</a> <strong>property</strong> and gets a direct child
   * component of this container.
   * @since 2.3.0
   * @param comp This parameter may be any of the following:
   * <ul>
   * <li>a <strong>String</strong> : representing the <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">itemId</a>
   * or <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a> of the child component.</li>
   * <li>a <strong>Number</strong> : representing the position of the child component
   * within the <a href="#!/api/Ext.container.Container-property-items" rel="Ext.container.Container-property-items" class="docClass">items</a> <strong>property</strong></li>
   * </ul>
   * <p>For additional information see <a href="#!/api/Ext.util.MixedCollection-method-get" rel="Ext.util.MixedCollection-method-get" class="docClass">Ext.util.MixedCollection.get</a>.</p>
   * @return The component (if found).
   */
  public native function getComponent(comp:*):ext.Component;

  /**
   * Finds the configured default focus item. See <a href="#!/api/Ext.container.Container-cfg-defaultFocus" rel="Ext.container.Container-cfg-defaultFocus" class="docClass">defaultFocus</a>.
   */
  public native function getDefaultFocus():ext.Component;

  /**
   * Returns the <a href="#!/api/Ext.layout.container.Container" rel="Ext.layout.container.Container" class="docClass">layout</a> instance currently associated with this Container.
   * If a layout has not been instantiated yet, that is done first
   * @return The layout
   */
  public native function getLayout():ext.layout.container.ContainerLayout;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Container-cfg-referenceHolder" rel="Ext.mixin.Container-cfg-referenceHolder" class="docClass">referenceHolder</a>.
   */
  public native function getReferenceHolder():Boolean;

  /**
   * Returns an object holding the descendants of this view keyed by their
   * <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code>. This object should not be held
   * past the scope of the function calling this method. It will not be valid if items
   * are added or removed from this or any sub-container.
   *
   * <p>The intended usage is shown here (assume there are 3 components with reference
   * values of "foo", "bar" and "baz" at some level below this container):</p>
   *
   * <pre><code> onClick: function () {
   *      var refs = this.getReferences();
   *
   *      // using "refs" we can access any descendant by its "reference"
   *
   *      refs.foo.getValue() + refs.bar.getValue() + refs.baz.getValue();
   *  }
   * </code></pre>
   *
   * <p>If <code>this</code> component has a <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code> assigned
   * to it, that is <strong>not</strong> included in this object. That reference is understood to
   * belong to the ancestor container configured as the <code>referenceHolder</code>.</p>
   * @since 5.0.0
   * @return An object with each child reference. This will be <code>null</code> if this
   * container has no descendants with a <code><a href="#!/api/Ext.Component-cfg-reference" rel="Ext.Component-cfg-reference" class="docClass">reference</a></code>
   * specified.
   */
  public native function getReferences():Object;

  /**
   * Inserts a Component into this Container at a specified index. Fires the
   * <a href="#!/api/Ext.container.Container-event-beforeadd" rel="Ext.container.Container-event-beforeadd" class="docClass">beforeadd</a> event before inserting, then fires the <a href="#!/api/Ext.container.Container-event-add" rel="Ext.container.Container-event-add" class="docClass">add</a>
   * event after the Component has been inserted.
   * @since 2.3.0
   * @param index The index at which the Component will be inserted
   * into the Container's items collection
   * @param component The child Component or config object to insert.
   * <p>Ext uses lazy rendering, and will only render the inserted Component should
   * it become necessary.</p>
   * <p>A Component config object may be passed in order to avoid the overhead of
   * constructing a real Component object if lazy rendering might mean that the
   * inserted Component will not be rendered immediately. To take advantage of
   * this 'lazy instantiation', set the <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">Ext.Component.xtype</a> config
   * property to the registered type of the Component wanted.</p>
   * <p>You can pass an array of Component instances and config objects.</p>
   * <p>For a list of all available xtypes, see <a href="#!/api/Ext.enums.Widget" rel="Ext.enums.Widget" class="docClass">Ext.enums.Widget</a>.</p>
   * @return component The Component (or config object) that was
   * inserted with the Container's default config values applied.
   */
  public native function insert(index:Number, component:ext.Component):ext.Component;

  /**
   * Gets a reference to the component with the specified reference value.
   *
   * <p>The method is a short-hand for the <a href="#!/api/Ext.mixin.Container-method-lookupReference" rel="Ext.mixin.Container-method-lookupReference" class="docClass">lookupReference</a> method.</p>
   * @since 6.0.1
   * @param key The name of the reference to lookup.
   * @return The referenced component or <code>null</code> if it is not found.
   */
  public native function lookup(key:String):ext.Component;

  /**
   * Gets a reference to the component with the specified reference value.
   *
   * <p>The <a href="#!/api/Ext.mixin.Container-method-lookup" rel="Ext.mixin.Container-method-lookup" class="docClass">lookup</a> method is a short-hand version of this method.</p>
   * @since 5.0
   * @param key The name of the reference to lookup.
   * @return The referenced component or <code>null</code> if it is not found.
   */
  public native function lookupReference(key:String):ext.Component;

  /**
   * Moves the given <code>item(s)</code> into this container following <code>after</code>. This method will
   * account for layout-generated components like splitters and should be used instead
   * of index based <code><a href="#!/api/Ext.container.Container-method-move" rel="Ext.container.Container-method-move" class="docClass">move</a></code>. If <code>after</code> is <code>null</code> then the <code>item</code> will be the
   * first item in this container.
   *
   * <pre><code>var tb = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'toolbar',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     items: [{
   *         text: 'one'
   *     }, {
   *         text: 'two'
   *     }]
   * });
   *
   * // moves the 'one' button after the 'two' button
   * tb.moveAfter(tb.getComponent(0), tb.getComponent(1));
   * </code></pre>
   * @since 5.0.0
   * @param item The item to move. May be a component,
   * component configuration object, or an array of either.
   * @param after The reference component. May be <code>null</code>.
   * @return The moved item(s).
   */
  public native function moveAfter(item:*, after:ext.Component):*;

  /**
   * Moves the given <code>item(s)</code> into this container in front of <code>before</code>. This method
   * will account for layout-generated components like splitters and should be used
   * instead of index based <code><a href="#!/api/Ext.container.Container-method-move" rel="Ext.container.Container-method-move" class="docClass">move</a></code>. If <code>before</code> is <code>null</code> then the
   * <code>item</code> will be the last item in this container.
   *
   * <pre><code>var tb = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'toolbar',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     items: [{
   *         text: 'one'
   *     }, {
   *         text: 'two'
   *     }]
   * });
   *
   * // moves the 'two' button before the 'one' button
   * tb.moveBefore(tb.getComponent(1), tb.getComponent(0));
   * </code></pre>
   * @since 5.0.0
   * @param item The item to move. May be a component,
   * component configuration object, or an array of either.
   * @param before The reference component. May be <code>null</code>.
   * @return The moved item(s).
   */
  public native function moveBefore(item:*, before:ext.Component):*;

  /**
   * A method to find a child component after the passed child parameter. If a selector is also provided,
   * the first child component matching the selector will be returned.
   * @param child The child to use as a starting point to find the next child.
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector to find the next child. This will return the next child matching this selector. This parameter is optional.
   * @return The next child found, <code>null</code> if no child found.
   */
  public native function nextChild(child:ext.Component, selector:String = null):ext.Component;

  /**
   * A method to find a child component before the passed child parameter. If a selector is also provided,
   * the first child component matching the selector will be returned.
   * @param child The child to use as a starting point to find the previous child.
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector to find the previous child. This will return the first child matching this selector. This parameter is optional.
   * @return The previous child found, <code>null</code> if no child found.
   */
  public native function prevChild(child:ext.Component, selector:String = null):ext.Component;

  [ArrayElementType("ext.Component")]
  /**
   * Retrieves all descendant components which match the passed selector.
   * Executes an <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a> using this container as its root.
   * @param selector Selector complying to an <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">Ext.ComponentQuery</a> selector.
   * If no selector is specified all items will be returned.
   * @return Components which matched the selector
   */
  public native function query(selector:String = null):Array;

  [Deprecated(message = "Use &lt;code>down(...)&lt;/code> or &lt;code>query(...)&lt;/code> with selector &lt;code>&#39;[prop=value]&#39;&lt;/code> instead.\n\n", since = "6.0")]
  [ArrayElementType("ext.Component")]
  /**
   * Find a component under this container at any level by property
   *
   * @param prop
   * @param value
   * @deprecated
   * @return Array of Ext.Components
   */
  public native function find(prop:String, value:String):Array;

  [Deprecated(message = "Use &lt;code>down(...)&lt;/code> or &lt;code>query(...)&lt;/code> with selector &lt;code>&#39;xtype&#39;&lt;/code> instead. Use selector &lt;code>&#39;xtype(true)&#39;&lt;/code> for shallow xtype lookup.\n\n", since = "6.0")]
  [ArrayElementType("ext.Component")]
  /**
   * Find a component under this container at any level by xtype or class
   *
   * @param xtype The xtype string for a component, or the class of the component directly
   * @param shallow False to check whether this Component is descended from the xtype (this is the default), or true to check whether this Component is directly of the specified xtype.
   * @return Array of Ext.Components
   */
  public native function findByType(xtype:*, shallow:Boolean = false):Array;

  [ArrayElementType("ext.Component")]
  /**
   * Retrieves all descendant components which match the passed function.
   * The function should return false for components that are to be
   * excluded from the selection.
   * @param fn The matcher function. It will be called with a single argument,
   * the component being tested.
   * @param scope The scope in which to run the function. If not specified,
   * it will default to the active component.
   * @return Components matched by the passed function
   */
  public native function queryBy(fn:Function, scope:Object = null):Array;

  /**
   * Finds a component at any level under this container matching the id/itemId.
   * This is a shorthand for calling ct.down('#' + id);
   * @param id The id to find
   * @return The matching id, null if not found
   */
  public native function queryById(id:String):ext.Component;

  /**
   * Removes a component from this container.  Fires the <a href="#!/api/Ext.container.Container-event-beforeremove" rel="Ext.container.Container-event-beforeremove" class="docClass">beforeremove</a> event
   * before removing, then fires the <a href="#!/api/Ext.container.Container-event-remove" rel="Ext.container.Container-event-remove" class="docClass">remove</a> event after the component has
   * been removed.
   * @since 2.3.0
   * @param component The component instance or id to remove.
   * @param disposition Flags to determine what to do with the removed component.
   * (May also be specified as a boolean <code>autoDestroy</code> flag for backward compatibility).
   * @param {Boolean} [disposition.destroy] Defaults to this Container's <a href="#!/api/Ext.container.Container-cfg-autoDestroy" rel="Ext.container.Container-cfg-autoDestroy" class="docClass">autoDestroy</a> config.
   * Specifies whether to destroy the component being removed.
   * @param [disposition.detach] Defaults to the <a href="#!/api/Ext.container.Container-cfg-detachOnRemove" rel="Ext.container.Container-cfg-detachOnRemove" class="docClass">detachOnRemove</a> configuration
   * Specifies whether to remove the component's DOM from the container and into
   * the <a href="#!/api/Ext-method-getDetachedBody" rel="Ext-method-getDetachedBody" class="docClass">detached body element</a>
   * @return component The Component that was removed.
   */
  public native function remove(component:*, disposition:Object = null):ext.Component;

  [ArrayElementType("ext.Component")]
  /**
   * Removes all components from this container.
   * @since 2.3.0
   * @param autoDestroy True to automatically invoke the removed
   * Component's <a href="#!/api/Ext.Component-method-destroy" rel="Ext.Component-method-destroy" class="docClass">Ext.Component.destroy</a> function.
   * Defaults to the value of this Container's <a href="#!/api/Ext.container.Container-cfg-autoDestroy" rel="Ext.container.Container-cfg-autoDestroy" class="docClass">autoDestroy</a> config.
   * @return Array of the removed components
   */
  public native function removeAll(autoDestroy:Boolean = false):Array;

  /**
   * Sets the value of <a href="#!/api/Ext.container.Container-cfg-actions" rel="Ext.container.Container-cfg-actions" class="docClass">actions</a>.
   * @since 6.2.0
   * @param actions The new value.
   */
  public native function setActions(actions:Object):void;

  /**
   * Sets a component as the active layout item. This only applies when using
   * a <a href="#!/api/Ext.layout.container.Card" rel="Ext.layout.container.Card" class="docClass">Ext.layout.container.Card</a> layout.
   *
   * <pre><code>var card1 = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {itemId: 'card-1'});
   * var card2 = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {itemId: 'card-2'});
   * var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
   *     layout: 'card',
   *     items: [card1, card2]
   * });
   * // These are all equivalent
   * panel.getLayout().setActiveItem(card2);
   * panel.getLayout().setActiveItem('card-2');
   * panel.getLayout().setActiveItem(1);
   * </code></pre>
   * @param item The component, component <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a>,
   * <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">itemId</a>, or index of component.
   * @return the activated component or false when nothing activated.
   * False is returned also when trying to activate an already active item.
   */
  public native function setActiveItem(item:*):ext.Component;

  /**
   * Reconfigures the initially configured <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>.
   *
   * <p>NOTE: this method cannot be used to change the "type" of layout after the component
   * has been rendered to the DOM. After rendering, this method can only modify the
   * existing layout's configuration properties. The reason for this restriction is that
   * many container layouts insert special wrapping elements into the dom, and the
   * framework does not currently support dynamically changing these elements once
   * rendered.</p>
   * @param configuration object for the layout
   */
  public native function setLayout(configuration:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Container-cfg-referenceHolder" rel="Ext.mixin.Container-cfg-referenceHolder" class="docClass">referenceHolder</a>.
   * @param referenceHolder The new value.
   */
  public native function setReferenceHolder(referenceHolder:Boolean):void;

  /**
   * Traverses the tree rooted at this node in post-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>after</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPostOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Traverses the tree rooted at this node in pre-order mode, calling the passed function on the nodes at each level.
   * That is the function is called upon each node <strong>before</strong> being called on its children).
   *
   * <p>This method is used at each level down the cascade. Currently <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s
   * and <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">TreeModel</a>s are queryable.</p>
   *
   * <p>If you have tree-structured data, you can make your nodes queryable, and use ComponentQuery on them.</p>
   * @param selector A ComponentQuery selector used to filter candidate nodes before calling the function.
   * An empty string matches any node.
   * @param fn The function to call. Return <code>false</code> to aborl the traverse.
   * @param {Object} fn.node The node being visited.
   * @param scope The context (<code>this</code> reference) in which the function is executed.
   * @param extraArgs A set of arguments to be appended to the function's argument list to pass down extra data known to the caller
   * <strong>after</strong> the node being visited.
   */
  public native function visitPreOrder(selector:Object, fn:Function, scope:Object = null, extraArgs:Array = null):void;

  /**
   * Invoked after the Container has laid out (and rendered if necessary)
   * its child Components.
   */
  protected native function afterLayout(layout:ext.layout.container.ContainerLayout):void;

  /**
   * Used by <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a>, <a href="#!/api/Ext.container.Container-method-child" rel="Ext.container.Container-method-child" class="docClass">child</a> and <a href="#!/api/Ext.container.Container-method-down" rel="Ext.container.Container-method-down" class="docClass">down</a> to retrieve all of the items
   * which can potentially be considered a child of this Container.
   *
   * <p>This may be overriden by Components which have ownership of Components
   * that are not contained in the <a href="#!/api/Ext.container.Container-property-items" rel="Ext.container.Container-property-items" class="docClass">items</a> collection.</p>
   *
   * <p>NOTE: IMPORTANT note for maintainers:
   * Items are returned in tree traversal order. Each item is appended to the result array
   * followed by the results of that child's getRefItems call.
   * Floating child items are appended after internal child items.</p>
   */
  protected native function getRefItems(deep:Object):Array;

  /**
   * This method is called to initialize the <code>items</code> collection. A derived class can
   * override this method to do any last minute manipulation of <code>items</code> and then call
   * this method using <code>callParent</code>. Upon return, the <code>items</code> will no longer be a simple
   * array.
   */
  protected native function initItems():void;

  /**
   * Called when a raw config object is added to this container either during initialization of the <a href="#!/api/Ext.container.Container-cfg-items" rel="Ext.container.Container-cfg-items" class="docClass">items</a> config,
   * or when new items are <a href="#!/api/Ext.container.Container-method-add" rel="Ext.container.Container-method-add" class="docClass">added</a>, or <a href="#!/api/Ext.container.Container-method-insert" rel="Ext.container.Container-method-insert" class="docClass">inserted</a>.
   *
   * <p>This method converts the passed object into an instanced child component.</p>
   *
   * <p>This may be overridden in subclasses when special processing needs to be applied to child creation.</p>
   * @param item The config object being added.
   * @return The component to be added.
   */
  protected native function lookupComponent(item:Object):ext.Component;

  /**
   * This method is invoked after a new Component has been added. It
   * is passed the Component which has been added. This method may
   * be used to update any internal structure which may depend upon
   * the state of the child items.
   */
  protected native function onAdd(component:ext.Component, position:Number):void;

  /**
   * This method is invoked before adding a new child Component. It
   * is passed the new Component, and may be used to modify the
   * Component, or prepare the Container in some way. Returning
   * false aborts the add operation.
   */
  protected native function onBeforeAdd(item:ext.Component):void;

  /**
   * This method is invoked after a new Component has been
   * removed. It is passed the Component which has been
   * removed. This method may be used to update any internal
   * structure which may depend upon the state of the child items.
   */
  protected native function onRemove(component:ext.Component, autoDestroy:Boolean):void;
}
}