package ext.container {
import ext.Component;

[Uses("ext.container.DockingContainer")]
[Mixin("ext.container.DockingContainer")]
/**
 * @see ext.container.DockingContainer
 */
public interface IDockingContainer {
  [ExtConfig]
  /**
   * This object holds the default weights applied to dockedItems that have no weight. These start with a
   * weight of 1, to allow negative weights to insert before top items and are odd numbers
   * so that even weights can be used to get between different dock orders.
   *
   * <p>To make default docking order match border layout, do this:</p>
   *
   * <pre><code> Ext.panel.Panel.prototype.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };
   * </code></pre>
   *
   * <p>Changing these defaults as above or individually on this object will effect all Panels.
   * To change the defaults on a single panel, you should replace the entire object:</p>
   *
   * <pre><code> initComponent: function () {
   *      // NOTE: Don't change members of defaultDockWeights since the object is shared.
   *      this.defaultDockWeights = { top: 1, bottom: 3, left: 5, right: 7 };
   *
   *      this.callParent();
   *  }
   * </code></pre>
   *
   * <p>To change only one of the default values, you do this:</p>
   *
   * <pre><code> initComponent: function () {
   *      // NOTE: Don't change members of defaultDockWeights since the object is shared.
   *      this.defaultDockWeights = <a href="#!/api/Ext-method-applyIf" rel="Ext-method-applyIf" class="docClass">Ext.applyIf</a>({ top: 10 }, this.defaultDockWeights);
   *
   *      this.callParent();
   *  }
   * </code></pre>
   * @default {top: {render: 1, visual: 1}, left: {render: 3, visual: 5}, right: {render: 5, visual: 7}, bottom: {render: 7, visual: 3}}
   */
  function get defaultDockWeights():Object;

  [ExtConfig]
  function set defaultDockWeights(value:Object):void;

  /**
   * Adds docked item(s) to the container.
   * @param items The Component or array of components to add. The components
   * must include a 'dock' parameter on each component to indicate where it should be docked
   * ('top', 'right', 'bottom', 'left').
   * @param pos The index at which the Component will be added
   * @return The added components.
   */
  function addDocked(items:*, pos:Number = NaN):Array;

  /**
   * Finds a docked component by id, itemId or position. Also see <a href="#!/api/Ext.container.DockingContainer-method-getDockedItems" rel="Ext.container.DockingContainer-method-getDockedItems" class="docClass">getDockedItems</a>
   * @param comp The id, itemId or position of the docked component (see <a href="#!/api/Ext.container.Container-method-getComponent" rel="Ext.container.Container-method-getComponent" class="docClass">getComponent</a> for details)
   * @return The docked component (if found)
   */
  function getDockedComponent(comp:*):ext.Component;

  /**
   * Retrieves an array of all currently docked Components.
   *
   * <p>For example to find a toolbar that has been docked at top:</p>
   *
   * <pre><code>panel.getDockedItems('toolbar[dock="top"]');
   * </code></pre>
   * @param selector A <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a> selector string to filter the returned items.
   * @param beforeBody An optional flag to limit the set of items to only those
   *  before the body (true) or after the body (false). All components are returned if this flag is not set.
   * @return The array of docked components meeting the specified criteria.
   */
  function getDockedItems(selector:String, beforeBody:Boolean = undefined):Array;

  /**
   * Inserts docked item(s) to the panel at the indicated position.
   * @param pos The index at which the Component will be inserted
   * @param items The Component or array of components to add. The components
   * must include a 'dock' paramater on each component to indicate where it should be docked ('top', 'right',
   * 'bottom', 'left').
   */
  function insertDocked(pos:Number, items:*):void;

  /**
   * Removes the docked item from the panel.
   * @param item The Component to remove.
   * @param autoDestroy Destroy the component after removal.
   */
  function removeDocked(item:ext.Component, autoDestroy:Boolean = false):void;
}
}