package ext.data {
import ext.data.operation.DestroyOperation;
import ext.tree.TreePanel;

[Native("Ext.data.NodeInterface", require)]
/**
 * Fires when a new child node is appended
 * @eventType ext.data.events.NodeInterface_node_indexEvent.APPEND
 */
[Event(name = 'onAppend', type = 'ext.data.events.NodeInterface_node_indexEvent')]
/**
 * Fires before a new child is appended, return false to cancel the append.
 * @eventType ext.data.events.NodeInterface_nodeEvent.BEFORE_APPEND
 */
[Event(name = 'onBeforeAppend', type = 'ext.data.events.NodeInterface_nodeEvent')]
/**
 * Fires before this node is collapsed.
 * @eventType ext.data.events.NodeInterfaceEvent.BEFORE_COLLAPSE
 */
[Event(name = 'onBeforeCollapse', type = 'ext.data.events.NodeInterfaceEvent')]
/**
 * Fires before this node is expanded.
 * @eventType ext.data.events.NodeInterfaceEvent.BEFORE_EXPAND
 */
[Event(name = 'onBeforeExpand', type = 'ext.data.events.NodeInterfaceEvent')]
/**
 * Fires before a new child is inserted, return false to cancel the insert.
 * @eventType ext.data.events.NodeInterface_node_refNodeEvent.BEFORE_INSERT
 */
[Event(name = 'onBeforeInsert', type = 'ext.data.events.NodeInterface_node_refNodeEvent')]
/**
 * Fires before this node is moved to a new location in the tree. Return false to cancel the move.
 * @eventType ext.data.events.NodeInterface_oldParent_newParent_indexEvent.BEFORE_MOVE
 */
[Event(name = 'onBeforeMove', type = 'ext.data.events.NodeInterface_oldParent_newParent_indexEvent')]
/**
 * Fires before a child is removed, return false to cancel the remove.
 * @eventType ext.data.events.NodeInterface_node_isMoveEvent.BEFORE_REMOVE
 */
[Event(name = 'onBeforeRemove', type = 'ext.data.events.NodeInterface_node_isMoveEvent')]
/**
 * Fires when this node is collapsed.
 * @eventType ext.data.events.NodeInterfaceEvent.COLLAPSE
 */
[Event(name = 'onCollapse', type = 'ext.data.events.NodeInterfaceEvent')]
/**
 * Fires when this node is expanded.
 * @eventType ext.data.events.NodeInterfaceEvent.EXPAND
 */
[Event(name = 'onExpand', type = 'ext.data.events.NodeInterfaceEvent')]
/**
 * Fires when a new child node is inserted.
 * @eventType ext.data.events.NodeInterface_node_refNodeEvent.INSERT
 */
[Event(name = 'onInsert', type = 'ext.data.events.NodeInterface_node_refNodeEvent')]
/**
 * Fires when this node is moved to a new location in the tree
 * @eventType ext.data.events.NodeInterface_oldParent_newParent_indexEvent.MOVE
 */
[Event(name = 'onMove', type = 'ext.data.events.NodeInterface_oldParent_newParent_indexEvent')]
/**
 * Fires when a child node is removed
 * @eventType ext.data.events.NodeInterface_node_isMove_contextEvent.REMOVE
 */
[Event(name = 'onRemove', type = 'ext.data.events.NodeInterface_node_isMove_contextEvent')]
/**
 * Fires when this node's childNodes are sorted.
 * @eventType ext.data.events.NodeInterface_childNodesEvent.SORT
 */
[Event(name = 'onSort', type = 'ext.data.events.NodeInterface_childNodesEvent')]
/**
 * This class is used as a set of methods that are applied to the prototype of a
 * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> to decorate it with a Node API. This means that models
 * used in conjunction with a tree will have all of the tree related methods available
 * on the model. In general, this class will not be used directly by the developer.
 *
 * <p>This class also creates extra <a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">fields</a> on the model, if they do
 * not exist, to help maintain the tree state and UI. These fields are documented as
 * config options.</p>
 *
 * <p>The data fields used to render a tree node are: <a href="#!/api/Ext.data.NodeInterface-cfg-text" rel="Ext.data.NodeInterface-cfg-text" class="docClass">text</a>, <a href="#!/api/Ext.data.NodeInterface-cfg-leaf" rel="Ext.data.NodeInterface-cfg-leaf" class="docClass">leaf</a>,
 * <a href="#!/api/Ext.data.NodeInterface-cfg-children" rel="Ext.data.NodeInterface-cfg-children" class="docClass">children</a>, and <a href="#!/api/Ext.data.NodeInterface-cfg-expanded" rel="Ext.data.NodeInterface-cfg-expanded" class="docClass">expanded</a>.  Once a node is loaded to the tree store
 * you can use <a href="#!/api/Ext.data.Model-method-get" rel="Ext.data.Model-method-get" class="docClass">get()</a> to fetch the value of a given field
 * name (provided there is not a convenience accessor on the Node for that field).</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>(); // not required when using Ext.application()
 *
 * var root = {
 *     expanded: true,
 *     children: [{
 *         text: "Leaf node (&lt;i&gt;no folder/arrow icon&lt;/i&gt;)",
 *         leaf: true,
 *         qtitle: 'Sample Tip Title',
 *         qtip: 'Tip body'
 *     }, {
 *         text: "Parent node expanded",
 *         expanded: true,
 *         children: [{
 *             text: "Expanded leaf node 1",
 *             leaf: true
 *         }, {
 *             text: "Expanded leaf node 2",
 *             leaf: true
 *         }]
 *     }, {
 *         text: "Parent node collapsed",
 *         children: [{
 *             text: "Collapsed leaf node 1",
 *             leaf: true
 *         }, {
 *             text: "Collapsed leaf node 2",
 *             leaf: true
 *         }]
 *     }]
 * };
 *
 * var tree = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>', {
 *     title: 'TreePanel',
 *     width: 260,
 *     height: 200,
 *     root: root,
 *     rootVisible: false,
 *     renderTo: document.body,
 *     bbar: ['The first node ', {
 *         text: 'is a leaf?',
 *         handler: function () {
 *             var firstChild = tree.getRootNode().getChildAt(0);
 *             <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Is Leaf?', firstChild.isLeaf());
 *         }
 *     }, {
 *         text: 'has text?',
 *         handler: function () {
 *             var firstChild = tree.getRootNode().getChildAt(0);
 *             <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Has Text:', firstChild.get('text'));
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <p>The following configs have methods used to set the value / state of the node at
 * runtime:</p>
 *
 * <p><strong><a href="#!/api/Ext.data.NodeInterface-cfg-children" rel="Ext.data.NodeInterface-cfg-children" class="docClass">children</a> / <a href="#!/api/Ext.data.NodeInterface-cfg-leaf" rel="Ext.data.NodeInterface-cfg-leaf" class="docClass">leaf</a></strong></p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-appendChild" rel="Ext.data.NodeInterface-method-appendChild" class="docClass">appendChild</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-hasChildNodes" rel="Ext.data.NodeInterface-method-hasChildNodes" class="docClass">hasChildNodes</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-insertBefore" rel="Ext.data.NodeInterface-method-insertBefore" class="docClass">insertBefore</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-insertChild" rel="Ext.data.NodeInterface-method-insertChild" class="docClass">insertChild</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-remove" rel="Ext.data.NodeInterface-method-remove" class="docClass">remove</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-removeAll" rel="Ext.data.NodeInterface-method-removeAll" class="docClass">removeAll</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-removeChild" rel="Ext.data.NodeInterface-method-removeChild" class="docClass">removeChild</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-replaceChild" rel="Ext.data.NodeInterface-method-replaceChild" class="docClass">replaceChild</a></li>
 * </ul>
 *
 *
 * <p><strong><a href="#!/api/Ext.data.NodeInterface-cfg-expanded" rel="Ext.data.NodeInterface-cfg-expanded" class="docClass">expanded</a></strong></p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-expand" rel="Ext.data.NodeInterface-method-expand" class="docClass">expand</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-expandChildren" rel="Ext.data.NodeInterface-method-expandChildren" class="docClass">expandChildren</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-collapse" rel="Ext.data.NodeInterface-method-collapse" class="docClass">collapse</a></li>
 * <li><a href="#!/api/Ext.data.NodeInterface-method-collapseChildren" rel="Ext.data.NodeInterface-method-collapseChildren" class="docClass">collapseChildren</a></li>
 * </ul>
 *
 *
 * <p>The remaining configs may be set using <a href="#!/api/Ext.data.Model-method-set" rel="Ext.data.Model-method-set" class="docClass">set()</a>.</p>
 *
 * <pre><code>node.set('text', 'Changed Text'); // example showing how to change the node label
 * </code></pre>
 *
 * <p>The <a href="#!/api/Ext.data.NodeInterface-cfg-qtip" rel="Ext.data.NodeInterface-cfg-qtip" class="docClass">qtip</a>, <a href="#!/api/Ext.data.NodeInterface-cfg-qtitle" rel="Ext.data.NodeInterface-cfg-qtitle" class="docClass">qtitle</a>, and <a href="#!/api/Ext.data.NodeInterface-cfg-qshowDelay" rel="Ext.data.NodeInterface-cfg-qshowDelay" class="docClass">qshowDelay</a> use QuickTips and
 * requires initializing <a href="#!/api/Ext.tip.QuickTipManager" rel="Ext.tip.QuickTipManager" class="docClass">Ext.tip.QuickTipManager</a> unless the application is
 * created using Ext.application.</p>
 *
 * <pre><code><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>();
 * </code></pre>
 *
 * <p>For additional information and examples see the description for
 * <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>.</p>
 *
 * <p>For decorating a model to implement NodeInterface, see the class NodeInterfaceDecorator.</p>
 */
public interface NodeInterface {
  /**
   * An array of this nodes children.  Array will be empty if this node has no children.
   */
  function get childNodes():Array;

  /**
   * @private
   */
  function set childNodes(value:Array):void;

  /**
   * A reference to this node's first child node. <code>null</code> if this node has no children.
   */
  function get firstChild():ext.data.NodeInterface;

  /**
   * @private
   */
  function set firstChild(value:ext.data.NodeInterface):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Node, or subclass thereof.
   * @default true
   */
  function get isNode():Boolean;

  /**
   * @private
   */
  function set isNode(value:Boolean):void;

  /**
   * A reference to this node's last child node. <code>null</code> if this node has no children.
   */
  function get lastChild():ext.data.NodeInterface;

  /**
   * @private
   */
  function set lastChild(value:ext.data.NodeInterface):void;

  /**
   * A reference to this node's next sibling node. <code>null</code> if this node does not have a next sibling.
   */
  function get nextSibling():ext.data.NodeInterface;

  /**
   * @private
   */
  function set nextSibling(value:ext.data.NodeInterface):void;

  /**
   * A reference to this node's parent node. <code>null</code> if this node is the root node.
   */
  function get parentNode():ext.data.NodeInterface;

  /**
   * @private
   */
  function set parentNode(value:ext.data.NodeInterface):void;

  /**
   * A reference to this node's previous sibling node. <code>null</code> if this node does not have a previous sibling.
   */
  function get previousSibling():ext.data.NodeInterface;

  /**
   * @private
   */
  function set previousSibling(value:ext.data.NodeInterface):void;

  [ExtConfig]
  /**
   * Set to false to deny dragging of this node.
   *
   * <p>Applicable when using the <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop" rel="Ext.tree.plugin.TreeViewDragDrop" class="docClass">TreeViewDragDrop</a> plugin.</p>
   * @default true
   */
  function get allowDrag():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  function set allowDrag(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to false to deny dropping on this node.
   *
   * <p>Applicable when using the <a href="#!/api/Ext.tree.plugin.TreeViewDragDrop" rel="Ext.tree.plugin.TreeViewDragDrop" class="docClass">TreeViewDragDrop</a> plugin.</p>
   * @default true
   */
  function get allowDrop():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  function set allowDrop(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to true or false to show a checkbox alongside this node.
   *
   * <p>To fetch an array of checked nodes use <a href="#!/api/Ext.tree.Panel-method-getChecked" rel="Ext.tree.Panel-method-getChecked" class="docClass">getChecked()</a>.</p>
   * @default null
   */
  function get checked():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  function set checked(value:Boolean):void;

  [ExtConfig]
  /**
   * Array of child nodes.
   *
   * <p><strong>Note:</strong> By default the child nodes root is <code>children</code>, but can be set using the
   * reader <a href="#!/api/Ext.data.reader.Reader-cfg-rootProperty" rel="Ext.data.reader.Reader-cfg-rootProperty" class="docClass">rootProperty</a> config on the
   * <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore's</a> <a href="#!/api/Ext.data.TreeStore-cfg-proxy" rel="Ext.data.TreeStore-cfg-proxy" class="docClass">proxy</a>.</p>
   */
  function get children():Array;

  [ExtConfig]
  /**
   * @private
   */
  function set children(value:Array):void;

  [ExtConfig]
  /**
   * CSS class to apply to this node.
   */
  function get cls():String;

  [ExtConfig]
  /**
   * @private
   */
  function set cls(value:String):void;

  [ExtConfig]
  /**
   * False to prevent expanding/collapsing of this node.
   *
   * <p>See also: <a href="#!/api/Ext.data.NodeInterface-method-isExpandable" rel="Ext.data.NodeInterface-method-isExpandable" class="docClass">isExpandable</a>.</p>
   * @default true
   */
  function get expandable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  function set expandable(value:Boolean):void;

  [ExtConfig]
  /**
   * True if the node is expanded.
   *
   * <p>When the tree is asynchronously remote loaded, expanding a collapsed node loads
   * the children of that node (if the node has not already been loaded previously).</p>
   *
   * <p>See also: <a href="#!/api/Ext.data.NodeInterface-method-isExpanded" rel="Ext.data.NodeInterface-method-isExpanded" class="docClass">isExpanded</a>.</p>
   * @default false
   */
  function get expanded():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  function set expanded(value:Boolean):void;

  [ExtConfig]
  /**
   * @since 6.2.0
   */
  function get glyph():String;

  [ExtConfig]
  /**
   * @private
   */
  function set glyph(value:String):void;

  [ExtConfig]
  /**
   * A URL for a link that's created when this config is specified.
   *
   * <p>See also <a href="#!/api/Ext.data.NodeInterface-cfg-hrefTarget" rel="Ext.data.NodeInterface-cfg-hrefTarget" class="docClass">hrefTarget</a>.</p>
   */
  function get href():String;

  [ExtConfig]
  /**
   * @private
   */
  function set href(value:String):void;

  [ExtConfig]
  /**
   * Target for link. Only applicable when <a href="#!/api/Ext.data.NodeInterface-cfg-href" rel="Ext.data.NodeInterface-cfg-href" class="docClass">href</a> is also specified.
   */
  function get hrefTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  function set hrefTarget(value:String):void;

  [ExtConfig]
  /**
   * Path to an image to use as an icon.
   *
   * <p>For instructions on how you can use icon fonts including those distributed in
   * the SDK see <a href="#!/api/Ext.data.NodeInterface-cfg-iconCls" rel="Ext.data.NodeInterface-cfg-iconCls" class="docClass">iconCls</a>.</p>
   */
  function get icon():String;

  [ExtConfig]
  /**
   * @private
   */
  function set icon(value:String):void;

  [ExtConfig]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   */
  function get iconCls():String;

  [ExtConfig]
  /**
   * @private
   */
  function set iconCls(value:String):void;

  [ExtConfig]
  /**
   * Set to true to indicate that this child can have no children. The expand icon/arrow will then not be
   * rendered for this node.
   *
   * <p>See also: <a href="#!/api/Ext.data.NodeInterface-method-isLeaf" rel="Ext.data.NodeInterface-method-isLeaf" class="docClass">isLeaf</a>.</p>
   * @default false
   */
  function get leaf():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  function set leaf(value:Boolean):void;

  [ExtConfig]
  /**
   * Tooltip showDelay.
   *
   * <p>See also <a href="#!/api/Ext.data.NodeInterface-cfg-qtip" rel="Ext.data.NodeInterface-cfg-qtip" class="docClass">qtip</a>.
   * See also <a href="#!/api/Ext.data.NodeInterface-cfg-qtitle" rel="Ext.data.NodeInterface-cfg-qtitle" class="docClass">qtitle</a>.</p>
   */
  function get qshowDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  function set qshowDelay(value:Number):void;

  [ExtConfig]
  /**
   * Tooltip text to show on this node.
   *
   * <p>See also <a href="#!/api/Ext.data.NodeInterface-cfg-qtitle" rel="Ext.data.NodeInterface-cfg-qtitle" class="docClass">qtitle</a>.
   * See also <a href="#!/api/Ext.data.NodeInterface-cfg-qshowDelay" rel="Ext.data.NodeInterface-cfg-qshowDelay" class="docClass">qshowDelay</a>.</p>
   */
  function get qtip():String;

  [ExtConfig]
  /**
   * @private
   */
  function set qtip(value:String):void;

  [ExtConfig]
  /**
   * Tooltip title.
   *
   * <p>See also <a href="#!/api/Ext.data.NodeInterface-cfg-qtip" rel="Ext.data.NodeInterface-cfg-qtip" class="docClass">qtip</a>.
   * See also <a href="#!/api/Ext.data.NodeInterface-cfg-qshowDelay" rel="Ext.data.NodeInterface-cfg-qshowDelay" class="docClass">qshowDelay</a>.</p>
   */
  function get qtitle():String;

  [ExtConfig]
  /**
   * @private
   */
  function set qtitle(value:String):void;

  [ExtConfig]
  /**
   * The text to show on node label (<em>html tags are accepted</em>).
   * The default text for the root node is <code>ROOT</code>.  All other nodes default to ''.
   *
   * <p><strong>Note:</strong> By default the node label is <code>text</code>, but can be set using the tree's
   * <a href="#!/api/Ext.tree.Panel-cfg-displayField" rel="Ext.tree.Panel-cfg-displayField" class="docClass">displayField</a> config.</p>
   */
  function get text():String;

  [ExtConfig]
  /**
   * @private
   */
  function set text(value:String):void;

  /**
   * Inserts node(s) as the last child node of this node.
   *
   * <p>If the node was previously a child node of another parent node, it will be removed from that node first.</p>
   * @param node The node or Array of nodes to append
   * @param suppressEvents True to suppress firing of
   * events.
   * @default false
   * @param commit
   * @default false
   * @return The appended node if single append, or null if an array was passed
   */
  function appendChild(node:*, suppressEvents:Boolean = false, commit:Boolean = false):ext.data.NodeInterface;

  /**
   * Bubbles up the tree from this node, calling the specified function with each node. The arguments to the function
   * will be the args provided or the current node. If the function returns false at any point,
   * the bubble is stopped.
   * @param fn The function to call
   * @param scope The scope (this reference) in which the function is executed. Defaults to the current Node.
   * @param args The args to call the function with. Defaults to passing the current Node.
   */
  function bubble(fn:Function, scope:Object = null, args:Array = null):void;

  /**
   * Cascades down the tree from this node, calling the specified functions with each node. The arguments to the function
   * will be the args provided or the current node. If the <code>before</code> function returns false at any point,
   * the cascade is stopped on that branch.
   *
   * <p>Note that the 3 argument form passing <code>fn, scope, args</code> is still supported. The <code>fn</code> function is as before, called
   * <em>before</em> cascading down into child nodes. If it returns <code>false</code>, the child nodes are not traversed.</p>
   * @param spec An object containing before and after functions, scope and an argument list.
   * @param {Function} [spec.before] A function to call on a node <em>before</em> cascading down into child nodes.
   * If it returns <code>false</code>, the child nodes are not traversed.
   * @param {Function} [spec.after] A function to call on a node <em>after</em> cascading down into child nodes.
   * @param {Object} [spec.scope] The scope (this reference) in which the functions are executed. Defaults to the current Node.
   * @param {Array} [spec.args] The args to call the function with. Defaults to passing the current Node.
   */
  function cascade(spec:Object):void;

  /**
   * Collapse this node.
   * @param recursive True to recursively collapse all the children
   * @default false
   * @param callback The function to execute once the collapse completes
   * @param scope The scope to run the callback in
   */
  function collapse(recursive:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * Collapse all the children of this node.
   * @param recursive True to recursively collapse all the children
   * @default false
   * @param callback The function to execute once all the children are collapsed
   * @param scope The scope to run the callback in
   */
  function collapseChildren(recursive:Function = null, callback:Function = null, scope:Object = null):void;

  /**
   * Returns true if this node is an ancestor (at any point) of the passed node.
   */
  function contains(node:ext.data.NodeInterface):Boolean;

  /**
   * Creates a copy (clone) of this Node.
   * @param id A new id, defaults to this Node's id.
   * @param session The session to which the new record belongs.
   * @param deep True to recursively copy all child Nodes into the new Node.
   * False to copy without child Nodes.
   * @default false
   * @return A copy of this Node.
   */
  function copyNode(id:String = null, session:Object = null, deep:Boolean = false):ext.data.NodeInterface;

  /**
   * Ensures that the passed object is an instance of a Record with the NodeInterface applied
   */
  function createNode(node:Object):ext.data.NodeInterface;

  /**
   * Iterates the child nodes of this node, calling the specified function
   * with each node. The arguments to the function will be the args
   * provided or the current node. If the function returns false at any
   * point, the iteration stops.
   * @param fn The function to call
   * @param scope The scope (<em>this</em> reference) in which the
   * function is executed. Defaults to the Node on which eachChild is
   * called.
   * @param args The args to call the function with. Defaults to
   * passing the current Node.
   */
  function eachChild(fn:Function, scope:Object = null, args:Array = null):void;

  /**
   * Destroys the node.
   */
  function erase(options:Object = null):ext.data.operation.DestroyOperation;

  /**
   * Expand this node.
   * @param recursive True to recursively expand all the children
   * @default false
   * @param callback The function to execute once the expand completes
   * @param scope The scope to run the callback in
   */
  function expand(recursive:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * Expand all the children of this node.
   * @param recursive True to recursively expand all the children
   * @default false
   * @param callback The function to execute once all the children are expanded
   * @param scope The scope to run the callback in
   */
  function expandChildren(recursive:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * Finds the first child that has the attribute with the specified value.
   * @param attribute The attribute name
   * @param value The value to search for
   * @param deep True to search through nodes deeper than the immediate children
   * @default false
   * @return The found child or null if none was found
   */
  function findChild(attribute:String, value:Object, deep:Boolean = false):ext.data.NodeInterface;

  /**
   * Finds the first child by a custom function. The child matches if the function passed returns true.
   * @param fn A function which must return true if the passed Node is the required Node.
   * @param scope The scope (this reference) in which the function is executed. Defaults to the Node being tested.
   * @param deep True to search through nodes deeper than the immediate children
   * @default false
   * @return The found child or null if none was found
   */
  function findChildBy(fn:Function, scope:Object = null, deep:Boolean = false):ext.data.NodeInterface;

  /**
   * Fires the specified event with the passed parameters (minus the event name, plus the <code>options</code> object passed
   * to <a href="#!/api/Ext.mixin.Observable-method-addListener" rel="Ext.mixin.Observable-method-addListener" class="docClass">addListener</a>).
   *
   * <p>An event may be set to bubble up an Observable parent hierarchy (See <a href="#!/api/Ext.Component-method-getBubbleTarget" rel="Ext.Component-method-getBubbleTarget" class="docClass">Ext.Component.getBubbleTarget</a>) by
   * calling <a href="#!/api/Ext.mixin.Observable-method-enableBubble" rel="Ext.mixin.Observable-method-enableBubble" class="docClass">enableBubble</a>.</p>
   * @param eventName The name of the event to fire.
   * @param args Variable number of parameters are passed to handlers.
   * @return returns false if any of the handlers return false otherwise it returns true.
   */
  function fireEvent(eventName:String, ...args):Boolean;

  /**
   * Returns the child node at the specified index.
   */
  function getChildAt(index:Number):ext.data.NodeInterface;

  /**
   * Returns depth of this node (the root node has a depth of 0)
   */
  function getDepth():Number;

  [Deprecated(since = "6.2.0")]
  /**
   * Returns the tree this node is in.
   * @return The tree panel which owns this node.
   */
  function getOwnerTree():ext.tree.TreePanel;

  /**
   * Gets the hierarchical path from the root of the current node.
   * @param field The field to construct the path from. Defaults to the model idProperty.
   * @param separator A separator to use.
   * @default '/'
   * @return The node path
   */
  function getPath(field:String = null, separator:String = '/'):String;

  /**
   * Returns the <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a> which owns this node.
   * @return The TreeStore which owns this node.
   */
  function getTreeStore():ext.data.TreeStore;

  /**
   * Returns true if this node has one or more child nodes, else false.
   */
  function hasChildNodes():Boolean;

  /**
   * Returns the index of a child node
   * @return The index of the node or -1 if it was not found
   */
  function indexOf(node:ext.data.NodeInterface):Number;

  /**
   * Returns the index of a child node that matches the id
   * @param id The id of the node to find
   * @return The index of the node or -1 if it was not found
   */
  function indexOfId(id:String):Number;

  /**
   * Inserts the first node before the second node in this nodes childNodes collection.
   * @param node The node to insert
   * @param refNode The node to insert before (if null the node is appended)
   * @return The inserted node
   */
  function insertBefore(node:*, refNode:ext.data.NodeInterface):ext.data.NodeInterface;

  /**
   * Inserts a node into this node.
   * @param index The zero-based index to insert the node at
   * @param node The node to insert
   * @return The node you just inserted
   */
  function insertChild(index:Number, node:*):ext.data.NodeInterface;

  /**
   * Returns true if the passed node is an ancestor (at any point) of this node.
   */
  function isAncestor(node:ext.data.NodeInterface):Boolean;

  /**
   * Returns true if this node is a branch node, and the entire branch is fully loaded.
   *
   * <p>Using this method, it is possible to ascertain whether an
   * <code>expandAll()</code> call (<em>classic toolkit TreePanel method</em>) will have
   * access to all descendant nodes without incurring a store load.</p>
   */
  function isBranchLoaded():Boolean;

  /**
   * Returns true if this node has one or more child nodes, or if the <tt>expandable</tt>
   * node attribute is explicitly specified as true, otherwise returns false.
   */
  function isExpandable():Boolean;

  /**
   * Returns <code>true</code> if this node is expanded.
   */
  function isExpanded():Boolean;

  /**
   * Returns true if this node is the first child of its parent
   */
  function isFirst():Boolean;

  /**
   * Returns true if this node is the last child of its parent
   */
  function isLast():Boolean;

  /**
   * Returns true if this node is a leaf
   */
  function isLeaf():Boolean;

  /**
   * Returns true if this node is loaded
   */
  function isLoaded():Boolean;

  /**
   * Returns true if this node is loading
   */
  function isLoading():Boolean;

  /**
   * Returns true if this node is the root node
   */
  function isRoot():Boolean;

  /**
   * Returns true if this node is visible. Note that visibility refers to
   * the structure of the tree, the <a href="#!/api/Ext.tree.Panel-cfg-rootVisible" rel="Ext.tree.Panel-cfg-rootVisible" class="docClass">Ext.tree.Panel.rootVisible</a>
   * configuration is not taken into account here. If this method is called
   * on the root node, it will always be visible.
   */
  function isVisible():Boolean;

  /**
   * Implement this method in a tree record subclass if it needs to track whenever it is registered
   * with a <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore</a>.
   * @param treeStore The TreeStore to which the node is being registered.
   */
  function onRegisterTreeNode(treeStore:ext.data.TreeStore):void;

  /**
   * Implement this method in a tree record subclass if it needs to track whenever it is unregistered
   * from a <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore</a>.
   * @param treeStore The TreeStore from which the node is being unregistered.
   */
  function onUnregisterTreeNode(treeStore:ext.data.TreeStore):void;

  /**
   * Removes this node from its parent.
   *
   * <p><strong>If</strong> the node is not phantom (only added in the client side), then it may be marked for removal.</p>
   *
   * <p>If the owning <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">tree store</a> is set to <a href="#!/api/Ext.data.ProxyStore-cfg-trackRemoved" rel="Ext.data.ProxyStore-cfg-trackRemoved" class="docClass">track removed</a>
   * then the node will be added to the stack of nodes due to be removed the next time the store is synced with the server.</p>
   *
   * <p>If the owning <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">tree store</a> is set to <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">auto synchronize</a>
   * then the synchronize request will be initiated immediately.</p>
   * @param erase True to erase the node using the configured proxy. This is only needed when the
   * owning <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">tree store</a> is not taking care of synchronization operations.
   * @default false
   * @return this
   */
  function remove(erase:Boolean = false):ext.data.NodeInterface;

  /**
   * Removes all child nodes from this node.
   * @param erase True to erase the node using the configured
   * proxy.
   * @default false
   * @return this
   */
  function removeAll(erase:Boolean = false):ext.data.NodeInterface;

  /**
   * Removes a child node from this node.
   * @param node The node to remove
   * @param erase True to erase the record using the
   * configured proxy.
   * @default false
   * @return The removed node
   */
  function removeChild(node:ext.data.NodeInterface, erase:Boolean = false):ext.data.NodeInterface;

  /**
   * Replaces one child node in this node with another.
   * @param newChild The replacement node
   * @param oldChild The node to replace
   * @return The replaced node
   */
  function replaceChild(newChild:ext.data.NodeInterface, oldChild:ext.data.NodeInterface):ext.data.NodeInterface;

  /**
   * Creates an object representation of this node including its children.
   */
  function serialize(writerParam:Object):void;

  /**
   * Sorts this nodes children using the supplied sort function.
   * @param sortFn A function which, when passed two Nodes, returns -1, 0 or 1 depending upon required sort order.
   * <p>It omitted, the node is sorted according to the existing sorters in the owning <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore</a>.</p>
   * @param recursive True to apply this sort recursively
   * @default false
   * @param suppressEvent True to not fire a sort event.
   * @default false
   */
  function sort(sortFn:Function = null, recursive:Boolean = false, suppressEvent:Boolean = false):void;

  [ArrayElementType("String")]
  /**
   * Updates general data of this node like isFirst, isLast, depth. This
   * method is internally called after a node is moved. This shouldn't
   * have to be called by the developer unless they are creating custom
   * Tree plugins.
   * @param info The info to update. May contain any of the following
   * @param {Object} info.isFirst
   * @param {Object} info.isLast
   * @param {Object} info.index
   * @param {Object} info.depth
   * @param {Object} info.parentId
   * @return The names of any persistent fields that were modified.
   */
  function updateInfo(commit:Boolean, info:Object):Array;
}
}