package ext.data {
import ext.data.proxy.DataProxy;
import ext.util.MixedCollection;

[Uses("ext.data.AbstractStore")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.ProxyStore", require)]
/**
 * Fires before a request is made for a new data object. If the beforeload handler returns false the load
 * action will be canceled.
 * @since 1.1.0
 * @eventType ext.data.events.ProxyStore_store_operationEvent.BEFORE_LOAD
 */
[Event(name = 'onBeforeLoad', type = 'ext.data.events.ProxyStore_store_operationEvent')]
/**
 * Fired before a call to <a href="#!/api/Ext.data.ProxyStore-method-sync" rel="Ext.data.ProxyStore-method-sync" class="docClass">sync</a> is executed. Return false from any listener to cancel the sync
 * @eventType ext.data.events.ProxyStore_optionsEvent.BEFORE_SYNC
 */
[Event(name = 'onBeforeSync', type = 'ext.data.events.ProxyStore_optionsEvent')]
/**
 * Fires whenever the store reads data from a remote data source.
 * @since 1.1.0
 * @eventType ext.data.events.ProxyStore_records_successful_operationEvent.LOAD
 */
[Event(name = 'onLoad', type = 'ext.data.events.ProxyStore_records_successful_operationEvent')]
/**
 * Fires when this store's underlying reader (available via the proxy) provides new metadata.
 * Metadata usually consists of new field definitions, but can include any configuration data
 * required by an application, and can be processed as needed in the event handler.
 * This event is currently only fired for JsonReaders.
 * @since 1.1.0
 * @eventType ext.data.events.ProxyStore_metaEvent.META_CHANGE
 */
[Event(name = 'onMetaChange', type = 'ext.data.events.ProxyStore_metaEvent')]
/**
 * Fires whenever a successful write has been made via the configured <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">Proxy</a>
 * @since 3.4.0
 * @eventType ext.data.events.ProxyStore_store_operationEvent.WRITE
 */
[Event(name = 'onWrite', type = 'ext.data.events.ProxyStore_store_operationEvent')]
/**
 * ProxyStore is a superclass of <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> and <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Ext.data.BufferedStore</a>. It's never used directly,
 * but offers a set of methods used by both of those subclasses.
 *
 * <p>We've left it here in the docs for reference purposes, but unless you need to make a whole new type of Store, what
 * you're probably looking for is <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>. If you're still interested, here's a brief description of what
 * ProxyStore is and is not.</p>
 *
 * <p>ProxyStore provides the basic configuration for anything that can be considered a Store. It expects to be
 * given a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> that represents the type of data in the Store. It also expects to be given a
 * <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a> that handles the loading of data into the Store.</p>
 *
 * <p>ProxyStore provides a few helpful methods such as <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> and <a href="#!/api/Ext.data.ProxyStore-method-sync" rel="Ext.data.ProxyStore-method-sync" class="docClass">sync</a>, which load and save data
 * respectively, passing the requests through the configured <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy</a>.</p>
 *
 * <p>Built-in Store subclasses add extra behavior to each of these functions. Note also that each ProxyStore subclass
 * has its own way of storing data - in <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> the data is saved as a flat <a href="#!/api/Ext.util.Collection" rel="Ext.util.Collection" class="docClass">Collection</a>,
 * whereas in <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">BufferedStore</a> we use a <a href="#!/api/Ext.data.PageMap" rel="Ext.data.PageMap" class="docClass">Ext.data.PageMap</a> to maintain a client side cache of pages of records.</p>
 *
 * <p>The store provides filtering and sorting support. This sorting/filtering can happen on the client side
 * or can be completed on the server. This is controlled by the <a href="#!/api/Ext.data.Store-cfg-remoteSort" rel="Ext.data.Store-cfg-remoteSort" class="docClass">remoteSort</a> and
 * <a href="#!/api/Ext.data.Store-cfg-remoteFilter" rel="Ext.data.Store-cfg-remoteFilter" class="docClass">remoteFilter</a> config options. For more information see the <a href="#!/api/Ext.data.ProxyStore-method-sort" rel="Ext.data.ProxyStore-method-sort" class="docClass">sort</a> and
 * <a href="#!/api/Ext.data.Store-method-filter" rel="Ext.data.Store-method-filter" class="docClass">filter</a> methods.</p>
 */
public class ProxyStore extends ext.data.AbstractStore {
  /**
   * @param config @inheritDoc
   */
  public function ProxyStore(config:ext.data.ProxyStore = null) {
    super();
  }

  /**
   * Property to hold the last options from a <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> method call. This object is used for the <a href="#!/api/Ext.data.ProxyStore-method-reload" rel="Ext.data.ProxyStore-method-reload" class="docClass">reload</a>
   * to reuse the same options. Please see <a href="#!/api/Ext.data.ProxyStore-method-reload" rel="Ext.data.ProxyStore-method-reload" class="docClass">reload</a> for a simple example on how to use the lastOptions property.
   */
  public native function get lastOptions():Object;

  /**
   * @private
   */
  public native function set lastOptions(value:Object):void;

  /**
   * Temporary cache in which removed model instances are kept until successfully
   * synchronised with a Proxy, at which point this is cleared.
   *
   * <p>This cache is maintained unless you set <code>trackRemoved</code> to <code>false</code>.</p>
   * @default []
   */
  public native function get removed():Array;

  /**
   * @private
   */
  public native function set removed(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This defaults to <code>true</code> when this store's <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy</a> is asynchronous, such as an
   * <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ajax proxy</a>.
   *
   * <p>When the proxy is synchronous, such as a <a href="#!/api/Ext.data.proxy.Memory" rel="Ext.data.proxy.Memory" class="docClass">Ext.data.proxy.Memory</a> memory proxy, this
   * defaults to <code>false</code>.</p>
   *
   * <p><em>NOTE:</em> This does not cause synchronous Ajax requests if configured <code>false</code> when an Ajax proxy
   * is used. It causes immediate issuing of an Ajax request when <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> is called
   * rather than issuing the request at the end of the current event handler run.</p>
   *
   * <p>What this means is that when using an Ajax proxy, calls to
   * <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> do not fire the request to the remote resource
   * immediately, but schedule a request to be made. This is so that multiple
   * requests are not fired when mutating a store's remote filters and sorters (as
   * happens during state restoration). The request is made only once after all
   * relevant store state is fully set.</p>
   * @since 6.0.1
   * @see #getAsynchronousLoad()
   * @see #setAsynchronousLoad()
   */
  public native function get asynchronousLoad():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set asynchronousLoad(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If data is not specified, and if autoLoad is true or an Object, this store's load method is automatically called
   * after creation. If the value of autoLoad is an Object, this Object will be passed to the store's load method.
   *
   * <p>It's important to note that <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Tree Stores</a> will<br/>
   * load regardless of autoLoad's value if expand is set to true on the
   * <a href="#!/api/Ext.data.TreeStore-cfg-root" rel="Ext.data.TreeStore-cfg-root" class="docClass">root node</a>.</p>
   * @since 2.3.0
   * @see #getAutoLoad()
   * @see #setAutoLoad()
   */
  public native function get autoLoad():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoLoad(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to automatically sync the Store with its Proxy after every edit to one of its Records. Defaults to false.
   * @default false
   * @see #getAutoSync()
   * @see #setAutoSync()
   */
  public native function get autoSync():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoSync(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Sets the updating behavior based on batch synchronization. 'operation' (the default) will update the Store's
   * internal representation of the data after each operation of the batch has completed, 'complete' will wait until
   * the entire batch has been completed before updating the Store's data. 'complete' is a good choice for local
   * storage proxies, 'operation' is better for remote proxies, where there is a comparatively high latency.
   * @default 'operation'
   * @see #getBatchUpdateMode()
   * @see #setBatchUpdateMode()
   */
  public native function get batchUpdateMode():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set batchUpdateMode(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An Array of <code><a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Ext.data.field.Field</a></code> config objects, simply the field
   * <a href="#!/api/Ext.data.field.Field-cfg-name" rel="Ext.data.field.Field-cfg-name" class="docClass">name</a>, or a mix of config objects and strings.
   * If just a name is given, the field type defaults to <code>auto</code>.
   *
   * <p>In a <a href="#!/api/Ext.data.field.Field" rel="Ext.data.field.Field" class="docClass">Field</a> config object you may pass the alias of
   * the <code>Ext.data.field.&#42;</code> type using the <code>type</code> config option.</p>
   *
   * <pre><code>// two fields are set:
   * // - an 'auto' field with a name of 'firstName'
   * // - and an <a href="#!/api/Ext.data.field.Integer" rel="Ext.data.field.Integer" class="docClass">Ext.data.field.Integer</a> field with a name of 'age'
   * fields: ['firstName', {
   *     type: 'int',
   *     name: 'age'
   * }]
   * </code></pre>
   *
   * <p>Fields will automatically be created at read time for any for any keys in the
   * data passed to the Model's <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy's</a>
   * <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">reader</a> whose name is not explicitly configured in
   * the <code>fields</code> config.</p>
   *
   * <p>Extending a Model class will inherit all the <code>fields</code> from the superclass /
   * ancestor classes.</p>
   * @since 2.3.0
   * @see #getFields()
   * @see #setFields()
   */
  public native function get fields():ext.util.MixedCollection;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fields(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Name of the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> associated with this store. See
   * <a href="#!/api/Ext.data.Model-property-entityName" rel="Ext.data.Model-property-entityName" class="docClass">Ext.data.Model.entityName</a>.
   *
   * <p>May also be the actual Model subclass.</p>
   *
   * <p>This config is required for the store to be able to read data unless you have defined
   * the <a href="#!/api/Ext.data.ProxyStore-cfg-fields" rel="Ext.data.ProxyStore-cfg-fields" class="docClass">fields</a> config which will create an anonymous <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code>.</p>
   * @see #getModel()
   * @see #setModel()
   */
  public native function get model():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set model(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The Proxy to use for this Store. This can be either a string, a config object or a Proxy instance -
   * see <a href="#!/api/Ext.data.ProxyStore-method-setProxy" rel="Ext.data.ProxyStore-method-setProxy" class="docClass">setProxy</a> for details.
   * @since 1.1.0
   * @see #getProxy()
   * @see #setProxy()
   */
  public native function get proxy():ext.data.proxy.DataProxy;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set proxy(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If true, any sorters attached to this Store will be run after loading data, before the datachanged event is fired.
   * Defaults to true, ignored if <a href="#!/api/Ext.data.Store-cfg-remoteSort" rel="Ext.data.Store-cfg-remoteSort" class="docClass">remoteSort</a> is true
   * @default true
   * @see #getSortOnLoad()
   * @see #setSortOnLoad()
   */
  public native function get sortOnLoad():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set sortOnLoad(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This config controls whether removed records are remembered by this store for
   * later saving to the server.
   * @default true
   * @see #getTrackRemoved()
   * @see #setTrackRemoved()
   */
  public native function get trackRemoved():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set trackRemoved(value:Boolean):void;

  /**
   * Returns true if the Store is currently performing a load operation
   * @return <code>true</code> if the Store is currently loading
   */
  override public native function isLoading():Boolean;

  /**
   * Called when the event handler which called the <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> method exits.
   */
  public native function flushLoad():void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-asynchronousLoad" rel="Ext.data.ProxyStore-cfg-asynchronousLoad" class="docClass">asynchronousLoad</a>.
   * @since 6.0.1
   */
  public native function getAsynchronousLoad():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-autoLoad" rel="Ext.data.ProxyStore-cfg-autoLoad" class="docClass">autoLoad</a>.
   * @since 2.3.0
   */
  public native function getAutoLoad():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">autoSync</a>.
   */
  public native function getAutoSync():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-batchUpdateMode" rel="Ext.data.ProxyStore-cfg-batchUpdateMode" class="docClass">batchUpdateMode</a>.
   */
  public native function getBatchUpdateMode():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-fields" rel="Ext.data.ProxyStore-cfg-fields" class="docClass">fields</a>.
   * @since 2.3.0
   */
  public native function getFields():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-model" rel="Ext.data.ProxyStore-cfg-model" class="docClass">model</a>.
   */
  public native function getModel():*;

  /**
   * Gets all <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">records</a> added or updated since the last commit. Note that the order of records
   * returned is not deterministic and does not indicate the order in which records were modified. Note also that
   * removed records are not included (use <a href="#!/api/Ext.data.ProxyStore-method-getRemovedRecords" rel="Ext.data.ProxyStore-method-getRemovedRecords" class="docClass">getRemovedRecords</a> for that).
   * @return The added and updated Model instances
   */
  public native function getModifiedRecords():Array;

  /**
   * Returns all <code><a href="#!/api/Ext.data.Model-property-phantom" rel="Ext.data.Model-property-phantom" class="docClass">phantom</a></code> records in this store.
   * @return A possibly empty array of <code>phantom</code> records.
   */
  public native function getNewRecords():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy</a>.
   * @since 1.1.0
   */
  public native function getProxy():*;

  /**
   * Returns any records that have been removed from the store but not yet destroyed on the proxy.
   * @return The removed Model instances. Note that this is a <em>copy</em> of the store's
   * array, so may be mutated.
   */
  public native function getRemovedRecords():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-sortOnLoad" rel="Ext.data.ProxyStore-cfg-sortOnLoad" class="docClass">sortOnLoad</a>.
   */
  public native function getSortOnLoad():Boolean;

  /**
   * Returns the total number of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> instances that the <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a>
   * indicates exist. This will usually differ from <a href="#!/api/Ext.data.ProxyStore-method-getCount" rel="Ext.data.ProxyStore-method-getCount" class="docClass">getCount</a> when using paging - getCount returns the
   * number of records loaded into the Store at the moment, getTotalCount returns the number of records that
   * could be loaded into the Store if the Store contained all data
   * @return The total number of Model instances available via the Proxy. 0 returned if
   * no value has been set via the reader.
   */
  public native function getTotalCount():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.ProxyStore-cfg-trackRemoved" rel="Ext.data.ProxyStore-cfg-trackRemoved" class="docClass">trackRemoved</a>.
   */
  public native function getTrackRemoved():Boolean;

  /**
   * Returns all valid, non-phantom Model instances that have been updated in the Store but not yet synchronized with the Proxy.
   * @return The updated Model instances
   */
  public native function getUpdatedRecords():Array;

  /**
   * Marks this store as needing a load. When the current executing event handler exits,
   * this store will send a request to load using its configured <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy</a>.
   *
   * <p>Upon return of the data from whatever data source the proxy connected to, the retrieved
   * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">records</a> will be loaded into this store, and the optional callback will be called.
   * Example usage:</p>
   *
   * <pre><code>store.load({
   *     scope: this,
   *     callback: function(records, operation, success) {
   *         // the operation object
   *         // contains all of the details of the load operation
   *         console.log(records);
   *     }
   * });
   * </code></pre>
   *
   * <p>If the callback scope does not need to be set, a function can simply be passed:</p>
   *
   * <pre><code>store.load(function(records, operation, success) {
   *     console.log('loaded records');
   * });
   * </code></pre>
   * @since 1.1.0
   * @param options This is passed into the <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a>
   * object that is created and then sent to the proxy's <a href="#!/api/Ext.data.proxy.Proxy-method-read" rel="Ext.data.proxy.Proxy-method-read" class="docClass">Ext.data.proxy.Proxy.read</a> function.
   * In addition to the options listed below, this object may contain properties to configure the
   * <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a>.
   * @param {Function} [options.callback] A function which is called when the response arrives.
   * @param {Array} options.callback.records Array of records.
   * @param {ext.data.operation.Operation} options.callback.operation The Operation itself.
   * @param {Boolean} options.callback.success <code>true</code> when operation completed successfully.
   * @param {Boolean} [options.addRecords] Specify as <code>true</code> to <em>add</em> the incoming records rather than the
   * default which is to have the incoming records <em>replace</em> the existing store contents.
   * @default false
   * @return this
   */
  public native function load(options:Object = null):ext.data.Store;

  /**
   * Reloads the store using the last options passed to the <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> method. You can use the reload method to reload the
   * store using the parameters from the last load() call. For example:
   *
   * <pre><code>store.load({
   *     params : {
   *         userid : 22216
   *     }
   * });
   *
   * //...
   *
   * store.reload();
   * </code></pre>
   *
   * <p>The initial <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> execution will pass the <code>userid</code> parameter in the request. The <a href="#!/api/Ext.data.ProxyStore-method-reload" rel="Ext.data.ProxyStore-method-reload" class="docClass">reload</a> execution
   * will also send the same <code>userid</code> parameter in its request as it will reuse the <code>params</code> object from the last <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> call.</p>
   *
   * <p>You can override a param by passing in the config object with the <code>params</code> object:</p>
   *
   * <pre><code>store.load({
   *     params : {
   *         userid : 22216,
   *         foo    : 'bar'
   *     }
   * });
   *
   * //...
   *
   * store.reload({
   *     params : {
   *         userid : 1234
   *     }
   * });
   * </code></pre>
   *
   * <p>The initial <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> execution sends the <code>userid</code> and <code>foo</code> parameters but in the <a href="#!/api/Ext.data.ProxyStore-method-reload" rel="Ext.data.ProxyStore-method-reload" class="docClass">reload</a> it only sends
   * the <code>userid</code> paramter because you are overriding the <code>params</code> config not just overriding the one param. To only change a single param
   * but keep other params, you will have to get the last params from the <a href="#!/api/Ext.data.ProxyStore-property-lastOptions" rel="Ext.data.ProxyStore-property-lastOptions" class="docClass">lastOptions</a> property:</p>
   *
   * <pre><code>var lastOptions = store.lastOptions,
   *     lastParams = <a href="#!/api/Ext-method-clone" rel="Ext-method-clone" class="docClass">Ext.clone</a>(lastOptions.params); // make a copy of the last params so we don't affect future reload() calls
   *
   * lastParams.userid = 1234;
   *
   * store.reload({
   *     params : lastParams
   * });
   * </code></pre>
   *
   * <p>This will now send the <code>userid</code> parameter as <code>1234</code> and the <code>foo</code> param as <code>'bar'</code>.</p>
   * @param options A config object which contains options which may override the options passed to the previous load call. See the
   * <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">load</a> method for valid configs.
   */
  public native function reload(options:Object = null):void;

  /**
   * Removes all records from the store. This method does a "fast remove",
   * individual remove events are not called. The <a href="#!/api/Ext.data.ProxyStore-event-clear" rel="Ext.data.ProxyStore-event-clear" class="docClass">clear</a> event is
   * fired upon completion.
   * @since 1.1.0
   */
  public native function removeAll():void;

  /**
   * Resumes automatically syncing the Store with its Proxy.  Only applicable if <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">autoSync</a> is <code>true</code>
   * @param syncNow Pass <code>true</code> to synchronize now. Only synchronizes with the Proxy if the suspension
   * count has gone to zero (We are not under a higher level of suspension)
   */
  public native function resumeAutoSync(syncNow:Boolean):void;

  [Deprecated(message = "Will be removed in the next major version\n", since = "4.0.0")]
  /**
   * Saves all pending changes via the configured <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy</a>. Use <a href="#!/api/Ext.data.ProxyStore-method-sync" rel="Ext.data.ProxyStore-method-sync" class="docClass">sync</a> instead.
   */
  public native function save():Number;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-asynchronousLoad" rel="Ext.data.ProxyStore-cfg-asynchronousLoad" class="docClass">asynchronousLoad</a>.
   * @since 6.0.1
   * @param asynchronousLoad The new value.
   */
  public native function setAsynchronousLoad(asynchronousLoad:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-autoLoad" rel="Ext.data.ProxyStore-cfg-autoLoad" class="docClass">autoLoad</a>.
   * @since 2.3.0
   * @param autoLoad The new value.
   */
  public native function setAutoLoad(autoLoad:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">autoSync</a>.
   * @param autoSync The new value.
   */
  public native function setAutoSync(autoSync:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-batchUpdateMode" rel="Ext.data.ProxyStore-cfg-batchUpdateMode" class="docClass">batchUpdateMode</a>.
   * @param batchUpdateMode The new value.
   */
  public native function setBatchUpdateMode(batchUpdateMode:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-fields" rel="Ext.data.ProxyStore-cfg-fields" class="docClass">fields</a>.
   * @since 2.3.0
   * @param fields The new value.
   */
  public native function setFields(fields:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-model" rel="Ext.data.ProxyStore-cfg-model" class="docClass">model</a>.
   * @param model The new value.
   */
  public native function setModel(model:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy</a>.
   * @since 1.1.0
   * @param proxy The new value.
   */
  public native function setProxy(proxy:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-sortOnLoad" rel="Ext.data.ProxyStore-cfg-sortOnLoad" class="docClass">sortOnLoad</a>.
   * @param sortOnLoad The new value.
   */
  public native function setSortOnLoad(sortOnLoad:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.ProxyStore-cfg-trackRemoved" rel="Ext.data.ProxyStore-cfg-trackRemoved" class="docClass">trackRemoved</a>.
   * @param trackRemoved The new value.
   */
  public native function setTrackRemoved(trackRemoved:Boolean):void;

  /**
   * Suspends automatically syncing the Store with its Proxy.  Only applicable if <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">autoSync</a> is <code>true</code>
   */
  public native function suspendAutoSync():void;

  /**
   * Synchronizes the store with its <a href="#!/api/Ext.data.ProxyStore-cfg-proxy" rel="Ext.data.ProxyStore-cfg-proxy" class="docClass">proxy</a>. This asks the proxy to batch together any new, updated
   * and deleted records in the store, updating the store's internal representation of the records
   * as each operation completes.
   * @param options Object containing one or more properties supported by the sync method (these get
   * passed along to the underlying proxy's <a href="#!/api/Ext.data.proxy.Proxy-method-batch" rel="Ext.data.proxy.Proxy-method-batch" class="docClass">batch</a> method):
   * @param [options.batch] A <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">Ext.data.Batch</a> object (or batch config to apply
   * to the created batch). If unspecified a default batch will be auto-created as needed.
   * @param {Function} [options.callback] The function to be called upon completion of the sync.
   * The callback is called regardless of success or failure and is passed the following parameters:
   * @param {ext.data.Batch} options.callback.batch The <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">batch</a> that was processed,
   * containing all operations in their current state after processing
   * @param {Object} options.callback.options The options argument that was originally passed into sync
   * @param {Function} [options.success] The function to be called upon successful completion of the sync. The
   * success function is called only if no exceptions were reported in any operations. If one or more exceptions
   * occurred then the failure function will be called instead. The success function is called
   * with the following parameters:
   * @param {ext.data.Batch} options.success.batch The <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">batch</a> that was processed,
   * containing all operations in their current state after processing
   * @param {Object} options.success.options The options argument that was originally passed into sync
   * @param {Function} [options.failure] The function to be called upon unsuccessful completion of the sync. The
   * failure function is called when one or more operations returns an exception during processing (even if some
   * operations were also successful). In this case you can check the batch's <a href="#!/api/Ext.data.Batch-property-exceptions" rel="Ext.data.Batch-property-exceptions" class="docClass">exceptions</a> array to see exactly which operations had exceptions. The failure function is called with the
   * following parameters:
   * @param {ext.data.Batch} options.failure.batch The <a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">Ext.data.Batch</a> that was processed, containing all
   * operations in their current state after processing
   * @param {Object} options.failure.options The options argument that was originally passed into sync
   * @param {Object} [options.params] Additional params to send during the sync Operation(s).
   * @param {Object} [options.scope] The scope in which to execute any callbacks (i.e. the <code>this</code> object inside
   * the callback, success and/or failure functions). Defaults to the store's proxy.
   * @return this
   */
  public native function sync(options:Object = null):ext.data.Store;
}
}