package ext.data.identifier {
import ext.Base;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.identifier.Generator", require)]
/**
 * This class is a base for all id generators. It also provides lookup of id generators by
 * their id.
 *
 * <p>Generally, id generators are used to generate a primary key for new model instances. There
 * are different approaches to solving this problem, so this mechanism has both simple use
 * cases and is open to custom implementations. A <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> requests id generation
 * using the <a href="#!/api/Ext.data.Model-cfg-identifier" rel="Ext.data.Model-cfg-identifier" class="docClass">Ext.data.Model.identifier</a> property.</p>
 *
 * <p>The following types of <code>identifiers</code> are provided:</p>
 *
 * <ul>
 * <li><code><a href="#!/api/Ext.data.identifier.Sequential" rel="Ext.data.identifier.Sequential" class="docClass">sequential</a></code></li>
 * <li><code><a href="#!/api/Ext.data.identifier.Negative" rel="Ext.data.identifier.Negative" class="docClass">negative</a></code></li>
 * <li><code><a href="#!/api/Ext.data.identifier.Uuid" rel="Ext.data.identifier.Uuid" class="docClass">uuid</a></code></li>
 * </ul>
 *
 *
 * <p>In most cases (other than <code>uuid</code>), the server is the only party that can generate
 * authoritative id values. This means that any id generated by an <code>identifier</code> should be
 * consider "provisional" and must eventually be reconciled with the server. This makes a
 * <code>uuid</code> very attractive as an <code>identifier</code> because they are designed to be generated in
 * a distributed manner and therefore never require reconciliation.</p>
 *
 * <p>It is common for id values to be generated as increasing integer values (1, 2, etc.) by
 * the server when records are inserted. A <code><a href="#!/api/Ext.data.identifier.Negative" rel="Ext.data.identifier.Negative" class="docClass">negative</a></code>
 * <code>identifier</code> may be useful as it generates client-side values of -1, -2, etc.. These
 * values are of the same data type (integer) and so can typically be read by servers
 * using typed languages (such as Java or C#) and easily recognized as provisional.</p>
 *
 * <p>In the end, the choice of <code>identifier</code> strategy requires agreement between client and
 * server.</p>
 *
 * <h1>Identity, Type and Shared Generators</h1>
 *
 * <p>It is often desirable to share Generators to ensure uniqueness or common configuration.
 * This is done by giving Generator instances an id property by which they can be looked
 * up using the <a href="#!/api/Ext.Factory-method-dataIdentifier" rel="Ext.Factory-method-dataIdentifier" class="docClass">dataIdentifier</a> method. To configure two <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> classes
 * to share one <a href="#!/api/Ext.data.identifier.Sequential" rel="Ext.data.identifier.Sequential" class="docClass">sequential</a> id generator, you simply
 * assign them the same id:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelA', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: {
 *         type: 'sequential',
 *         id: 'foo'
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelB', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: {
 *         type: 'sequential',
 *         id: 'foo'
 *     }
 * });
 * </code></pre>
 *
 * <p>To make this as simple as possible for generator types that are shared by many (or all)
 * Models, the Generator types (such as 'sequential' or 'uuid') are also reserved as
 * generator ids. This is used by the <a href="#!/api/Ext.data.identifier.Uuid" rel="Ext.data.identifier.Uuid" class="docClass">Ext.data.identifier.Uuid</a> which has an id equal
 * to its type ('uuid'). In other words, the following Models share the same generator:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelX', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: 'uuid'
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelY', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: 'uuid'
 * });
 * </code></pre>
 *
 * <p>This can be overridden (by specifying the id explicitly), but there is no particularly
 * good reason to do so for this generator type.</p>
 *
 * <h1>Creating Custom Generators</h1>
 *
 * <p>An id generator should derive from this class and implement the <a href="#!/api/Ext.data.identifier.Generator-method-generate" rel="Ext.data.identifier.Generator-method-generate" class="docClass">generate</a> method.</p>
 *
 * <p>To register an id generator type, a derived class should provide an <code>alias</code> like so:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.identifier.Custom', {
 *     extend: '<a href="#!/api/Ext.data.identifier.Generator" rel="Ext.data.identifier.Generator" class="docClass">Ext.data.identifier.Generator</a>',
 *     alias: 'data.identifier.custom',
 *     config: {
 *         configProp: 42 // some config property w/default value
 *     }
 *
 *     generate: function () {
 *         return ... // a new id
 *     }
 * });
 * </code></pre>
 *
 * <p>Using the custom id generator is then straightforward:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModel', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: 'custom'
 * });
 * // or...
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModel', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: {
 *         type: 'custom',
 *         configProp: value
 *     }
 * });
 * </code></pre>
 *
 * <p>It is not recommended to mix shared generators with generator configuration. This leads
 * to unpredictable results unless all configurations match (which is also redundant). In
 * such cases, a custom generator with a default id is the best approach.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.identifier.Custom', {
 *     extend: '<a href="#!/api/Ext.data.identifier.Sequential" rel="Ext.data.identifier.Sequential" class="docClass">Ext.data.identifier.Sequential</a>',
 *     alias: 'data.identifier.custom',
 *
 *     config: {
 *         id: 'custom',
 *         prefix: 'ID_',
 *         seed: 1000
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelX', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: 'custom'
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.data.MyModelY', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     identifier: 'custom'
 * });
 *
 * // the above models share a generator that produces ID_1000, ID_1001, etc..
 * </code></pre>
 */
public class GeneratorIdentifier extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * Initializes a new instance.
   * @param config Configuration object to be applied to the new instance.
   */
  public function GeneratorIdentifier(config:ext.data.identifier.GeneratorIdentifier = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated IdGenerator, or subclass thereof.
   * @default true
   */
  public native function get isGenerator():Boolean;

  /**
   * @private
   */
  public native function set isGenerator(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The id for this generator.
   * @see #getId()
   * @see #setId()
   */
  public native function get id():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set id(value:String):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Generates and returns the next id. This method must be implemented by the derived
   * class.
   * @return The next id.
   */
  public native function generate():*;

  /**
   * Returns the value of <a href="#!/api/Ext.data.identifier.Generator-cfg-id" rel="Ext.data.identifier.Generator-cfg-id" class="docClass">id</a>.
   */
  public native function getId():String;

  /**
   * Sets the value of <a href="#!/api/Ext.data.identifier.Generator-cfg-id" rel="Ext.data.identifier.Generator-cfg-id" class="docClass">id</a>.
   * @param id The new value.
   */
  public native function setId(id:String):void;
}
}