package ext.data.proxy {
import ext.data.Request;

[Uses("ext.data.proxy.ServerProxy")]
[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.Ajax", require)]
/**
 * AjaxProxy is one of the most widely-used ways of getting data into your application. It uses AJAX requests to load
 * data from the server, usually to be placed into a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>. Let's take a look at a typical setup.
 * Here we're going to set up a Store that has an AjaxProxy. To prepare, we'll also set up a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>:
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name', 'email']
 * });
 *
 * //The Store contains the AjaxProxy as an inline configuration
 * var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User',
 *     proxy: {
 *         type: 'ajax',
 *         url : 'users.json'
 *     }
 * });
 *
 * store.load();
 * </code></pre>
 *
 * <p>Our example is going to load user data into a Store, so we start off by defining a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> with
 * the fields that we expect the server to return. Next we set up the Store itself, along with a
 * <a href="#!/api/Ext.data.Store-cfg-proxy" rel="Ext.data.Store-cfg-proxy" class="docClass">proxy</a> configuration. This configuration was automatically turned into an
 * <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a> instance, with the url we specified being passed into AjaxProxy's constructor.
 * It's as if we'd done this:</p>
 *
 * <pre><code>new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *     url: 'users.json',
 *     model: 'User',
 *     reader: 'json'
 * });
 * </code></pre>
 *
 * <p>A couple of extra configurations appeared here - <a href="#!/api/Ext.data.proxy.Ajax-cfg-model" rel="Ext.data.proxy.Ajax-cfg-model" class="docClass">model</a> and <a href="#!/api/Ext.data.proxy.Ajax-cfg-reader" rel="Ext.data.proxy.Ajax-cfg-reader" class="docClass">reader</a>. These are set by default when we
 * create the proxy via the Store - the Store already knows about the Model, and Proxy's default <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Reader</a> is <a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">JsonReader</a>.</p>
 *
 * <p>Now when we call store.load(), the AjaxProxy springs into action, making a request to the url we configured
 * ('users.json' in this case). As we're performing a read, it sends a GET request to that url (see
 * <a href="#!/api/Ext.data.proxy.Ajax-cfg-actionMethods" rel="Ext.data.proxy.Ajax-cfg-actionMethods" class="docClass">actionMethods</a> to customize this - by default any kind of read will be sent as a GET request and any kind of write
 * will be sent as a POST request).</p>
 *
 * <h1>Limitations</h1>
 *
 * <p>AjaxProxy cannot be used to retrieve data from other domains. If your application is running on http://domainA.com it
 * cannot load data from http://domainB.com because browsers have a built-in security policy that prohibits domains
 * talking to each other via AJAX.</p>
 *
 * <p>If you need to read data from another domain and can't set up a proxy server (some software that runs on your own
 * domain's web server and transparently forwards requests to http://domainB.com, making it look like they actually came
 * from http://domainA.com), you can use <a href="#!/api/Ext.data.proxy.JsonP" rel="Ext.data.proxy.JsonP" class="docClass">Ext.data.proxy.JsonP</a> and a technique known as JSON-P (JSON with
 * Padding), which can help you get around the problem so long as the server on http://domainB.com is set up to support
 * JSON-P responses. See <a href="#!/api/Ext.data.proxy.JsonP" rel="Ext.data.proxy.JsonP" class="docClass">JsonPProxy</a>'s introduction docs for more details.</p>
 *
 * <h1>Readers and Writers</h1>
 *
 * <p>AjaxProxy can be configured to use any type of <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Reader</a> to decode the server's response.
 * If no Reader is supplied, AjaxProxy will default to using a <a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">JsonReader</a>. Reader
 * configuration can be passed in as a simple object, which the Proxy automatically turns into a <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Reader</a> instance:</p>
 *
 * <pre><code>var proxy = new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *     model: 'User',
 *     reader: {
 *         type: 'xml',
 *         rootProperty: 'users'
 *     }
 * });
 *
 * proxy.getReader(); //returns an XmlReader instance based on the config we supplied
 * </code></pre>
 *
 * <h1>Url generation</h1>
 *
 * <p>AjaxProxy automatically inserts any sorting, filtering, paging and grouping options into the url it generates for
 * each request. These are controlled with the following configuration options:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.proxy.Ajax-cfg-pageParam" rel="Ext.data.proxy.Ajax-cfg-pageParam" class="docClass">pageParam</a> - controls how the page number is sent to the server (see also <a href="#!/api/Ext.data.proxy.Ajax-cfg-startParam" rel="Ext.data.proxy.Ajax-cfg-startParam" class="docClass">startParam</a> and <a href="#!/api/Ext.data.proxy.Ajax-cfg-limitParam" rel="Ext.data.proxy.Ajax-cfg-limitParam" class="docClass">limitParam</a>)</li>
 * <li><a href="#!/api/Ext.data.proxy.Ajax-cfg-sortParam" rel="Ext.data.proxy.Ajax-cfg-sortParam" class="docClass">sortParam</a> - controls how sort information is sent to the server</li>
 * <li><a href="#!/api/Ext.data.proxy.Ajax-cfg-groupParam" rel="Ext.data.proxy.Ajax-cfg-groupParam" class="docClass">groupParam</a> - controls how grouping information is sent to the server</li>
 * <li><a href="#!/api/Ext.data.proxy.Ajax-cfg-filterParam" rel="Ext.data.proxy.Ajax-cfg-filterParam" class="docClass">filterParam</a> - controls how filter information is sent to the server</li>
 * </ul>
 *
 *
 * <p>Each request sent by AjaxProxy is described by an <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a>. To see how we can customize
 * the generated urls, let's say we're loading the Proxy with the following Operation:</p>
 *
 * <pre><code>var proxy = new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *     url: '/users'
 * });
 *
 * var operation = proxy.createOperation('read', {
 *     page  : 2
 * });
 * </code></pre>
 *
 * <p>Now we'll issue the request for this Operation by calling <a href="#!/api/Ext.data.proxy.Ajax-method-read" rel="Ext.data.proxy.Ajax-method-read" class="docClass">read</a>:</p>
 *
 * <pre><code>proxy.read(operation); //GET /users?page=2
 * </code></pre>
 *
 * <p>Easy enough - the Proxy just copied the page property from the Operation. We can customize how this page data is sent
 * to the server:</p>
 *
 * <pre><code>var proxy = new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *     url: '/users',
 *     pageParam: 'pageNumber'
 * });
 *
 * proxy.read(operation); //GET /users?pageNumber=2
 * </code></pre>
 *
 * <p>Alternatively, our Operation could have been configured to send start and limit parameters instead of page:</p>
 *
 * <pre><code>var proxy = new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *     url: '/users'
 * });
 *
 * var operation = proxy.createOperation('read', {
 *     start : 50,
 *     limit : 25
 * });
 *
 * proxy.read(operation); //GET /users?start=50&amp;limit;=25
 * </code></pre>
 *
 * <p>Again we can customize this url:</p>
 *
 * <pre><code>var proxy = new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *     url: '/users',
 *     startParam: 'startIndex',
 *     limitParam: 'limitIndex'
 * });
 *
 * proxy.read(operation); //GET /users?startIndex=50&amp;limitIndex;=25
 * </code></pre>
 *
 * <p>AjaxProxy will also send sort and filter information to the server. Let's take a look at how this looks with a more
 * expressive Operation object:</p>
 *
 * <pre><code>var operation = proxy.createOperation('read', {
 *     sorters: [
 *         new <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Ext.util.Sorter</a>({
 *             property : 'name',
 *             direction: 'ASC'
 *         }),
 *         new <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Ext.util.Sorter</a>({
 *             property : 'age',
 *             direction: 'DESC'
 *         })
 *     ],
 *     filters: [
 *         new <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Ext.util.Filter</a>({
 *             property: 'eyeColor',
 *             value   : 'brown'
 *         })
 *     ]
 * });
 * </code></pre>
 *
 * <p>This is the type of object that is generated internally when loading a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> with sorters and
 * filters defined. By default the AjaxProxy will JSON encode the sorters and filters, resulting in something like this
 * (note that the url is escaped before sending the request, but is left unescaped here for clarity):</p>
 *
 * <pre><code>var proxy = new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *     url: '/users'
 * });
 *
 * proxy.read(operation); //GET /users?sort=[{"property":"name","direction":"ASC"},{"property":"age","direction":"DESC"}]&amp;filter;=[{"property":"eyeColor","value":"brown"}]
 * </code></pre>
 *
 * <p>We can again customize how this is created by supplying a few configuration options. Let's say our server is set up
 * to receive sorting information is a format like "sortBy=name#ASC,age#DESC". We can configure AjaxProxy to provide
 * that format like this:</p>
 *
 * <pre><code> var proxy = new <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a>({
 *      url: '/users',
 *      sortParam: 'sortBy',
 *      filterParam: 'filterBy',
 *
 *      //our custom implementation of sorter encoding - turns our sorters into "name#ASC,age#DESC"
 *      encodeSorters: function(sorters) {
 *          var length   = sorters.length,
 *              sortStrs = [],
 *              sorter, i;
 *
 *          for (i = 0; i &lt; length; i++) {
 *              sorter = sorters[i];
 *
 *              sortStrs[i] = sorter.property + '#' + sorter.direction
 *          }
 *
 *          return sortStrs.join(",");
 *      }
 *  });
 *
 *  proxy.read(operation); //GET /users?sortBy=name#ASC,age#DESC&amp;filterBy;=[{"property":"eyeColor","value":"brown"}]
 * </code></pre>
 *
 * <p>We can also provide a custom <a href="#!/api/Ext.data.proxy.Ajax-method-encodeFilters" rel="Ext.data.proxy.Ajax-method-encodeFilters" class="docClass">encodeFilters</a> function to encode our filters.</p>
 *
 * <h1>Debugging your Ajax Proxy</h1>
 *
 * <p>If the data is not being loaded into the store as expected, it could be due to a mismatch between the the way that the <a href="#!/api/Ext.data.proxy.Ajax-cfg-reader" rel="Ext.data.proxy.Ajax-cfg-reader" class="docClass">reader</a>
 * is configured, and the shape of the incoming data.</p>
 *
 * <p>To debug from the point that your data arrives back from the network, set a breakpoint inside the callback function
 * created in the <code>createRequestCallback</code> method of the Ajax Proxy class, and follow the data to where the <a href="#!/api/Ext.data.proxy.Ajax-cfg-reader" rel="Ext.data.proxy.Ajax-cfg-reader" class="docClass">reader</a> attempts
 * to consume it.</p>
 */
public class AjaxProxy extends ext.data.proxy.ServerProxy {
  /**
   * Note that if this HttpProxy is being used by a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>, then the Store's call to
   * <a href="#!/api/Ext.data.Store-method-load" rel="Ext.data.Store-method-load" class="docClass">load</a> will override any specified callback and params options. In this case, use the
   * <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>'s events to modify parameters, or react to loading events.
   * @param config Config object.
   * If an options parameter is passed, the singleton <a href="#!/api/Ext.Ajax" rel="Ext.Ajax" class="docClass">Ext.Ajax</a> object will be used to make the request.
   */
  public function AjaxProxy(config:ext.data.proxy.AjaxProxy = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Mapping of action name to HTTP request method. In the basic AjaxProxy these are set to 'GET' for 'read' actions
   * and 'POST' for 'create', 'update' and 'destroy' actions. The <a href="#!/api/Ext.data.proxy.Rest" rel="Ext.data.proxy.Rest" class="docClass">Ext.data.proxy.Rest</a> maps these to the
   * correct RESTful methods.
   * @default {create: 'POST', read: 'GET', update: 'POST', destroy: 'POST'}
   * @see #getActionMethods()
   * @see #setActionMethods()
   */
  public native function get actionMethods():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set actionMethods(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to request binary data from the server.  This feature requires
   * the use of a binary reader such as AMF Reader
   * @default false
   * @see #getBinary()
   * @see #setBinary()
   */
  public native function get binary():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set binary(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Any headers to add to the Ajax request.
   *
   * <p>example:</p>
   *
   * <pre><code>proxy: {
   *     headers: {'Content-Type': "text/plain" }
   *     ...
   * }
   * </code></pre>
   * @see #getHeaders()
   * @see #setHeaders()
   */
  public native function get headers():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set headers(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to have any request parameters sent as <a href="#!/api/Ext.data.Connection-method-request" rel="Ext.data.Connection-method-request" class="docClass">jsonData</a>
   * where they can be parsed from the raw request. By default, parameters are sent via the
   * <a href="#!/api/Ext.data.Connection-method-request" rel="Ext.data.Connection-method-request" class="docClass">params</a> property. <strong>Note</strong>: This setting does not apply when the
   * request is sent as a 'GET' request. See <a href="#!/api/Ext.data.proxy.Ajax-cfg-actionMethods" rel="Ext.data.proxy.Ajax-cfg-actionMethods" class="docClass">actionMethods</a> for controlling the HTTP verb
   * that is used when sending requests.
   * @default false
   * @see #getParamsAsJson()
   * @see #setParamsAsJson()
   */
  public native function get paramsAsJson():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set paramsAsJson(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Most oData feeds require basic HTTP authentication. This configuration allows
   * you to specify the password.
   * @see #getPassword()
   * @see #setPassword()
   */
  public native function get password():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set password(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set this to false to not send the default Xhr header (X-Requested-With) with every request.
   * This should be set to false when making CORS (cross-domain) requests.
   * @default true
   * @see #getUseDefaultXhrHeader()
   * @see #setUseDefaultXhrHeader()
   */
  public native function get useDefaultXhrHeader():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set useDefaultXhrHeader(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Most oData feeds require basic HTTP authentication. This configuration allows
   * you to specify the username.
   * @see #getUsername()
   * @see #setUsername()
   */
  public native function get username():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set username(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This configuration is sometimes necessary when using cross-origin resource sharing.
   * @default false
   * @see #getWithCredentials()
   * @see #setWithCredentials()
   */
  public native function get withCredentials():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set withCredentials(value:Boolean):void;

  /**
   * Aborts a running request.
   * @param request The request to abort. If not passed, the most recent active
   * request will be aborted.
   */
  public native function abort(request:ext.data.Request = null):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-actionMethods" rel="Ext.data.proxy.Ajax-cfg-actionMethods" class="docClass">actionMethods</a>.
   */
  public native function getActionMethods():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-binary" rel="Ext.data.proxy.Ajax-cfg-binary" class="docClass">binary</a>.
   */
  public native function getBinary():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-headers" rel="Ext.data.proxy.Ajax-cfg-headers" class="docClass">headers</a>.
   */
  public native function getHeaders():Object;

  /**
   * Returns the HTTP method name for a given request. By default this returns based on a lookup on
   * <a href="#!/api/Ext.data.proxy.Ajax-cfg-actionMethods" rel="Ext.data.proxy.Ajax-cfg-actionMethods" class="docClass">actionMethods</a>.
   * @param request The request object
   * @return The HTTP method to use (should be one of 'GET', 'POST', 'PUT' or 'DELETE')
   */
  public native function getMethod(request:ext.data.Request):String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-paramsAsJson" rel="Ext.data.proxy.Ajax-cfg-paramsAsJson" class="docClass">paramsAsJson</a>.
   */
  public native function getParamsAsJson():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-password" rel="Ext.data.proxy.Ajax-cfg-password" class="docClass">password</a>.
   */
  public native function getPassword():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-useDefaultXhrHeader" rel="Ext.data.proxy.Ajax-cfg-useDefaultXhrHeader" class="docClass">useDefaultXhrHeader</a>.
   */
  public native function getUseDefaultXhrHeader():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-username" rel="Ext.data.proxy.Ajax-cfg-username" class="docClass">username</a>.
   */
  public native function getUsername():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-withCredentials" rel="Ext.data.proxy.Ajax-cfg-withCredentials" class="docClass">withCredentials</a>.
   */
  public native function getWithCredentials():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-actionMethods" rel="Ext.data.proxy.Ajax-cfg-actionMethods" class="docClass">actionMethods</a>.
   * @param actionMethods The new value.
   */
  public native function setActionMethods(actionMethods:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-binary" rel="Ext.data.proxy.Ajax-cfg-binary" class="docClass">binary</a>.
   * @param binary The new value.
   */
  public native function setBinary(binary:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-headers" rel="Ext.data.proxy.Ajax-cfg-headers" class="docClass">headers</a>.
   * @param headers The new value.
   */
  public native function setHeaders(headers:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-paramsAsJson" rel="Ext.data.proxy.Ajax-cfg-paramsAsJson" class="docClass">paramsAsJson</a>.
   * @param paramsAsJson The new value.
   */
  public native function setParamsAsJson(paramsAsJson:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-password" rel="Ext.data.proxy.Ajax-cfg-password" class="docClass">password</a>.
   * @param password The new value.
   */
  public native function setPassword(password:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-useDefaultXhrHeader" rel="Ext.data.proxy.Ajax-cfg-useDefaultXhrHeader" class="docClass">useDefaultXhrHeader</a>.
   * @param useDefaultXhrHeader The new value.
   */
  public native function setUseDefaultXhrHeader(useDefaultXhrHeader:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-username" rel="Ext.data.proxy.Ajax-cfg-username" class="docClass">username</a>.
   * @param username The new value.
   */
  public native function setUsername(username:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Ajax-cfg-withCredentials" rel="Ext.data.proxy.Ajax-cfg-withCredentials" class="docClass">withCredentials</a>.
   * @param withCredentials The new value.
   */
  public native function setWithCredentials(withCredentials:Boolean):void;
}
}