package ext.data.proxy {
import ext.data.Request;

[Uses("ext.data.proxy.ServerProxy")]
[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.JsonP", require)]
/**
 * The JsonP proxy is useful when you need to load data from a domain other than the one your application is running on. If
 * your application is running on http://domainA.com it cannot use <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ajax</a> to load its data
 * from http://domainB.com because cross-domain ajax requests are prohibited by the browser.
 *
 * <p>We can get around this using a JsonP proxy. JsonP proxy injects a <code>&lt;script&gt;</code> tag into the DOM whenever an AJAX request
 * would usually be made. Let's say we want to load data from http://domainB.com/users - the script tag that would be
 * injected might look like this:</p>
 *
 * <pre><code>&lt;script src="http://domainB.com/users?callback=someCallback"&gt;&lt;/script&gt;
 * </code></pre>
 *
 * <p>When we inject the tag above, the browser makes a request to that url and includes the response as if it was any
 * other type of JavaScript include. By passing a callback in the url above, we're telling domainB's server that we want
 * to be notified when the result comes in and that it should call our callback function with the data it sends back. So
 * long as the server formats the response to look like this, everything will work:</p>
 *
 * <pre><code>someCallback({
 *     users: [
 *         {
 *             id: 1,
 *             name: "Ed Spencer",
 *             email: "ed&#64;sencha.com"
 *         }
 *     ]
 * });
 * </code></pre>
 *
 * <p>As soon as the script finishes loading, the 'someCallback' function that we passed in the url is called with the JSON
 * object that the server returned.</p>
 *
 * <p>JsonP proxy takes care of all of this automatically. It formats the url you pass, adding the callback parameter
 * automatically. It even creates a temporary callback function, waits for it to be called and then puts the data into
 * the Proxy making it look just like you loaded it through a normal <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">AjaxProxy</a>. Here's how
 * we might set that up:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name', 'email']
 * });
 *
 * var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User',
 *     proxy: {
 *         type: 'jsonp',
 *         url : 'http://domainB.com/users'
 *     }
 * });
 *
 * store.load();
 * </code></pre>
 *
 * <p>That's all we need to do - JsonP proxy takes care of the rest. In this case the Proxy will have injected a script tag
 * like this:</p>
 *
 * <pre><code>&lt;script src="http://domainB.com/users?callback=callback1"&gt;&lt;/script&gt;
 * </code></pre>
 *
 * <h1>Customization</h1>
 *
 * <p>This script tag can be customized using the <a href="#!/api/Ext.data.proxy.JsonP-cfg-callbackKey" rel="Ext.data.proxy.JsonP-cfg-callbackKey" class="docClass">callbackKey</a> configuration. For example:</p>
 *
 * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User',
 *     proxy: {
 *         type: 'jsonp',
 *         url : 'http://domainB.com/users',
 *         callbackKey: 'theCallbackFunction'
 *     }
 * });
 *
 * store.load();
 * </code></pre>
 *
 * <p>Would inject a script tag like this:</p>
 *
 * <pre><code>&lt;script src="http://domainB.com/users?theCallbackFunction=callback1"&gt;&lt;/script&gt;
 * </code></pre>
 *
 * <h1>Implementing on the server side</h1>
 *
 * <p>The remote server side needs to be configured to return data in this format. Here are suggestions for how you might
 * achieve this using Java, PHP and ASP.net:</p>
 *
 * <p>Java:</p>
 *
 * <pre><code>boolean jsonP = false;
 * String cb = request.getParameter("callback");
 * if (cb != null) {
 *     jsonP = true;
 *     response.setContentType("text/javascript");
 * } else {
 *     response.setContentType("application/x-json");
 * }
 * Writer out = response.getWriter();
 * if (jsonP) {
 *     out.write(cb + "(");
 * }
 * out.print(dataBlock.toJsonString());
 * if (jsonP) {
 *     out.write(");");
 * }
 * </code></pre>
 *
 * <p>PHP:</p>
 *
 * <pre><code>$callback = $_REQUEST['callback'];
 *
 * // Create the output object.
 * $output = array('a' =&gt; 'Apple', 'b' =&gt; 'Banana');
 *
 * //start output
 * if ($callback) {
 *     header('Content-Type: text/javascript');
 *     echo $callback . '(' . json_encode($output) . ');';
 * } else {
 *     header('Content-Type: application/x-json');
 *     echo json_encode($output);
 * }
 * </code></pre>
 *
 * <p>ASP.net:</p>
 *
 * <pre><code>String jsonString = "{success: true}";
 * String cb = Request.Params.Get("callback");
 * String responseString = "";
 * if (!String.IsNullOrEmpty(cb)) {
 *     responseString = cb + "(" + jsonString + ")";
 * } else {
 *     responseString = jsonString;
 * }
 * Response.Write(responseString);
 * </code></pre>
 */
public class JsonPProxy extends ext.data.proxy.ServerProxy {
  /**
   * @param config @inheritDoc
   */
  public function JsonPProxy(config:ext.data.proxy.JsonPProxy = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to automatically append the request's params to the generated url. Defaults to true
   * @default true
   * @see #getAutoAppendParams()
   * @see #setAutoAppendParams()
   */
  public native function get autoAppendParams():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoAppendParams(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * See <a href="#!/api/Ext.data.JsonP-property-callbackKey" rel="Ext.data.JsonP-property-callbackKey" class="docClass">Ext.data.JsonP.callbackKey</a>.
   * @default 'callback'
   * @see #getCallbackKey()
   * @see #setCallbackKey()
   */
  public native function get callbackKey():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set callbackKey(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The HTTP parameter name to use when passing records to the server and the <a href="#!/api/Ext.data.proxy.JsonP-cfg-writer" rel="Ext.data.proxy.JsonP-cfg-writer" class="docClass">Json writer</a> is not configured
   * to <a href="#!/api/Ext.data.writer.Json-cfg-encode" rel="Ext.data.writer.Json-cfg-encode" class="docClass">encode</a> records into a parameter.
   *
   * <p>The <a href="#!/api/Ext.data.proxy.JsonP-method-encodeRecords" rel="Ext.data.proxy.JsonP-method-encodeRecords" class="docClass">encodeRecords</a> method is used to encode the records to create this parameter's value.</p>
   * @default 'records'
   * @see #getRecordParam()
   * @see #setRecordParam()
   */
  public native function get recordParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set recordParam(value:String):void;

  /**
   * Generates a url based on a given <a href="#!/api/Ext.data.Request" rel="Ext.data.Request" class="docClass">Ext.data.Request</a> object. Adds the params and callback function name to the url
   * @param request The request object
   * @return The url
   */
  override public native function buildUrl(request:ext.data.Request):String;

  /**
   * Aborts a server request. If no request is passed, the most recent request
   * will be aborted.
   * @param request The request to abort.
   */
  public native function abort(request:ext.data.Request = null):void;

  /**
   * Encodes an array of records into a value suitable to be added to the request <code>params</code> as the <a href="#!/api/Ext.data.proxy.JsonP-cfg-recordParam" rel="Ext.data.proxy.JsonP-cfg-recordParam" class="docClass">recordParam</a> parameter.
   * This is broken out into its own function so that it can be easily overridden.
   *
   * <p>The default implementation</p>
   * @param records The records array
   * @return An array of record data objects
   */
  public native function encodeRecords(records:Array):Array;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.JsonP-cfg-autoAppendParams" rel="Ext.data.proxy.JsonP-cfg-autoAppendParams" class="docClass">autoAppendParams</a>.
   */
  public native function getAutoAppendParams():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.JsonP-cfg-callbackKey" rel="Ext.data.proxy.JsonP-cfg-callbackKey" class="docClass">callbackKey</a>.
   */
  public native function getCallbackKey():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.JsonP-cfg-recordParam" rel="Ext.data.proxy.JsonP-cfg-recordParam" class="docClass">recordParam</a>.
   */
  public native function getRecordParam():String;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.JsonP-cfg-autoAppendParams" rel="Ext.data.proxy.JsonP-cfg-autoAppendParams" class="docClass">autoAppendParams</a>.
   * @param autoAppendParams The new value.
   */
  public native function setAutoAppendParams(autoAppendParams:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.JsonP-cfg-callbackKey" rel="Ext.data.proxy.JsonP-cfg-callbackKey" class="docClass">callbackKey</a>.
   * @param callbackKey The new value.
   */
  public native function setCallbackKey(callbackKey:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.JsonP-cfg-recordParam" rel="Ext.data.proxy.JsonP-cfg-recordParam" class="docClass">recordParam</a>.
   * @param recordParam The new value.
   */
  public native function setRecordParam(recordParam:String):void;
}
}