package ext.data.proxy {
import ext.data.operation.Operation;

[Uses("ext.data.proxy.ClientProxy")]
[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.Memory", require)]
/**
 * In-memory proxy. This proxy simply uses a local variable for data storage/retrieval, so its contents are lost on
 * every page refresh.
 *
 * <p>Usually this Proxy isn't used directly, serving instead as a helper to a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> where a reader
 * is required to load data. For example, say we have a Store for a User model and have some inline data we want to
 * load, but this data isn't in quite the right format: we can use a MemoryProxy with a JsonReader to read it into our
 * Store:</p>
 *
 * <pre><code>//this is the model we will be using in the store
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         {name: 'id',    type: 'int'},
 *         {name: 'name',  type: 'string'},
 *         {name: 'phone', type: 'string', mapping: 'phoneNumber'}
 *     ]
 * });
 *
 * //this data does not line up to our model fields - the phone field is called phoneNumber
 * var data = {
 *     users: [
 *         {
 *             id: 1,
 *             name: 'Ed Spencer',
 *             phoneNumber: '555 1234'
 *         },
 *         {
 *             id: 2,
 *             name: 'Abe Elias',
 *             phoneNumber: '666 1234'
 *         }
 *     ]
 * };
 *
 * //note how we set the 'root' in the reader to match the data structure above
 * var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     autoLoad: true,
 *     model: 'User',
 *     data : data,
 *     proxy: {
 *         type: 'memory',
 *         reader: {
 *             type: 'json',
 *             rootProperty: 'users'
 *         }
 *     }
 * });
 * </code></pre>
 */
public class MemoryProxy extends ext.data.proxy.ClientProxy {
  /**
   * @param config @inheritDoc
   */
  public function MemoryProxy(config:ext.data.proxy.MemoryProxy = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional data to pass to configured Reader.
   * @see #getData()
   * @see #setData()
   */
  public native function get data():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set data(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configure as <code>true</code> to enable this MemoryProxy to honour a read operation's <code>start</code> and <code>limit</code> options.
   *
   * <p>When <code>true</code>, read operations will be able to read <em>pages</em> of records from the data object.</p>
   * @default false
   * @see #getEnablePaging()
   * @see #setEnablePaging()
   */
  public native function get enablePaging():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set enablePaging(value:Boolean):void;

  /**
   * Currently this is a hard-coded method that simply commits any records and sets the operation to successful,
   * then calls the callback function, if provided. It is essentially mocking a server call in memory, but since
   * there is no real back end in this case there's not much else to do. This method can be easily overridden to
   * implement more complex logic if needed.
   * @param operation The Operation to perform
   */
  override public native function create(operation:ext.data.operation.Operation):void;

  /**
   * Currently this is a hard-coded method that simply commits any records and sets the operation to successful,
   * then calls the callback function, if provided. It is essentially mocking a server call in memory, but since
   * there is no real back end in this case there's not much else to do. This method can be easily overridden to
   * implement more complex logic if needed.
   * @param operation The Operation to perform
   */
  override public native function erase(operation:ext.data.operation.Operation):void;

  /**
   * Reads data from the configured <a href="#!/api/Ext.data.proxy.Memory-cfg-data" rel="Ext.data.proxy.Memory-cfg-data" class="docClass">data</a> object. Uses the Proxy's <a href="#!/api/Ext.data.proxy.Memory-cfg-reader" rel="Ext.data.proxy.Memory-cfg-reader" class="docClass">reader</a>, if present.
   * @param operation The read Operation
   */
  override public native function read(operation:ext.data.operation.Operation):void;

  /**
   * Currently this is a hard-coded method that simply commits any records and sets the operation to successful,
   * then calls the callback function, if provided. It is essentially mocking a server call in memory, but since
   * there is no real back end in this case there's not much else to do. This method can be easily overridden to
   * implement more complex logic if needed.
   * @param operation The Operation to perform
   */
  override public native function update(operation:ext.data.operation.Operation):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Memory-cfg-data" rel="Ext.data.proxy.Memory-cfg-data" class="docClass">data</a>.
   */
  public native function getData():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Memory-cfg-enablePaging" rel="Ext.data.proxy.Memory-cfg-enablePaging" class="docClass">enablePaging</a>.
   */
  public native function getEnablePaging():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Memory-cfg-data" rel="Ext.data.proxy.Memory-cfg-data" class="docClass">data</a>.
   * @param data The new value.
   */
  public native function setData(data:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Memory-cfg-enablePaging" rel="Ext.data.proxy.Memory-cfg-enablePaging" class="docClass">enablePaging</a>.
   * @param enablePaging The new value.
   */
  public native function setEnablePaging(enablePaging:Boolean):void;
}
}