package ext.data.proxy {
import ext.data.Request;

[Uses("ext.data.proxy.AjaxProxy")]
[Uses("ext.data.proxy.ServerProxy")]
[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.Rest", require)]
/**
 * The Rest proxy is a specialization of the <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">AjaxProxy</a> which simply maps the four actions
 * (create, read, update and destroy) to RESTful HTTP verbs. For example, let's set up a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>
 * with an inline Rest proxy
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name', 'email'],
 *
 *     proxy: {
 *         type: 'rest',
 *         url : '/users'
 *     }
 * });
 * </code></pre>
 *
 * <p>Now we can create a new User instance and save it via the Rest proxy. Doing this will cause the Proxy to send a POST
 * request to '/users':</p>
 *
 * <pre><code>var user = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('User', {name: 'Ed Spencer', email: 'ed&#64;sencha.com'});
 *
 * user.save(); //POST /users
 * </code></pre>
 *
 * <p>Let's expand this a little and provide a callback for the <a href="#!/api/Ext.data.Model-method-save" rel="Ext.data.Model-method-save" class="docClass">Ext.data.Model.save</a> call to update the Model once
 * it has been created. We'll assume the creation went successfully and that the server gave this user an ID of 123:</p>
 *
 * <pre><code>user.save({
 *     success: function(user) {
 *         user.set('name', 'Khan Noonien Singh');
 *
 *         user.save(); //PUT /users/123
 *     }
 * });
 * </code></pre>
 *
 * <p>Now that we're no longer creating a new Model instance, the request method is changed to an HTTP PUT, targeting the
 * relevant url for that user. Now let's delete this user, which will use the DELETE method:</p>
 *
 * <pre><code>    user.erase(); //DELETE /users/123
 * </code></pre>
 *
 * <p>Finally, when we perform a load of a Model or Store, Rest proxy will use the GET method:</p>
 *
 * <pre><code>//1. Load via Store
 *
 * //the Store automatically picks up the Proxy from the User model
 * var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User'
 * });
 *
 * store.load(); //GET /users
 *
 * //2. Load directly from the Model
 *
 * //GET /users/123
 * User.load(123, {
 *     success: function(user) {
 *         console.log(user.getId()); //outputs 123
 *     }
 * });
 * </code></pre>
 *
 * <h1>Url generation</h1>
 *
 * <p>The Rest proxy is able to automatically generate the urls above based on two configuration options - <a href="#!/api/Ext.data.proxy.Rest-cfg-appendId" rel="Ext.data.proxy.Rest-cfg-appendId" class="docClass">appendId</a> and
 * <a href="#!/api/Ext.data.proxy.Rest-cfg-format" rel="Ext.data.proxy.Rest-cfg-format" class="docClass">format</a>. If appendId is true (it is by default) then Rest proxy will automatically append the ID of the Model
 * instance in question to the configured url, resulting in the '/users/123' that we saw above.</p>
 *
 * <p>If the request is not for a specific Model instance (e.g. loading a Store), the url is not appended with an id.
 * The Rest proxy will automatically insert a '/' before the ID if one is not already present.</p>
 *
 * <pre><code>new <a href="#!/api/Ext.data.proxy.Rest" rel="Ext.data.proxy.Rest" class="docClass">Ext.data.proxy.Rest</a>({
 *     url: '/users',
 *     appendId: true //default
 * });
 *
 * // Collection url: /users
 * // Instance url  : /users/123
 * </code></pre>
 *
 * <p>The Rest proxy can also optionally append a format string to the end of any generated url:</p>
 *
 * <pre><code>new <a href="#!/api/Ext.data.proxy.Rest" rel="Ext.data.proxy.Rest" class="docClass">Ext.data.proxy.Rest</a>({
 *     url: '/users',
 *     format: 'json'
 * });
 *
 * // Collection url: /users.json
 * // Instance url  : /users/123.json
 * </code></pre>
 *
 * <p>If further customization is needed, simply implement the <a href="#!/api/Ext.data.proxy.Rest-method-buildUrl" rel="Ext.data.proxy.Rest-method-buildUrl" class="docClass">buildUrl</a> method and add your custom generated url
 * onto the <a href="#!/api/Ext.data.Request" rel="Ext.data.Request" class="docClass">Request</a> object that is passed to buildUrl. See <a href="source/Rest.html#Ext-data-proxy-Rest-method-buildUrl">Rest proxy's implementation</a> for
 * an example of how to achieve this.</p>
 *
 * <p>Note that Rest proxy inherits from <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">AjaxProxy</a>, which already injects all of the sorter,
 * filter, group and paging options into the generated url. See the <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">AjaxProxy docs</a> for more
 * details.</p>
 */
public class RestProxy extends ext.data.proxy.AjaxProxy {
  /**
   * @param config @inheritDoc
   */
  public function RestProxy(config:ext.data.proxy.RestProxy = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Mapping of action name to HTTP request method. These default to RESTful conventions for the 'create', 'read',
   * 'update' and 'destroy' actions (which map to 'POST', 'GET', 'PUT' and 'DELETE' respectively). This object
   * should not be changed except globally via <a href="#!/api/Ext-method-override" rel="Ext-method-override" class="docClass">Ext.override</a> - the <a href="#!/api/Ext.data.proxy.Rest-method-getMethod" rel="Ext.data.proxy.Rest-method-getMethod" class="docClass">getMethod</a> function
   * can be overridden instead.
   * @see #getActionMethods()
   * @see #setActionMethods()
   */
  override public native function get actionMethods():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set actionMethods(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to batch actions of a particular type when synchronizing the store. Defaults to false.
   * @default false
   * @see #getBatchActions()
   * @see #setBatchActions()
   */
  override public native function get batchActions():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set batchActions(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to automatically append the ID of a Model instance when performing a request based on that single instance.
   * See Rest proxy intro docs for more details. Defaults to true.
   * @default true
   * @see #getAppendId()
   * @see #setAppendId()
   */
  public native function get appendId():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set appendId(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional data format to send to the server when making any request (e.g. 'json'). See the Rest proxy intro docs
   * for full details. Defaults to undefined.
   * @see #getFormat()
   * @see #setFormat()
   */
  public native function get format():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set format(value:String):void;

  /**
   * Specialized version of buildUrl that incorporates the <a href="#!/api/Ext.data.proxy.Rest-cfg-appendId" rel="Ext.data.proxy.Rest-cfg-appendId" class="docClass">appendId</a> and <a href="#!/api/Ext.data.proxy.Rest-cfg-format" rel="Ext.data.proxy.Rest-cfg-format" class="docClass">format</a> options into the
   * generated url. Override this to provide further customizations, but remember to call the superclass buildUrl so
   * that additional parameters like the cache buster string are appended.
   * @param request The request object
   */
  override public native function buildUrl(request:ext.data.Request):String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Rest-cfg-appendId" rel="Ext.data.proxy.Rest-cfg-appendId" class="docClass">appendId</a>.
   */
  public native function getAppendId():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Rest-cfg-format" rel="Ext.data.proxy.Rest-cfg-format" class="docClass">format</a>.
   */
  public native function getFormat():String;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Rest-cfg-appendId" rel="Ext.data.proxy.Rest-cfg-appendId" class="docClass">appendId</a>.
   * @param appendId The new value.
   */
  public native function setAppendId(appendId:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Rest-cfg-format" rel="Ext.data.proxy.Rest-cfg-format" class="docClass">format</a>.
   * @param format The new value.
   */
  public native function setFormat(format:String):void;
}
}