package ext.data.reader {
import ext.data.ResultSet;

[Uses("ext.data.reader.DataReader")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.reader.Xml", require)]
/**
 * The XML Reader is used by a Proxy to read a server response that is sent back in XML format. This usually happens as
 * a result of loading a Store - for example we might create something like this:
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name', 'email']
 * });
 *
 * var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     model: 'User',
 *     proxy: {
 *         type: 'ajax',
 *         url : 'users.xml',
 *         reader: {
 *             type: 'xml',
 *             record: 'user',
 *             rootProperty: 'users'
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <p>The example above creates a 'User' model. Models are explained in the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> docs if you're not
 * already familiar with them.</p>
 *
 * <p>We created the simplest type of XML Reader possible by simply telling our <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>'s <a href="#!/api/Ext.data.proxy.Proxy" rel="Ext.data.proxy.Proxy" class="docClass">Proxy</a> that we want a XML Reader. The Store automatically passes the configured model to the
 * Store, so it is as if we passed this instead:</p>
 *
 * <pre><code>reader: {
 *     type : 'xml',
 *     model: 'User',
 *     record: 'user',
 *     rootProperty: 'users'
 * }
 * </code></pre>
 *
 * <p>The reader we set up is ready to read data from our server - at the moment it will accept a response like this:</p>
 *
 * <pre><code>&lt;?xml version="1.0" encoding="UTF-8"?&gt;
 * &lt;users&gt;
 *     &lt;user&gt;
 *         &lt;id&gt;1&lt;/id&gt;
 *         &lt;name&gt;Ed Spencer&lt;/name&gt;
 *         &lt;email&gt;ed&#64;sencha.com&lt;/email&gt;
 *     &lt;/user&gt;
 *     &lt;user&gt;
 *         &lt;id&gt;2&lt;/id&gt;
 *         &lt;name&gt;Abe Elias&lt;/name&gt;
 *         &lt;email&gt;abe&#64;sencha.com&lt;/email&gt;
 *     &lt;/user&gt;
 * &lt;/users&gt;
 * </code></pre>
 *
 * <p>First off there's <a href="#!/api/Ext.data.reader.Xml-cfg-rootProperty" rel="Ext.data.reader.Xml-cfg-rootProperty" class="docClass">rootProperty</a> option to define the root node <code>&lt;users&gt;</code> (there should be only one in a well-formed
 * XML document). Then the XML Reader uses the configured <a href="#!/api/Ext.data.reader.Xml-cfg-record" rel="Ext.data.reader.Xml-cfg-record" class="docClass">record</a> option to pull out the data for each record -
 * in this case we set record to 'user', so each <code>&lt;user&gt;</code> above will be converted into a User model.</p>
 *
 * <p>Note that XmlReader doesn't care whether your <a href="#!/api/Ext.data.reader.Xml-cfg-rootProperty" rel="Ext.data.reader.Xml-cfg-rootProperty" class="docClass">rootProperty</a> and <a href="#!/api/Ext.data.reader.Xml-cfg-record" rel="Ext.data.reader.Xml-cfg-record" class="docClass">record</a> elements are nested deep inside a
 * larger structure, so a response like this will still work:</p>
 *
 * <pre><code>&lt;?xml version="1.0" encoding="UTF-8"?&gt;
 * &lt;deeply&gt;
 *     &lt;nested&gt;
 *         &lt;xml&gt;
 *             &lt;users&gt;
 *                 &lt;user&gt;
 *                     &lt;id&gt;1&lt;/id&gt;
 *                     &lt;name&gt;Ed Spencer&lt;/name&gt;
 *                     &lt;email&gt;ed&#64;sencha.com&lt;/email&gt;
 *                 &lt;/user&gt;
 *                 &lt;user&gt;
 *                     &lt;id&gt;2&lt;/id&gt;
 *                     &lt;name&gt;Abe Elias&lt;/name&gt;
 *                     &lt;email&gt;abe&#64;sencha.com&lt;/email&gt;
 *                 &lt;/user&gt;
 *             &lt;/users&gt;
 *         &lt;/xml&gt;
 *     &lt;/nested&gt;
 * &lt;/deeply&gt;
 * </code></pre>
 *
 * <p>If this Reader is being used by a <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore</a> to read tree-structured data in which records
 * are nested as descendant nodes of other records, then this lenient behaviour must be overridden by using a more specific
 * child node selector as your <a href="#!/api/Ext.data.reader.Xml-cfg-record" rel="Ext.data.reader.Xml-cfg-record" class="docClass">record</a> selector which will not select all descendants, such as:</p>
 *
 * <p>   record: '>user'</p>
 *
 * <h1>Response metadata</h1>
 *
 * <p>The server can return additional data in its response, such as the <a href="#!/api/Ext.data.reader.Xml-cfg-totalProperty" rel="Ext.data.reader.Xml-cfg-totalProperty" class="docClass">total number of records</a> and
 * the <a href="#!/api/Ext.data.reader.Xml-cfg-successProperty" rel="Ext.data.reader.Xml-cfg-successProperty" class="docClass">success status of the response</a>. These are typically included in the XML response like
 * this:</p>
 *
 * <pre><code>&lt;?xml version="1.0" encoding="UTF-8"?&gt;
 * &lt;users&gt;
 *     &lt;total&gt;100&lt;/total&gt;
 *     &lt;success&gt;true&lt;/success&gt;
 *     &lt;user&gt;
 *         &lt;id&gt;1&lt;/id&gt;
 *         &lt;name&gt;Ed Spencer&lt;/name&gt;
 *         &lt;email&gt;ed&#64;sencha.com&lt;/email&gt;
 *     &lt;/user&gt;
 *     &lt;user&gt;
 *         &lt;id&gt;2&lt;/id&gt;
 *         &lt;name&gt;Abe Elias&lt;/name&gt;
 *         &lt;email&gt;abe&#64;sencha.com&lt;/email&gt;
 *     &lt;/user&gt;
 * &lt;/users&gt;
 * </code></pre>
 *
 * <p>If these properties are present in the XML response they can be parsed out by the XmlReader and used by the Store
 * that loaded it. We can set up the names of these properties by specifying a final pair of configuration options:</p>
 *
 * <pre><code>reader: {
 *     type: 'xml',
 *     rootProperty: 'users',
 *     totalProperty  : 'total',
 *     successProperty: 'success'
 * }
 * </code></pre>
 *
 * <p>These final options are not necessary to make the Reader work, but can be useful when the server needs to report an
 * error or if it needs to indicate that there is a lot of data available of which only a subset is currently being
 * returned.</p>
 *
 * <h1>Response format</h1>
 *
 * <p><strong>Note:</strong> in order for the browser to parse a returned XML document, the Content-Type header in the HTTP response
 * must be set to "text/xml" or "application/xml". This is very important - the XmlReader will not work correctly
 * otherwise.</p>
 */
public class XmlReader extends ext.data.reader.DataReader {
  /**
   * @param config @inheritDoc
   */
  public function XmlReader(config:ext.data.reader.XmlReader = null) {
    super();
  }

  [Deprecated(replacement = "rawData", since = "5.1.1")]
  /**
   * Copy of <a href="#!/api/Ext.data.reader.Xml-property-rawData" rel="Ext.data.reader.Xml-property-rawData" class="docClass">rawData</a>.
   */
  public native function get xmlData():Object;

  /**
   * @private
   */
  public native function set xmlData(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A namespace prefix that will be prepended to the field name when reading a
   * field from an XML node.  Take, for example, the following Model:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo', {
   *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *     fields: ['bar', 'baz']
   * });
   * </code></pre>
   *
   * <p>The reader would need to be configured with a namespace of 'n' in order to read XML
   * data in the following format:</p>
   *
   * <pre><code>&lt;foo&gt;
   *     &lt;n:bar&gt;bar&lt;/n:bar&gt;
   *     &lt;n:baz&gt;baz&lt;/n:baz&gt;
   * &lt;/foo&gt;
   * </code></pre>
   * @default ''
   * @see #getNamespace()
   * @see #setNamespace()
   */
  public native function get namespace():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set namespace(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The DomQuery path to the repeated element which contains record information.
   *
   * <p>By default, the elements which match the selector may be nested at any level
   * below the <a href="#!/api/Ext.data.reader.Xml-cfg-rootProperty" rel="Ext.data.reader.Xml-cfg-rootProperty" class="docClass">rootProperty</a></p>
   *
   * <p>If this Reader is being used by a <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore</a> to read tree-structured data,
   * then only first generation child nodes of the root element must be selected, so the record selector must be
   * specified with a more specific selector which will not select all descendants. For example:</p>
   *
   * <p>   record: '>node'</p>
   * @default ''
   * @see #getRecord()
   * @see #setRecord()
   */
  public native function get record():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set record(value:String):void;

  /**
   * Normalizes the data object.
   * @param data The data object
   * @return The documentElement property of the data object if present, or the same object if not.
   */
  override protected native function getData(data:Object):Object;

  /**
   * Parses an XML document and returns a ResultSet containing the model instances.
   * @param doc Parsed XML document
   * @param readOptions See <a href="#!/api/Ext.data.reader.Xml-method-read" rel="Ext.data.reader.Xml-method-read" class="docClass">read</a> for details.
   * @return The parsed result set
   */
  override public native function readRecords(doc:Object, readOptions:Object = null):ext.data.ResultSet;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Xml-cfg-namespace" rel="Ext.data.reader.Xml-cfg-namespace" class="docClass">namespace</a>.
   */
  public native function getNamespace():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.reader.Xml-cfg-record" rel="Ext.data.reader.Xml-cfg-record" class="docClass">record</a>.
   */
  public native function getRecord():String;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Xml-cfg-namespace" rel="Ext.data.reader.Xml-cfg-namespace" class="docClass">namespace</a>.
   * @param namespace The new value.
   */
  public native function setNamespace(namespace:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.reader.Xml-cfg-record" rel="Ext.data.reader.Xml-cfg-record" class="docClass">record</a>.
   * @param record The new value.
   */
  public native function setRecord(record:String):void;
}
}