package ext.dd {
import ext.event.Event;

[Uses("ext.dd.DragSource")]
[Uses("ext.dd.DDProxy")]
[Uses("ext.dd.DD")]
[Uses("ext.dd.DragDrop")]
[Uses("ext.Base")]
[Native("Ext.dd.DragZone", require)]
/**
 * This class provides a container DD instance that allows dragging of multiple child source nodes.
 *
 * <p>This class does not move the drag target nodes, but a proxy element which may contain any DOM structure you wish. The
 * DOM element to show in the proxy is provided by either a provided implementation of <a href="#!/api/Ext.dd.DragZone-method-getDragData" rel="Ext.dd.DragZone-method-getDragData" class="docClass">getDragData</a>, or by
 * registered draggables registered with <a href="#!/api/Ext.dd.Registry" rel="Ext.dd.Registry" class="docClass">Ext.dd.Registry</a></p>
 *
 * <p>If you wish to provide draggability for an arbitrary number of DOM nodes, each of which represent some application
 * object (For example nodes in a <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">DataView</a>) then use of this class is the most efficient way to
 * "activate" those nodes.</p>
 *
 * <p>By default, this class requires that draggable child nodes are registered with <a href="#!/api/Ext.dd.Registry" rel="Ext.dd.Registry" class="docClass">Ext.dd.Registry</a>. However a
 * simpler way to allow a DragZone to manage any number of draggable elements is to configure the DragZone with an
 * implementation of the <a href="#!/api/Ext.dd.DragZone-method-getDragData" rel="Ext.dd.DragZone-method-getDragData" class="docClass">getDragData</a> method which interrogates the passed mouse event to see if it has taken
 * place within an element, or class of elements. This is easily done by using the event's <a href="#!/api/Ext.event.Event-method-getTarget" rel="Ext.event.Event-method-getTarget" class="docClass">getTarget</a> method to identify a node based on a CSS selector. For example,
 * to make the nodes of a DataView draggable, use the following technique. Knowledge of the use of the DataView is
 * required:</p>
 *
 * <pre><code>myDataView.on('render', function(v) {
 *     myDataView.dragZone = new <a href="#!/api/Ext.dd.DragZone" rel="Ext.dd.DragZone" class="docClass">Ext.dd.DragZone</a>(v.getEl(), {
 *
 * //      On receipt of a mousedown event, see if it is within a DataView node.
 * //      Return a drag data object if so.
 *         getDragData: function(e) {
 *
 * //          Use the DataView's own itemSelector (a mandatory property) to
 * //          test if the mousedown is within one of the DataView's nodes.
 *             var sourceEl = e.getTarget(v.itemSelector, 10);
 *
 * //          If the mousedown is within a DataView node, clone the node to produce
 * //          a ddel element for use by the drag proxy. Also add application data
 * //          to the returned data object.
 *             if (sourceEl) {
 *                 d = sourceEl.cloneNode(true);
 *                 d.id = <a href="#!/api/Ext-method-id" rel="Ext-method-id" class="docClass">Ext.id</a>();
 *                 return {
 *                     ddel: d,
 *                     sourceEl: sourceEl,
 *                     repairXY: <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>(sourceEl).getXY(),
 *                     sourceStore: v.store,
 *                     draggedRecord: v.getRecord(sourceEl)
 *                 }
 *             }
 *         },
 *
 * //      Provide coordinates for the proxy to slide back to on failed drag.
 * //      This is the original XY coordinates of the draggable element captured
 * //      in the getDragData method.
 *         getRepairXY: function() {
 *             return this.dragData.repairXY;
 *         }
 *     });
 * });
 * </code></pre>
 *
 * <p>See the <a href="#!/api/Ext.dd.DropZone" rel="Ext.dd.DropZone" class="docClass">DropZone</a> documentation for details about building a DropZone which cooperates with
 * this DragZone.</p>
 */
public class DragZone extends ext.dd.DragSource {
  /**
   * Creates new DragZone.
   * @param el The container element or ID of it.
   */
  public function DragZone(el:*, config:ext.dd.DragZone = null) {
    super(undefined);
  }

  [ExtConfig]
  /**
   * True to register this container with the Scrollmanager for auto scrolling during drag operations.
   * A <a href="#!/api/Ext.dd.ScrollManager" rel="Ext.dd.ScrollManager" class="docClass">Ext.dd.ScrollManager</a> configuration may also be passed.
   */
  public native function get containerScroll():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set containerScroll(value:*):void;

  [ExtConfig]
  /**
   * An element to register with the ScrollManager if <a href="#!/api/Ext.dd.DragZone-cfg-containerScroll" rel="Ext.dd.DragZone-cfg-containerScroll" class="docClass">containerScroll</a>
   * is set. Defaults to the drag element.
   */
  public native function get scrollEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scrollEl(value:*):void;

  /**
   * Called when a mousedown occurs in this container. Looks in <a href="#!/api/Ext.dd.Registry" rel="Ext.dd.Registry" class="docClass">Ext.dd.Registry</a> for a valid target to drag
   * based on the mouse down. Override this method to provide your own lookup logic (e.g. finding a child by class
   * name). Make sure your returned object has a "ddel" attribute (with an HTML Element) for other functions to work.
   * @return The dragData
   */
  override public native function getDragData(e:ext.event.Event):Object;

  /**
   * Called before a repair of an invalid drop to get the XY to animate to. By default returns the XY of
   * this.dragData.ddel
   * @param e The mouse up event
   * @return The xy location (e.g. <code>[100, 200]</code>)
   */
  public native function getRepairXY(e:ext.event.Event):Array;

  /**
   * Called once drag threshold has been reached to initialize the proxy element. By default, it clones the
   * this.dragData.ddel
   * @param x The x position of the click on the dragged object
   * @param y The y position of the click on the dragged object
   * @return true to continue the drag, false to cancel
   */
  public native function onInitDrag(x:Number, y:Number):Boolean;
}
}