package ext.direct {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native]
/**
 * Fires after an event.
 * @eventType ext.direct.events.DirectManager_event_providerEvent.EVENT
 */
[Event(name = 'onEvent', type = 'ext.direct.events.DirectManager_event_providerEvent')]
/**
 * Fires after an event exception.
 * @eventType ext.direct.events.DirectManager_event_providerEvent.EXCEPTION
 */
[Event(name = 'onException', type = 'ext.direct.events.DirectManager_event_providerEvent')]
/**
 * Fired by <a href="#!/api/Ext.direct.Manager-method-loadProvider" rel="Ext.direct.Manager-method-loadProvider" class="docClass">loadProvider</a> after successfully loading RemotingProvider API
 * declaration and creating a new Provider instance.
 * @eventType ext.direct.events.DirectManager_url_providerEvent.PROVIDER_LOAD
 */
[Event(name = 'onProviderLoad', type = 'ext.direct.events.DirectManager_url_providerEvent')]
/**
 * Fired by <a href="#!/api/Ext.direct.Manager-method-loadProvider" rel="Ext.direct.Manager-method-loadProvider" class="docClass">loadProvider</a> when remoting API could not be loaded, or
 * Provider instance could not be created.
 * @eventType ext.direct.events.DirectManager_url_errorEvent.PROVIDER_LOAD_ERROR
 */
[Event(name = 'onProviderLoadError', type = 'ext.direct.events.DirectManager_url_errorEvent')]
/**
 * Ext Direct aims to streamline communication between the client and server
 * by providing a single interface that reduces the amount of common code
 * typically required to validate data and handle returned data packets
 * (reading data, error conditions, etc).
 *
 * <p>The <code>Ext.direct</code> namespace includes several classes for a closer integration
 * with the server-side. The Ext.data namespace also includes classes for working
 * with Ext.data.Stores which are backed by data from an Ext Direct method.</p>
 *
 * <h1>Specification</h1>
 *
 * <p>For additional information consult the <a href="http://sencha.com/products/extjs/extdirect">Ext Direct Specification</a>.</p>
 *
 * <h1>Providers</h1>
 *
 * <p>Ext Direct uses a provider architecture, where one or more providers are used
 * to transport data to and from the server. There are several providers that exist
 * in the core at the moment:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.direct.JsonProvider" rel="Ext.direct.JsonProvider" class="docClass">JsonProvider</a> for simple JSON operations</li>
 * <li><a href="#!/api/Ext.direct.PollingProvider" rel="Ext.direct.PollingProvider" class="docClass">PollingProvider</a> for repeated requests</li>
 * <li><a href="#!/api/Ext.direct.RemotingProvider" rel="Ext.direct.RemotingProvider" class="docClass">RemotingProvider</a> exposes server side to the client.</li>
 * </ul>
 *
 *
 * <p>A provider does not need to be invoked directly, providers are added via
 * <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">#addProvider</a>. RemotingProviders' API declarations
 * can also be loaded with <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">#loadProvider</a>, with
 * Provider instance created automatically after successful retrieval.</p>
 *
 * <h1>Router</h1>
 *
 * <p>Ext Direct RemotingProviders utilize a "router" on the server to direct
 * requests from the client to the appropriate server-side method. Because
 * the Ext Direct API is platform-agnostic, you could completely swap out
 * a Java based server solution and replace it with one that uses C#
 * without changing the client side JavaScript at all, or vice versa.</p>
 *
 * <h1>Server side events</h1>
 *
 * <p>Custom events from the server may be handled by the client by adding listeners, for example:</p>
 *
 * <pre><code>{"type":"event","name":"message","data":"Successfully polled at: 11:19:30 am"}
 *
 * // add a handler for a 'message' event sent by the server
 * <a href="#!/api/Ext.direct.Manager-method-on" rel="Ext.direct.Manager-method-on" class="docClass">Ext.direct.Manager.on</a>('message', function(e){
 *     out.append(String.format('&lt;p&gt;&lt;i&gt;{0}&lt;/i&gt;&lt;/p&gt;', e.data));
 *     out.el.scrollTo('t', 100000, true);
 * });
 * </code></pre>
 * <p>Type of singleton DirectManager.</p>
 * @see ext.direct.#DirectManager ext.direct.DirectManager
 */
public class SDirectManager extends ext.mixin.Observable {
  /**
   * @inheritDoc
   */
  public function SDirectManager(config:ext.mixin.Observable = null) {
    super();
  }

  /**
   * Exception types.
   * @default {TRANSPORT: 'xhr', PARSE: 'parse', DATA: 'data', LOGIN: 'login', SERVER: 'exception'}
   */
  public native function get exceptions():Object;

  /**
   * @private
   */
  public native function set exceptions(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Default variable name to use for Ext Direct API declaration.
   * @default "Ext.REMOTING_API"
   * @see #getVarName()
   * @see #setVarName()
   */
  public native function get varName():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set varName(value:String):void;

  /**
   * Adds an Ext Direct Provider and creates the proxy or stub methods to execute
   * server-side methods for RemotingProviders. If the provider is not already connected,
   * it will auto-connect.
   *
   * <pre><code> var pollProv = new <a href="#!/api/Ext.direct.PollingProvider" rel="Ext.direct.PollingProvider" class="docClass">Ext.direct.PollingProvider</a>({
   *      id: 'polling1',
   *      url: 'php/poll2.php'
   *  });
   *
   *  <a href="#!/api/Ext.direct.Manager-method-addProvider" rel="Ext.direct.Manager-method-addProvider" class="docClass">Ext.direct.Manager.addProvider</a>({
   *      id: 'remoting1',
   *      type: 'remoting',           // create a <a href="#!/api/Ext.direct.RemotingProvider" rel="Ext.direct.RemotingProvider" class="docClass">Ext.direct.RemotingProvider</a>
   *      url:  'php/router.php',     // url to connect to the Ext Direct server-side router.
   *      actions: {                  // each property within the actions object represents an Action
   *          TestAction: [{          // array of Methods within each server side Action
   *              name: 'doEcho',     // name of method
   *              len:  1
   *          }, {
   *              name: 'multiply',
   *              len:  1
   *          }, {
   *              name: 'doForm',
   *              formHandler: true   // handle form on server with <a href="#!/api/Ext.direct.Transaction" rel="Ext.direct.Transaction" class="docClass">Ext.direct.Transaction</a>
   *          }]
   *      },
   *      namespace: 'myApplication', // namespace to create the Remoting Provider in
   *  }, {
   *      id:   'polling2',
   *      type: 'polling',            // create an <a href="#!/api/Ext.direct.PollingProvider" rel="Ext.direct.PollingProvider" class="docClass">Ext.direct.PollingProvider</a>
   *      url:  'php/poll.php'
   *  },
   *  pollProv);                      // reference to previously created instance
   * </code></pre>
   * @param provider Accepts any number of Provider descriptions (an instance or config object for
   * a Provider). Each Provider description instructs Ext Direct how to create
   * client-side stub methods.
   */
  public native function addProvider(...provider):void;

  /**
   * Retrieves a <a href="#!/api/Ext.direct.Provider" rel="Ext.direct.Provider" class="docClass">provider</a> by the id specified when the
   * provider is added.
   * @param id The id of the provider, or the provider instance.
   */
  public native function getProvider(id:*):ext.direct.Provider;

  /**
   * Returns the value of <a href="#!/api/Ext.direct.Manager-cfg-varName" rel="Ext.direct.Manager-cfg-varName" class="docClass">varName</a>.
   */
  public native function getVarName():String;

  /**
   * Load Ext Direct Provider API declaration from the server and construct
   * a new Provider instance. The new Provider will then auto-connect and
   * create stub functions for the methods exposed by the server side. See
   * <a href="#!/api/Ext.direct.Manager-method-addProvider" rel="Ext.direct.Manager-method-addProvider" class="docClass">addProvider</a>.
   *
   * <pre><code> <a href="#!/api/Ext.direct.Manager-method-loadProvider" rel="Ext.direct.Manager-method-loadProvider" class="docClass">Ext.direct.Manager.loadProvider</a>({
   *      url: 'php/api.php',
   *      varName: 'MY_REMOTING_API' // defaults to 'Ext.REMOTING_API'
   *  });
   * </code></pre>
   * @param config Remoting API configuration.
   * @param {String} config.url URL to retrieve remoting API declaration from.
   * @param {String} config.varName Name of the variable that will hold
   * RemotingProvider configuration block, including its Actions.
   * @param callback Optional callback to execute when
   * Provider is created, or when an error has occured.
   * @param scope Optional scope to execute callback function in.
   * <p>For additional information see the [Ext Direct specification][1].</p>
   */
  public native function loadProvider(config:Object, callback:Function = null, scope:Object = null):void;

  /**
   * Parses a direct function. It may be passed in a string format, for example:
   * "MyApp.Person.read".
   * @param fn The direct function
   * @return The function to use in the direct call. Null if not found
   */
  public native function parseMethod(fn:*):Function;

  /**
   * Removes the provider.
   * @param provider The provider instance or the id of the provider.
   * @return The provider, null if not found.
   */
  public native function removeProvider(provider:*):ext.direct.Provider;

  /**
   * Sets the value of <a href="#!/api/Ext.direct.Manager-cfg-varName" rel="Ext.direct.Manager-cfg-varName" class="docClass">varName</a>.
   * @param varName The new value.
   */
  public native function setVarName(varName:String):void;
}
}