package ext.dom {

[Uses("ext.dom.Element")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.util.Positionable")]
[Native("Ext.dom.CompositeElementLite", require)]
/**
 * This class encapsulates a <em>collection</em> of DOM elements, providing methods to filter members, or to perform collective
 * actions upon the whole set.
 *
 * <p>Although they are not listed, this class supports all of the methods of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>. The
 * methods from these classes will be performed on all the elements in this collection.</p>
 *
 * <p>Example:</p>
 *
 * <pre><code>var els = <a href="#!/api/Ext-method-select" rel="Ext-method-select" class="docClass">Ext.select</a>("#some-el div.some-class");
 * // or select directly from an existing element
 * var el = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('some-el');
 * el.select('div.some-class');
 *
 * els.setWidth(100); // all elements become 100 width
 * els.hide(true); // all elements fade out and hide
 * // or
 * els.setWidth(100).hide(true);
 * </code></pre>
 */
public class CompositeElementLite extends ext.dom.Element {
  /**
   * @inheritDoc
   */
  public function CompositeElementLite(elements:Array) {
    super(null);
  }

  /**
   * The Array of DOM elements which this CompositeElement encapsulates.
   *
   * <p>This will not <em>usually</em> be accessed in developers' code, but developers wishing to augment the capabilities
   * of the CompositeElementLite class may use it when adding methods to the class.</p>
   *
   * <p>For example to add the <code>nextAll</code> method to the class to <strong>add</strong> all following siblings of selected elements,
   * the code would be</p>
   *
   * <pre><code><a href="#!/api/Ext-method-override" rel="Ext-method-override" class="docClass">Ext.override</a>(<a href="#!/api/Ext.dom.CompositeElementLite" rel="Ext.dom.CompositeElementLite" class="docClass">Ext.dom.CompositeElementLite</a>, {
   *     nextAll: function() {
   *         var elements = this.elements, i, l = elements.length, n, r = [], ri = -1;
   *
   *         // Loop through all elements in this Composite, accumulating
   *         // an Array of all siblings.
   *         for (i = 0; i &lt; l; i++) {
   *             for (n = elements[i].nextSibling; n; n = n.nextSibling) {
   *                 r[++ri] = n;
   *             }
   *         }
   *
   *         // Add all found siblings to this Composite
   *         return this.add(r);
   *     }
   * });
   * </code></pre>
   */
  public native function get elements():Array;

  /**
   * <code>true</code> in this class to identify an object as an instantiated CompositeElement, or subclass thereof.
   * @default true
   */
  public native function get isComposite():Boolean;

  /**
   * @private
   */
  public native function set isComposite(value:Boolean):void;

  [ExtConfig]
  public native function get bubbleEvents():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bubbleEvents(value:Object):void;

  /**
   * Returns <code>true</code> if this composite contains the passed element
   * @param el The id of an element, or an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>, or an HtmlElement to
   * find within the composite collection.
   */
  override public native function contains(el:*):Boolean;

  /**
   * Returns the first Element
   * @param selector Find the next sibling that matches the passed simple selector.
   * See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param returnDom <code>true</code> to return a raw DOM node instead of an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   */
  override public native function first(selector:String = null, returnDom:Boolean = false):*;

  /**
   * Returns the last Element
   * @param selector Find the previous sibling that matches the passed simple selector.
   * See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   * @param returnDom <code>true</code> to return a raw DOM node instead of an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default false
   */
  override public native function last(selector:String = null, returnDom:Boolean = false):*;

  /**
   * Adds elements to this Composite object.
   * @param els Either an Array of DOM elements to add, or another Composite
   * object who's elements should be added.
   * @param root The root element of the query or id of the root.
   * @return This Composite object.
   */
  public native function add(els:*, root:* = undefined):ext.dom.CompositeElementLite;

  /**
   * Removes all elements from this Composite.
   * @param removeDom True to also remove the elements from the document.
   */
  public native function clear(removeDom:Boolean = false):void;

  /**
   * Calls the passed function for each element in this composite.
   * @param fn The function to call.
   * @param {ext.dom.Element} fn.el The current Element in the iteration. <strong>This is the flyweight
   * (shared) <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instance, so if you require a a reference to the dom node, use el.dom.</strong>
   * @param {ext.dom.CompositeElementLite} fn.c This Composite object.
   * @param {Number} fn.index The zero-based index in the iteration.
   * @param scope The scope (this reference) in which the function is executed.
   * Defaults to the Element.
   * @return this
   */
  public native function each(fn:Function, scope:Object = null):ext.dom.CompositeElementLite;

  /**
   * Clears this Composite and adds the elements passed.
   * @param els Either an array of DOM elements, or another Composite from which
   * to fill this Composite.
   * @return this
   */
  public native function fill(els:*):ext.dom.CompositeElementLite;

  /**
   * Filters this composite to only elements that match the passed selector.
   * @param selector A string CSS selector or a comparison function. The comparison function will be
   * called with the following arguments:
   * @param {ext.dom.Element} selector.el The current DOM element.
   * @param {Number} selector.index The current index within the collection.
   * @return this
   */
  public native function filter(selector:*):ext.dom.CompositeElementLite;

  /**
   * Returns the number of elements in this Composite.
   */
  public native function getCount():Number;

  /**
   * Find the index of the passed element within the composite collection.
   * @param el The id of an element, or an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>, or an HtmlElement
   * to find within the composite collection.
   * @return The index of the passed <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> in the composite collection, or -1 if not found.
   */
  public native function indexOf(el:*):Number;

  /**
   * Returns a flyweight Element of the dom element object at the specified index.
   */
  public native function item(index:Number):ext.dom.Element;

  /**
   * Removes the specified element(s).
   * @param el The id of an element, the Element itself, the index of the
   * element in this composite or an array of any of those.
   * @param removeDom <code>true</code> to also remove the element from the document
   * @return this
   */
  public native function removeElement(el:*, removeDom:Boolean = false):ext.dom.CompositeElementLite;

  /**
   * Replaces the specified element with the passed element.
   * @param el The id of an element, the Element itself, the index of the
   * element in this composite to replace.
   * @param replacement The id of an element or the Element itself.
   * @param domReplace <code>true</code> to remove and replace the element in the document too.
   * @return this
   */
  public native function replaceElement(el:*, replacement:*, domReplace:Boolean = false):ext.dom.CompositeElementLite;

  /**
   * Gets a range nodes.
   * @param start The index of the first node in the range
   * @param end The index of the last node in the range
   * @return An array of nodes
   */
  public native function slice(start:Number = NaN, end:Number = NaN):Array;
}
}