package ext.dom {

[Uses("ext.dom.SDomHelper")]
[Uses("ext.Base")]
[Native("Ext.dom.Helper", require)]
/**
 * The DomHelper class provides a layer of abstraction from DOM and transparently supports creating elements via DOM or
 * using HTML fragments. It also has the ability to create HTML fragment templates from your DOM building code.
 *
 * <h2>DomHelper element specification object</h2>
 *
 * <p>A specification object is used when creating elements. Attributes of this object are assumed to be element
 * attributes, except for 4 special attributes:</p>
 *
 * <ul>
 * <li><strong>tag</strong>: The tag name of the element</li>
 * <li><strong>children (or cn)</strong>: An array of the same kind of element definition objects to be created and appended. These
 * can be nested as deep as you want.</li>
 * <li><strong>cls</strong>: The class attribute of the element. This will end up being either the "class" attribute on a HTML
 * fragment or className for a DOM node, depending on whether DomHelper is using fragments or DOM.</li>
 * <li><strong>html</strong>: The innerHTML for the element</li>
 * </ul>
 *
 *
 * <h2>Insertion methods</h2>
 *
 * <p>Commonly used insertion methods:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.dom.Helper-method-append" rel="Ext.dom.Helper-method-append" class="docClass">append</a></li>
 * <li><a href="#!/api/Ext.dom.Helper-method-insertBefore" rel="Ext.dom.Helper-method-insertBefore" class="docClass">insertBefore</a></li>
 * <li><a href="#!/api/Ext.dom.Helper-method-insertAfter" rel="Ext.dom.Helper-method-insertAfter" class="docClass">insertAfter</a></li>
 * <li><a href="#!/api/Ext.dom.Helper-method-overwrite" rel="Ext.dom.Helper-method-overwrite" class="docClass">overwrite</a></li>
 * <li><a href="#!/api/Ext.dom.Helper-method-insertHtml" rel="Ext.dom.Helper-method-insertHtml" class="docClass">insertHtml</a></li>
 * </ul>
 *
 *
 * <h2>Example</h2>
 *
 * <p>This is an example, where an unordered list with 3 children items is appended to an existing element with id
 * 'my-div':</p>
 *
 * <pre><code>var dh = <a href="#!/api/Ext.dom.Helper" rel="Ext.dom.Helper" class="docClass">Ext.DomHelper</a>; // create shorthand alias
 * // specification object
 * var spec = {
 *     id: 'my-ul',
 *     tag: 'ul',
 *     cls: 'my-list',
 *     // append children after creating
 *     children: [     // may also specify 'cn' instead of 'children'
 *         {tag: 'li', id: 'item0', html: 'List Item 0'},
 *         {tag: 'li', id: 'item1', html: 'List Item 1'},
 *         {tag: 'li', id: 'item2', html: 'List Item 2'}
 *     ]
 * };
 * var list = dh.append(
 *     'my-div', // the context element 'my-div' can either be the id or the actual node
 *     spec      // the specification object
 * );
 * </code></pre>
 *
 * <p>Element creation specification parameters in this class may also be passed as an Array of specification objects.
 * This can be used to insert multiple sibling nodes into an existing container very efficiently. For example, to add
 * more list items to the example above:</p>
 *
 * <pre><code>dh.append('my-ul', [
 *     {tag: 'li', id: 'item3', html: 'List Item 3'},
 *     {tag: 'li', id: 'item4', html: 'List Item 4'}
 * ]);
 * </code></pre>
 *
 * <h2>Templating</h2>
 *
 * <p>The real power is in the built-in templating. Instead of creating or appending any elements, createTemplate returns
 * a Template object which can be used over and over to insert new elements. Revisiting the example above, we could
 * utilize templating this time:</p>
 *
 * <pre><code>// create the node
 * var list = dh.append('my-div', {tag: 'ul', cls: 'my-list'});
 * // get template
 * var tpl = dh.createTemplate({tag: 'li', id: 'item{0}', html: 'List Item {0}'});
 *
 * for(var i = 0; i &lt; 5; i++){
 *     tpl.append(list, i); // use template to append to the actual node
 * }
 * </code></pre>
 *
 * <p>An example using a template:</p>
 *
 * <pre><code>var html = '"{0}" href="{1}" class="nav"&gt;{2}';
 *
 * var tpl = new <a href="#!/api/Ext.dom.Helper-method-createTemplate" rel="Ext.dom.Helper-method-createTemplate" class="docClass">Ext.DomHelper.createTemplate</a>(html);
 * tpl.append('blog-roll', ['link1', 'http://www.foxmulder.com/', "Fox's Site"]);
 * tpl.append('blog-roll', ['link2', 'http://www.danascully.org/', "Scully's Site"]);
 * </code></pre>
 *
 * <p>The same example using named parameters:</p>
 *
 * <pre><code>var html = '"{id}" href="{url}" class="nav"&gt;{text}';
 *
 * var tpl = new <a href="#!/api/Ext.dom.Helper-method-createTemplate" rel="Ext.dom.Helper-method-createTemplate" class="docClass">Ext.DomHelper.createTemplate</a>(html);
 * tpl.append('blog-roll', {
 *     id: 'link1',
 *     url: 'http://www.danascully.org/',
 *     text: "Scully's Site"
 * });
 * tpl.append('blog-roll', {
 *     id: 'link2',
 *     url: 'http://www.foxmulder.com/',
 *     text: "Fox's Site"
 * });
 * </code></pre>
 *
 * <h2>Compiling Templates</h2>
 *
 * <p>Templates are applied using regular expressions. The performance is great, but if you are adding a bunch of DOM
 * elements using the same template, you can increase performance even further by "compiling" the template. The way
 * "compile()" works is the template is parsed and broken up at the different variable points and a dynamic function is
 * created and eval'ed. The generated function performs string concatenation of these parts and the passed variables
 * instead of using regular expressions.</p>
 *
 * <pre><code>var html = '"{id}" href="{url}" class="nav"&gt;{text}';
 *
 * var tpl = new <a href="#!/api/Ext.dom.Helper-method-createTemplate" rel="Ext.dom.Helper-method-createTemplate" class="docClass">Ext.DomHelper.createTemplate</a>(html);
 * tpl.compile();
 *
 * // ... use template like normal
 * </code></pre>
 *
 * <h2>Performance Boost</h2>
 *
 * <p>DomHelper will transparently create HTML fragments when it can. Using HTML fragments instead of DOM can
 * significantly boost performance.</p>
 *
 * <p>Element creation specification parameters may also be strings which are used as innerHTML.</p>
 */
public const DomHelper:ext.dom.SDomHelper;
}