package ext.dom {
import ext.Base;
import js.HTMLElement;

[Uses("ext.Base")]
[Native]
/**
 * Provides high performance selector/xpath processing by compiling queries into reusable functions. New pseudo classes
 * and matchers can be plugged. It works on HTML and XML documents (if a content node is passed in).
 *
 * <p>DomQuery supports most of the <a href="http://www.w3.org/TR/2005/WD-css3-selectors-20051215/#selectors">CSS3 selectors spec</a>, along with some custom selectors and basic XPath.</p>
 *
 * <p>All selectors, attribute filters and pseudos below can be combined infinitely in any order. For example
 * <code>div.foo:nth-child(odd)[&#64;foo=bar].bar:first</code> would be a perfectly valid selector. Node filters are processed
 * in the order in which they appear, which allows you to optimize your queries for your document structure.</p>
 *
 * <h2>Simple Selectors</h2>
 *
 * <p>For performance reasons, some query methods accept selectors that are termed as <strong>simple selectors</strong>. A simple
 * selector is a selector that does not include contextual information about any parent/sibling elements.</p>
 *
 * <p>Some examples of valid simple selectors:</p>
 *
 * <pre><code>var simple = '.foo'; // Only asking for the class name on the element
 * var simple = 'div.bar'; // Only asking for the tag/class name on the element
 * var simple = '[href];' // Asking for an attribute on the element.
 * var simple = ':not(.foo)'; // Only asking for the non-matches against the class name
 * var simple = 'span:first-child'; // Doesn't require any contextual information about the parent node
 * </code></pre>
 *
 * <p>Simple examples of invalid simple selectors:</p>
 *
 * <pre><code>var notSimple = 'div.foo div.bar'; // Requires matching a parent node by class name
 * var notSimple = 'span + div'; //  Requires matching a sibling by tag name
 * </code></pre>
 *
 * <h2>Element Selectors:</h2>
 *
 * <ul>
 * <li><strong><code>*</code></strong> any element</li>
 * <li><strong><code>E</code></strong> an element with the tag E</li>
 * <li><strong><code>E F</code></strong> All descendent elements of E that have the tag F</li>
 * <li><strong><code>E &gt; F</code></strong> or <strong>E/F</strong> all direct children elements of E that have the tag F</li>
 * <li><strong><code>E + F</code></strong> all elements with the tag F that are immediately preceded by an element with the tag E</li>
 * <li><strong><code>E ~ F</code></strong> all elements with the tag F that are preceded by a sibling element with the tag E</li>
 * </ul>
 *
 *
 * <h2>Attribute Selectors:</h2>
 *
 * <p>The use of <code>&#64;</code> and quotes are optional. For example, <code>div[&#64;foo='bar']</code> is also a valid attribute selector.</p>
 *
 * <ul>
 * <li><strong><code>E[foo]</code></strong> has an attribute "foo"</li>
 * <li><strong><code>E[foo=bar]</code></strong> has an attribute "foo" that equals "bar"</li>
 * <li><strong><code>E[foo^=bar]</code></strong> has an attribute "foo" that starts with "bar"</li>
 * <li><strong><code>E[foo$=bar]</code></strong> has an attribute "foo" that ends with "bar"</li>
 * <li><strong><code>E[foo*=bar]</code></strong> has an attribute "foo" that contains the substring "bar"</li>
 * <li><strong><code>E[foo%=2]</code></strong> has an attribute "foo" that is evenly divisible by 2</li>
 * <li><strong><code>E[foo!=bar]</code></strong> attribute "foo" does not equal "bar"</li>
 * </ul>
 *
 *
 * <h2>Pseudo Classes:</h2>
 *
 * <ul>
 * <li><strong><code>E:first-child</code></strong> E is the first child of its parent</li>
 * <li><strong><code>E:last-child</code></strong> E is the last child of its parent</li>
 * <li><strong><code>E:nth-child(_n_)</code></strong> E is the _n_th child of its parent (1 based as per the spec)</li>
 * <li><strong><code>E:nth-child(odd)</code></strong> E is an odd child of its parent</li>
 * <li><strong><code>E:nth-child(even)</code></strong> E is an even child of its parent</li>
 * <li><strong><code>E:only-child</code></strong> E is the only child of its parent</li>
 * <li><strong><code>E:checked</code></strong> E is an element that is has a checked attribute that is true (e.g. a radio or checkbox)</li>
 * <li><strong><code>E:first</code></strong> the first E in the resultset</li>
 * <li><strong><code>E:last</code></strong> the last E in the resultset</li>
 * <li><strong><code>E:nth(_n_)</code></strong> the _n_th E in the resultset (1 based)</li>
 * <li><strong><code>E:odd</code></strong> shortcut for :nth-child(odd)</li>
 * <li><strong><code>E:even</code></strong> shortcut for :nth-child(even)</li>
 * <li><strong><code>E:contains(foo)</code></strong> E's innerHTML contains the substring "foo"</li>
 * <li><strong><code>E:nodeValue(foo)</code></strong> E contains a textNode with a nodeValue that equals "foo"</li>
 * <li><strong><code>E:not(S)</code></strong> an E element that does not match simple selector S</li>
 * <li><strong><code>E:has(S)</code></strong> an E element that has a descendent that matches simple selector S</li>
 * <li><strong><code>E:next(S)</code></strong> an E element whose next sibling matches simple selector S</li>
 * <li><strong><code>E:prev(S)</code></strong> an E element whose previous sibling matches simple selector S</li>
 * <li><strong><code>E:any(S1|S2|S2)</code></strong> an E element which matches any of the simple selectors S1, S2 or S3</li>
 * <li><strong><code>E:visible(true)</code></strong> an E element which is deeply visible according to <a href="#!/api/Ext.dom.Element-method-isVisible" rel="Ext.dom.Element-method-isVisible" class="docClass">Ext.dom.Element.isVisible</a></li>
 * </ul>
 *
 *
 * <h2>CSS Value Selectors:</h2>
 *
 * <ul>
 * <li><strong><code>E{display=none}</code></strong> css value "display" that equals "none"</li>
 * <li><strong><code>E{display^=none}</code></strong> css value "display" that starts with "none"</li>
 * <li><strong><code>E{display$=none}</code></strong> css value "display" that ends with "none"</li>
 * <li><strong><code>E{display*=none}</code></strong> css value "display" that contains the substring "none"</li>
 * <li><strong><code>E{display%=2}</code></strong> css value "display" that is evenly divisible by 2</li>
 * <li><strong><code>E{display!=none}</code></strong> css value "display" that does not equal "none"</li>
 * </ul>
 *
 *
 * <h2>XML Namespaces:</h2>
 *
 * <ul>
 * <li><strong><code>ns|E</code></strong> an element with tag E and namespace prefix ns</li>
 * </ul>
 * <p>Type of singleton DomQuery.</p>
 * @see ext.dom.#DomQuery ext.dom.DomQuery
 */
public class SDomQuery extends ext.Base {
  /**
   * Collection of matching regular expressions and code snippets.
   * Each capture group within <code>()</code> will be replace the <code>{}</code> in the select
   * statement as specified by their index.
   */
  public native function get matchers():Object;

  /**
   * @private
   */
  public native function set matchers(value:Object):void;

  /**
   * Collection of operator comparison functions.
   * The default operators are <code>=</code>, <code>!=</code>, <code>^=</code>, <code>$=</code>, <code>*=</code>, <code>%=</code>, <code>|=</code> and <code>~=</code>.
   *
   * <p>New operators can be added as long as the match the format <em>c</em><code>=</code> where <em>c</em>
   * is any character other than space, <code>&gt;</code>, or <code>&lt;</code>.</p>
   *
   * <p>Operator functions are passed the following parameters:</p>
   *
   * <ul>
   * <li><code>propValue</code> : The property value to test.</li>
   * <li><code>compareTo</code> : The value to compare to.</li>
   * </ul>
   */
  public native function get operators():Object;

  /**
   * @private
   */
  public native function set operators(value:Object):void;

  /**
   * Object hash of "pseudo class" filter functions which are used when filtering selections.
   * Each function is passed two parameters:
   *
   * <ul>
   * <li><p><strong>c</strong> : Array
   *   An Array of DOM elements to filter.</p></li>
   * <li><p><strong>v</strong> : String
   *   The argument (if any) supplied in the selector.</p></li>
   * </ul>
   *
   *
   * <p>A filter function returns an Array of DOM elements which conform to the pseudo class.
   * In addition to the provided pseudo classes listed above such as <code>first-child</code> and <code>nth-child</code>,
   * developers may add additional, custom psuedo class filters to select elements according to application-specific requirements.</p>
   *
   * <p>For example, to filter <code>a</code> elements to only return links to <strong>external</strong> resources:</p>
   *
   * <pre><code>Ext.DomQuery.pseudos.external = function(c, v) {
   *     var r = [], ri = -1;
   *     for(var i = 0, ci; ci = c[i]; i++) {
   *         // Include in result set only if it's a link to an external resource
   *         if (ci.hostname != location.hostname) {
   *             r[++ri] = ci;
   *         }
   *     }
   *     return r;
   * };
   * </code></pre>
   *
   * <p>Then external links could be gathered with the following statement:</p>
   *
   * <pre><code>var externalLinks = <a href="#!/api/Ext-method-select" rel="Ext-method-select" class="docClass">Ext.select</a>("a:external");
   * </code></pre>
   */
  public native function get pseudos():Object;

  /**
   * @private
   */
  public native function set pseudos(value:Object):void;

  /**
   * Compiles a selector/xpath query into a reusable function. The returned function
   * takes one parameter "root" (optional), which is the context node from where the query should start.
   * @param selector The selector/xpath query
   * @param type Either "select" or "simple" for a simple selector match
   * @default "select"
   */
  public native function compile(selector:String, type:String = "select"):Function;

  /**
   * Filters an array of elements to only include matches of a simple selector
   * @param el An array of elements to filter
   * @param selector The simple selector to test
   * @param nonMatches If true, it returns the elements that DON'T match the selector instead of the
   * ones that match
   * @return An Array of DOM elements which match the selector. If there are no matches, and empty
   * Array is returned.
   */
  public native function filter(el:Array, selector:String, nonMatches:Boolean):Array;

  /**
   * Get the text value for a node, optionally replacing null with the defaultValue.
   * @param node The node
   * @param defaultValue When specified, this is return as empty value.
   * @return The value
   */
  public native function getNodeValue(node:Object, defaultValue:String = null):String;

  /**
   * Returns true if the passed element(s) match the passed simple selector
   * @param el An element id, element or array of elements
   * @param selector The simple selector to test
   */
  public native function matches(el:*, selector:String):Boolean;

  /**
   * Selects an array of DOM nodes using JavaScript-only implementation.
   *
   * <p>Use <a href="#!/api/Ext.dom.Query-method-select" rel="Ext.dom.Query-method-select" class="docClass">select</a> to take advantage of browsers built-in support for CSS selectors.</p>
   * @param selector The selector/xpath query (can be a comma separated list of selectors)
   * @param root (Default document) The start of the query.
   * @default document
   * @return An Array of DOM elements which match the selector. If there are
   * no matches, and empty Array is returned.
   */
  public native function jsSelect(selector:String, root:* = undefined):Array;

  /**
   * Selects an array of DOM nodes by CSS/XPath selector.
   *
   * <p>Uses <a href="https://developer.mozilla.org/en/DOM/document.querySelectorAll">document.querySelectorAll</a> if browser supports that, otherwise falls back to
   * <a href="#!/api/Ext.dom.Query-method-jsSelect" rel="Ext.dom.Query-method-jsSelect" class="docClass">jsSelect</a> to do the work.</p>
   * @param path The selector/xpath query
   * @param root (Default document) The start of the query.
   * @default document
   * @param type Either "select" or "simple" for a simple selector match (only valid when
   * used when the call is deferred to the jsSelect method)
   * @default "select"
   * @param single Pass <code>true</code> to select only the first matching node using <code>document.querySelector</code> (where available)
   * @return An array of DOM elements (not a NodeList as returned by <code>querySelectorAll</code>).
   */
  public native function select(path:String, root:js.HTMLElement = null, type:String = "select", single:Boolean = false):Array;

  /**
   * Selects a single element.
   * @param selector The selector/xpath query
   * @param root (Default document) The start of the query.
   * @default document
   * @return The DOM element which matched the selector.
   */
  public native function selectNode(selector:String, root:js.HTMLElement = null):js.HTMLElement;

  /**
   * Selects the value of a node, parsing integers and floats.
   * Returns the defaultValue, or 0 if none is specified.
   * @param selector The selector/xpath query
   * @param root (Default document) The start of the query.
   * @default document
   * @param defaultValue When specified, this is return as empty value.
   */
  public native function selectNumber(selector:String, root:js.HTMLElement = null, defaultValue:Number = NaN):Number;

  /**
   * Selects the value of a node, optionally replacing null with the defaultValue.
   * @param selector The selector/xpath query
   * @param root (Default document) The start of the query.
   * @default document
   * @param defaultValue When specified, this is return as empty value.
   */
  public native function selectValue(selector:String, root:js.HTMLElement = null, defaultValue:String = null):String;
}
}