package ext.drag {
import ext.mixin.Observable;

[Uses("ext.drag.DragItem")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Identifiable")]
[Native("Ext.drag.Source", require)]
/**
 * Fires before drag starts on this source. Return <code>false</code> to cancel the drag.
 * @eventType ext.drag.events.DragSource_The_eventEvent.BEFORE_DRAG_START
 */
[Event(name = 'onBeforeDragStart', type = 'ext.drag.events.DragSource_The_eventEvent')]
/**
 * Fires when a drag is cancelled.
 * @eventType ext.drag.events.DragSource_The_eventEvent.DRAG_CANCEL
 */
[Event(name = 'onDragCancel', type = 'ext.drag.events.DragSource_The_eventEvent')]
/**
 * Fires when the drag ends on this source.
 * @eventType ext.drag.events.DragSource_The_eventEvent.DRAG_END
 */
[Event(name = 'onDragEnd', type = 'ext.drag.events.DragSource_The_eventEvent')]
/**
 * Fires continuously as this source is dragged.
 * @eventType ext.drag.events.DragSource_The_eventEvent.DRAG_MOVE
 */
[Event(name = 'onDragMove', type = 'ext.drag.events.DragSource_The_eventEvent')]
/**
 * Fires when the drag starts on this source.
 * @eventType ext.drag.events.DragSource_The_eventEvent.DRAG_START
 */
[Event(name = 'onDragStart', type = 'ext.drag.events.DragSource_The_eventEvent')]
/**
 * A wrapper around a DOM element that allows it to be dragged.
 *
 * <h2>Constraining</h2>
 *
 * <p>The <a href="#!/api/Ext.drag.Source-cfg-constrain" rel="Ext.drag.Source-cfg-constrain" class="docClass">constrain</a> config gives various options for limiting drag, for example:
 * - Vertical or horizontal only
 * - Minimum/maximum x/y values.
 * - Snap to grid
 * - Constrain to an element or region.</p>
 *
 * <p>See Ext.drag.Constrain for detailed options.</p>
 *
 * <pre><code> new <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">Ext.drag.Source</a>({
 *      element: dragEl,
 *      constrain: {
 *          // Drag only vertically in 30px increments
 *          vertical: true,
 *          snap: {
 *              y: 30
 *          }
 *      }
 *  });
 * </code></pre>
 *
 * <h2>Data</h2>
 *
 * <p>Data representing the underlying drag is driven by the <a href="#!/api/Ext.drag.Source-cfg-describe" rel="Ext.drag.Source-cfg-describe" class="docClass">describe</a> method. This method
 * is called once at the beginning of the drag. It should populate the info object with data using
 * the setData method. It accepts 2 arguments.</p>
 *
 * <ul>
 * <li>The <code>type</code> is used to indicate to <a href="#!/api/Ext.drag.Target" rel="Ext.drag.Target" class="docClass">targets</a> the type(s) of data being provided.
 * This allows the <a href="#!/api/Ext.drag.Target" rel="Ext.drag.Target" class="docClass">target</a> to decide whether it is able to interact with the source.
 * All types added are available in types.</li>
 * <li>The value can be a static value, or a function reference. In the latter case, the function is evaluated
 * when the data is requested.</li>
 * </ul>
 *
 *
 * <p>The <a href="#!/api/Ext.drag.Info-method-getData" rel="Ext.drag.Info-method-getData" class="docClass">Ext.drag.Info.getData</a> method may be called once the drop completes. The data for the relevant type
 * is retrieved. All values from this method return a <a href="#!/api/Ext.Promise" rel="Ext.Promise" class="docClass">Ext.Promise</a> to allow for consistency when dealing
 * with synchronous and asynchronous data.</p>
 *
 * <h2>Proxy</h2>
 *
 * <p>A <a href="#!/api/Ext.drag.Source-cfg-proxy" rel="Ext.drag.Source-cfg-proxy" class="docClass">proxy</a> is an element that follows the mouse cursor during a drag. This may be the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">element</a>,
 * a newly created element, or none at all (if the purpose is to just track the cursor).</p>
 *
 * <p>See <a href="#!/api/Ext.drag.proxy.None" rel="Ext.drag.proxy.None" class="docClass">for details</a>.</p>
 *
 * <pre><code> var data = [{
 *      id: 1,
 *      name: 'Adam'
 *  }, {
 *      id: 2,
 *      name: 'Barbara'
 *  }, {
 *      id: 3,
 *      name: 'Charlie'
 *  }];
 *
 *  var tpl = new <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>(
 *      '&lt;div class="container"&gt;',
 *          '&lt;tpl for="."&gt;',
 *              '&lt;div class="child" data-id="{id}"&gt;{name}&lt;/div&gt;',
 *          '&lt;/tpl&gt;',
 *      '&lt;/div&gt;'
 *  );
 *
 *  var el = tpl.append(<a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(), data);
 *
 *  new <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">Ext.drag.Source</a>({
 *      element: el,
 *      handle: '.child',
 *      proxy: {
 *          type: 'placeholder',
 *          getElement: function(info) {
 *              return <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().createChild({
 *                  cls: 'foo',
 *                  html: info.eventTarget.innerHTML
 *              });
 *          }
 *      }
 *  });
 * </code></pre>
 *
 * <h2>Handle</h2>
 *
 * <p>A <a href="#!/api/Ext.drag.Source-cfg-handle" rel="Ext.drag.Source-cfg-handle" class="docClass">handle</a> is a CSS selector that allows certain child elements of the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">element</a>
 * to begin a drag. This is useful in 2 case:
 * - Where only a certain part of the element should trigger a drag, but the whole element should move.
 * - When there are several repeated elements that may represent objects.</p>
 *
 * <p>In the example below, each child element becomes draggable and
 * the describe method is used to extract the id from the DOM element.</p>
 *
 * <pre><code> var data = [{
 *      id: 1,
 *      name: 'Adam'
 *  }, {
 *      id: 2,
 *      name: 'Barbara'
 *  }, {
 *      id: 3,
 *      name: 'Charlie'
 *  }];
 *
 *  var tpl = new <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>(
 *      '&lt;div class="container"&gt;',
 *          '&lt;tpl for="."&gt;',
 *              '&lt;div class="child" data-id="{id}"&gt;{name}&lt;/div&gt;',
 *          '&lt;/tpl&gt;',
 *      '&lt;/div&gt;'
 *  );
 *
 *  var el = tpl.append(<a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(), data);
 *
 *  new <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">Ext.drag.Source</a>({
 *      element: el,
 *      handle: '.child',
 *      describe: function(info) {
 *          info.setData('item', <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">Ext.fly</a>(info.eventTarget).getAttribute('data-id'));
 *      }
 *  });
 * </code></pre>
 */
public class DragSource extends ext.drag.DragItem {
  /**
   * @inheritDoc
   */
  public function DragSource(config:ext.mixin.Observable = null) {
    super();
  }

  /**
   * <code>true</code> if this source is currently dragging.
   * @default false
   */
  public native function get dragging():Boolean;

  /**
   * @private
   */
  public native function set dragging(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A css class to add to the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">element</a> while dragging is
   * active.
   * @see #getActiveCls()
   * @see #setActiveCls()
   */
  public native function get activeCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set activeCls(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to always begin a drag with longpress. <code>false</code> to
   * never drag with longpress. If a string (or strings) are passed, it should
   * correspond to the pointer event type that should initiate a a drag on
   * longpress. See {&#64;<a href="#!/api/Ext.event.Event-property-pointerType" rel="Ext.event.Event-property-pointerType" class="docClass">Ext.event.Event.pointerType</a>} for available types.
   */
  public native function get activeOnLongPress():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeOnLongPress(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Adds constraining behavior for this drag source. See <a href="#!/api/Ext.drag.Constraint" rel="Ext.drag.Constraint" class="docClass">Ext.drag.Constraint</a> for
   * configuration options. As a shortcut, a <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a>
   * or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> may be passed, which will be mapped to the
   * appropriate configuration on the constraint.
   * @see #getConstrain()
   * @see #setConstrain()
   */
  public native function get constrain():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set constrain(value:*):void;

  [ExtConfig("describe")]
  /**
   * See <a href="#!/api/Ext.drag.Source-method-describe" rel="Ext.drag.Source-method-describe" class="docClass">describe</a>.
   */
  public native function get describe_():Function;

  [ExtConfig("describe")]
  /**
   * @private
   */
  public native function set describe_(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A CSS selector to identify child elements of the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">element</a> that will cause
   * a drag to be activated. If this is not specified, the entire <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">element</a> will
   * be draggable.
   * @see #getHandle()
   * @see #setHandle()
   */
  public native function get handle():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set handle(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * proxy
   * The proxy to show while this element is dragging. This may be
   * the alias, a config, or instance of a proxy.
   *
   * <p>See <a href="#!/api/Ext.drag.proxy.None" rel="Ext.drag.proxy.None" class="docClass">None</a>, <a href="#!/api/Ext.drag.proxy.Original" rel="Ext.drag.proxy.Original" class="docClass">Original</a>,
   * Status.</p>
   * @default 'original'
   * @see #getProxy()
   * @see #setProxy()
   */
  public native function get proxy():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set proxy(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> (or an animation configuration) to animate the <a href="#!/api/Ext.drag.Source-cfg-proxy" rel="Ext.drag.Source-cfg-proxy" class="docClass">proxy</a> (which may be
   * the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">element</a>) back to the original position after drag.
   * @default false
   * @see #getRevert()
   * @see #setRevert()
   */
  public native function get revert():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set revert(value:*):void;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Source-cfg-activeCls" rel="Ext.drag.Source-cfg-activeCls" class="docClass">activeCls</a>.
   */
  public native function getActiveCls():String;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Source-cfg-constrain" rel="Ext.drag.Source-cfg-constrain" class="docClass">constrain</a>.
   */
  public native function getConstrain():*;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Source-cfg-handle" rel="Ext.drag.Source-cfg-handle" class="docClass">handle</a>.
   */
  public native function getHandle():String;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Source-cfg-proxy" rel="Ext.drag.Source-cfg-proxy" class="docClass">proxy</a>.
   */
  public native function getProxy():String;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Source-cfg-revert" rel="Ext.drag.Source-cfg-revert" class="docClass">revert</a>.
   */
  public native function getRevert():*;

  /**
   * Checks whether this source is actively dragging.
   * @return <code>true</code> if this source is dragging.
   */
  public native function isDragging():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Source-cfg-activeCls" rel="Ext.drag.Source-cfg-activeCls" class="docClass">activeCls</a>.
   * @param activeCls The new value.
   */
  public native function setActiveCls(activeCls:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Source-cfg-constrain" rel="Ext.drag.Source-cfg-constrain" class="docClass">constrain</a>.
   * @param constrain The new value.
   */
  public native function setConstrain(constrain:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Source-cfg-handle" rel="Ext.drag.Source-cfg-handle" class="docClass">handle</a>.
   * @param handle The new value.
   */
  public native function setHandle(handle:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Source-cfg-proxy" rel="Ext.drag.Source-cfg-proxy" class="docClass">proxy</a>.
   * @param proxy The new value.
   */
  public native function setProxy(proxy:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Source-cfg-revert" rel="Ext.drag.Source-cfg-revert" class="docClass">revert</a>.
   * @param revert The new value.
   */
  public native function setRevert(revert:*):void;

  /**
   * Called before a drag starts. Return <code>false</code> to veto the drag.
   * @param The drag info.
   * @return <code>false</code> to veto the drag.
   */
  protected native function beforeDragStart(The:Object):Boolean;

  /**
   * Sets up the underlying data that describes the drag. This method
   * is called once at the start of the drag operation.
   *
   * <p>Data should be set on the <a href="#!/api/Ext.drag.Info" rel="Ext.drag.Info" class="docClass">info</a> using the
   * <a href="#!/api/Ext.drag.Info-method-setData" rel="Ext.drag.Info-method-setData" class="docClass">setData</a> method. See
   * <a href="#!/api/Ext.drag.Info-method-setData" rel="Ext.drag.Info-method-setData" class="docClass">setData</a> for more information.</p>
   *
   * <p>This method should not be called by user code.</p>
   * @param info The drag info.
   */
  protected native function describe(info:ext.drag.DragInfo):void;

  /**
   * Called when a drag is cancelled.
   */
  protected native function onDragCancel():void;

  /**
   * Called when a drag ends.
   */
  protected native function onDragEnd():void;

  /**
   * Called for each move in a drag.
   */
  protected native function onDragMove():void;

  /**
   * Called when a drag starts.
   */
  protected native function onDragStart():void;
}
}