package ext.drag {
import ext.mixin.Observable;

[Uses("ext.drag.DragItem")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Identifiable")]
[Native("Ext.drag.Target", require)]
/**
 * Fires before a valid drop occurs. Return <code>false</code> to prevent the drop from
 * completing.
 * @eventType ext.drag.events.DragTarget_infoEvent.BEFORE_DROP
 */
[Event(name = 'onBeforeDrop', type = 'ext.drag.events.DragTarget_infoEvent')]
/**
 * Fires when a drag enters this target.
 * @eventType ext.drag.events.DragTarget_infoEvent.DRAG_ENTER
 */
[Event(name = 'onDragEnter', type = 'ext.drag.events.DragTarget_infoEvent')]
/**
 * Fires when a source leaves this target.
 * @eventType ext.drag.events.DragTarget_infoEvent.DRAG_LEAVE
 */
[Event(name = 'onDragLeave', type = 'ext.drag.events.DragTarget_infoEvent')]
/**
 * Fires when a drag moves while inside this target.
 * @eventType ext.drag.events.DragTarget_infoEvent.DRAG_MOVE
 */
[Event(name = 'onDragMove', type = 'ext.drag.events.DragTarget_infoEvent')]
/**
 * Fires when a valid drop occurs.
 * @eventType ext.drag.events.DragTarget_infoEvent.DROP
 */
[Event(name = 'onDrop', type = 'ext.drag.events.DragTarget_infoEvent')]
/**
 * A wrapper around a DOM element that allows it to receive drops.
 *
 * <h2>Validity of drag operations</h2>
 *
 * <p>There are certain conditions that govern whether a <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">source</a>
 * and a target can interact. By default (without configuration), all <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">sources</a>
 * and targets can interact with each other, the conditions are evaluated in this order:</p>
 *
 * <h3><a href="#!/api/Ext.drag.Target-method-isDisabled" rel="Ext.drag.Target-method-isDisabled" class="docClass">Disabled State</a></h3>
 *
 * <p>If the target is disabled, the <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">source</a>
 * cannot interact with it.</p>
 *
 * <h3><a href="#!/api/Ext.drag.Target-cfg-groups" rel="Ext.drag.Target-cfg-groups" class="docClass">Groups</a></h3>
 *
 * <p>Both the <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">source</a> and target can belong to multiple groups.
 * They may interact if:
 * - Neither has a group
 * - Both have one (or more) of the same group</p>
 *
 * <h3><a href="#!/api/Ext.drag.Target-cfg-accepts" rel="Ext.drag.Target-cfg-accepts" class="docClass">Accept</a></h3>
 *
 * <p>This method is called each time a <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">source</a> enters this
 * target. If the method returns <code>false</code>, the drag is not considered valid.</p>
 *
 * <h2>Asynchronous drop processing</h2>
 *
 * <p> When the drop completes, the <a href="#!/api/Ext.drag.Target-event-drop" rel="Ext.drag.Target-event-drop" class="docClass">drop</a> event will fire, however the underlying data
 * may not be ready to be consumed. By returning a <a href="#!/api/Ext.Promise" rel="Ext.Promise" class="docClass">Promise</a> from the data,
 * it allows either:
 * - The data to be fetched (either from a remote source or generated if expensive).
 * - Any validation to take place before the drop is finalized.</p>
 *
 * <p>Once the promise is <a href="#!/api/Ext.Promise-static-method-resolve" rel="Ext.Promise-static-method-resolve" class="docClass">resolved</a> or <a href="#!/api/Ext.Promise-static-method-resolve" rel="Ext.Promise-static-method-resolve" class="docClass">rejected</a>,
 * further processing can be completed.</p>
 *
 * <p>Validation example:</p>
 *
 * <pre><code> var confirmSource = new <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">Ext.drag.Source</a>({
 *      element: dragEl,
 *      describe: function(info) {
 *          // Provide the data up front
 *          info.setData('records', theRecords);
 *      }
 *  });
 *
 *  var confirmTarget = new <a href="#!/api/Ext.drag.Target" rel="Ext.drag.Target" class="docClass">Ext.drag.Target</a>({
 *      element: dropEl,
 *      listeners: {
 *          drop: function(target, info) {
 *              <a href="#!/api/Ext.MessageBox-method-confirm" rel="Ext.MessageBox-method-confirm" class="docClass">Ext.MessageBox.confirm</a>('Really', 'Are you sure?', function(btn) {
 *                  if (btn === 'yes') {
 *                      info.getData('records').then(function(data) {
 *                          // Process the data
 *                      });
 *                  }
 *              });
 *          }
 *      }
 *  });
 * </code></pre>
 *
 * <p>Remote data example:</p>
 *
 * <pre><code> var fetchSource = new <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">Ext.drag.Source</a>({
 *      element: dragEl,
 *      // The resulting drag data will be a binary blob
 *      // of image data, we don't want to fetch it up front, so
 *      // pass a callback to be executed when data is requested.
 *      describe: function(info) {
 *          info.setData('image', function() {
 *              return <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
 *                  url: 'data.json'
 *                  // some options
 *              }).then(function(result) {
 *                  var imageData;
 *                  // Do some post-processing
 *                  return imageData;
 *              }, function() {
 *                  return <a href="#!/api/Ext.Promise-static-method-reject" rel="Ext.Promise-static-method-reject" class="docClass">Ext.Promise.reject</a>('Something went wrong!');
 *              });
 *          });
 *      }
 *  });
 *
 *  var fetchTarget = new <a href="#!/api/Ext.drag.Target" rel="Ext.drag.Target" class="docClass">Ext.drag.Target</a>({
 *      element: dropEl,
 *      accepts: function(info) {
 *          return info.types.indexOf('image') &gt; -1;
 *      },
 *      listeners: {
 *          drop: function(target, info) {
 *              info.getData('image').then(function() {
 *                  // All good, show the image
 *              }, function() {
 *                  // Handle failure case
 *              });
 *          }
 *      }
 *  });
 * </code></pre>
 */
public class DragTarget extends ext.drag.DragItem {
  /**
   * @inheritDoc
   */
  public function DragTarget(config:ext.mixin.Observable = null) {
    super();
  }

  [ExtConfig("accepts")]
  /**
   * See <a href="#!/api/Ext.drag.Target-method-accepts" rel="Ext.drag.Target-method-accepts" class="docClass">accepts</a>.
   */
  public native function get accepts_():Function;

  [ExtConfig("accepts")]
  /**
   * @private
   */
  public native function set accepts_(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A class to add to the <a href="#!/api/Ext.drag.Target-cfg-element" rel="Ext.drag.Target-cfg-element" class="docClass">element</a> when an
   * invalid drag is over this target.
   * @see #getInvalidCls()
   * @see #setInvalidCls()
   */
  public native function get invalidCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set invalidCls(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Target-cfg-invalidCls" rel="Ext.drag.Target-cfg-invalidCls" class="docClass">invalidCls</a>.
   */
  public native function getInvalidCls():String;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Target-cfg-invalidCls" rel="Ext.drag.Target-cfg-invalidCls" class="docClass">invalidCls</a>.
   * @param invalidCls The new value.
   */
  public native function setInvalidCls(invalidCls:String):void;

  /**
   * Called each time a <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">source</a> enters this target.
   * Allows this target to indicate whether it will interact with
   * the given drag. Determined after <a href="#!/api/Ext.drag.Target-method-isDisabled" rel="Ext.drag.Target-method-isDisabled" class="docClass">isDisabled</a> and
   * <a href="#!/api/Ext.drag.Target-cfg-groups" rel="Ext.drag.Target-cfg-groups" class="docClass">groups</a> checks. If either of the aforementioned conditions
   * means the target is not valid, this will not be called.
   *
   * <p>Defaults to returning <code>true</code>.</p>
   * @param info The drag info.
   * @return <code>true</code> if the drag is valid for this target.
   */
  protected native function accepts(info:ext.drag.DragInfo):Boolean;

  /**
   * Called before a drag finishes on this target. Return <code>false</code> to veto
   * the drop.
   * @param info The drag info.
   * @return <code>false</code> to veto the drop.
   */
  protected native function beforeDrop(info:ext.drag.DragInfo):Boolean;

  /**
   * Called when a drag enters this target.
   * @param info The drag info.
   */
  protected native function onDragEnter(info:ext.drag.DragInfo):void;

  /**
   * Called when a source leaves this target.
   * @param info The drag info.
   */
  protected native function onDragLeave(info:ext.drag.DragInfo):void;

  /**
   * Called when a drag is moved while inside this target.
   * @param info The drag info.
   */
  protected native function onDragMove(info:ext.drag.DragInfo):void;

  /**
   * Called when a drag is dropped on this target.
   * @param info The drag info.
   */
  protected native function onDrop(info:ext.drag.DragInfo):void;
}
}