package ext.form {
import ext.container.Container;
import ext.data.Model;
import ext.data.reader.DataReader;
import ext.form.field.Field;
import ext.util.MixedCollection;
import ext.util.Observable;

[Uses("ext.util.Observable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.form.Basic", require)]
/**
 * Fires when an action is completed.
 * @eventType ext.form.events.BasicForm_actionEvent.ACTION_COMPLETE
 */
[Event(name = 'onActionComplete', type = 'ext.form.events.BasicForm_actionEvent')]
/**
 * Fires when an action fails.
 * @eventType ext.form.events.BasicForm_actionEvent.ACTION_FAILED
 */
[Event(name = 'onActionFailed', type = 'ext.form.events.BasicForm_actionEvent')]
/**
 * Fires before any action is performed. Return false to cancel the action.
 * @eventType ext.form.events.BasicForm_actionEvent.BEFORE_ACTION
 */
[Event(name = 'onBeforeAction', type = 'ext.form.events.BasicForm_actionEvent')]
/**
 * Fires when the dirty state of the entire form changes.
 * @eventType ext.form.events.BasicForm_dirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.form.events.BasicForm_dirtyEvent')]
/**
 * Fires when the validity of the entire form changes.
 * @eventType ext.form.events.BasicForm_validEvent.VALIDITY_CHANGE
 */
[Event(name = 'onValidityChange', type = 'ext.form.events.BasicForm_validEvent')]
/**
 * Provides input field management, validation, submission, and form loading services for the collection
 * of <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Field</a> instances within a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>. It is recommended
 * that you use a <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a> as the form container, as that has logic to automatically
 * hook up an instance of <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> (plus other conveniences related to field configuration.)
 *
 * <h2>Form Actions</h2>
 *
 * <p>The Basic class delegates the handling of form loads and submits to instances of <a href="#!/api/Ext.form.action.Action" rel="Ext.form.action.Action" class="docClass">Ext.form.action.Action</a>.
 * See the various Action implementations for specific details of each one's functionality, as well as the
 * documentation for <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">doAction</a> which details the configuration options that can be specified in
 * each action call.</p>
 *
 * <p>The default submit Action is <a href="#!/api/Ext.form.action.Submit" rel="Ext.form.action.Submit" class="docClass">Ext.form.action.Submit</a>, which uses an Ajax request to submit the
 * form's values to a configured URL. To enable normal browser submission of an Ext form, use the
 * <a href="#!/api/Ext.form.Basic-cfg-standardSubmit" rel="Ext.form.Basic-cfg-standardSubmit" class="docClass">standardSubmit</a> config option.</p>
 *
 * <h2>File uploads</h2>
 *
 * <p>File uploads are not performed using normal 'Ajax' techniques; see the description for
 * <a href="#!/api/Ext.form.Basic-method-hasUpload" rel="Ext.form.Basic-method-hasUpload" class="docClass">hasUpload</a> for details. If you're using file uploads you should read the method description.</p>
 *
 * <h2>Example usage:</h2>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Basic Form',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     bodyPadding: 5,
 *     width: 350,
 *
 *     // Any configuration items here will be automatically passed along to
 *     // the <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> instance when it gets created.
 *
 *     // The form will submit an AJAX request to this URL when submitted
 *     url: 'save-form.php',
 *
 *     items: [{
 *         xtype: 'textfield',
 *         fieldLabel: 'Field',
 *         name: 'theField'
 *     }],
 *
 *     buttons: [{
 *         text: 'Submit',
 *         handler: function() {
 *             // The getForm() method returns the <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> instance:
 *             var form = this.up('form').getForm();
 *             if (form.isValid()) {
 *                 // Submit the Ajax request and handle the response
 *                 form.submit({
 *                     success: function(form, action) {
 *                        <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Success', action.result.message);
 *                     },
 *                     failure: function(form, action) {
 *                         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Failed', action.result ? action.result.message : 'No response');
 *                     }
 *                 });
 *             }
 *         }
 *     }]
 * });
 * </code></pre>
 */
public class BasicForm extends ext.util.Observable {
  /**
   * Creates new form.
   * @param owner The component that is the container for the form, usually a <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>
   * @param config Configuration options. These are normally specified in the config to the
   * <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a> constructor, which passes them along to the BasicForm automatically.
   */
  public function BasicForm(owner:ext.container.Container, config:ext.form.BasicForm = null) {
    super();
  }

  /**
   * The container component to which this BasicForm is attached.
   */
  public native function get owner():ext.container.Container;

  /**
   * @private
   */
  public native function set owner(value:ext.container.Container):void;

  [ExtConfig]
  /**
   * If specified, load and submit actions will be handled with <a href="#!/api/Ext.form.action.DirectLoad" rel="Ext.form.action.DirectLoad" class="docClass">DirectLoad</a>
   * and <a href="#!/api/Ext.form.action.DirectSubmit" rel="Ext.form.action.DirectSubmit" class="docClass">DirectSubmit</a>.  Methods which have been imported by
   * <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a> can be specified here to load and submit forms. API methods may also be
   * specified as strings. See <a href="#!/api/Ext.data.proxy.Direct-cfg-directFn" rel="Ext.data.proxy.Direct-cfg-directFn" class="docClass">Ext.data.proxy.Direct.directFn</a>.  Such as the following:
   *
   * <pre><code>api: {
   *     load: App.ss.MyProfile.load,
   *     submit: App.ss.MyProfile.submit
   * }
   * </code></pre>
   *
   * <p>Load actions can use <a href="#!/api/Ext.form.Basic-cfg-paramOrder" rel="Ext.form.Basic-cfg-paramOrder" class="docClass">paramOrder</a> or <a href="#!/api/Ext.form.Basic-cfg-paramsAsHash" rel="Ext.form.Basic-cfg-paramsAsHash" class="docClass">paramsAsHash</a> to customize how the load method
   * is invoked.  Submit actions will always use a standard form submit. The <code>formHandler</code> configuration
   * (see Ext.direct.RemotingProvider#action) must be set on the associated server-side method which has
   * been imported by <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a>.</p>
   */
  public native function get api():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set api(value:Object):void;

  [ExtConfig]
  /**
   * Parameters to pass with all requests. e.g. baseParams: <code>{id: '123', foo: 'bar'}</code>.
   *
   * <p>Parameters are encoded as standard HTTP parameters using <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>.</p>
   */
  public native function get baseParams():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseParams(value:Object):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> (e.g. <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Ext.data.reader.Xml</a>) instance or
   * configuration to be used to read field error messages returned from 'submit' actions.
   * This is optional as there is built-in support for processing JSON responses.
   *
   * <p>The Records which provide messages for the invalid Fields must use the
   * Field name (or id) as the Record ID, and must contain a field called 'msg'
   * which contains the error message.</p>
   *
   * <p>The errorReader does not have to be a full-blown implementation of a
   * Reader. It simply needs to implement a <code>read(xhr)</code> function
   * which returns an Array of Records in an object with the following
   * structure:</p>
   *
   * <pre><code>{
   *     records: recordArray
   * }
   * </code></pre>
   */
  public native function get errorReader():ext.data.reader.DataReader;

  [ExtConfig]
  /**
   * @private
   */
  public native function set errorReader(value:*):void;

  [ExtConfig]
  /**
   * If set to true, the field values are sent as JSON in the request body.
   * All of the field values, plus any additional params configured via <a href="#!/api/Ext.form.Basic-cfg-baseParams" rel="Ext.form.Basic-cfg-baseParams" class="docClass">baseParams</a>
   * and/or the <code>options</code> to <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submit</a>, will be included in the values POSTed in the body of the request.
   */
  public native function get jsonSubmit():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set jsonSubmit(value:Boolean):void;

  [ExtConfig]
  /**
   * Optional metadata to pass with the actions when Ext Direct <a href="#!/api/Ext.form.Basic-cfg-api" rel="Ext.form.Basic-cfg-api" class="docClass">api</a> is used.
   * See <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a> for more information.
   */
  public native function get metadata():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set metadata(value:*):void;

  [ExtConfig]
  /**
   * The request method to use (GET or POST) for form actions if one isn't supplied in the action options.
   */
  public native function get method():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set method(value:String):void;

  [ExtConfig]
  /**
   * A list of params to be executed server side. Only used for the <a href="#!/api/Ext.form.Basic-cfg-api" rel="Ext.form.Basic-cfg-api" class="docClass">api</a> <code>load</code>
   * configuration.
   *
   * <p>Specify the params in the order in which they must be executed on the
   * server-side as either (1) an Array of String values, or (2) a String of params
   * delimited by either whitespace, comma, or pipe. For example,
   * any of the following would be acceptable:</p>
   *
   * <pre><code>paramOrder: ['param1','param2','param3']
   * paramOrder: 'param1 param2 param3'
   * paramOrder: 'param1,param2,param3'
   * paramOrder: 'param1|param2|param'
   * </code></pre>
   */
  public native function get paramOrder():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set paramOrder(value:*):void;

  [ExtConfig]
  /**
   * Only used for the <a href="#!/api/Ext.form.Basic-cfg-api" rel="Ext.form.Basic-cfg-api" class="docClass">api</a> <code>load</code> configuration. If true, parameters will be sent as a
   * single hash collection of named arguments. Providing a <a href="#!/api/Ext.form.Basic-cfg-paramOrder" rel="Ext.form.Basic-cfg-paramOrder" class="docClass">paramOrder</a> nullifies this
   * configuration.
   * @default false
   */
  public native function get paramsAsHash():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set paramsAsHash(value:Boolean):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> (e.g. <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Ext.data.reader.Xml</a>) instance or
   * configuration to be used to read data when executing 'load' actions. This
   * is optional as there is built-in support for processing JSON responses.
   */
  public native function get reader():ext.data.reader.DataReader;

  [ExtConfig]
  /**
   * @private
   */
  public native function set reader(value:*):void;

  [ExtConfig]
  /**
   * If set to true, a standard HTML form submit is used instead of a XHR (Ajax) style form submission.
   * All of the field values, plus any additional params configured via <a href="#!/api/Ext.form.Basic-cfg-baseParams" rel="Ext.form.Basic-cfg-baseParams" class="docClass">baseParams</a>
   * and/or the <code>options</code> to <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submit</a>, will be included in the values submitted in the form.
   */
  public native function get standardSubmit():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set standardSubmit(value:Boolean):void;

  [ExtConfig]
  /**
   * Timeout for form actions in seconds.
   * @default 30
   */
  public native function get timeout():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set timeout(value:Number):void;

  [ExtConfig]
  /**
   * If set to true, <a href="#!/api/Ext.form.Basic-method-reset" rel="Ext.form.Basic-method-reset" class="docClass">reset</a>() resets to the last loaded or
   * <a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a>() data instead of when the form was first
   * created.
   * @default false
   */
  public native function get trackResetOnLoad():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set trackResetOnLoad(value:Boolean):void;

  [ExtConfig]
  /**
   * The URL to use for form actions if one isn't supplied in the
   * <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">doAction</a> options.
   */
  public native function get url():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set url(value:String):void;

  [ExtConfig]
  /**
   * By default wait messages are displayed with <a href="#!/api/Ext.MessageBox-method-wait" rel="Ext.MessageBox-method-wait" class="docClass">Ext.MessageBox.wait</a>. You can target a specific
   * element by passing it or its id or mask the form itself by passing in true.
   */
  public native function get waitMsgTarget():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set waitMsgTarget(value:*):void;

  [ExtConfig]
  /**
   * The default title to show for the waiting message box
   * @default 'Please Wait...'
   */
  public native function get waitTitle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set waitTitle(value:String):void;

  /**
   * Destroys this object.
   */
  override public native function destroy(...params):void;

  /**
   * Calls <a href="#!/api/Ext-method-applyIf" rel="Ext-method-applyIf" class="docClass">Ext.applyIf</a> for all field in this form with the passed object.
   * @param obj The object to be applied
   * @return this
   */
  public native function applyIfToFields(obj:Object):ext.form.BasicForm;

  /**
   * Calls <a href="#!/api/Ext-method-apply" rel="Ext-method-apply" class="docClass">Ext.apply</a> for all fields in this form with the passed object.
   * @param obj The object to be applied
   * @return this
   */
  public native function applyToFields(obj:Object):ext.form.BasicForm;

  /**
   * Check whether the dirty state of the entire form has changed since it was last checked, and
   * if so fire the <a href="#!/api/Ext.form.Basic-event-dirtychange" rel="Ext.form.Basic-event-dirtychange" class="docClass">dirtychange</a> event. This is automatically invoked
   * when an individual field's <code>dirty</code> state changes.
   */
  public native function checkDirty():void;

  /**
   * Check whether the validity of the entire form has changed since it was last checked, and
   * if so fire the <a href="#!/api/Ext.form.Basic-event-validitychange" rel="Ext.form.Basic-event-validitychange" class="docClass">validitychange</a> event. This is automatically invoked
   * when an individual field's validity changes.
   */
  public native function checkValidity():void;

  /**
   * Clears all invalid field messages in this form.
   * @return this
   */
  public native function clearInvalid():ext.form.BasicForm;

  /**
   * Performs a predefined action (an implementation of <a href="#!/api/Ext.form.action.Action" rel="Ext.form.action.Action" class="docClass">Ext.form.action.Action</a>) to perform application-
   * specific processing.
   * @param action The name of the predefined action type, or instance of <a href="#!/api/Ext.form.action.Action" rel="Ext.form.action.Action" class="docClass">Ext.form.action.Action</a> to perform.
   * @param options The options to pass to the <a href="#!/api/Ext.form.action.Action" rel="Ext.form.action.Action" class="docClass">Ext.form.action.Action</a> that will get created,
   * if the action argument is a String.
   * <p>All of the config options listed below are supported by both the <a href="#!/api/Ext.form.action.Submit" rel="Ext.form.action.Submit" class="docClass">submit</a> and
   * <a href="#!/api/Ext.form.action.Load" rel="Ext.form.action.Load" class="docClass">load</a> actions unless otherwise noted (custom actions could also accept other
   * config options):</p>
   * @param {String} options.url The url for the action (defaults to the form's <a href="#!/api/Ext.form.Basic-cfg-url" rel="Ext.form.Basic-cfg-url" class="docClass">url</a>.)
   * @param {String} options.method The form method to use (defaults to the form's method, or POST if not defined)
   * @param options.params The params to pass (defaults to the form's baseParams, or none if not defined)
   * <p>Parameters are encoded as standard HTTP parameters using <a href="#!/api/Ext-method-urlEncode" rel="Ext-method-urlEncode" class="docClass">Ext.Object.toQueryString</a>.</p>
   * @param {Object} options.headers Request headers to set for the action.
   * @param {Function} options.success The callback that will be invoked after a successful response (see top of <a href="#!/api/Ext.form.action.Submit" rel="Ext.form.action.Submit" class="docClass">submit</a>
   * and <a href="#!/api/Ext.form.action.Load" rel="Ext.form.action.Load" class="docClass">load</a> for a description of what constitutes a successful response).
   * @param {ext.form.BasicForm} options.success.form The form that requested the action.
   * @param {ext.form.action.Action} options.success.action The Action object which performed the operation.
   * The action object contains these properties of interest:
   * <ul>
   * <li><a href="#!/api/Ext.form.action.Action-property-response" rel="Ext.form.action.Action-property-response" class="docClass">response</a></li>
   * <li><a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a> - interrogate for custom post-processing</li>
   * <li><a href="#!/api/Ext.form.action.Action-property-type" rel="Ext.form.action.Action-property-type" class="docClass">type</a></li>
   * </ul>
   * @param {Function} options.failure The callback that will be invoked after a failed transaction attempt.
   * @param {ext.form.BasicForm} options.failure.form The form that requested the action.
   * @param {ext.form.action.Action} options.failure.action The Action object which performed the operation.
   * The action object contains these properties of interest:
   * <ul>
   * <li><a href="#!/api/Ext.form.action.Action-property-failureType" rel="Ext.form.action.Action-property-failureType" class="docClass">failureType</a></li>
   * <li><a href="#!/api/Ext.form.action.Action-property-response" rel="Ext.form.action.Action-property-response" class="docClass">response</a></li>
   * <li><a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a> - interrogate for custom post-processing</li>
   * <li><a href="#!/api/Ext.form.action.Action-property-type" rel="Ext.form.action.Action-property-type" class="docClass">type</a></li>
   * </ul>
   * @param {Object} options.scope The scope in which to call the callback functions (The this reference for the callback functions).
   * @param {Boolean} options.clientValidation Submit Action only. Determines whether a Form's fields are validated in a final call to <a href="#!/api/Ext.form.Basic-method-isValid" rel="Ext.form.Basic-method-isValid" class="docClass">isValid</a> prior to submission. Set to false to prevent this. If undefined, pre-submission
   * field validation is performed.
   * @return this
   */
  public native function doAction(action:*, options:Object = null):ext.form.BasicForm;

  /**
   * Find a specific <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> in this form by id or name.
   * @param id The value to search for (specify either a <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a> or
   * <a href="#!/api/Ext.form.field.Field-method-getName" rel="Ext.form.field.Field-method-getName" class="docClass">name or hiddenName</a>).
   * @return The first matching field, or <code>null</code> if none was found.
   */
  public native function findField(id:String):ext.form.field.Field;

  /**
   * Retrieves the fields in the form as a set of key/value pairs, using their
   * <a href="#!/api/Ext.form.field.Field-method-getModelData" rel="Ext.form.field.Field-method-getModelData" class="docClass">getModelData()</a> method to collect the values.
   * If multiple fields return values under the same name those values will be combined into an Array.
   * This is similar to <a href="#!/api/Ext.form.Basic-method-getValues" rel="Ext.form.Basic-method-getValues" class="docClass">getValues</a> except that this method collects type-specific data values
   * (e.g. Date objects for date fields) while getValues returns only String values for submission.
   * @param dirtyOnly If true, only fields that are dirty will be included in the result.
   * @default false
   */
  public native function getFieldValues(dirtyOnly:Boolean = false):Object;

  /**
   * Return all the <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> components in the owner container.
   * @return Collection of the Field objects
   */
  public native function getFields():ext.util.MixedCollection;

  /**
   * Returns the last <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> instance that was loaded via <a href="#!/api/Ext.form.Basic-method-loadRecord" rel="Ext.form.Basic-method-loadRecord" class="docClass">loadRecord</a>
   * @return The record
   */
  public native function getRecord():ext.data.Model;

  /**
   * Retrieves the fields in the form as a set of key/value pairs, using their
   * <a href="#!/api/Ext.form.field.Field-method-getSubmitData" rel="Ext.form.field.Field-method-getSubmitData" class="docClass">getSubmitData()</a> method to collect the values.
   * If multiple fields return values under the same name those values will be combined into an Array.
   * This is similar to <a href="#!/api/Ext.form.Basic-method-getFieldValues" rel="Ext.form.Basic-method-getFieldValues" class="docClass">getFieldValues</a> except that this method
   * collects only String values for submission, while getFieldValues collects type-specific data
   * values (e.g. Date objects for date fields.)
   * @param asString If true, will return the key/value collection as a single
   * URL-encoded param string.
   * @default false
   * @param dirtyOnly If true, only fields that are dirty will be included in the result.
   * @default false
   * @param includeEmptyText If true, the configured emptyText of empty fields will be used.
   * @default false
   * @param useDataValues If true, the <a href="#!/api/Ext.form.field.Field-method-getModelData" rel="Ext.form.field.Field-method-getModelData" class="docClass">getModelData</a>
   * method is used to retrieve values from fields, otherwise the <a href="#!/api/Ext.form.field.Field-method-getSubmitData" rel="Ext.form.field.Field-method-getSubmitData" class="docClass">getSubmitData</a>
   * method is used.
   * @default false
   */
  public native function getValues(asString:Boolean = false, dirtyOnly:Boolean = false, includeEmptyText:Boolean = false, useDataValues:Boolean = false):*;

  /**
   * Returns true if the form contains any invalid fields. No fields will be marked as invalid
   * as a result of calling this; to trigger marking of fields use <a href="#!/api/Ext.form.Basic-method-isValid" rel="Ext.form.Basic-method-isValid" class="docClass">isValid</a> instead.
   */
  public native function hasInvalidField():void;

  /**
   * Returns <code>true</code> if the form contains a file upload field. This is used to determine the method for submitting the
   * form: File uploads are not performed using normal 'Ajax' techniques, that is they are <strong>not</strong> performed using
   * XMLHttpRequests. Instead a hidden <code>&lt;form&gt;</code> element containing all the fields is created temporarily and submitted
   * with its <a href="http://www.w3.org/TR/REC-html40/present/frames.html#adef-target">target</a> set to refer to a dynamically generated, hidden <code>&lt;iframe&gt;</code> which is inserted into the document
   * but removed after the return data has been gathered.
   *
   * <p>The server response is parsed by the browser to create the document for the IFRAME. If the server is using JSON
   * to send the return object, then the <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17">Content-Type</a> header should be set to "text/plain" in order to tell the
   * browser to insert the text unchanged into a '&lt;pre>' element in the document body from which it can be retrieved.</p>
   *
   * <p>If the <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17">Content-Type</a> header is sent as the default, "text/html", then characters which are significant to an HTML
   * parser must be sent as HTML entities, so encode <code>"&lt;"</code> as <code>"&amp;lt;"</code>, <code>"&amp;"</code> as <code>"&amp;amp;"</code> etc.</p>
   *
   * <p>The response text is retrieved from the document, and a fake XMLHttpRequest object is created containing a
   * responseText property in order to conform to the requirements of event handlers and callbacks.</p>
   *
   * <p>Be aware that file upload packets are sent with the content type <a href="http://www.faqs.org/rfcs/rfc2388.html">multipart/form</a> and some server technologies
   * (notably JEE) may require some custom processing in order to retrieve parameter names and parameter values from
   * the packet content.</p>
   */
  public native function hasUpload():Boolean;

  /**
   * Returns <code>true</code> if any fields in this form have changed from their original values.
   *
   * <p>Note that if this BasicForm was configured with <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> then the Fields' <em>original values</em> are updated when the values are
   * loaded by <a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a> or <a href="#!/api/Ext.form.Basic-method-loadRecord" rel="Ext.form.Basic-method-loadRecord" class="docClass">loadRecord</a>. This means
   * that:</p>
   *
   * <ul>
   * <li><a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>: <code>false</code> -> Will return <code>true</code> after calling this method.</li>
   * <li><a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>: <code>true</code> -> Will return <code>false</code> after calling this method.</li>
   * </ul>
   */
  public native function isDirty():Boolean;

  /**
   * Returns true if client-side validation on the form is successful. Any invalid fields will be
   * marked as invalid. If you only want to determine overall form validity without marking anything,
   * use <a href="#!/api/Ext.form.Basic-method-hasInvalidField" rel="Ext.form.Basic-method-hasInvalidField" class="docClass">hasInvalidField</a> instead.
   */
  public native function isValid():Boolean;

  /**
   * Shortcut to <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">do</a> a <a href="#!/api/Ext.form.action.Load" rel="Ext.form.action.Load" class="docClass">load action</a>.
   * @param options The options to pass to the action (see <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">doAction</a> for details)
   * @return this
   */
  public native function load(options:Object):ext.form.BasicForm;

  /**
   * Loads an <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> into this form by calling <a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a> with the
   * <a href="#!/api/Ext.data.Model-method-getData" rel="Ext.data.Model-method-getData" class="docClass">record data</a>. The fields in the model are mapped to
   * fields in the form by matching either the <a href="#!/api/Ext.form.field.Base-cfg-name" rel="Ext.form.field.Base-cfg-name" class="docClass">Ext.form.field.Base.name</a> or <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">Ext.Component.itemId</a>.<br/>
   * See also <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>.
   * @param record The record to load
   * @return this
   */
  public native function loadRecord(record:ext.data.Model):ext.form.BasicForm;

  /**
   * This method allows you to mark one or more fields in a form as invalid along with
   * one or more invalid messages per field.
   *
   * <pre><code>var formPanel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
   *     title: 'Contact Info',
   *     width: 300,
   *     bodyPadding: 10,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     items: [{
   *         xtype: 'textfield',
   *         name: 'name',
   *         id: 'nameId',
   *         fieldLabel: 'Name'
   *     }, {
   *         xtype: 'textfield',
   *         name: 'email',
   *         id: 'emailId',
   *         fieldLabel: 'Email Address'
   *     }],
   *     bbar: [{
   *         text: 'Mark both fields invalid',
   *         handler: function() {
   *             formPanel.getForm().markInvalid([{
   *                 field: 'name',
   *                 message: 'Name invalid message'
   *             }, {
   *                 field: 'email',
   *                 message: ['First invalid message', 'Second message']
   *             }]);
   *         }
   *     }]
   * });
   * </code></pre>
   *
   * <p><strong>Note</strong>: this method does not cause the Field's
   * <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">Ext.form.field.Field.validate</a> or <a href="#!/api/Ext.form.field.Base-method-isValid" rel="Ext.form.field.Base-method-isValid" class="docClass">Ext.form.field.Base.isValid</a>
   * methods to return <code>false</code> if the value does <em>pass</em> validation.  So simply marking
   * a Field as invalid will not prevent submission of forms submitted with the
   * <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> option set.</p>
   *
   * <p>For additional information on how the fields are marked invalid see field's
   * <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> method.</p>
   * @param errors The errors param may be in one of two forms: Object[] or Object
   * <ul>
   * <li><strong>Array:</strong> An array of Objects with the following keys:
   * <ul>
   * <li><em>field</em> (String): The <a href="#!/api/Ext.form.field.Base-cfg-name" rel="Ext.form.field.Base-cfg-name" class="docClass">name</a> or
   * <a href="#!/api/Ext.form.field.Base-cfg-id" rel="Ext.form.field.Base-cfg-id" class="docClass">id</a> of the form field to receive the error message</li>
   * <li><em>message</em> (String/String[]): The error message or an array
   * of messages</li>
   * </ul>
   * </li>
   * </ul>
   * <p>Example Array syntax:</p>
   * <pre><code>form.markInvalid([{
   *     field: 'email', // the field name
   *     message: 'Error message'
   * }]);
   * </code></pre>
   * <ul>
   * <li><strong>Object:</strong> An Object hash with key/value pairs where the key is the field name
   * or field ID and the value is the message or array of messages to display.</li>
   * </ul>
   * <p>Example Object syntax:</p>
   * <pre><code>form.markInvalid({
   *     name: 'Err. message',
   *     emailId: ['Error1', 'Error 2']
   * });
   * </code></pre>
   * @return basicForm The <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> instance
   */
  public native function markInvalid(errors:*):ext.form.BasicForm;

  /**
   * Resets all fields in this form. By default, any record bound by <a href="#!/api/Ext.form.Basic-method-loadRecord" rel="Ext.form.Basic-method-loadRecord" class="docClass">loadRecord</a>
   * will be retained.
   * @param resetRecord True to unbind any record set
   * by <a href="#!/api/Ext.form.Basic-method-loadRecord" rel="Ext.form.Basic-method-loadRecord" class="docClass">loadRecord</a>
   * @default false
   * @return this
   */
  public native function reset(resetRecord:Boolean = false):ext.form.BasicForm;

  /**
   * Set values for fields in this form in bulk.
   * @param values Either an array in the form:
   * <pre><code>[{id:'clientName', value:'Fred. Olsen Lines'},
   *  {id:'portOfLoading', value:'FXT'},
   *  {id:'portOfDischarge', value:'OSL'} ]
   * </code></pre>
   * <p>or an object hash of the form:</p>
   * <pre><code>{
   *     clientName: 'Fred. Olsen Lines',
   *     portOfLoading: 'FXT',
   *     portOfDischarge: 'OSL'
   * }
   * </code></pre>
   * @return this
   */
  public native function setValues(values:*):ext.form.BasicForm;

  /**
   * Shortcut to <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">do</a> a <a href="#!/api/Ext.form.action.Submit" rel="Ext.form.action.Submit" class="docClass">submit action</a>. This will use the
   * <a href="#!/api/Ext.form.action.Submit" rel="Ext.form.action.Submit" class="docClass">AJAX submit action</a> by default. If the <a href="#!/api/Ext.form.Basic-cfg-standardSubmit" rel="Ext.form.Basic-cfg-standardSubmit" class="docClass">standardSubmit</a> config
   * is enabled it will use a standard form element to submit, or if the <a href="#!/api/Ext.form.Basic-cfg-api" rel="Ext.form.Basic-cfg-api" class="docClass">api</a> config is present
   * it will use the <a href="#!/api/Ext.form.action.DirectLoad" rel="Ext.form.action.DirectLoad" class="docClass">Ext.direct.Direct submit action</a>.
   *
   * <p>The following code:</p>
   *
   * <pre><code>myFormPanel.getForm().submit({
   *     clientValidation: true,
   *     url: 'updateConsignment.php',
   *     params: {
   *         newStatus: 'delivered'
   *     },
   *     success: function(form, action) {
   *        <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Success', action.result.msg);
   *     },
   *     failure: function(form, action) {
   *         switch (action.failureType) {
   *             case <a href="#!/api/Ext.form.action.Action-static-property-CLIENT_INVALID" rel="Ext.form.action.Action-static-property-CLIENT_INVALID" class="docClass">Ext.form.action.Action.CLIENT_INVALID</a>:
   *                 <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Failure', 'Form fields may not be submitted with invalid values');
   *                 break;
   *             case <a href="#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE" rel="Ext.form.action.Action-static-property-CONNECT_FAILURE" class="docClass">Ext.form.action.Action.CONNECT_FAILURE</a>:
   *                 <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Failure', 'Ajax communication failed');
   *                 break;
   *             case <a href="#!/api/Ext.form.action.Action-static-property-SERVER_INVALID" rel="Ext.form.action.Action-static-property-SERVER_INVALID" class="docClass">Ext.form.action.Action.SERVER_INVALID</a>:
   *                <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Failure', action.result.msg);
   *        }
   *     }
   * });
   * </code></pre>
   *
   * <p>would process the following server response for a successful submission:</p>
   *
   * <pre><code>{
   *     "success":true, // note this is Boolean, not string
   *     "msg":"Consignment updated"
   * }
   * </code></pre>
   *
   * <p>and the following server response for a failed submission:</p>
   *
   * <pre><code>{
   *     "success":false, // note this is Boolean, not string
   *     "msg":"You do not have permission to perform this operation"
   * }
   * </code></pre>
   * @param options The options to pass to the action (see <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">doAction</a> for details).
   * @return this
   */
  public native function submit(options:Object):ext.form.BasicForm;

  /**
   * Persists the values in this form into the passed <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> object in a beginEdit/endEdit block.
   * If the record is not specified, it will attempt to update (if it exists) the record provided to loadRecord.
   * @param record The record to edit
   * @return this
   */
  public native function updateRecord(record:ext.data.Model = null):ext.form.BasicForm;
}
}