package ext.form.action {
import ext.Base;
import ext.form.BasicForm;

[Uses("ext.Base")]
[Native("Ext.form.action.Action", require)]
/**
 * The subclasses of this class provide actions to perform upon <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a>s.
 *
 * <p>Instances of this class are only created by a <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a> when the Form needs to perform an action
 * such as submit or load. The Configuration options listed for this class are set through the Form's action methods:
 * <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submit</a>, <a href="#!/api/Ext.form.Basic-method-load" rel="Ext.form.Basic-method-load" class="docClass">load</a> and <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">doAction</a></p>
 *
 * <p>The instance of Action which performed the action is passed to the success and failure callbacks of the Form's action
 * methods (<a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submit</a>, <a href="#!/api/Ext.form.Basic-method-load" rel="Ext.form.Basic-method-load" class="docClass">load</a> and
 * <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">doAction</a>), and to the <a href="#!/api/Ext.form.Basic-event-actioncomplete" rel="Ext.form.Basic-event-actioncomplete" class="docClass">actioncomplete</a> and
 * <a href="#!/api/Ext.form.Basic-event-actionfailed" rel="Ext.form.Basic-event-actionfailed" class="docClass">actionfailed</a> event handlers.</p>
 */
public class Action extends ext.Base {
  /**
   * Failure type returned when client side validation of the Form fails thus aborting a submit action. Client
   * side validation is performed unless <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> is explicitly set to
   * false.
   * @default 'client'
   */
  public static native function get CLIENT_INVALID():String;

  /**
   * Failure type returned when a communication error happens when attempting to send a request to the remote
   * server. The <a href="#!/api/Ext.form.action.Action-property-response" rel="Ext.form.action.Action-property-response" class="docClass">response</a> may be examined to provide further information.
   * @default 'connect'
   */
  public static native function get CONNECT_FAILURE():String;

  /**
   * Failure type returned when the response's <code>success</code> property is set to false, or no field values are returned
   * in the response's data property.
   * @default 'load'
   */
  public static native function get LOAD_FAILURE():String;

  /**
   * Failure type returned when server side processing fails and the <a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a>'s <code>success</code> property is set to
   * false.
   *
   * <p>In the case of a form submission, field-specific error messages may be returned in the <a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a>'s
   * errors property.</p>
   * @default 'server'
   */
  public static native function get SERVER_INVALID():String;

  /**
   * Creates new Action.
   * @param config Config object.
   */
  public function Action(config:ext.form.action.Action = null) {
    super();
  }

  /**
   * The type of failure detected will be one of these:
   * <a href="#!/api/Ext.form.action.Action-static-property-CLIENT_INVALID" rel="Ext.form.action.Action-static-property-CLIENT_INVALID" class="docClass">CLIENT_INVALID</a>, <a href="#!/api/Ext.form.action.Action-static-property-SERVER_INVALID" rel="Ext.form.action.Action-static-property-SERVER_INVALID" class="docClass">SERVER_INVALID</a>, <a href="#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE" rel="Ext.form.action.Action-static-property-CONNECT_FAILURE" class="docClass">CONNECT_FAILURE</a>, or <a href="#!/api/Ext.form.action.Action-static-property-LOAD_FAILURE" rel="Ext.form.action.Action-static-property-LOAD_FAILURE" class="docClass">LOAD_FAILURE</a>.
   *
   * <p>Usage:</p>
   *
   * <pre><code>var fp = new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
   * ...
   * buttons: [{
   *     text: 'Save',
   *     formBind: true,
   *     handler: function(){
   *         if(fp.getForm().isValid()){
   *             fp.getForm().submit({
   *                 url: 'form-submit.php',
   *                 waitMsg: 'Submitting your data...',
   *                 success: function(form, action){
   *                     // server responded with success = true
   *                     var result = action.result;
   *                 },
   *                 failure: function(form, action){
   *                     if (action.<a href="#!/api/Ext.form.action.Action-property-failureType" rel="Ext.form.action.Action-property-failureType" class="docClass">failureType</a> === <a href="#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE" rel="Ext.form.action.Action-static-property-CONNECT_FAILURE" class="docClass">Ext.form.action.Action.CONNECT_FAILURE</a>) {
   *                         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Error',
   *                             'Status:'+action.response.status+': '+
   *                             action.response.statusText);
   *                     }
   *                     if (action.failureType === <a href="#!/api/Ext.form.action.Action-static-property-SERVER_INVALID" rel="Ext.form.action.Action-static-property-SERVER_INVALID" class="docClass">Ext.form.action.Action.SERVER_INVALID</a>){
   *                         // server responded with success = false
   *                         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Invalid', action.result.errormsg);
   *                     }
   *                 }
   *             });
   *         }
   *     }
   * },{
   *     text: 'Reset',
   *     handler: function(){
   *         fp.getForm().reset();
   *     }
   * }]
   * </code></pre>
   */
  public native function get failureType():String;

  /**
   * @private
   */
  public native function set failureType(value:String):void;

  /**
   * The raw XMLHttpRequest object used to perform the action.
   */
  public native function get response():Object;

  /**
   * @private
   */
  public native function set response(value:Object):void;

  /**
   * The decoded response object containing a boolean <code>success</code> property and other, action-specific properties.
   */
  public native function get result():Object;

  /**
   * @private
   */
  public native function set result(value:Object):void;

  /**
   * The type of action this Action instance performs. Currently only "submit" and "load" are supported.
   */
  public native function get type():String;

  /**
   * @private
   */
  public native function set type(value:String):void;

  [ExtConfig]
  /**
   * The function to call when a failure packet was received, or when an error
   * occurred in the Ajax communication.
   * <ul>
   *   <li><code>form</code> : ext.form.BasicForm
   * The form that requested the action
   *   </li>
   *   <li><code>action</code> : ext.form.action.Action
   * The Action class. If an Ajax error
   * occurred, the failure type will be in <a href="#!/api/Ext.form.action.Action-property-failureType" rel="Ext.form.action.Action-property-failureType" class="docClass">failureType</a>. The <a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a>
   * property of this object may be examined to perform custom post-processing.
   * <p>&#64;controllable</p>
   *   </li>
   * </ul>
   */
  public native function get failure():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set failure(value:*):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">BasicForm</a> instance that is invoking this Action. Required.
   */
  public native function get form():ext.form.BasicForm;

  [ExtConfig]
  /**
   * @private
   */
  public native function set form(value:ext.form.BasicForm):void;

  [ExtConfig]
  /**
   * Extra headers to be sent in the AJAX request for submit and load actions.
   * See <a href="#!/api/Ext.data.proxy.Ajax-cfg-headers" rel="Ext.data.proxy.Ajax-cfg-headers" class="docClass">Ext.data.proxy.Ajax.headers</a>.
   *
   * <p><strong>Note:</strong> Headers are not sent during file upload.</p>
   */
  public native function get headers():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set headers(value:Object):void;

  [ExtConfig]
  /**
   * The HTTP method to use to access the requested URL.
   * Defaults to the <a href="#!/api/Ext.form.Basic-cfg-method" rel="Ext.form.Basic-cfg-method" class="docClass">BasicForm's method</a>, or 'POST' if not specified.
   */
  public native function get method():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set method(value:String):void;

  [ExtConfig]
  /**
   * Extra parameter values to pass. These are added to the Form's <a href="#!/api/Ext.form.Basic-cfg-baseParams" rel="Ext.form.Basic-cfg-baseParams" class="docClass">Ext.form.Basic.baseParams</a> and passed to the
   * specified URL along with the Form's input fields.
   *
   * <p>Parameters are encoded as standard HTTP parameters using <a href="#!/api/Ext-method-urlEncode" rel="Ext-method-urlEncode" class="docClass">Ext.Object.toQueryString</a>.</p>
   */
  public native function get params():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set params(value:*):void;

  [ExtConfig]
  /**
   * When set to <strong>true</strong>, causes the Form to be <a href="#!/api/Ext.form.Basic-method-reset" rel="Ext.form.Basic-method-reset" class="docClass">reset</a> on Action success. If specified,
   * this happens before the <a href="#!/api/Ext.form.action.Action-cfg-success" rel="Ext.form.action.Action-cfg-success" class="docClass">success</a> callback is called and before the Form's
   * <a href="#!/api/Ext.form.Basic-event-actioncomplete" rel="Ext.form.Basic-event-actioncomplete" class="docClass">actioncomplete</a> event fires.
   */
  public native function get reset():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set reset(value:Boolean):void;

  [ExtConfig]
  /**
   * The scope in which to call the configured <a href="#!/api/Ext.form.action.Action-cfg-success" rel="Ext.form.action.Action-cfg-success" class="docClass">success</a> and <a href="#!/api/Ext.form.action.Action-cfg-failure" rel="Ext.form.action.Action-cfg-failure" class="docClass">failure</a> callback functions
   * (the <code>this</code> reference for the callback functions).
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * If set to true, the emptyText value will be sent with the form when it is submitted.
   * @default true
   */
  public native function get submitEmptyText():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitEmptyText(value:Boolean):void;

  [ExtConfig]
  /**
   * The function to call when a valid success return packet is received.
   * <ul>
   *   <li><code>form</code> : ext.form.BasicForm
   * The form that requested the action
   *   </li>
   *   <li><code>action</code> : ext.form.action.Action
   * The Action class. The <a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a> property of this object may
   * be examined to perform custom post-processing.
   * <p>&#64;controllable</p>
   *   </li>
   * </ul>
   */
  public native function get success():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set success(value:*):void;

  [ExtConfig]
  /**
   * The number of seconds to wait for a server response before failing with the <a href="#!/api/Ext.form.action.Action-property-failureType" rel="Ext.form.action.Action-property-failureType" class="docClass">failureType</a> as
   * <a href="#!/api/Ext.form.action.Action-static-property-CONNECT_FAILURE" rel="Ext.form.action.Action-static-property-CONNECT_FAILURE" class="docClass">CONNECT_FAILURE</a>. If not specified, defaults to the configured
   * <a href="#!/api/Ext.form.Basic-cfg-timeout" rel="Ext.form.Basic-cfg-timeout" class="docClass">timeout</a> of the <a href="#!/api/Ext.form.action.Action-cfg-form" rel="Ext.form.action.Action-cfg-form" class="docClass">form</a>.
   */
  public native function get timeout():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set timeout(value:Number):void;

  [ExtConfig]
  /**
   * The URL that the Action is to invoke. Will default to the <a href="#!/api/Ext.form.Basic-cfg-url" rel="Ext.form.Basic-cfg-url" class="docClass">url</a> configured on the
   * <a href="#!/api/Ext.form.action.Action-cfg-form" rel="Ext.form.action.Action-cfg-form" class="docClass">form</a>.
   */
  public native function get url():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set url(value:String):void;

  [ExtConfig]
  /**
   * The message to be displayed by a call to <a href="#!/api/Ext.window.MessageBox-method-wait" rel="Ext.window.MessageBox-method-wait" class="docClass">Ext.window.MessageBox.wait</a> during the time the action is being
   * processed.
   */
  public native function get waitMsg():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set waitMsg(value:String):void;

  [ExtConfig]
  /**
   * The title to be displayed by a call to <a href="#!/api/Ext.window.MessageBox-method-wait" rel="Ext.window.MessageBox-method-wait" class="docClass">Ext.window.MessageBox.wait</a> during the time the action is being
   * processed.
   */
  public native function get waitTitle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set waitTitle(value:String):void;

  /**
   * Invokes this action using the current configuration.
   */
  public native function run():void;
}
}