package ext.form.field {
import ext.dom.Element;

[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Checkbox", require)]
/**
 * Single checkbox field. Can be used as a direct replacement for traditional checkbox fields. Also serves as a
 * parent class for <a href="#!/api/Ext.form.field.Radio" rel="Ext.form.field.Radio" class="docClass">radio buttons</a>.
 *
 * <h1>Labeling</h1>
 *
 * <p>In addition to the <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">standard field labeling options</a>, checkboxes
 * may be given an optional <a href="#!/api/Ext.form.field.Checkbox-cfg-boxLabel" rel="Ext.form.field.Checkbox-cfg-boxLabel" class="docClass">boxLabel</a> which will be displayed immediately after checkbox. Also see
 * <a href="#!/api/Ext.form.CheckboxGroup" rel="Ext.form.CheckboxGroup" class="docClass">Ext.form.CheckboxGroup</a> for a convenient method of grouping related checkboxes.</p>
 *
 * <h1>Values</h1>
 *
 * <p>The main value of a checkbox is a boolean, indicating whether or not the checkbox is checked.
 * The following values will check the checkbox:</p>
 *
 * <ul>
 * <li><code>true</code></li>
 * <li><code>'true'</code></li>
 * <li><code>'1'</code></li>
 * <li><code>'on'</code></li>
 * </ul>
 *
 *
 * <p>Any other value will un-check the checkbox.</p>
 *
 * <p>In addition to the main boolean value, you may also specify a separate <a href="#!/api/Ext.form.field.Checkbox-cfg-inputValue" rel="Ext.form.field.Checkbox-cfg-inputValue" class="docClass">inputValue</a>. This will be
 * sent as the parameter value when the form is <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a>. You will want to set
 * this value if you have multiple checkboxes with the same <a href="#!/api/Ext.form.field.Checkbox-cfg-name" rel="Ext.form.field.Checkbox-cfg-name" class="docClass">name</a>. If not specified, the value <code>on</code>
 * will be used.</p>
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     bodyPadding: 10,
 *     width: 300,
 *     title: 'Pizza Order',
 *     items: [
 *         {
 *             xtype: 'fieldcontainer',
 *             fieldLabel: 'Toppings',
 *             defaultType: 'checkboxfield',
 *             items: [
 *                 {
 *                     boxLabel  : 'Anchovies',
 *                     name      : 'topping',
 *                     inputValue: '1',
 *                     id        : 'checkbox1'
 *                 }, {
 *                     boxLabel  : 'Artichoke Hearts',
 *                     name      : 'topping',
 *                     inputValue: '2',
 *                     checked   : true,
 *                     id        : 'checkbox2'
 *                 }, {
 *                     boxLabel  : 'Bacon',
 *                     name      : 'topping',
 *                     inputValue: '3',
 *                     id        : 'checkbox3'
 *                 }
 *             ]
 *         }
 *     ],
 *     bbar: [
 *         {
 *             text: 'Select Bacon',
 *             handler: function() {
 *                 <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('checkbox3').setValue(true);
 *             }
 *         },
 *         '-',
 *         {
 *             text: 'Select All',
 *             handler: function() {
 *                 <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('checkbox1').setValue(true);
 *                 <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('checkbox2').setValue(true);
 *                 <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('checkbox3').setValue(true);
 *             }
 *         },
 *         {
 *             text: 'Deselect All',
 *             handler: function() {
 *                 <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('checkbox1').setValue(false);
 *                 <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('checkbox2').setValue(false);
 *                 <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('checkbox3').setValue(false);
 *             }
 *         }
 *     ],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class Checkbox extends ext.form.field.BaseField {
  /**
   * @param config @inheritDoc
   */
  public function Checkbox(config:ext.form.field.Checkbox = null) {
    super();
  }

  /**
   * The original value of the field as configured in the <a href="#!/api/Ext.form.field.Checkbox-cfg-checked" rel="Ext.form.field.Checkbox-cfg-checked" class="docClass">checked</a> configuration, or as loaded by the last
   * form load operation if the form's <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> setting is <code>true</code>.
   */
  override public native function get originalValue():*;

  /**
   * @private
   */
  override public native function set originalValue(value:*):void;

  [ExtConfig]
  /**
   * The default CSS class for the checkbox
   * @default 'x-form-field'
   */
  override public native function get fieldCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set fieldCls(value:String):void;

  [ExtConfig]
  /**
   * The CSS class to use when the checkbox receives focus
   * @default 'x-form-checkbox-focus'
   */
  override public native function get focusCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set focusCls(value:String):void;

  /**
   * A reference to the label element created for the <a href="#!/api/Ext.form.field.Checkbox-cfg-boxLabel" rel="Ext.form.field.Checkbox-cfg-boxLabel" class="docClass">boxLabel</a>. Only present if the component has been
   * rendered and has a boxLabel configured.
   */
  public native function get boxLabelEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set boxLabelEl(value:ext.dom.Element):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Checkbox, or subclass thereof.
   * @default true
   */
  public native function get isCheckbox():Boolean;

  /**
   * @private
   */
  public native function set isCheckbox(value:Boolean):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the box label text. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get afterBoxLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterBoxLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the box label element. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get afterBoxLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterBoxLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the box label text. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get beforeBoxLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeBoxLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the box label element. If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get beforeBoxLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeBoxLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional text label that will appear next to the checkbox. Whether it appears before or after the checkbox is
   * determined by the <a href="#!/api/Ext.form.field.Checkbox-cfg-boxLabelAlign" rel="Ext.form.field.Checkbox-cfg-boxLabelAlign" class="docClass">boxLabelAlign</a> config.
   * @see #setBoxLabel()
   */
  public native function get boxLabel():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set boxLabel(value:String):void;

  [ExtConfig]
  /**
   * The position relative to the checkbox where the <a href="#!/api/Ext.form.field.Checkbox-cfg-boxLabel" rel="Ext.form.field.Checkbox-cfg-boxLabel" class="docClass">boxLabel</a> should appear. Recognized values are 'before'
   * and 'after'.
   * @default 'after'
   */
  public native function get boxLabelAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set boxLabelAlign(value:String):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * inside the box label element (as attributes). If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get boxLabelAttrTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set boxLabelAttrTpl(value:*):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the <a href="#!/api/Ext.form.field.Checkbox-cfg-boxLabel" rel="Ext.form.field.Checkbox-cfg-boxLabel" class="docClass">boxLabel</a> element
   * @default 'x-form-cb-label'
   */
  public native function get boxLabelCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set boxLabelCls(value:String):void;

  [ExtConfig]
  /**
   * true if the checkbox should render initially checked
   * @default false
   */
  public native function get checked():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checked(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class(es) added to the component's main element when it is in the checked state.
   * You can add your own class (checkedCls='myClass x-form-cb-checked') or replace the default
   * class altogether (checkedCls='myClass').
   * @default 'x-form-cb-checked'
   */
  public native function get checkedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkedCls(value:String):void;

  [ExtConfig]
  /**
   * A function called when the <a href="#!/api/Ext.form.field.Checkbox-cfg-checked" rel="Ext.form.field.Checkbox-cfg-checked" class="docClass">checked</a> value changes (can be used instead of handling the <a href="#!/api/Ext.form.field.Checkbox-event-change" rel="Ext.form.field.Checkbox-event-change" class="docClass">change event</a>).
   * @default undefined
   * <ul>
   *   <li><code>checkbox</code> : ext.form.field.Checkbox
   * The Checkbox being toggled.
   *   </li>
   *   <li><code>checked</code> : Boolean
   * The new checked state of the checkbox.
   * &#64;controllable
   *   </li>
   * </ul>
   */
  public native function get handler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handler(value:*):void;

  [ExtConfig]
  /**
   * The value that should go into the generated input element's value attribute and should be used as the parameter
   * value when submitting as part of a form.
   * @default 'on'
   */
  public native function get inputValue():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inputValue(value:String):void;

  [ExtConfig]
  /**
   * An object to use as the scope ('this' reference) of the <a href="#!/api/Ext.form.field.Checkbox-cfg-handler" rel="Ext.form.field.Checkbox-cfg-handler" class="docClass">handler</a> function.
   *
   * <p>Defaults to this Checkbox.</p>
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * If configured, this will be submitted as the checkbox's value during form submit if the checkbox is unchecked. By
   * default this is undefined, which results in nothing being submitted for the checkbox field when the form is
   * submitted (the default behavior of HTML checkboxes).
   */
  public native function get uncheckedValue():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set uncheckedValue(value:String):void;

  /**
   * Returns the checked state of the checkbox.
   * @return True if checked, else false
   */
  override public native function getRawValue():String;

  /**
   * Returns the submit value for the checkbox which can be used when submitting forms.
   * @return If checked the <a href="#!/api/Ext.form.field.Checkbox-cfg-inputValue" rel="Ext.form.field.Checkbox-cfg-inputValue" class="docClass">inputValue</a> is returned; otherwise the <a href="#!/api/Ext.form.field.Checkbox-cfg-uncheckedValue" rel="Ext.form.field.Checkbox-cfg-uncheckedValue" class="docClass">uncheckedValue</a>
   * (or null if the latter is not configured).
   */
  override public native function getSubmitValue():String;

  /**
   * Returns the checked state of the checkbox.
   * @return True if checked, else false
   */
  override public native function getValue():*;

  /**
   * Sets the checked state of the checkbox.
   * @param value The value to set
   * @return the new checked state of the checkbox
   */
  override public native function setRawValue(value:Object):Object;

  /**
   * Sets the checked state of the checkbox, and invokes change detection.
   * @param value The value to set
   * @return this
   */
  override public native function setValue(value:*):ext.form.field.IField;

  /**
   * Sets the <a href="#!/api/Ext.form.field.Checkbox-cfg-boxLabel" rel="Ext.form.field.Checkbox-cfg-boxLabel" class="docClass">boxLabel</a> for this checkbox.
   * @param boxLabel The new label
   */
  public native function setBoxLabel(boxLabel:String):void;
}
}