package ext.form.field {

[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Spinner", require)]
/**
 * Fires when the spinner is made to spin up or down.
 * @eventType ext.form.field.events.SpinnerField_directionEvent.SPIN
 */
[Event(name = 'onSpin', type = 'ext.form.field.events.SpinnerField_directionEvent')]
/**
 * Fires when the spinner is made to spin down.
 * @eventType ext.form.field.events.SpinnerFieldEvent.SPIN_DOWN
 */
[Event(name = 'onSpinDown', type = 'ext.form.field.events.SpinnerFieldEvent')]
/**
 * Fires when a spin command has been finished. For example on mouseup
 * on the spin buttons, when an <code>UP</code> or <code>DOWN</code> arrow key is released
 * of when a mousewheel stops spinning.
 *
 * <p>When this event fires, the field's value has stabilized.</p>
 * @since 6.2.0
 * @eventType ext.form.field.events.SpinnerFieldEvent.SPIN_END
 */
[Event(name = 'onSpinEnd', type = 'ext.form.field.events.SpinnerFieldEvent')]
/**
 * Fires when the spinner is made to spin up.
 * @eventType ext.form.field.events.SpinnerFieldEvent.SPIN_UP
 */
[Event(name = 'onSpinUp', type = 'ext.form.field.events.SpinnerFieldEvent')]
/**
 * A field with a pair of up/down spinner buttons. This class is not normally instantiated directly,
 * instead it is subclassed and the <a href="#!/api/Ext.form.field.Spinner-method-onSpinUp" rel="Ext.form.field.Spinner-method-onSpinUp" class="docClass">onSpinUp</a> and <a href="#!/api/Ext.form.field.Spinner-method-onSpinDown" rel="Ext.form.field.Spinner-method-onSpinDown" class="docClass">onSpinDown</a> methods are implemented
 * to handle when the buttons are clicked. A good example of this is the <a href="#!/api/Ext.form.field.Number" rel="Ext.form.field.Number" class="docClass">Ext.form.field.Number</a>
 * field which uses the spinner to increment and decrement the field's value by its
 * <a href="#!/api/Ext.form.field.Number-cfg-step" rel="Ext.form.field.Number-cfg-step" class="docClass">step</a> config value.
 *
 * <p>For example:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.ux.CustomSpinner', {
 *     extend: '<a href="#!/api/Ext.form.field.Spinner" rel="Ext.form.field.Spinner" class="docClass">Ext.form.field.Spinner</a>',
 *     alias: 'widget.customspinner',
 *
 *     // override onSpinUp (using step isn't neccessary)
 *     onSpinUp: function() {
 *         var me = this;
 *         if (!me.readOnly) {
 *             var val = parseInt(me.getValue().split(' '), 10)||0; // gets rid of " Pack", defaults to zero on parse failure
 *             me.setValue((val + me.step) + ' Pack');
 *         }
 *     },
 *
 *     // override onSpinDown
 *     onSpinDown: function() {
 *         var me = this;
 *         if (!me.readOnly) {
 *            var val = parseInt(me.getValue().split(' '), 10)||0; // gets rid of " Pack", defaults to zero on parse failure
 *            if (val &lt;= me.step) {
 *                me.setValue('Dry!');
 *            } else {
 *                me.setValue((val - me.step) + ' Pack');
 *            }
 *         }
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.FormPanel</a>', {
 *     title: 'Form with SpinnerField',
 *     bodyPadding: 5,
 *     width: 350,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items:[{
 *         xtype: 'customspinner',
 *         fieldLabel: 'How Much Beer?',
 *         step: 6
 *     }]
 * });
 * </code></pre>
 *
 * <p>By default, pressing the up and down arrow keys will also trigger the onSpinUp and onSpinDown methods;
 * to prevent this, set <code><a href="#!/api/Ext.form.field.Spinner-cfg-keyNavEnabled" rel="Ext.form.field.Spinner-cfg-keyNavEnabled" class="docClass">keyNavEnabled</a> = false</code>.</p>
 */
public class SpinnerField extends ext.form.field.TextField {
  /**
   * @param config @inheritDoc
   */
  public function SpinnerField(config:ext.form.field.SpinnerField = null) {
    super();
  }

  [ExtConfig]
  /**
   * Whether a <a href="#!/api/Ext.util.ClickRepeater" rel="Ext.util.ClickRepeater" class="docClass">click repeater</a> should be attached to the spinner buttons.
   * Defaults to true.
   * @default true
   */
  override public native function get repeatTriggerClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set repeatTriggerClick(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether the up and down arrow keys should trigger spinning up and down. Defaults to true.
   * @default true
   */
  public native function get keyNavEnabled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set keyNavEnabled(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether the mouse wheel should trigger spinning up and down while the field has focus.
   * Defaults to true.
   * @default true
   */
  public native function get mouseWheelEnabled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set mouseWheelEnabled(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether the down spinner button is enabled. Defaults to true. To change this after the component is
   * created, use the <a href="#!/api/Ext.form.field.Spinner-method-setSpinDownEnabled" rel="Ext.form.field.Spinner-method-setSpinDownEnabled" class="docClass">setSpinDownEnabled</a> method.
   * @default true
   * @see #setSpinDownEnabled()
   */
  public native function get spinDownEnabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set spinDownEnabled(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether the up spinner button is enabled. Defaults to true. To change this after the component is
   * created, use the <a href="#!/api/Ext.form.field.Spinner-method-setSpinUpEnabled" rel="Ext.form.field.Spinner-method-setSpinUpEnabled" class="docClass">setSpinUpEnabled</a> method.
   * @default true
   * @see #setSpinUpEnabled()
   */
  public native function get spinUpEnabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set spinUpEnabled(value:Boolean):void;

  /**
   * Sets whether the spinner down button is enabled.
   * @param enabled true to enable the button, false to disable it.
   */
  public native function setSpinDownEnabled(enabled:Boolean):void;

  /**
   * Sets whether the spinner up button is enabled.
   * @param enabled true to enable the button, false to disable it.
   */
  public native function setSpinUpEnabled(enabled:Boolean):void;

  /**
   * Triggers the spinner to step down; fires the <a href="#!/api/Ext.form.field.Spinner-event-spin" rel="Ext.form.field.Spinner-event-spin" class="docClass">spin</a> and <a href="#!/api/Ext.form.field.Spinner-event-spindown" rel="Ext.form.field.Spinner-event-spindown" class="docClass">spindown</a> events and calls the
   * <a href="#!/api/Ext.form.field.Spinner-method-onSpinDown" rel="Ext.form.field.Spinner-method-onSpinDown" class="docClass">onSpinDown</a> method. Does nothing if the field is <a href="#!/api/Ext.form.field.Spinner-cfg-disabled" rel="Ext.form.field.Spinner-cfg-disabled" class="docClass">disabled</a> or if <a href="#!/api/Ext.form.field.Spinner-cfg-spinDownEnabled" rel="Ext.form.field.Spinner-cfg-spinDownEnabled" class="docClass">spinDownEnabled</a>
   * is false.
   */
  public native function spinDown():void;

  /**
   * Triggers the spinner to step up; fires the <a href="#!/api/Ext.form.field.Spinner-event-spin" rel="Ext.form.field.Spinner-event-spin" class="docClass">spin</a> and <a href="#!/api/Ext.form.field.Spinner-event-spinup" rel="Ext.form.field.Spinner-event-spinup" class="docClass">spinup</a> events and calls the
   * <a href="#!/api/Ext.form.field.Spinner-method-onSpinUp" rel="Ext.form.field.Spinner-method-onSpinUp" class="docClass">onSpinUp</a> method. Does nothing if the field is <a href="#!/api/Ext.form.field.Spinner-cfg-disabled" rel="Ext.form.field.Spinner-cfg-disabled" class="docClass">disabled</a> or if <a href="#!/api/Ext.form.field.Spinner-cfg-spinUpEnabled" rel="Ext.form.field.Spinner-cfg-spinUpEnabled" class="docClass">spinUpEnabled</a>
   * is false.
   */
  public native function spinUp():void;

  /**
   * This method is called when the spinner down button is clicked, or when the down arrow key is pressed if
   * <a href="#!/api/Ext.form.field.Spinner-cfg-keyNavEnabled" rel="Ext.form.field.Spinner-cfg-keyNavEnabled" class="docClass">keyNavEnabled</a> is true. Must be implemented by subclasses.
   */
  protected native function onSpinDown():void;

  /**
   * This method is called when the spinner up button is clicked, or when the up arrow key is pressed if
   * <a href="#!/api/Ext.form.field.Spinner-cfg-keyNavEnabled" rel="Ext.form.field.Spinner-cfg-keyNavEnabled" class="docClass">keyNavEnabled</a> is true. Must be implemented by subclasses.
   */
  protected native function onSpinUp():void;
}
}