package ext.form.field {

[Uses("ext.form.field.SVTypes")]
[Uses("ext.Base")]
[Native("Ext.form.field.VTypes", require)]
/**
 * This is a singleton object which contains a set of commonly used field validation functions
 * and provides a mechanism for creating reusable custom field validations.
 * The following field validation functions are provided out of the box:
 *
 * <ul>
 * <li><a href="#!/api/Ext.form.field.VTypes-method-alpha" rel="Ext.form.field.VTypes-method-alpha" class="docClass">alpha</a></li>
 * <li><a href="#!/api/Ext.form.field.VTypes-method-alphanum" rel="Ext.form.field.VTypes-method-alphanum" class="docClass">alphanum</a></li>
 * <li><a href="#!/api/Ext.form.field.VTypes-method-email" rel="Ext.form.field.VTypes-method-email" class="docClass">email</a></li>
 * <li><a href="#!/api/Ext.form.field.VTypes-method-url" rel="Ext.form.field.VTypes-method-url" class="docClass">url</a></li>
 * </ul>
 *
 *
 * <p>VTypes can be applied to a <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Text Field</a> using the <code><a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a></code> configuration:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
 *     fieldLabel: 'Email Address',
 *     name: 'email',
 *     vtype: 'email' // applies email validation rules to this field
 * });
 * </code></pre>
 *
 * <p>To create custom VTypes:</p>
 *
 * <pre><code>// custom Vtype for vtype:'time'
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Override.form.field.VTypes', {
 *     override: '<a href="#!/api/Ext.form.field.VTypes" rel="Ext.form.field.VTypes" class="docClass">Ext.form.field.VTypes</a>',
 *
 *     // vtype validation function
 *     time: function(value) {
 *         return this.timeRe.test(value);
 *     },
 *     // RegExp for the value to be tested against within the validation function
 *     timeRe: /^([1-9]|1[0-9]):([0-5][0-9])(\s[a|p]m)$/i,
 *     // vtype Text property: The error text to display when the validation function returns false
 *     timeText: 'Not a valid time.  Must be in the format "12:34 PM".',
 *     // vtype Mask property: The keystroke filter mask
 *     timeMask: /[\d\s:amp]/i
 * });
 * </code></pre>
 *
 * <p>In the above example the <code>time</code> function is the validator that will run when field validation occurs,
 * <code>timeText</code> is the error message, and <code>timeMask</code> limits what characters can be typed into the field.
 * Note that the <code>Text</code> and <code>Mask</code> functions must begin with the same name as the validator function.</p>
 *
 * <p>Using a custom validator is the same as using one of the build-in validators - just use the name of the validator function
 * as the <code><a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a></code> configuration on a <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Text Field</a>:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
 *     fieldLabel: 'Departure Time',
 *     name: 'departureTime',
 *     vtype: 'time' // applies custom time validation rules to this field
 * });
 * </code></pre>
 *
 * <p>Another example of a custom validator:</p>
 *
 * <pre><code>// custom Vtype for vtype:'IPAddress'
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Override.form.field.VTypes', {
 *     override: '<a href="#!/api/Ext.form.field.VTypes" rel="Ext.form.field.VTypes" class="docClass">Ext.form.field.VTypes</a>',
 *
 *     IPAddress:  function(value) {
 *         return this.IPAddressRe.test(value);
 *     },
 *     IPAddressRe: /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/,
 *     IPAddressText: 'Must be a numeric IP address',
 *     IPAddressMask: /[\d\.]/i
 * });
 * </code></pre>
 *
 * <p>It's important to note that using <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define()</a> with the <a href="#!/api/Ext.Class-cfg-override" rel="Ext.Class-cfg-override" class="docClass">override</a> property
 * means that the custom validator function as well as <code>Text</code> and <code>Mask</code> fields are added as properties
 * of the <code><a href="#!/api/Ext.form.field.VTypes" rel="Ext.form.field.VTypes" class="docClass">Ext.form.field.VTypes</a></code> singleton.</p>
 */
public const VTypes:ext.form.field.SVTypes;
}