package ext.grid {
import ext.Base;
import ext.data.Model;
import ext.grid.column.Column;
import ext.view.TableView;

[Uses("ext.Base")]
[Native("Ext.grid.CellContext", require)]
/**
 * Instances of this class encapsulate a position in a grid's row/column coordinate system.
 *
 * <p>Cells are addressed using the owning <a href="#!/api/Ext.grid.CellContext-property-record" rel="Ext.grid.CellContext-property-record" class="docClass">record</a> and <a href="#!/api/Ext.grid.CellContext-property-column" rel="Ext.grid.CellContext-property-column" class="docClass">column</a> for robustness.
 * the column may be moved, the store may be sorted, and the CellContext will still reference
 * the same <em>logical</em> cell. Be aware that due to buffered rendering the <em>physical</em> cell may not exist.</p>
 *
 * <p>The <a href="#!/api/Ext.grid.CellContext-method-setPosition" rel="Ext.grid.CellContext-method-setPosition" class="docClass">setPosition</a> method however allows a numeric row and column to be passed in. These
 * are immediately converted.</p>
 *
 * <p>Be careful not to make <code>CellContext</code> objects <em>too</em> persistent. If the owning record is removed, or the owning column
 * is removed, the reference will be stale.</p>
 *
 * <p>Freshly created context objects, such as those exposed by events from the <a href="#!/api/Ext.grid.selection.SpreadsheetModel" rel="Ext.grid.selection.SpreadsheetModel" class="docClass">spreadsheet selection model</a>
 * are safe to use until your application mutates the store, or changes the column set.</p>
 */
public class CellContext extends ext.Base {
  /**
   * Creates a new CellContext which references a <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">GridView</a>
   * @param view The <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">GridView</a> for which the cell context is needed.
   * <p>To complete creation of a valid context, use the <a href="#!/api/Ext.grid.CellContext-method-setPosition" rel="Ext.grid.CellContext-method-setPosition" class="docClass">setPosition</a> method.</p>
   */
  public function CellContext(view:ext.view.TableView) {
    super();
  }

  /**
   * The column index in the owning View's leaf column set of the referenced cell.
   *
   * <p><em>Be aware that after the initial call to <a href="#!/api/Ext.grid.CellContext-method-setPosition" rel="Ext.grid.CellContext-method-setPosition" class="docClass">setPosition</a>, this value may become stale due to subsequent column mutation.</em></p>
   */
  public native function get colIdx():Number;

  /**
   * The grid column which owns the referenced cell.
   */
  public native function get column():ext.grid.column.Column;

  /**
   * <code>true</code> in this class to identify an object as an instantiated CellContext, or subclass thereof.
   * @default true
   */
  public native function get isCellContext():Boolean;

  /**
   * The store record which maps to the referenced cell.
   */
  public native function get record():ext.data.Model;

  /**
   * The row number in the store which owns the referenced cell.
   *
   * <p><em>Be aware that after the initial call to <a href="#!/api/Ext.grid.CellContext-method-setPosition" rel="Ext.grid.CellContext-method-setPosition" class="docClass">setPosition</a>, this value may become stale due to subsequent store mutation.</em></p>
   */
  public native function get rowIdx():Number;

  /**
   * Creates a clone of this CellContext.
   *
   * <p>The clone may be retargeted without affecting the reference of this context.</p>
   * @return A copy of this context, referencing the same cell.
   */
  public native function clone():ext.grid.CellContext;

  /**
   * Returns the cell object referenced <em>at the time of calling</em>. Note that grid DOM is transient, and
   * the cell referenced may be removed from the DOM due to paging or buffered rendering or column or record removal.
   * @param returnDom Pass <code>true</code> to return a DOM object instead of an {&#64;link <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> Element).
   * @return The cell referenced by this context.
   */
  public native function getCell(returnDom:Boolean = false):*;

  /**
   * Returns the view node object (the encapsulating element of a data row) referenced <em>at the time of
   * calling</em>. Note that grid DOM is transient, and the node referenced may be removed from the DOM due
   * to paging or buffered rendering or column or record removal.
   * @param returnDom Pass <code>true</code> to return a DOM object instead of an {&#64;link <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> Element).
   * @return The grid item referenced by this context.
   */
  public native function getNode(returnDom:Boolean):*;

  /**
   * Returns the row object referenced <em>at the time of calling</em>. Note that grid DOM is transient, and
   * the row referenced may be removed from the DOM due to paging or buffered rendering or column or record removal.
   * @param returnDom Pass <code>true</code> to return a DOM object instead of an {&#64;link <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> Element).
   * @return The grid row referenced by this context.
   */
  public native function getRow(returnDom:Boolean):*;

  /**
   * Compares this CellContext object to another CellContext to see if they refer to the same cell.
   * @param other The CellContext to compare.
   * @return <code>true</code> if the other cell context references the same cell as this.
   */
  public native function isEqual(other:ext.grid.CellContext):Boolean;

  /**
   * Binds this cell context to a logical cell defined by the <a href="#!/api/Ext.grid.CellContext-property-record" rel="Ext.grid.CellContext-property-record" class="docClass">record</a> and <a href="#!/api/Ext.grid.CellContext-property-column" rel="Ext.grid.CellContext-property-column" class="docClass">column</a>.
   * @param row The row index or record which owns the required cell.
   * @param col The column index (In the owning View's leaf column set), or the owning <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">column</a>.
   * <p>A one argument form may be used in the form of an array:</p>
   * <pre><code>[column, row]
   * </code></pre>
   * <p>Or another CellContext may be passed.</p>
   * @return this CellContext object.
   */
  public native function setPosition(row:*, col:*):ext.grid.CellContext;
}
}