package ext.grid.feature {
import ext.grid.GridPanel;
import ext.view.TableView;

[Uses("ext.grid.feature.Feature")]
[Mixin("ext.grid.feature.Feature")]
/**
 * A feature is a type of plugin that is specific to the <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>. It provides several
 * hooks that allows the developer to inject additional functionality at certain points throughout the
 * grid creation cycle. This class provides the base template methods that are available to the developer,
 * it should be extended.
 *
 * <p>There are several built in features that extend this class, for example:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.grid.feature.Grouping" rel="Ext.grid.feature.Grouping" class="docClass">Ext.grid.feature.Grouping</a> - Shows grid rows in groups as specified by the <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></li>
 * <li><a href="#!/api/Ext.grid.feature.RowBody" rel="Ext.grid.feature.RowBody" class="docClass">Ext.grid.feature.RowBody</a> - Adds a body section for each grid row that can contain markup.</li>
 * <li><a href="#!/api/Ext.grid.feature.Summary" rel="Ext.grid.feature.Summary" class="docClass">Ext.grid.feature.Summary</a> - Adds a summary row at the bottom of the grid with aggregate totals for a column.</li>
 * </ul>
 *
 *
 * <h2>Using Features</h2>
 *
 * <p>A feature is added to the grid by specifying it an array of features in the configuration:</p>
 *
 * <pre><code>var groupingFeature = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.feature.Grouping" rel="Ext.grid.feature.Grouping" class="docClass">Ext.grid.feature.Grouping</a>');
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     // other options
 *     features: [groupingFeature]
 * });
 * </code></pre>
 *
 * <h2>Writing Features</h2>
 *
 * <p>A Feature may add new DOM structure within the structure of a grid.</p>
 *
 * <p>A grid is essentially a <code>&lt;table&gt;</code> element. A <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">TableView</a> instance uses four <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplates</a>
 * to render the grid, <code>tpl</code>, <code>itemTpl</code>, <code>rowTpl</code>, <code>cellTpl</code>.</p>
 *
 * <p>A <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">TableView</a> uses its <code>tpl</code> to emit the item encapsulating HTML tags into its output stream.
 * To render the rows, it invokes <a href="#!/api/Ext.view.Table-method-renderRows" rel="Ext.view.Table-method-renderRows" class="docClass">Ext.view.Table.renderRows</a> passing the <code>rows</code> member of its data object and the <code>columns</code> member of its data object.</p>
 *
 * <p>The <code>tpl</code>'s data object Looks like this:
 *     {
 *         view: owningTableView,
 *         rows: recordsToRender,
 *         viewStartIndex: indexOfFirstRecordInStore,
 *         tableStyle: styleString
 *     }</p>
 *
 * <ul>
 * <li>A <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">TableView</a> uses its <code>rowTpl</code> to emit a <code>&lt;tr&gt;</code> HTML tag to its output stream. To render cells,
 * it invokes <a href="#!/api/Ext.view.Table-method-renderCell" rel="Ext.view.Table-method-renderCell" class="docClass">Ext.view.Table.renderCell</a> passing the <code>rows</code> member of its data object.</li>
 * </ul>
 *
 *
 * <p>The <code>itemTpl</code> and <code>rowTpl</code>'s data object looks like this:</p>
 *
 * <pre><code>{
 *     view:        owningTableView,
 *     record:      recordToRender,
 *     recordIndex: indexOfRecordInStore,
 *     rowIndex:    indexOfRowInView,
 *     columns:     arrayOfColumnDefinitions,
 *     itemClasses: arrayOfClassNames, // For outermost row in case of wrapping
 *     rowClasses:  arrayOfClassNames,  // For internal data bearing row in case of wrapping
 *     rowStyle:    styleString
 * }
 * </code></pre>
 *
 * <ul>
 * <li>A <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">TableView</a> uses its <code>cellTpl</code> to emit a <code>&lt;td&gt;</code> HTML tag to its output stream.</li>
 * </ul>
 *
 *
 * <p>The <code>cellTpl's</code> data object looks like this:</p>
 *
 * <pre><code>{
 *     record: recordToRender
 *     column: columnToRender;
 *     recordIndex: indexOfRecordInStore,
 *     rowIndex:    indexOfRowInView,
 *     columnIndex: columnIndex,
 *     align: columnAlign,
 *     tdCls: classForCell
 * }
 * </code></pre>
 *
 * <p>A Feature may inject its own tpl or rowTpl or cellTpl into the <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">TableView</a>'s rendering by
 * calling <a href="#!/api/Ext.view.Table-method-addTpl" rel="Ext.view.Table-method-addTpl" class="docClass">Ext.view.Table.addTpl</a> or <a href="#!/api/Ext.view.Table-method-addRowTpl" rel="Ext.view.Table-method-addRowTpl" class="docClass">Ext.view.Table.addRowTpl</a> or <a href="#!/api/Ext.view.Table-method-addCellTpl" rel="Ext.view.Table-method-addCellTpl" class="docClass">Ext.view.Table.addCellTpl</a>.</p>
 *
 * <p>The passed XTemplate is added <em>upstream</em> of the default template for the table element in a link list of XTemplates which contribute
 * to the element's HTML. It may emit appropriate HTML strings into the output stream <em>around</em> a call to</p>
 *
 * <pre><code>this.nextTpl.apply(values, out, parent);
 * </code></pre>
 *
 * <p>This passes the current value context, output stream and the parent value context to the next XTemplate in the list.</p>
 * @see ext.grid.feature.Feature
 */
public interface IFeature {
  /**
   * True when feature is disabled.
   * @default false
   */
  function get disabled():Boolean;

  function set disabled(value:Boolean):void;

  /**
   * Prefix to use when firing events on the view.
   * For example a prefix of group would expose "groupclick", "groupcontextmenu", "groupdblclick".
   */
  function get eventPrefix():String;

  function set eventPrefix(value:String):void;

  /**
   * Selector used to determine when to fire the event with the eventPrefix.
   */
  function get eventSelector():String;

  function set eventSelector(value:String):void;

  /**
   * Reference to the grid panel
   */
  function get grid():ext.grid.GridPanel;

  function set grid(value:ext.grid.GridPanel):void;

  /**
   * Most features will expose additional events, some may not and will
   * need to change this to false.
   * @default true
   */
  function get hasFeatureEvent():Boolean;

  function set hasFeatureEvent(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Feature, or subclass thereof.
   * @default true
   */
  function get isFeature():Boolean;

  function set isFeature(value:Boolean):void;

  /**
   * Reference to the TableView.
   */
  function get view():ext.view.TableView;

  function set view(value:ext.view.TableView):void;

  /**
   * Disables the feature.
   */
  function disable():void;

  /**
   * Enables the feature.
   */
  function enable():void;

  /**
   * Abstract method to be overriden when a feature should add additional
   * arguments to its event signature. By default the event will fire:
   *
   * <ul>
   * <li>view - The underlying <a href="#!/api/Ext.view.Table" rel="Ext.view.Table" class="docClass">Ext.view.Table</a></li>
   * <li>featureTarget - The matched element by the defined <a href="#!/api/Ext.grid.feature.Feature-property-eventSelector" rel="Ext.grid.feature.Feature-property-eventSelector" class="docClass">eventSelector</a></li>
   * </ul>
   *
   *
   * <p>The method must also return the eventName as the first index of the array
   * to be passed to fireEvent.</p>
   */
  function getFireEventArgs(eventName:Object, view:Object, featureTarget:Object, e:Object):Array;
}
}