package ext.grid.filters {
import ext.data.AbstractStore;
import ext.data.Store;
import ext.plugin.AbstractPlugin;
import ext.util.IStoreHolder;

[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.Plugin")]
[Native("Ext.grid.filters.Filters", require)]
/**
 * This class is a grid <a href="#!/api/Ext.plugin.Abstract" rel="Ext.plugin.Abstract" class="docClass">plugin</a> that adds a simple and flexible
 * presentation for <a href="#!/api/Ext.data.AbstractStore-cfg-filters" rel="Ext.data.AbstractStore-cfg-filters" class="docClass">store filters</a>.
 *
 * <p>Filters can be modified by the end-user using the grid's column header menu. Through
 * this menu users can configure, enable, and disable filters for each column.</p>
 *
 * <h1>Example Usage</h1>
 *
 * <pre class='inline-example '><code>var shows = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['id','show'],
 *     data: [
 *         {id: 0, show: 'Battlestar Galactica'},
 *         {id: 1, show: 'Doctor Who'},
 *         {id: 2, show: 'Farscape'},
 *         {id: 3, show: 'Firefly'},
 *         {id: 4, show: 'Star Trek'},
 *         {id: 5, show: 'Star Wars: Christmas Special'}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     title: 'Sci-Fi Television',
 *     height: 250,
 *     width: 250,
 *     store: shows,
 *     plugins: 'gridfilters',
 *     columns: [{
 *         dataIndex: 'id',
 *         text: 'ID',
 *         width: 50
 *     },{
 *         dataIndex: 'show',
 *         text: 'Show',
 *         flex: 1,
 *         filter: {
 *             // required configs
 *             type: 'string',
 *             // optional configs
 *             value: 'star',  // setting a value makes the filter active.
 *             itemDefaults: {
 *                 // any <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a> configs accepted
 *             }
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Features</h1>
 *
 * <h2>Filtering implementations</h2>
 *
 * <p>Currently provided filter types are:</p>
 *
 * <ul>
 * <li><code><a href="#!/api/Ext.grid.filters.filter.Boolean" rel="Ext.grid.filters.filter.Boolean" class="docClass">boolean</a></code></li>
 * <li><code><a href="#!/api/Ext.grid.filters.filter.Date" rel="Ext.grid.filters.filter.Date" class="docClass">date</a></code></li>
 * <li><code><a href="#!/api/Ext.grid.filters.filter.List" rel="Ext.grid.filters.filter.List" class="docClass">list</a></code></li>
 * <li><code><a href="#!/api/Ext.grid.filters.filter.Number" rel="Ext.grid.filters.filter.Number" class="docClass">number</a></code></li>
 * <li><code><a href="#!/api/Ext.grid.filters.filter.String" rel="Ext.grid.filters.filter.String" class="docClass">string</a></code></li>
 * </ul>
 *
 *
 * <p><strong>Note:</strong> You can find inline examples for each filter on its specific filter page.</p>
 *
 * <h2>Graphical Indicators</h2>
 *
 * <p>Columns that are filtered have <a href="#!/api/Ext.grid.filters.Filters-property-filterCls" rel="Ext.grid.filters.Filters-property-filterCls" class="docClass">CSS class</a> applied to their column
 * headers. This style can be managed using that CSS class or by setting these Sass
 * variables in your theme or application:</p>
 *
 * <pre><code> $grid-filters-column-filtered-font-style: italic !default;
 *
 *  $grid-filters-column-filtered-font-weight: bold !default;
 * </code></pre>
 *
 * <h2>Stateful</h2>
 *
 * <p>Filter information will be persisted across page loads by specifying a <code>stateId</code>
 * in the Grid configuration. In actuality this state is saved by the <code>store</code>, but this
 * plugin ensures that saved filters are properly identified and reclaimed on subsequent
 * visits to the page.</p>
 *
 * <h2>Grid Changes</h2>
 *
 * <ul>
 * <li>A <code>filters</code> property is added to the Grid using this plugin.</li>
 * </ul>
 *
 *
 * <h1>Upgrading From Ext.ux.grid.FilterFeature</h1>
 *
 * <p>The biggest change for developers converting from the user extension is most likely the
 * conversion to standard <a href="#!/api/Ext.data.AbstractStore-cfg-filters" rel="Ext.data.AbstractStore-cfg-filters" class="docClass">store filters</a>. In the
 * process, the "like" and "in" operators are now supported by <code><a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Ext.util.Filter</a></code>.
 * These filters and all other filters added to the store will be sent in the standard
 * way (using the "filters" parameter by default).</p>
 *
 * <p>Since this plugin now uses actual store filters, the <code>onBeforeLoad</code> listener and all
 * helper methods that were used to clean and build the params have been removed. The store
 * will send the filters managed by this plugin along in its normal request.</p>
 */
public class Filters extends ext.plugin.AbstractPlugin implements ext.util.IStoreHolder {
  /**
   * @param config @inheritDoc
   */
  public function Filters(config:ext.grid.filters.Filters = null) {
    super();
  }

  /**
   * This property allows the object
   * to destroy bound stores that have #Ext.data.AbstractStore.autoDestroy
   * option set to <code>true</code>.
   * @default false
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @private
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * The CSS applied to column headers with active filters.
   * @default "x-grid-filters-filtered-column"
   */
  public native function get filterCls():String;

  /**
   * @private
   */
  public native function set filterCls(value:String):void;

  [ExtConfig]
  /**
   * The text for the filters menu.
   * @default "Filters"
   */
  public native function get menuFilterText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menuFilterText(value:String):void;

  [ExtConfig]
  /**
   * Defaults to true, including a filter submenu in the default header menu.
   * @default true
   */
  public native function get showMenu():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showMenu(value:Boolean):void;

  [ExtConfig]
  /**
   * Name of the value to be used to store state information.
   */
  public native function get stateId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stateId(value:String):void;

  /**
   * Adds a filter to the collection and creates a store filter if has a <code>value</code> property.
   * @param filters A filter
   * configuration or a filter object.
   */
  public native function addFilter(filters:*):void;

  /**
   * Adds filters to the collection.
   * @param filters An Array of filter configuration objects.
   */
  public native function addFilters(filters:Array):void;

  /**
   * Binds a store to this instance.
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  /**
   * Turns all filters off. This does not clear the configuration information.
   */
  public native function clearFilters():void;

  /**
   * Gets the current store instance.
   * @return The store, null if one does not exist.
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * Checks the plugin's grid for statefulness.
   */
  public native function isStateful():Boolean;

  /**
   * Sets the store to the specified store.
   * @since 5.0.0
   */
  public native function setStore(store:Object):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Gets the listeners to bind to a new store.
   * @param store The Store which is being bound to for which a listeners object should be returned.
   * @return The listeners to be bound to the store in object literal form. The scope
   * may be omitted, it is assumed to be the current instance.
   */
  protected native function getStoreListeners(store:ext.data.Store):Object;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}