package ext.grid.filters.filter {
import ext.data.Store;

[Uses("ext.grid.filters.filter.SingleFilter")]
[Uses("ext.grid.filters.filter.BaseFilter")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.grid.filters.filter.List", require)]
/**
 * The list grid filter allows you to create a filter selection that limits results
 * to values matching an element in a list.  The filter can be set programmatically or via
 * user input with a configurable <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">check box field</a> in the filter section
 * of the column header.
 *
 * <p>List filters are able to be preloaded/backed by an <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> to load
 * their options the first time they are shown.  They are also able to create their own
 * list of values from  all unique values of the specified <a href="#!/api/Ext.grid.filters.filter.List-cfg-dataIndex" rel="Ext.grid.filters.filter.List-cfg-dataIndex" class="docClass">dataIndex</a> field in
 * the store at first time of filter invocation.</p>
 *
 * <p>Example List Filter Usage:</p>
 *
 * <pre class='inline-example '><code>var shows = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['id','show','rating'],
 *     data: [
 *         {id: 0, show: 'Battlestar Galactica', rating: 2},
 *         {id: 1, show: 'Doctor Who', rating: 4},
 *         {id: 2, show: 'Farscape', rating: 3},
 *         {id: 3, show: 'Firefly', rating: 4},
 *         {id: 4, show: 'Star Trek', rating: 1},
 *         {id: 5, show: 'Star Wars: Christmas Special', rating: 5}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     title: 'Sci-Fi Television',
 *     height: 250,
 *     width: 350,
 *     store: shows,
 *     plugins: 'gridfilters',
 *     columns: [{
 *         dataIndex: 'id',
 *         text: 'ID',
 *         width: 50
 *     },{
 *         dataIndex: 'show',
 *         text: 'Show',
 *         flex: 1
 *     },{
 *         dataIndex: 'rating',
 *         text: 'Rating',
 *         width: 75,
 *         filter: {
 *             type: 'list',
 *             value: 5
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <h2>Options</h2>
 *
 * <p>There are three means to determine the list of options to present to the user:</p>
 *
 * <ul>
 * <li>The <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-options" rel="Ext.grid.filters.filter.List-cfg-options" class="docClass">options</a></code> config.</li>
 * <li>The <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-store" rel="Ext.grid.filters.filter.List-cfg-store" class="docClass">store</a></code> config. In this mode, the <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-idField" rel="Ext.grid.filters.filter.List-cfg-idField" class="docClass">idField</a></code>
 * and <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-labelField" rel="Ext.grid.filters.filter.List-cfg-labelField" class="docClass">labelField</a></code> configs are used to extract the presentation and
 * filtering values from the <code>store</code> and apply to the menu items and grid store
 * filter, respectively.</li>
 * <li>If none of the above is specified, the associated grid's store is used. In this
 * case, the <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-dataIndex" rel="Ext.grid.filters.filter.List-cfg-dataIndex" class="docClass">dataIndex</a></code> is used to determine the filter values and
 * the <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-labelIndex" rel="Ext.grid.filters.filter.List-cfg-labelIndex" class="docClass">labelIndex</a></code> is used to populate the menu items. These fields
 * are extracted from the records in the associated grid's store. Both of these
 * configs default to the column's <code>dataIndex</code> property.</li>
 * </ul>
 *
 *
 * <p>In all of these modes, a store is created that is synchronized with the menu items.
 * The records in this store have <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-idField" rel="Ext.grid.filters.filter.List-cfg-idField" class="docClass">idField</a></code> and <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-labelField" rel="Ext.grid.filters.filter.List-cfg-labelField" class="docClass">labelField</a></code>
 * fields that get populated from which ever source was provided.</p>
 *
 * <pre><code>var filters = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     ...
 *     columns: [{
 *         text: 'Size',
 *         dataIndex: 'size',
 *
 *         filter: {
 *             type: 'list',
 *             // options will be used as data to implicitly creates an ArrayStore
 *             options: ['extra small', 'small', 'medium', 'large', 'extra large']
 *         }
 *     }],
 *     ...
 * });
 * </code></pre>
 */
public class ListFilter extends ext.grid.filters.filter.SingleFilter {
  /**
   * @param config @inheritDoc
   */
  public function ListFilter(config:ext.grid.filters.filter.ListFilter = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * See the <a href="#!/api/Ext.grid.filters.filter.Base-cfg-itemDefaults" rel="Ext.grid.filters.filter.Base-cfg-itemDefaults" class="docClass">documentation</a> for
   * the base class for details.
   *
   * <p>In the case of this class, however, note that the <code>checked</code> config should <strong>not</strong> be
   * specified.</p>
   * @default {checked: false, hideOnClick: false}
   * @see #getItemDefaults()
   * @see #setItemDefaults()
   */
  override public native function get itemDefaults():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set itemDefaults(value:Object):void;

  [ExtConfig]
  /**
   * The field name for the <code>id</code> of records in this list's <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-store" rel="Ext.grid.filters.filter.List-cfg-store" class="docClass">store</a></code>. These values are
   * used to populate the filter for the grid's store.
   * @default "id"
   */
  public native function get idField():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set idField(value:String):void;

  [ExtConfig]
  /**
   * The field name for the menu item text in the records in this list's <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-store" rel="Ext.grid.filters.filter.List-cfg-store" class="docClass">store</a></code>.
   * @default "text"
   */
  public native function get labelField():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelField(value:String):void;

  [ExtConfig]
  /**
   * The field in the records of the grid's store from which the menu item text should be retrieved.
   * This field is only used when no <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-options" rel="Ext.grid.filters.filter.List-cfg-options" class="docClass">options</a></code> and no <code><a href="#!/api/Ext.grid.filters.filter.List-cfg-store" rel="Ext.grid.filters.filter.List-cfg-store" class="docClass">store</a></code> is provided
   * and the distinct value of the grid's store need to be generated dynamically.
   *
   * <p>If not provided, this field defaults to the column's <code>dataIndex</code> property.</p>
   * @since 5.1.0
   */
  public native function get labelIndex():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelIndex(value:String):void;

  [ExtConfig]
  /**
   * Defaults to true.
   * @default true
   */
  public native function get loadOnShow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadOnShow(value:Boolean):void;

  [ExtConfig]
  /**
   * The text that is displayed while the configured store is loading.
   * @default "Loading..."
   */
  public native function get loadingText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadingText(value:String):void;

  [ExtConfig]
  /**
   * The data to be used to implicitly create a data store to back this list. This is used only when
   * the data source is <strong>local</strong>. If the data for the list is remote, use the <a href="#!/api/Ext.grid.filters.filter.List-cfg-store" rel="Ext.grid.filters.filter.List-cfg-store" class="docClass">store</a>
   * config instead.
   *
   * <p>If neither store nor <a href="#!/api/Ext.grid.filters.filter.List-cfg-options" rel="Ext.grid.filters.filter.List-cfg-options" class="docClass">options</a> is specified, then the choices list is automatically
   * populated from all unique values of the specified <a href="#!/api/Ext.grid.filters.filter.List-cfg-dataIndex" rel="Ext.grid.filters.filter.List-cfg-dataIndex" class="docClass">dataIndex</a> field in the store at first
   * time of filter invocation.</p>
   *
   * <p>Each item within the provided array may be in one of the following formats:</p>
   *
   * <ul>
   * <li><p><strong>Array</strong> :</p>
   *
   * <pre><code>options: [
   *     [11, 'extra small'],
   *     [18, 'small'],
   *     [22, 'medium'],
   *     [35, 'large'],
   *     [44, 'extra large']
   * ]
   * </code></pre></li>
   * <li><p><strong>Object</strong> :</p>
   *
   * <pre><code>labelField: 'name', // override default of 'text'
   * options: [
   *     {id: 11, name:'extra small'},
   *     {id: 18, name:'small'},
   *     {id: 22, name:'medium'},
   *     {id: 35, name:'large'},
   *     {id: 44, name:'extra large'}
   * ]
   * </code></pre></li>
   * <li><p><strong>String</strong> :</p>
   *
   * <pre><code>options: ['extra small', 'small', 'medium', 'large', 'extra large']
   * </code></pre></li>
   * </ul>
   */
  public native function get options():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set options(value:Array):void;

  [ExtConfig]
  /**
   * Specify true to group all items in this list into a single-select
   * radio button group. Defaults to false.
   * @default false
   */
  public native function get single():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set single(value:Boolean):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> this list should use as its data source.
   *
   * <p>If neither store nor <a href="#!/api/Ext.grid.filters.filter.List-cfg-options" rel="Ext.grid.filters.filter.List-cfg-options" class="docClass">options</a> is specified, then the choices list is automatically
   * populated from all unique values of the specified <a href="#!/api/Ext.grid.filters.filter.List-cfg-dataIndex" rel="Ext.grid.filters.filter.List-cfg-dataIndex" class="docClass">dataIndex</a> field in the store at first
   * time of filter invocation.</p>
   */
  public native function get store():ext.data.Store;

  [ExtConfig]
  /**
   * @private
   */
  public native function set store(value:ext.data.Store):void;
}
}