package ext.grid.plugin {
import ext.grid.CellContext;

[Uses("ext.grid.plugin.GridEditingPlugin")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Native("Ext.grid.plugin.CellEditing", require)]
/**
 * Fires before cell editing is triggered. Return false from event handler to stop the editing.
 * @eventType ext.grid.plugin.events.CellEditingPlugin_contextEvent.BEFORE_EDIT
 */
[Event(name = 'onBeforeEdit', type = 'ext.grid.plugin.events.CellEditingPlugin_contextEvent')]
/**
 * Fires when the user started editing a cell but then cancelled the edit.
 * @eventType ext.grid.plugin.events.CellEditingPlugin_contextEvent.CANCEL_EDIT
 */
[Event(name = 'onCancelEdit', type = 'ext.grid.plugin.events.CellEditingPlugin_contextEvent')]
/**
 * Fires after a cell is edited. Usage example:
 *
 * <pre><code>grid.on('edit', function(editor, e) {
 *     // commit the changes right after editing finished
 *     e.record.commit();
 * });
 * </code></pre>
 * @eventType ext.grid.plugin.events.CellEditingPlugin_contextEvent.EDIT
 */
[Event(name = 'onEdit', type = 'ext.grid.plugin.events.CellEditingPlugin_contextEvent')]
/**
 * Fires after a cell is edited, but before the value is set in the record.
 * There are three possible outcomes when handling the validateedit event:
 *
 * <ul>
 * <li>Return <code>true</code> - Return true to commit the change to the underlying record and
 * hide the editor</li>
 * <li>Return 'false' - Return false to prevent 1) the edit from being committed to
 * the underlying record and 2) the editor from hiding / blurring.</li>
 * <li>Set context.cancel: true and return <code>false</code> - Set the context param's cancel property
 * to true and returning false will 1) prevent the edit from being committed to
 * the underlying record but <em>will</em> allow the edit to hide once blurred.</li>
 * </ul>
 *
 *
 * <p>In the following example, entering 10 in the editor field and tabbing out /
 * blurring the editor field will result in the the editor remaining focused as the
 * required validation criteria has not been met.</p>
 *
 * <pre><code>grid.on('validateedit', function(editor, context) {
 *     if (context.value &lt; 10) {
 *         return false;
 *     }
 * });
 * </code></pre>
 *
 * <p>If we modify the previous example by setting context.cancel to true then changing
 * the editor value from 2 to 10 and tabbing out of the field will result in the
 * editor hiding and the grid cell retaining the initial value of 2.</p>
 *
 * <pre><code>grid.on('validateedit', function(editor, context) {
 *     if (context.value &lt; 10) {
 *         context.cancel = true;
 *         return false;
 *     }
 * });
 * </code></pre>
 *
 * <p>Below is a usage example showing how to remove the red triangle (dirty-record
 * indicator) from some records (not all). By observing the grid's validateedit
 * event, it can be cancelled if the edit occurs on a targeted row (for example) and
 * then setting the field's new value in the Record directly:</p>
 *
 * <pre><code>grid.on('validateedit', function(editor, e) {
 *   var myTargetRow = 6;
 *
 *   if (e.row == myTargetRow) {
 *     e.cancel = true;
 *     e.record.data[e.field] = e.value;
 *   }
 * });
 * </code></pre>
 * @eventType ext.grid.plugin.events.CellEditingPlugin_contextEvent.VALIDATE_EDIT
 */
[Event(name = 'onValidateEdit', type = 'ext.grid.plugin.events.CellEditingPlugin_contextEvent')]
/**
 * The <a href="#!/api/Ext.grid.plugin.CellEditing" rel="Ext.grid.plugin.CellEditing" class="docClass">Ext.grid.plugin.CellEditing</a> plugin injects editing at a cell level for a Grid. Only a single
 * cell will be editable at a time. The field that will be used for the editor is defined at the
 * <a href="#!/api/Ext.grid.column.Column-cfg-editor" rel="Ext.grid.column.Column-cfg-editor" class="docClass">editor</a>. The editor can be a field instance or a field configuration.
 *
 * <p>If an editor is not specified for a particular column then that cell will not be editable and it will
 * be skipped when activated via the mouse or the keyboard.</p>
 *
 * <p>The editor may be shared for each column in the grid, or a different one may be specified for each column.
 * An appropriate field type should be chosen to match the data structure that it will be editing. For example,
 * to edit a date, it would be useful to specify <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a> as the editor.</p>
 *
 * <p>If the <code>editor</code> config on a column contains a <code>field</code> property, then the <code>editor</code> config is used to create
 * the wrapping Ext.grid.CellEditorCellEditor, and the <code>field</code> property is used to create the editing
 * input field.</p>
 *
 * <h2>Example</h2>
 *
 * <p>A grid with editor for the name and the email columns:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     storeId: 'simpsonsStore',
 *     fields:[ 'name', 'email', 'phone'],
 *     data: [
 *         { name: 'Lisa', email: 'lisa&#64;simpsons.com', phone: '555-111-1224' },
 *         { name: 'Bart', email: 'bart&#64;simpsons.com', phone: '555-222-1234' },
 *         { name: 'Homer', email: 'homer&#64;simpsons.com', phone: '555-222-1244' },
 *         { name: 'Marge', email: 'marge&#64;simpsons.com', phone: '555-222-1254' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('simpsonsStore'),
 *     columns: [
 *         {header: 'Name', dataIndex: 'name', editor: 'textfield'},
 *         {header: 'Email', dataIndex: 'email', flex:1,
 *             editor: {
 *                 completeOnEnter: false,
 *
 *                 // If the editor config contains a field property, then
 *                 // the editor config is used to create the <a href="#!/api/Ext.grid.CellEditor" rel="Ext.grid.CellEditor" class="docClass">CellEditor</a>
 *                 // and the field property is used to create the editing input field.
 *                 field: {
 *                     xtype: 'textfield',
 *                     allowBlank: false
 *                 }
 *             }
 *         },
 *         {header: 'Phone', dataIndex: 'phone'}
 *     ],
 *     selModel: 'cellmodel',
 *     plugins: {
 *         ptype: 'cellediting',
 *         clicksToEdit: 1
 *     },
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>This requires a little explanation. We're passing in <code>store</code> and <code>columns</code> as normal, but
 * we also specify a <a href="#!/api/Ext.grid.column.Column-cfg-field" rel="Ext.grid.column.Column-cfg-field" class="docClass">field</a> on two of our columns. For the
 * Name column we just want a default textfield to edit the value, so we specify 'textfield'.
 * For the Email column we customized the editor slightly by passing allowBlank: false, which
 * will provide inline validation.</p>
 *
 * <p>To support cell editing, we also specified that the grid should use the 'cellmodel'
 * <a href="#!/api/Ext.grid.Panel-cfg-selModel" rel="Ext.grid.Panel-cfg-selModel" class="docClass">selModel</a>, and created an instance of the CellEditing plugin,
 * which we configured to activate each editor after a single click.</p>
 */
public class CellEditingPlugin extends ext.grid.plugin.GridEditingPlugin {
  /**
   * @param config @inheritDoc
   */
  public function CellEditingPlugin(config:ext.grid.plugin.CellEditingPlugin = null) {
    super();
  }

  /**
   * Cancels any active editing.
   */
  override public native function cancelEdit():void;

  [Deprecated(replacement = "setActionableMode", since = "5.5.0")]
  /**
   * @param record The Store data record which backs the row to be edited, or index of the record.
   * @param columnHeader The Column object defining the column to be edited, or index of the column.
   */
  public native function startEdit(record:*, columnHeader:*):void;

  /**
   * Starts editing by position (row/column)
   * @param position A position with keys of row and column.
   * Example usage:
   * <pre><code>cellEditing.startEditByPosition({
   *     row: 3,
   *     column: 2
   * });
   * </code></pre>
   */
  public native function startEditByPosition(position:Object):void;

  /**
   * This method is called when actionable mode is requested for a cell.
   * @param position The position at which actionable mode was requested.
   * @param skipBeforeCheck Pass <code>true</code> to skip the possible vetoing conditions like event firing.
   * @param doFocus Pass <code>true</code> to immediately focus the active editor.
   * @return <code>true</code> if this cell is actionable (editable)
   */
  protected native function activateCell(position:ext.grid.CellContext, skipBeforeCheck:Boolean = false, doFocus:Boolean = false):Boolean;

  /**
   * Cancels the currently focused operation. In this case CellEditing.
   * the view is being changed.
   */
  protected native function deactivate():void;

  /**
   * Called by TableView#resumeActionableMode to resume actionable processing after
   * the view has been changed.
   * @param position The position at which to resume actionable processing.
   * @return <code>true</code> if this Actionable has successfully resumed.
   */
  protected native function resume(position:ext.grid.CellContext):Boolean;

  /**
   * Called by TableView#suspendActionableMode to suspend actionable processing while
   * the view is being changed.
   */
  protected native function suspend():void;
}
}