package ext.grid.selection {
import ext.grid.column.Column;
import ext.selection.SelectionModel;

[Uses("ext.selection.SelectionModel")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.util.StoreHolder")]
[Native("Ext.grid.selection.SpreadsheetModel", require)]
/**
 * Fired <em>by the grid</em> after the selection changes. Return <code>false</code> to veto the selection extension.
 *
 * <p>Note that the behavior of selectionchange is different in Ext 6.x vs. Ext 5.  In Ext 6.x, if rows
 * are being selected, a block of records is passed as the second parameter.  In Ext 5, the selection
 * object was passed.</p>
 * @eventType ext.grid.selection.events.SpreadsheetModelSelection_grid_selectionEvent.SELECTION_CHANGE
 */
[Event(name = 'onSelectionChange', type = 'ext.grid.selection.events.SpreadsheetModelSelection_grid_selectionEvent')]
/**
 * A selection model for <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">grids</a> which allows you to select data in
 * a spreadsheet-like manner.
 *
 * <p>Supported features:</p>
 *
 * <ul>
 * <li>Single / Range / Multiple individual row selection.</li>
 * <li>Single / Range cell selection.</li>
 * <li>Column selection by click selecting column headers.</li>
 * <li>Select / deselect all by clicking in the top-left, header.</li>
 * <li>Adds row number column to enable row selection.</li>
 * <li>Optionally you can enable row selection using checkboxes</li>
 * </ul>
 *
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['name', 'email', 'phone'],
 *     data: [
 *         { name: 'Lisa', email: 'lisa&#64;simpsons.com', phone: '555-111-1224' },
 *         { name: 'Bart', email: 'bart&#64;simpsons.com', phone: '555-222-1234' },
 *         { name: 'Homer', email: 'homer&#64;simpsons.com', phone: '555-222-1244' },
 *         { name: 'Marge', email: 'marge&#64;simpsons.com', phone: '555-222-1254' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     store: store,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     columns: [
 *         { text: 'Name', dataIndex: 'name' },
 *         { text: 'Email', dataIndex: 'email', flex: 1 },
 *         { text: 'Phone', dataIndex: 'phone' }
 *     ],
 *     selModel: {
 *        type: 'spreadsheet'
 *     }
 * });
 * </code></pre>
 *
 * <h1>Using <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Ext.data.BufferedStore</a>s</h1>
 *
 * <p>It is very important to remember that a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Ext.data.BufferedStore</a> does <em>not</em> contain the
 * full dataset. The purpose of a BufferedStore is to only hold in the client, a range of
 * pages from the dataset that corresponds with what is currently visible in the grid
 * (plus a few pages above and below the visible range to allow fast scrolling).</p>
 *
 * <p>When using "select all" rows and a BufferedStore, an <code>allSelected</code> flag is set, and so all
 * records which are read into the client side cache will thenceforth be selected, and will
 * be rendered as selected in the grid.</p>
 *
 * <p><em>But records which have not been read into the cache will obviously not be available
 * when interrogating selected records. As you scroll through the dataset, and more
 * pages are read from the server, they will become available to add to the selection.</em></p>
 */
public class SpreadsheetModelSelection extends ext.selection.SelectionModel {
  /**
   * @param config @inheritDoc
   */
  public function SpreadsheetModelSelection(config:ext.grid.selection.SpreadsheetModelSelection = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set to <code>true</code> to enable selection of individual cells or a single rectangular
   * range of cells. This will provide cell range selection using click, and
   * potentially drag to select a rectangular range. You can also use "SHIFT + arrow"
   * key navigation to select a range of cells.
   * @default true
   * @see #getCellSelect()
   * @see #setCellSelect()
   */
  public native function get cellSelect():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set cellSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * Displays the configured text in the check column's header.
   *
   * <p>if <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-showHeaderCheckbox" rel="Ext.grid.selection.SpreadsheetModel-cfg-showHeaderCheckbox" class="docClass">showHeaderCheckbox</a> is <code>true</code>, the text is shown <em>above</em> the checkbox.</p>
   * @since 6.0.1
   */
  public native function get checkColumnHeaderText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkColumnHeaderText(value:String):void;

  [ExtConfig]
  /**
   * The index at which to insert the checkbox column.
   * Supported values are a numeric index, and the strings 'first' and 'last'. Only valid when set
   * <em>before</em> render.
   * @default 0
   */
  public native function get checkboxColumnIndex():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkboxColumnIndex(value:*):void;

  [ExtConfig]
  /**
   * Width of checkbox column.
   * @default 24
   */
  public native function get checkboxHeaderWidth():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkboxHeaderWidth(value:*):void;

  [ExtConfig]
  /**
   * [checkboxSelect=false]
   * Enables selection of the row via clicking on checkbox. Note: this feature will add
   * new column at position specified by <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-checkboxColumnIndex" rel="Ext.grid.selection.SpreadsheetModel-cfg-checkboxColumnIndex" class="docClass">checkboxColumnIndex</a>.
   * @default false
   */
  public native function get checkboxSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkboxSelect(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set to <code>true</code> to enable selection of columns.
   *
   * <p><strong>NOTE</strong>: This will remove sorting on header click and instead provide column
   * selection and deselection. Sorting is still available via column header menu.</p>
   * @default false
   * @see #getColumnSelect()
   * @see #setColumnSelect()
   */
  public native function get columnSelect():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set columnSelect(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set to <code>true</code> to enables cell range selection by cell dragging.
   * @default true
   * @see #getDragSelect()
   * @see #setDragSelect()
   */
  public native function get dragSelect():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set dragSelect(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This configures whether this selection model is to implement a mouse based dragging gesture to extend a <em>contiguou</em>s selection.
   *
   * <p>Note that if there are multiple, discontiguous selected rows or columns, selection extension is not available.</p>
   *
   * <p>If set, then the bottom right corner of the contiguous selection will display a drag handle. By dragging this, an extension area
   * may be defined into which the selection is extended.</p>
   *
   * <p>Upon the end of the drag, the <a href="#!/api/Ext.panel.Table-event-beforeselectionextend" rel="Ext.panel.Table-event-beforeselectionextend" class="docClass">beforeselectionextend</a> event will be fired though the
   * encapsulating grid. Event handlers may manipulate the store data in any way.</p>
   *
   * <p>Possible values for this configuration are</p>
   *
   * <ul>
   * <li> <code>"x"</code> Only allow extending the block to the left or right.</li>
   * <li> <code>"y"</code> Only allow extending the block above or below.</li>
   * <li> <code>"xy"</code> Allow extending the block in both dimensions.</li>
   * <li> <code>"both"</code> Allow extending the block in both dimensions.</li>
   * <li> <code>true</code> Allow extending the block in both dimensions.</li>
   * <li> <code>false</code> Disable the extensible feature</li>
   * <li> <code>null</code> Disable the extensible feature</li>
   * </ul>
   *
   *
   * <p>It's importante to notice that setting this to <code>"both"</code>, <code>"xy"</code> or <code>true</code> will allow you to extend the selection in both
   * directions, but only one direction at a time. It will NOT be possible to drag it diagonally.</p>
   * @default {$value: true, lazy: true}
   * @see #getExtensible()
   * @see #setExtensible()
   */
  public native function get extensible():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set extensible(value:String):void;

  [ExtConfig]
  /**
   * Width of row numbering column.
   * @default 46
   */
  public native function get rowNumbererHeaderWidth():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rowNumbererHeaderWidth(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set to <code>true</code> to enable selection of rows by clicking on a row number column.
   *
   * <p><em>Note</em>: This feature will add the row number as the first column.</p>
   * @default true
   * @see #getRowSelect()
   * @see #setRowSelect()
   */
  public native function get rowSelect():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set rowSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * Pass an instance of one of the subclasses of <a href="#!/api/Ext.grid.selection.Selection" rel="Ext.grid.selection.Selection" class="docClass">Ext.grid.selection.Selection</a>.
   */
  public native function get selected():ext.grid.selection.Selection;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selected(value:ext.grid.selection.Selection):void;

  [ExtConfig]
  /**
   * Configure as <code>false</code> to not display the header checkbox at the top of the checkbox column
   * when <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-checkboxSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-checkboxSelect" class="docClass">checkboxSelect</a> is set.
   * @default true
   */
  public native function get showHeaderCheckbox():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showHeaderCheckbox(value:Boolean):void;

  /**
   * Clears the selection.
   * @param suppressEvent Pass <code>true</code> to prevent firing the
   * <code><a href="#!/api/Ext.grid.selection.SpreadsheetModel-event-selectionchange" rel="Ext.grid.selection.SpreadsheetModel-event-selectionchange" class="docClass">selectionchange</a></code> event.
   */
  override public native function deselectAll(suppressEvent:Boolean = false):void;

  /**
   * This method is not supported by SpreadsheetModel.
   *
   * <p>To interrogate the selection use <a href="#!/api/Ext.grid.selection.SpreadsheetModel-method-getSelected" rel="Ext.grid.selection.SpreadsheetModel-method-getSelected" class="docClass">getSelected</a> which will return an instance of one
   * of the three selection types, or <code>null</code> if no selection.</p>
   *
   * <p>The three selection types are:</p>
   *
   * <ul>
   * <li> <a href="#!/api/Ext.grid.selection.Rows" rel="Ext.grid.selection.Rows" class="docClass">Ext.grid.selection.Rows</a></li>
   * <li> <a href="#!/api/Ext.grid.selection.Columns" rel="Ext.grid.selection.Columns" class="docClass">Ext.grid.selection.Columns</a></li>
   * <li> <a href="#!/api/Ext.grid.selection.Cells" rel="Ext.grid.selection.Cells" class="docClass">Ext.grid.selection.Cells</a></li>
   * </ul>
   */
  override public native function getCount():Number;

  /**
   * This method is not supported by SpreadsheetModel.
   */
  override public native function getSelectionMode():String;

  /**
   * This method is not currently supported by SpreadsheetModel.
   */
  override public native function isLocked():Boolean;

  /**
   * This method is not supported by SpreadsheetModel.
   *
   * <p>To interrogate the selection use <a href="#!/api/Ext.grid.selection.SpreadsheetModel-method-getSelected" rel="Ext.grid.selection.SpreadsheetModel-method-getSelected" class="docClass">getSelected</a> which will return an instance of one
   * of the three selection types, or <code>null</code> if no selection.</p>
   *
   * <p>The three selection types are:</p>
   *
   * <ul>
   * <li> <a href="#!/api/Ext.grid.selection.Rows" rel="Ext.grid.selection.Rows" class="docClass">Ext.grid.selection.Rows</a></li>
   * <li> <a href="#!/api/Ext.grid.selection.Columns" rel="Ext.grid.selection.Columns" class="docClass">Ext.grid.selection.Columns</a></li>
   * <li> <a href="#!/api/Ext.grid.selection.Cells" rel="Ext.grid.selection.Cells" class="docClass">Ext.grid.selection.Cells</a></li>
   * </ul>
   * @param from The start of the range to check.
   * @param to The end of the range to check.
   */
  override public native function isRangeSelected(from:*, to:*):Boolean;

  /**
   * Select all the data if possible.
   *
   * <p>If <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-rowSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-rowSelect" class="docClass">rowSelect</a> is <code>true</code>, then all <em>records</em> will be selected.</p>
   *
   * <p>If <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-cellSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-cellSelect" class="docClass">cellSelect</a> is <code>true</code>, then all <em>rendered cells</em> will be selected.</p>
   *
   * <p>If <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-columnSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-columnSelect" class="docClass">columnSelect</a> is <code>true</code>, then all <em>columns</em> will be selected.</p>
   * @param suppressEvent True to suppress any select events
   */
  override public native function selectAll(suppressEvent:Boolean):void;

  /**
   * This method is not currently supported by SpreadsheetModel.
   * @param locked True to lock, false to unlock.
   */
  override public native function setLocked(locked:Boolean):void;

  /**
   * This method is not supported by SpreadsheetModel.
   * @param selMode 'SINGLE', 'MULTI' or 'SIMPLE'.
   */
  override public native function setSelectionMode(selMode:String):void;

  /**
   * Deselects a column.
   * @param column Column to deselect.
   * @param suppressEvent Pass <code>true</code> to prevent firing the
   * <code><a href="#!/api/Ext.grid.selection.SpreadsheetModel-event-selectionchange" rel="Ext.grid.selection.SpreadsheetModel-event-selectionchange" class="docClass">selectionchange</a></code> event.
   */
  public native function deselectColumn(column:ext.grid.column.Column, suppressEvent:Boolean = false):void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-cellSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-cellSelect" class="docClass">cellSelect</a>.
   */
  public native function getCellSelect():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-columnSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-columnSelect" class="docClass">columnSelect</a>.
   */
  public native function getColumnSelect():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-dragSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-dragSelect" class="docClass">dragSelect</a>.
   */
  public native function getDragSelect():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-extensible" rel="Ext.grid.selection.SpreadsheetModel-cfg-extensible" class="docClass">extensible</a>.
   */
  public native function getExtensible():String;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-rowSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-rowSelect" class="docClass">rowSelect</a>.
   */
  public native function getRowSelect():Boolean;

  /**
   * This method allows programmatic selection of the cell range.
   *
   * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     fields  : ['name', 'email', 'phone'],
   *     data    : {
   *         items : [
   *             { name : 'Lisa',  email : 'lisa&#64;simpsons.com',  phone : '555-111-1224' },
   *             { name : 'Bart',  email : 'bart&#64;simpsons.com',  phone : '555-222-1234' },
   *             { name : 'Homer', email : 'homer&#64;simpsons.com', phone : '555-222-1244' },
   *             { name : 'Marge', email : 'marge&#64;simpsons.com', phone : '555-222-1254' }
   *         ]
   *     },
   *     proxy   : {
   *         type   : 'memory',
   *         reader : {
   *             type : 'json',
   *             root : 'items'
   *         }
   *     }
   * });
   *
   * var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     title    : 'Simpsons',
   *     store    : store,
   *     width    : 400,
   *     renderTo : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     columns  : [
   *        columns: [
   *           { text: 'Name',  dataIndex: 'name' },
   *           { text: 'Email', dataIndex: 'email', flex: 1 },
   *           { text: 'Phone', dataIndex: 'phone', width:120 },
   *           {
   *               text:'Combined', dataIndex: 'name', width : 300,
   *               renderer: function(value, metaData, record, rowIndex, colIndex, store, view) {
   *                   console.log(arguments);
   *                   return value + ' has email: ' + record.get('email');
   *               }
   *           }
   *       ],
   *     ],
   *     selType: 'spreadsheet'
   * });
   *
   * var model = grid.getSelectionModel();  // get selection model
   *
   * // We will create range of 4 cells.
   *
   * // Now set the range  and prevent rangeselect event from being fired.
   * // We can use a simple array when we have no locked columns.
   * model.selectCells([0, 0], [1, 1], true);
   * </code></pre>
   * @param rangeStart Range starting position. Can be either Cell context or a <code>[rowIndex, columnIndex]</code> numeric array.
   * <p>Note that when a numeric array is used in a locking grid, the column indices are relative to the outermost grid, encompassing locked <em>and</em> normal sides.</p>
   * @param rangeEnd Range end position. Can be either Cell context or a <code>[rowIndex, columnIndex]</code> numeric array.
   * <p>Note that when a numeric array is used in a locking grid, the column indices are relative to the outermost grid, encompassing locked <em>and</em> normal sides.</p>
   * @param suppressEvent Pass <code>true</code> to prevent firing the
   * <code><a href="#!/api/Ext.grid.selection.SpreadsheetModel-event-selectionchange" rel="Ext.grid.selection.SpreadsheetModel-event-selectionchange" class="docClass">selectionchange</a></code> event.
   */
  public native function selectCells(rangeStart:Object, rangeEnd:Object, suppressEvent:Boolean = false):void;

  /**
   * Selects a column.
   * @param column Column to select.
   * @param keepSelection Pass <code>true</code> to keep previous selection.
   * @default false
   * @param suppressEvent Pass <code>true</code> to prevent firing the
   * <code><a href="#!/api/Ext.grid.selection.SpreadsheetModel-event-selectionchange" rel="Ext.grid.selection.SpreadsheetModel-event-selectionchange" class="docClass">selectionchange</a></code> event.
   */
  public native function selectColumn(column:ext.grid.column.Column, keepSelection:Boolean = false, suppressEvent:Boolean = false):void;

  /**
   * Select one or more rows.
   * @param rows Records to select.
   * @param keepSelection Pass <code>true</code> to keep previous selection.
   * @default false
   * @param suppressEvent Pass <code>true</code> to prevent firing the
   * <code><a href="#!/api/Ext.grid.selection.SpreadsheetModel-event-selectionchange" rel="Ext.grid.selection.SpreadsheetModel-event-selectionchange" class="docClass">selectionchange</a></code> event.
   */
  public native function selectRows(rows:Object, keepSelection:Boolean = false, suppressEvent:Boolean = false):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-cellSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-cellSelect" class="docClass">cellSelect</a>.
   * @param cellSelect The new value.
   */
  public native function setCellSelect(cellSelect:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-columnSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-columnSelect" class="docClass">columnSelect</a>.
   * @param columnSelect The new value.
   */
  public native function setColumnSelect(columnSelect:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-dragSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-dragSelect" class="docClass">dragSelect</a>.
   * @param dragSelect The new value.
   */
  public native function setDragSelect(dragSelect:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-extensible" rel="Ext.grid.selection.SpreadsheetModel-cfg-extensible" class="docClass">extensible</a>.
   * @param extensible The new value.
   */
  public native function setExtensible(extensible:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.selection.SpreadsheetModel-cfg-rowSelect" rel="Ext.grid.selection.SpreadsheetModel-cfg-rowSelect" class="docClass">rowSelect</a>.
   * @param rowSelect The new value.
   */
  public native function setRowSelect(rowSelect:Boolean):void;

  /**
   * Retrieve a configuration to be used in a HeaderContainer.
   * This should be used when checkboxSelect is set to false.
   */
  protected native function getCheckboxHeaderConfig():ext.grid.column.Column;
}
}