package ext.layout.container {

[Uses("ext.layout.container.AutoLayout")]
[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Anchor", require)]
/**
 * This is a layout that enables anchoring of contained elements relative to the container's dimensions.
 * If the container is resized, all anchored items are automatically rerendered according to their
 * <code><a href="#!/api/Ext.layout.container.Anchor-cfg-anchor" rel="Ext.layout.container.Anchor-cfg-anchor" class="docClass">anchor</a></code> rules.
 *
 * <p>This class is intended to be extended or created via the <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>: 'anchor'
 * config, and should generally not need to be created directly via the new keyword.</p>
 *
 * <p>AnchorLayout does not have any direct config options (other than inherited ones). By default,
 * AnchorLayout will calculate anchor measurements based on the size of the container itself. However, the
 * container using the AnchorLayout can supply an anchoring-specific config property of <code>anchorSize</code>.</p>
 *
 * <p>If anchorSize is specifed, the layout will use it as a virtual container for the purposes of calculating
 * anchor measurements based on it instead, allowing the container to be sized independently of the anchoring
 * logic if necessary.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.Panel</a>', {
 *     width: 500,
 *     height: 400,
 *     title: "AnchorLayout Panel",
 *     layout: 'anchor',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [
 *         {
 *             xtype: 'panel',
 *             title: '75% Width and 20% Height',
 *             anchor: '75% 20%'
 *         },
 *         {
 *             xtype: 'panel',
 *             title: 'Offset -300 Width &amp; -200 Height',
 *             anchor: '-300 -200'
 *         },
 *         {
 *             xtype: 'panel',
 *             title: 'Mixed Offset and Percent',
 *             anchor: '-250 20%'
 *         }
 *     ]
 * });
 * </code></pre>
 */
public class AnchorLayout extends ext.layout.container.AutoLayout {
  /**
   * @param config @inheritDoc
   */
  public function AnchorLayout(config:ext.layout.container.AnchorLayout = null) {
    super();
  }

  [ExtConfig]
  /**
   * This configuration option is to be applied to <strong>child <code>items</code></strong> of a container managed
   * by an <a href="#!/api/Ext.layout.container.Anchor" rel="Ext.layout.container.Anchor" class="docClass">Anchor Layout</a>.
   *
   * <p>This value is what tells the layout how an item should be anchored to the container. <code>items</code>
   * added to an AnchorLayout accept an anchoring-specific config property of <strong>anchor</strong> which is a string
   * containing two values: the horizontal anchor value and the vertical anchor value (for example, '100% 50%').
   * The following types of anchor values are supported:</p>
   *
   * <ul>
   * <li><p><strong>Percentage</strong> : Any value between 1 and 100, expressed as a percentage.</p>
   *
   * <p>The first anchor is the percentage width that the item should take up within the container, and the
   * second is the percentage height.  For example:</p>
   *
   * <pre><code>// two values specified
   * anchor: '100% 50%' // render item complete width of the container and
   *                    // 1/2 height of the container
   * // one value specified
   * anchor: '100%'     // the width value; the height will default to auto
   * </code></pre></li>
   * <li><p><strong>Offsets</strong> : Any positive or negative integer value.</p>
   *
   * <p>This is a raw adjustment where the first anchor is the offset from the right edge of the container,
   * and the second is the offset from the bottom edge. For example:</p>
   *
   * <pre><code>// two values specified
   * anchor: '-50 -100' // render item the complete width of the container
   *                    // minus 50 pixels and
   *                    // the complete height minus 100 pixels.
   * // one value specified
   * anchor: '-50'      // anchor value is assumed to be the right offset value
   *                    // bottom offset will default to 0
   * </code></pre></li>
   * <li><p><strong>Sides</strong> : Valid values are <code>right</code> (or <code>r</code>) and <code>bottom</code> (or <code>b</code>).</p>
   *
   * <p>Either the container must have a fixed size or an anchorSize config value defined at render time in
   * order for these to have any effect.</p></li>
   * <li><p><strong>Mixed</strong> :</p>
   *
   * <p>Anchor values can also be mixed as needed.  For example, to render the width offset from the container
   * right edge by 50 pixels and 75% of the container's height use:</p>
   *
   * <pre><code>anchor:   '-50 75%'
   * </code></pre></li>
   * </ul>
   */
  public native function get anchor():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set anchor(value:String):void;

  [ExtConfig]
  /**
   * Default anchor for all child <strong>container</strong> items applied if no anchor or specific width is set on the child item.
   * @default '100%'
   */
  public native function get defaultAnchor():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultAnchor(value:String):void;
}
}