package ext.mixin {
import ext.Base;

[Uses("ext.mixin.Observable")]
[Mixin("ext.mixin.Observable")]
/**
 * Base class that provides a common interface for publishing events. Subclasses are
 * expected to have a property "events" which is populated as event listeners register,
 * and, optionally, a property "listeners" with configured listeners defined.
 *
 * <p><em>Note</em>: This mixin requires the constructor to be called, which is typically done
 * during the construction of your object. The Observable constructor will call
 * <a href="#!/api/Ext.mixin.Observable-method-initConfig" rel="Ext.mixin.Observable-method-initConfig" class="docClass">initConfig</a>, so it does not need to be called a second time.</p>
 *
 * <p>For example:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Employee', {
 *     mixins: ['<a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">Ext.mixin.Observable</a>'],
 *
 *     config: {
 *         name: ''
 *     },
 *
 *     constructor: function (config) {
 *         // The `listeners` property is processed to add listeners and the config
 *         // is applied to the object.
 *         this.mixins.observable.constructor.call(this, config);
 *         // Config has been initialized
 *         console.log(this.getEmployeeName());
 *     }
 * });
 * </code></pre>
 *
 * <p>This could then be used like this:</p>
 *
 * <pre><code>var newEmployee = new Employee({
 *     name: employeeName,
 *     listeners: {
 *         quit: function() {
 *             // By default, "this" will be the object that fired the event.
 *             alert(this.getName() + " has quit!");
 *         }
 *     }
 * });
 * </code></pre>
 * @see ext.mixin.Observable
 */
public interface IObservable {
  /**
   * This object holds a key for any event that has a listener. The listener may be set
   * directly on the instance, or on its class or a super class (via <a href="#!/api/Ext.mixin.Observable-static-method-observe" rel="Ext.mixin.Observable-static-method-observe" class="docClass">observe</a>) or
   * on the <a href="#!/api/Ext.app.EventBus" rel="Ext.app.EventBus" class="docClass">MVC EventBus</a>. The values of this object are truthy
   * (a non-zero number) and falsy (0 or undefined). They do not represent an exact count
   * of listeners. The value for an event is truthy if the event must be fired and is
   * falsy if there is no need to fire the event.
   *
   * <p>The intended use of this property is to avoid the expense of fireEvent calls when
   * there are no listeners. This can be particularly helpful when one would otherwise
   * have to call fireEvent hundreds or thousands of times. It is used like this:</p>
   *
   * <pre><code> if (this.hasListeners.foo) {
   *      this.fireEvent('foo', this, arg1);
   *  }
   * </code></pre>
   */
  function get hasListeners():Object;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Observable, or subclass thereof.
   * @default true
   */
  function get isObservable():Boolean;

  function set isObservable(value:Boolean):void;

  [ExtConfig]
  /**
   * A config object containing one or more event handlers to be added to this object during initialization. This
   * should be a valid listeners config object as specified in the
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> example for attaching
   * multiple handlers at once.
   *
   * <p><strong>DOM events from Ext JS <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a></strong></p>
   *
   * <p>While <em>some</em> Ext JS Component classes export selected DOM events (e.g. "click", "mouseover" etc), this is usually
   * only done when extra value can be added. For example the <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">DataView</a>'s <strong><code><a href="#!/api/Ext.view.View-event-itemclick" rel="Ext.view.View-event-itemclick" class="docClass">itemclick</a></code></strong> event passing the node clicked on. To access DOM events directly from a
   * child element of a Component, we need to specify the <code>element</code> option to identify the Component property to add a
   * DOM listener to:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *     width: 400,
   *     height: 200,
   *     dockedItems: [{
   *         xtype: 'toolbar'
   *     }],
   *     listeners: {
   *         click: {
   *             element: 'el', //bind to the underlying el property on the panel
   *             fn: function(){ console.log('click el'); }
   *         },
   *         dblclick: {
   *             element: 'body', //bind to the underlying body property on the panel
   *             fn: function(){ console.log('dblclick body'); }
   *         }
   *     }
   * });
   * </code></pre>
   * @see #setListeners()
   */
  function get listeners():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  function set listeners(value:Object):void;

  function get events():Object;

  /**
   * Alias for <a href="#!/api/Ext.mixin.Observable-method-onAfter" rel="Ext.mixin.Observable-method-onAfter" class="docClass">onAfter</a>.
   */
  function addAfterListener():void;

  /**
   * Alias for <a href="#!/api/Ext.mixin.Observable-method-onBefore" rel="Ext.mixin.Observable-method-onBefore" class="docClass">onBefore</a>.
   */
  function addBeforeListener():void;

  /**
   * The <a href="#!/api/Ext.mixin.Observable-method-on" rel="Ext.mixin.Observable-method-on" class="docClass">on</a> method is shorthand for
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>.
   *
   * <p>Appends an event handler to this object.  For example:</p>
   *
   * <pre><code>myGridPanel.on("itemclick", this.onItemClick, this);
   * </code></pre>
   *
   * <p>The method also allows for a single argument to be passed which is a config object
   * containing properties which specify multiple events. For example:</p>
   *
   * <pre><code>myGridPanel.on({
   *     cellclick: this.onCellClick,
   *     select: this.onSelect,
   *     viewready: this.onViewReady,
   *     scope: this // Important. Ensure "this" is correct during handler execution
   * });
   * </code></pre>
   *
   * <p>One can also specify options for each event handler separately:</p>
   *
   * <pre><code>myGridPanel.on({
   *     cellclick: {fn: this.onCellClick, scope: this, single: true},
   *     viewready: {fn: panel.onViewReady, scope: panel}
   * });
   * </code></pre>
   *
   * <p><em>Names</em> of methods in a specified scope may also be used:</p>
   *
   * <pre><code>myGridPanel.on({
   *     cellclick: {fn: 'onCellClick', scope: this, single: true},
   *     viewready: {fn: 'onViewReady', scope: panel}
   * });
   * </code></pre>
   * @param eventName The name of the event to listen for.
   * May also be an object who's property names are event names.
   * @param fn The method the event invokes or the <em>name</em> of
   * the method within the specified <code>scope</code>.  Will be called with arguments
   * given to <a href="#!/api/Ext.util.Observable-method-fireEvent" rel="Ext.util.Observable-method-fireEvent" class="docClass">Ext.util.Observable.fireEvent</a> plus the <code>options</code> parameter described
   * below.
   * @param scope The scope (<code>this</code> reference) in which the handler function is
   * executed. <strong>If omitted, defaults to the object which fired the event.</strong>
   * @param options An object containing handler configuration.
   * <p><strong>Note:</strong> The options object will also be passed as the last argument to every
   * event handler.</p>
   * <p>This object may contain any of the following properties:</p>
   * @param {Object} options.scope The scope (<code>this</code> reference) in which the handler function is executed. <strong>If omitted,
   *   defaults to the object which fired the event.</strong>
   * @param {Number} options.delay The number of milliseconds to delay the invocation of the handler after the event
   *   fires.
   * @param {Boolean} options.single True to add a handler to handle just the next firing of the event, and then remove
   *   itself.
   * @param {Number} options.buffer Causes the handler to be scheduled to run in an <a href="#!/api/Ext.util.DelayedTask" rel="Ext.util.DelayedTask" class="docClass">Ext.util.DelayedTask</a> delayed
   *   by the specified number of milliseconds. If the event fires again within that time,
   *   the original handler is <em>not</em> invoked, but the new handler is scheduled in its place.
   * @param {Number} options.onFrame Causes the handler to be scheduled to run at the next
   *   <a href="#!/api/Ext.Function-method-requestAnimationFrame" rel="Ext.Function-method-requestAnimationFrame" class="docClass">animation frame event</a>. If the
   *   event fires again before that time, the handler is not rescheduled - the handler
   *   will only be called once when the next animation frame is fired, with the last set
   *   of arguments passed.
   * @param {ext.util.Observable} options.target Only call the handler if the event was fired on the target Observable, <em>not</em> if the
   *   event was bubbled up from a child Observable.
   * @param {String} options.element <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a>.</strong>
   *   The name of a Component property which references an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">element</a>
   *   to add a listener to.
   * <p>  This option is useful during Component construction to add DOM event listeners to
   *   elements of <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a> which will exist only after the
   *   Component is rendered.</p>
   * <p>  For example, to add a click listener to a Panel's body:</p>
   * <pre><code>  var panel = new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *       title: 'The title',
   *       listeners: {
   *           click: this.handlePanelClick,
   *           element: 'body'
   *       }
   *   });
   * </code></pre>
   * <p>In order to remove listeners attached using the element, you'll need to reference
   * the element itself as seen below.</p>
   * <pre><code> panel.body.un(...)
   * </code></pre>
   * @param {String} [options.delegate] A simple selector to filter the event target or look for a descendant of the target.
   * <p>  The "delegate" option is only available on <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances (or
   *   when attaching a listener to a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> via a Component using the
   *   element option).</p>
   * <p>  See the <em>delegate</em> example below.</p>
   * @param {Boolean} [options.capture] When set to <code>true</code>, the listener is fired in the capture phase of the event propagation
   *  sequence, instead of the default bubble phase.
   * <p>  The <code>capture</code> option is only available on <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances (or
   *   when attaching a listener to a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> via a Component using the
   *   element option).</p>
   * @param {Boolean} [options.stopPropagation] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-stopPropagation" rel="Ext.event.Event-method-stopPropagation" class="docClass">stopPropagation</a> on the event object
   *   before firing the handler.
   * @param {Boolean} [options.preventDefault] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-preventDefault" rel="Ext.event.Event-method-preventDefault" class="docClass">preventDefault</a> on the event object
   *   before firing the handler.
   * @param {Boolean} [options.stopEvent] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-stopEvent" rel="Ext.event.Event-method-stopEvent" class="docClass">stopEvent</a> on the event object
   *   before firing the handler.
   * @param {Array} [options.args] Optional arguments to pass to the handler function. Any additional arguments
   *   passed to <a href="#!/api/Ext.util.Observable-method-fireEvent" rel="Ext.util.Observable-method-fireEvent" class="docClass">fireEvent</a> will be appended
   *   to these arguments.
   * @param {Boolean} [options.destroyable] When specified as <code>true</code>, the function returns a <code>destroyable</code> object. An object
   *   which implements the <code>destroy</code> method which removes all listeners added in this call.
   *   This syntax can be a helpful shortcut to using <a href="#!/api/Ext-method-un" rel="Ext-method-un" class="docClass">un</a>; particularly when
   *   removing multiple listeners.  <em>NOTE</em> - not compatible when using the <em>element</em>
   *   option.  See <a href="#!/api/Ext-method-un" rel="Ext-method-un" class="docClass">un</a> for the proper syntax for removing listeners added using the
   *   <em>element</em> config.
   * @default false
   * @param {Number} [options.priority] An optional numeric priority that determines the order in which event handlers
   *   are run. Event handlers with no priority will be run as if they had a priority
   *   of 0. Handlers with a higher priority will be prioritized to run sooner than
   *   those with a lower priority.  Negative numbers can be used to set a priority
   *   lower than the default. Internally, the framework uses a range of 1000 or
   *   greater, and -1000 or lesser for handlers that are intended to run before or
   *   after all others, so it is recommended to stay within the range of -999 to 999
   *   when setting the priority of event handlers in application-level code.
   *   A priority must be an integer to be valid.  Fractional values are reserved for
   *   internal framework use.
   * @param {String} [options.order] A legacy option that is provided for backward compatibility.
   *   It is recommended to use the <code>priority</code> option instead.  Available options are:
   * <ul>
   * <li><code>'before'</code>: equal to a priority of <code>100</code></li>
   * <li><code>'current'</code>: equal to a priority of <code>0</code> or default priority</li>
   * <li><code>'after'</code>: equal to a priority of <code>-100</code></li>
   * </ul>
   * @default 'current'
   * @param order A shortcut for the <code>order</code> event option.  Provided for backward compatibility.
   *   Please use the <code>priority</code> event option instead.
   * <p><strong>Combining Options</strong></p>
   * <p>Using the options argument, it is possible to combine different types of listeners:</p>
   * <p>A delayed, one-time listener.</p>
   * <pre><code>myPanel.on('hide', this.handleClick, this, {
   *     single: true,
   *     delay: 100
   * });
   * </code></pre>
   * <p><strong>Attaching multiple handlers in 1 call</strong></p>
   * <p>The method also allows for a single argument to be passed which is a config object
   * containing properties which specify multiple handlers and handler configs.</p>
   * <pre><code>grid.on({
   *     itemclick: 'onItemClick',
   *     itemcontextmenu: grid.onItemContextmenu,
   *     destroy: {
   *         fn: function () {
   *             // function called within the 'altCmp' scope instead of grid
   *         },
   *         scope: altCmp // unique scope for the destroy handler
   *     },
   *     scope: grid       // default scope - provided for example clarity
   * });
   * </code></pre>
   * <p><strong>Delegate</strong></p>
   * <p>This is a configuration option that you can pass along when registering a handler for
   * an event to assist with event delegation. By setting this configuration option
   * to a simple selector, the target element will be filtered to look for a
   * descendant of the target. For example:</p>
   * <pre><code>var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'panel',
   *     renderTo: document.body,
   *     title: 'Delegate Handler Example',
   *     frame: true,
   *     height: 220,
   *     width: 220,
   *     html: '&lt;h1 class="myTitle"&gt;BODY TITLE&lt;/h1&gt;Body content'
   * });
   * // The click handler will only be called when the click occurs on the
   * // delegate: h1.myTitle ("h1" tag with class "myTitle")
   * panel.on({
   *     click: function (e) {
   *         console.log(e.getTarget().innerHTML);
   *     },
   *     element: 'body',
   *     delegate: 'h1.myTitle'
   *  });
   * </code></pre>
   * @default 'current'
   * @return <strong>Only when the <code>destroyable</code> option is specified. </strong>
   * <p> A <code>Destroyable</code> object. An object which implements the <code>destroy</code> method which removes
   *  all listeners added in this call. For example:</p>
   * <pre><code>this.btnListeners =  = myButton.on({
   *     destroyable: true
   *     mouseover:   function() { console.log('mouseover'); },
   *     mouseout:    function() { console.log('mouseout'); },
   *     click:       function() { console.log('click'); }
   * });
   * </code></pre>
   * <p>And when those listeners need to be removed:</p>
   * <pre><code><a href="#!/api/Ext-method-destroy" rel="Ext-method-destroy" class="docClass">Ext.destroy</a>(this.btnListeners);
   * </code></pre>
   * <p>or</p>
   * <pre><code>this.btnListeners.destroy();
   * </code></pre>
   */
  function addListener(eventName:String, fn:Function, scope:Object = null, options:Object = null, order:String = 'current'):ext.Base;

  /**
   * Attach Flex-style event listener.
   * @param type the Flex event type corresponding to the Ext event to listen to.
   *   The event type has to be specified as a constant from a Flex event class.
   * @param listener  the Flex-style event listener. It receives a single parameter <code>event</code>
   *   of the corresponding Flex event class. All arguments of the Ext event handler are provided as
   *   (typed) properties of this event object.
   * @param options see <code>addListener()</code>, <code>options</code> parameter
   */
  function addEventListener(type:String, listener:Function, options:Object = null):void;

  /**
   * The addManagedListener method is used when some object (call it "A") is listening
   * to an event on another observable object ("B") and you want to remove that listener
   * from "B" when "A" is destroyed. This is not an issue when "B" is destroyed because
   * all of its listeners will be removed at that time.
   *
   * <p>Example:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo', {
   *     extend: '<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>',
   *
   *     initComponent: function () {
   *         this.addManagedListener(MyApp.SomeGlobalSharedMenu, 'show', this.doSomething);
   *         this.callParent();
   *     }
   * });
   * </code></pre>
   *
   * <p>As you can see, when an instance of Foo is destroyed, it ensures that the 'show'
   * listener on the menu (<code>MyApp.SomeGlobalSharedMenu</code>) is also removed.</p>
   *
   * <p>As of version 5.1 it is no longer necessary to use this method in most cases because
   * listeners are automatically managed if the scope object provided to
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> is an Observable instance.
   * However, if the observable instance and scope are not the same object you
   * still need to use <code>mon</code> or <code>addManagedListener</code> if you want the listener to be
   * managed.</p>
   * @param item The item to which to add a listener/listeners.
   * @param ename The event name, or an object containing event name properties.
   * @param fn If the <code>ename</code> parameter was an event
   * name, this is the handler function or the name of a method on the specified
   * <code>scope</code>.
   * @param scope If the <code>ename</code> parameter was an event name, this is the scope (<code>this</code> reference)
   * in which the handler function is executed.
   * @param options If the <code>ename</code> parameter was an event name, this is the
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> options.
   * @return <strong>Only when the <code>destroyable</code> option is specified. </strong>
   * <p> A <code>Destroyable</code> object. An object which implements the <code>destroy</code> method which removes all listeners added in this call. For example:</p>
   * <pre><code>this.btnListeners = myButton.mon({
   *     destroyable: true
   *     mouseover:   function() { console.log('mouseover'); },
   *     mouseout:    function() { console.log('mouseout'); },
   *     click:       function() { console.log('click'); }
   * });
   * </code></pre>
   * <p>And when those listeners need to be removed:</p>
   * <pre><code><a href="#!/api/Ext-method-destroy" rel="Ext-method-destroy" class="docClass">Ext.destroy</a>(this.btnListeners);
   * </code></pre>
   * <p>or</p>
   * <pre><code>this.btnListeners.destroy();
   * </code></pre>
   */
  function addManagedListener(item:*, ename:*, fn:* = undefined, scope:Object = null, options:Object = null):Object;

  /**
   * Removes all listeners for this object including the managed listeners
   */
  function clearListeners():void;

  /**
   * Removes all managed listeners for this object.
   */
  function clearManagedListeners():void;

  /**
   * Enables events fired by this Observable to bubble up an owner hierarchy by calling <code>this.getBubbleTarget()</code> if
   * present. There is no implementation in the Observable base class.
   *
   * <p>This is commonly used by Ext.Components to bubble events to owner Containers.
   * See <a href="#!/api/Ext.Component-method-getBubbleTarget" rel="Ext.Component-method-getBubbleTarget" class="docClass">Ext.Component.getBubbleTarget</a>. The default implementation in <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> returns the
   * Component's immediate owner. But if a known target is required, this can be overridden to access the
   * required target more quickly.</p>
   *
   * <p>Example:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.overrides.form.field.Base', {
   *     override: '<a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a>',
   *
   *     //  Add functionality to Field's initComponent to enable the change event to bubble
   *     initComponent: function () {
   *         this.callParent();
   *         this.enableBubble('change');
   *     }
   * });
   *
   * var myForm = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
   *     title: 'User Details',
   *     items: [{
   *         ...
   *     }],
   *     listeners: {
   *         change: function() {
   *             // Title goes red if form has been modified.
   *             myForm.header.setStyle('color', 'red');
   *         }
   *     }
   * });
   * </code></pre>
   * @param eventNames The event name to bubble, or an Array of event names.
   */
  function enableBubble(eventNames:*):void;

  [Deprecated(replacement = "fireEventedAction", since = "5.5")]
  /**
   * Fires the specified event with the passed parameters and executes a function (action).
   * By default, the action function will be executed after any "before" event handlers
   * (as specified using the <code>order</code> option of
   * <code><a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a></code>), but before any other
   * handlers are fired.  This gives the "before" handlers an opportunity to
   * cancel the event by returning <code>false</code>, and prevent the action function from
   * being called.
   *
   * <p>The action can also be configured to run after normal handlers, but before any "after"
   * handlers (as specified using the <code>order</code> event option) by passing <code>'after'</code>
   * as the <code>order</code> parameter.  This configuration gives any event handlers except
   * for "after" handlers the opportunity to cancel the event and prevent the action
   * function from being called.</p>
   * @param eventName The name of the event to fire.
   * @param args Arguments to pass to handlers and to the action function.
   * @param fn The action function.
   * @param scope The scope (<code>this</code> reference) in which the handler function is
   * executed. <strong>If omitted, defaults to the object which fired the event.</strong>
   * @param options Event options for the action function.  Accepts any
   * of the options of <code><a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a></code>
   * @param order The order to call the action function relative
   * too the event handlers (<code>'before'</code> or <code>'after'</code>).  Note that this option is
   * simply used to sort the action function relative to the event handlers by "priority".
   * An order of <code>'before'</code> is equivalent to a priority of <code>99.5</code>, while an order of
   * <code>'after'</code> is equivalent to a priority of <code>-99.5</code>.  See the <code>priority</code> option
   * of <code><a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a></code> for more details.
   * @default 'before'
   */
  function fireAction(eventName:String, args:Array, fn:Function, scope:Object = null, options:Object = null, order:String = 'before'):void;

  /**
   * Fires the specified event with the passed parameters (minus the event name, plus the <code>options</code> object passed
   * to <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>).
   *
   * <p>An event may be set to bubble up an Observable parent hierarchy (See <a href="#!/api/Ext.Component-method-getBubbleTarget" rel="Ext.Component-method-getBubbleTarget" class="docClass">Ext.Component.getBubbleTarget</a>) by
   * calling <a href="#!/api/Ext.mixin.Observable-method-enableBubble" rel="Ext.mixin.Observable-method-enableBubble" class="docClass">enableBubble</a>.</p>
   * @param eventName The name of the event to fire.
   * @param args Variable number of parameters are passed to handlers.
   * @return returns false if any of the handlers return false otherwise it returns true.
   */
  function fireEvent(eventName:String, ...args):Boolean;

  /**
   * Fires the specified event with the passed parameter list.
   *
   * <p>An event may be set to bubble up an Observable parent hierarchy (See <a href="#!/api/Ext.Component-method-getBubbleTarget" rel="Ext.Component-method-getBubbleTarget" class="docClass">Ext.Component.getBubbleTarget</a>) by
   * calling <a href="#!/api/Ext.mixin.Observable-method-enableBubble" rel="Ext.mixin.Observable-method-enableBubble" class="docClass">enableBubble</a>.</p>
   * @param eventName The name of the event to fire.
   * @param args An array of parameters which are passed to handlers.
   * @return returns false if any of the handlers return false otherwise it returns true.
   */
  function fireEventArgs(eventName:String, args:Array):Boolean;

  /**
   * Fires the specified event with the passed parameters and executes a function (action).
   * Evented Actions will automatically dispatch a 'before' event passing. This event will
   * be given a special controller that allows for pausing/resuming of the event flow.
   *
   * <p>By pausing the controller the updater and events will not run until resumed. Pausing,
   * however, will not stop the processing of any other before events.</p>
   * @param eventName The name of the event to fire.
   * @param args Arguments to pass to handlers and to the action function.
   * @param fn The action function.
   * @param scope The scope (<code>this</code> reference) in which the handler function is
   * executed. <strong>If omitted, defaults to the object which fired the event.</strong>
   * @param fnArgs Optional arguments for the action <code>fn</code>. If not
   * given, the normal <code>args</code> will be used to call <code>fn</code>. If <code>false</code> is passed, the
   * <code>args</code> are used but if the first argument is this instance it will be removed
   * from the args passed to the action function.
   */
  function fireEventedAction(eventName:String, args:Array, fn:*, scope:Object = null, fnArgs:* = undefined):void;

  /**
   * Checks to see if this object has any listeners for a specified event, or whether the event bubbles. The answer
   * indicates whether the event needs firing or not.
   * @param eventName The name of the event to check for
   * @return <code>true</code> if the event is being listened for or bubbles, else <code>false</code>
   */
  function hasListener(eventName:String):Boolean;

  /**
   * Checks if all events, or a specific event, is suspended.
   * @param event The name of the specific event to check
   * @return <code>true</code> if events are suspended
   */
  function isSuspended(event:String = null):Boolean;

  /**
   * Shorthand for <a href="#!/api/Ext.mixin.Observable-method-addManagedListener" rel="Ext.mixin.Observable-method-addManagedListener" class="docClass">addManagedListener</a>.
   * @param item The item to which to add a listener/listeners.
   * @param ename The event name, or an object containing event name properties.
   * @param fn If the <code>ename</code> parameter was an event
   * name, this is the handler function or the name of a method on the specified
   * <code>scope</code>.
   * @param scope If the <code>ename</code> parameter was an event name, this is the scope (<code>this</code> reference)
   * in which the handler function is executed.
   * @param options If the <code>ename</code> parameter was an event name, this is the
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> options.
   * @return <strong>Only when the <code>destroyable</code> option is specified. </strong>
   * <p> A <code>Destroyable</code> object. An object which implements the <code>destroy</code> method which removes all listeners added in this call. For example:</p>
   * <pre><code>this.btnListeners = myButton.mon({
   *     destroyable: true
   *     mouseover:   function() { console.log('mouseover'); },
   *     mouseout:    function() { console.log('mouseout'); },
   *     click:       function() { console.log('click'); }
   * });
   * </code></pre>
   * <p>And when those listeners need to be removed:</p>
   * <pre><code><a href="#!/api/Ext-method-destroy" rel="Ext-method-destroy" class="docClass">Ext.destroy</a>(this.btnListeners);
   * </code></pre>
   * <p>or</p>
   * <pre><code>this.btnListeners.destroy();
   * </code></pre>
   */
  function mon(item:*, ename:*, fn:* = undefined, scope:Object = null, options:Object = null):Object;

  /**
   * Shorthand for <a href="#!/api/Ext.mixin.Observable-method-removeManagedListener" rel="Ext.mixin.Observable-method-removeManagedListener" class="docClass">removeManagedListener</a>.
   * @param item The item from which to remove a listener/listeners.
   * @param ename The event name, or an object containing event name properties.
   * @param fn If the <code>ename</code> parameter was an event name, this is the handler function.
   * @param scope If the <code>ename</code> parameter was an event name, this is the scope (<code>this</code> reference)
   * in which the handler function is executed.
   */
  function mun(item:*, ename:*, fn:Function = null, scope:Object = null):void;

  /**
   * The <a href="#!/api/Ext.mixin.Observable-method-on" rel="Ext.mixin.Observable-method-on" class="docClass">on</a> method is shorthand for
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>.
   *
   * <p>Appends an event handler to this object.  For example:</p>
   *
   * <pre><code>myGridPanel.on("itemclick", this.onItemClick, this);
   * </code></pre>
   *
   * <p>The method also allows for a single argument to be passed which is a config object
   * containing properties which specify multiple events. For example:</p>
   *
   * <pre><code>myGridPanel.on({
   *     cellclick: this.onCellClick,
   *     select: this.onSelect,
   *     viewready: this.onViewReady,
   *     scope: this // Important. Ensure "this" is correct during handler execution
   * });
   * </code></pre>
   *
   * <p>One can also specify options for each event handler separately:</p>
   *
   * <pre><code>myGridPanel.on({
   *     cellclick: {fn: this.onCellClick, scope: this, single: true},
   *     viewready: {fn: panel.onViewReady, scope: panel}
   * });
   * </code></pre>
   *
   * <p><em>Names</em> of methods in a specified scope may also be used:</p>
   *
   * <pre><code>myGridPanel.on({
   *     cellclick: {fn: 'onCellClick', scope: this, single: true},
   *     viewready: {fn: 'onViewReady', scope: panel}
   * });
   * </code></pre>
   * @param eventName The name of the event to listen for.
   * May also be an object who's property names are event names.
   * @param fn The method the event invokes or the <em>name</em> of
   * the method within the specified <code>scope</code>.  Will be called with arguments
   * given to <a href="#!/api/Ext.util.Observable-method-fireEvent" rel="Ext.util.Observable-method-fireEvent" class="docClass">Ext.util.Observable.fireEvent</a> plus the <code>options</code> parameter described
   * below.
   * @param scope The scope (<code>this</code> reference) in which the handler function is
   * executed. <strong>If omitted, defaults to the object which fired the event.</strong>
   * @param options An object containing handler configuration.
   * <p><strong>Note:</strong> The options object will also be passed as the last argument to every
   * event handler.</p>
   * <p>This object may contain any of the following properties:</p>
   * @param {Object} options.scope The scope (<code>this</code> reference) in which the handler function is executed. <strong>If omitted,
   *   defaults to the object which fired the event.</strong>
   * @param {Number} options.delay The number of milliseconds to delay the invocation of the handler after the event
   *   fires.
   * @param {Boolean} options.single True to add a handler to handle just the next firing of the event, and then remove
   *   itself.
   * @param {Number} options.buffer Causes the handler to be scheduled to run in an <a href="#!/api/Ext.util.DelayedTask" rel="Ext.util.DelayedTask" class="docClass">Ext.util.DelayedTask</a> delayed
   *   by the specified number of milliseconds. If the event fires again within that time,
   *   the original handler is <em>not</em> invoked, but the new handler is scheduled in its place.
   * @param {Number} options.onFrame Causes the handler to be scheduled to run at the next
   *   <a href="#!/api/Ext.Function-method-requestAnimationFrame" rel="Ext.Function-method-requestAnimationFrame" class="docClass">animation frame event</a>. If the
   *   event fires again before that time, the handler is not rescheduled - the handler
   *   will only be called once when the next animation frame is fired, with the last set
   *   of arguments passed.
   * @param {ext.util.Observable} options.target Only call the handler if the event was fired on the target Observable, <em>not</em> if the
   *   event was bubbled up from a child Observable.
   * @param {String} options.element <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a>.</strong>
   *   The name of a Component property which references an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">element</a>
   *   to add a listener to.
   * <p>  This option is useful during Component construction to add DOM event listeners to
   *   elements of <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a> which will exist only after the
   *   Component is rendered.</p>
   * <p>  For example, to add a click listener to a Panel's body:</p>
   * <pre><code>  var panel = new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *       title: 'The title',
   *       listeners: {
   *           click: this.handlePanelClick,
   *           element: 'body'
   *       }
   *   });
   * </code></pre>
   * <p>In order to remove listeners attached using the element, you'll need to reference
   * the element itself as seen below.</p>
   * <pre><code> panel.body.un(...)
   * </code></pre>
   * @param {String} [options.delegate] A simple selector to filter the event target or look for a descendant of the target.
   * <p>  The "delegate" option is only available on <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances (or
   *   when attaching a listener to a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> via a Component using the
   *   element option).</p>
   * <p>  See the <em>delegate</em> example below.</p>
   * @param {Boolean} [options.capture] When set to <code>true</code>, the listener is fired in the capture phase of the event propagation
   *  sequence, instead of the default bubble phase.
   * <p>  The <code>capture</code> option is only available on <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances (or
   *   when attaching a listener to a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> via a Component using the
   *   element option).</p>
   * @param {Boolean} [options.stopPropagation] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-stopPropagation" rel="Ext.event.Event-method-stopPropagation" class="docClass">stopPropagation</a> on the event object
   *   before firing the handler.
   * @param {Boolean} [options.preventDefault] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-preventDefault" rel="Ext.event.Event-method-preventDefault" class="docClass">preventDefault</a> on the event object
   *   before firing the handler.
   * @param {Boolean} [options.stopEvent] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-stopEvent" rel="Ext.event.Event-method-stopEvent" class="docClass">stopEvent</a> on the event object
   *   before firing the handler.
   * @param {Array} [options.args] Optional arguments to pass to the handler function. Any additional arguments
   *   passed to <a href="#!/api/Ext.util.Observable-method-fireEvent" rel="Ext.util.Observable-method-fireEvent" class="docClass">fireEvent</a> will be appended
   *   to these arguments.
   * @param {Boolean} [options.destroyable] When specified as <code>true</code>, the function returns a <code>destroyable</code> object. An object
   *   which implements the <code>destroy</code> method which removes all listeners added in this call.
   *   This syntax can be a helpful shortcut to using <a href="#!/api/Ext-method-un" rel="Ext-method-un" class="docClass">un</a>; particularly when
   *   removing multiple listeners.  <em>NOTE</em> - not compatible when using the <em>element</em>
   *   option.  See <a href="#!/api/Ext-method-un" rel="Ext-method-un" class="docClass">un</a> for the proper syntax for removing listeners added using the
   *   <em>element</em> config.
   * @default false
   * @param {Number} [options.priority] An optional numeric priority that determines the order in which event handlers
   *   are run. Event handlers with no priority will be run as if they had a priority
   *   of 0. Handlers with a higher priority will be prioritized to run sooner than
   *   those with a lower priority.  Negative numbers can be used to set a priority
   *   lower than the default. Internally, the framework uses a range of 1000 or
   *   greater, and -1000 or lesser for handlers that are intended to run before or
   *   after all others, so it is recommended to stay within the range of -999 to 999
   *   when setting the priority of event handlers in application-level code.
   *   A priority must be an integer to be valid.  Fractional values are reserved for
   *   internal framework use.
   * @param {String} [options.order] A legacy option that is provided for backward compatibility.
   *   It is recommended to use the <code>priority</code> option instead.  Available options are:
   * <ul>
   * <li><code>'before'</code>: equal to a priority of <code>100</code></li>
   * <li><code>'current'</code>: equal to a priority of <code>0</code> or default priority</li>
   * <li><code>'after'</code>: equal to a priority of <code>-100</code></li>
   * </ul>
   * @default 'current'
   * @param order A shortcut for the <code>order</code> event option.  Provided for backward compatibility.
   *   Please use the <code>priority</code> event option instead.
   * <p><strong>Combining Options</strong></p>
   * <p>Using the options argument, it is possible to combine different types of listeners:</p>
   * <p>A delayed, one-time listener.</p>
   * <pre><code>myPanel.on('hide', this.handleClick, this, {
   *     single: true,
   *     delay: 100
   * });
   * </code></pre>
   * <p><strong>Attaching multiple handlers in 1 call</strong></p>
   * <p>The method also allows for a single argument to be passed which is a config object
   * containing properties which specify multiple handlers and handler configs.</p>
   * <pre><code>grid.on({
   *     itemclick: 'onItemClick',
   *     itemcontextmenu: grid.onItemContextmenu,
   *     destroy: {
   *         fn: function () {
   *             // function called within the 'altCmp' scope instead of grid
   *         },
   *         scope: altCmp // unique scope for the destroy handler
   *     },
   *     scope: grid       // default scope - provided for example clarity
   * });
   * </code></pre>
   * <p><strong>Delegate</strong></p>
   * <p>This is a configuration option that you can pass along when registering a handler for
   * an event to assist with event delegation. By setting this configuration option
   * to a simple selector, the target element will be filtered to look for a
   * descendant of the target. For example:</p>
   * <pre><code>var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'panel',
   *     renderTo: document.body,
   *     title: 'Delegate Handler Example',
   *     frame: true,
   *     height: 220,
   *     width: 220,
   *     html: '&lt;h1 class="myTitle"&gt;BODY TITLE&lt;/h1&gt;Body content'
   * });
   * // The click handler will only be called when the click occurs on the
   * // delegate: h1.myTitle ("h1" tag with class "myTitle")
   * panel.on({
   *     click: function (e) {
   *         console.log(e.getTarget().innerHTML);
   *     },
   *     element: 'body',
   *     delegate: 'h1.myTitle'
   *  });
   * </code></pre>
   * @default 'current'
   * @return <strong>Only when the <code>destroyable</code> option is specified. </strong>
   * <p> A <code>Destroyable</code> object. An object which implements the <code>destroy</code> method which removes
   *  all listeners added in this call. For example:</p>
   * <pre><code>this.btnListeners =  = myButton.on({
   *     destroyable: true
   *     mouseover:   function() { console.log('mouseover'); },
   *     mouseout:    function() { console.log('mouseout'); },
   *     click:       function() { console.log('click'); }
   * });
   * </code></pre>
   * <p>And when those listeners need to be removed:</p>
   * <pre><code><a href="#!/api/Ext-method-destroy" rel="Ext-method-destroy" class="docClass">Ext.destroy</a>(this.btnListeners);
   * </code></pre>
   * <p>or</p>
   * <pre><code>this.btnListeners.destroy();
   * </code></pre>
   */
  function on(eventName:String, fn:Function, scope:Object = null, options:Object = null, order:String = 'current'):ext.Base;

  /**
   * Appends an after-event handler.
   *
   * <p>Same as <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> with <code>order</code> set
   * to <code>'after'</code>.</p>
   * @param eventName The name of the event to listen for.
   * @param fn The method the event invokes.
   * @param scope The scope for <code>fn</code>.
   * @param options An object containing handler configuration.
   */
  function onAfter(eventName:String, fn:Function, scope:Object = null, options:Object = null):void;

  /**
   * Appends a before-event handler.  Returning <code>false</code> from the handler will stop the event.
   *
   * <p>Same as <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> with <code>order</code> set
   * to <code>'before'</code>.</p>
   * @param eventName The name of the event to listen for.
   * @param fn The method the event invokes.
   * @param scope The scope for <code>fn</code>.
   * @param options An object containing handler configuration.
   */
  function onBefore(eventName:String, fn:Function, scope:Object = null, options:Object = null):void;

  /**
   * Relays selected events from the specified Observable as if the events were fired by <code>this</code>.
   *
   * <p>For example if you are extending Grid, you might decide to forward some events from store.
   * So you can do this inside your initComponent:</p>
   *
   * <pre><code>this.relayEvents(this.getStore(), ['load']);
   * </code></pre>
   *
   * <p>The grid instance will then have an observable 'load' event which will be passed
   * the parameters of the store's load event and any function fired with the grid's
   * load event would have access to the grid using the this keyword (unless the event
   * is handled by a controller's control/listen event listener in which case 'this'
   * will be the controller rather than the grid).</p>
   * @param origin The Observable whose events this object is to relay.
   * @param events Array of event names to relay or an Object with key/value
   * pairs translating to ActualEventName/NewEventName respectively. For example:
   *     this.relayEvents(this, {add:'push', remove:'pop'});
   * <p>Would now redispatch the add event of this as a push event and the remove event as a pop event.</p>
   * @param prefix A common prefix to prepend to the event names. For example:
   * <pre><code>this.relayEvents(this.getStore(), ['load', 'clear'], 'store');
   * </code></pre>
   * <p>Now the grid will forward 'load' and 'clear' events of store as 'storeload' and 'storeclear'.</p>
   * @return A <code>Destroyable</code> object. An object which implements the <code>destroy</code> method which, when destroyed, removes all relayers. For example:
   * <pre><code>this.storeRelayers = this.relayEvents(this.getStore(), ['load', 'clear'], 'store');
   * </code></pre>
   * <p>Can be undone by calling</p>
   * <pre><code><a href="#!/api/Ext-method-destroy" rel="Ext-method-destroy" class="docClass">Ext.destroy</a>(this.storeRelayers);
   * </code></pre>
   * <p>or
   *     this.store.relayers.destroy();</p>
   */
  function relayEvents(origin:Object, events:*, prefix:String = null):Object;

  /**
   * Alias for <a href="#!/api/Ext.mixin.Observable-method-unAfter" rel="Ext.mixin.Observable-method-unAfter" class="docClass">unAfter</a>.
   */
  function removeAfterListener():void;

  /**
   * Alias for <a href="#!/api/Ext.mixin.Observable-method-unBefore" rel="Ext.mixin.Observable-method-unBefore" class="docClass">unBefore</a>.
   */
  function removeBeforeListener():void;

  /**
   * Removes an event handler.
   * @param eventName The type of event the handler was associated with.
   * @param fn The handler to remove. <strong>This must be a reference to the function
   * passed into the
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> call.</strong>
   * @param scope The scope originally specified for the handler. It
   * must be the same as the scope argument specified in the original call to
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">Ext.util.Observable.addListener</a> or the listener will not be removed.
   * <p><strong>Convenience Syntax</strong></p>
   * <p>You can use the <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>
   * <code>destroyable: true</code> config option in place of calling un().  For example:</p>
   * <pre><code>var listeners = cmp.on({
   *     scope: cmp,
   *     afterrender: cmp.onAfterrender,
   *     beforehide: cmp.onBeforeHide,
   *     destroyable: true
   * });
   * // Remove listeners
   * listeners.destroy();
   * // or
   * cmp.un(
   *     scope: cmp,
   *     afterrender: cmp.onAfterrender,
   *     beforehide: cmp.onBeforeHide
   * );
   * </code></pre>
   * <p><strong>Exception - DOM event handlers using the element config option</strong></p>
   * <p>You must go directly through the element to detach an event handler attached using
   * the <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> <em>element</em> option.</p>
   * <pre><code>panel.on({
   *     element: 'body',
   *     click: 'onBodyCLick'
   * });
   * panel.body.un({
   *     click: 'onBodyCLick'
   * });
   * </code></pre>
   */
  function removeListener(eventName:String, fn:Function, scope:Object = null):void;

  /**
   * Removes listeners that were added by the <a href="#!/api/Ext.mixin.Observable-method-mon" rel="Ext.mixin.Observable-method-mon" class="docClass">mon</a> method.
   * @param item The item from which to remove a listener/listeners.
   * @param ename The event name, or an object containing event name properties.
   * @param fn If the <code>ename</code> parameter was an event name, this is the handler function.
   * @param scope If the <code>ename</code> parameter was an event name, this is the scope (<code>this</code> reference)
   * in which the handler function is executed.
   */
  function removeManagedListener(item:*, ename:*, fn:Function = null, scope:Object = null):void;

  /**
   * Resumes firing of the named event(s).
   *
   * <p>After calling this method to resume events, the events will fire when requested to fire.</p>
   *
   * <p><strong>Note that if the <a href="#!/api/Ext.mixin.Observable-method-suspendEvent" rel="Ext.mixin.Observable-method-suspendEvent" class="docClass">suspendEvent</a> method is called multiple times for a certain event,
   * this converse method will have to be called the same number of times for it to resume firing.</strong></p>
   * @param eventName Multiple event names to resume.
   */
  function resumeEvent(...eventName):void;

  /**
   * Resumes firing events (see <a href="#!/api/Ext.mixin.Observable-method-suspendEvents" rel="Ext.mixin.Observable-method-suspendEvents" class="docClass">suspendEvents</a>).
   *
   * <p>If events were suspended using the <code>queueSuspended</code> parameter, then all events fired
   * during event suspension will be sent to any listeners now.</p>
   * @param discardQueue <code>true</code> to prevent any previously queued events from firing
   * while we were suspended. See <a href="#!/api/Ext.mixin.Observable-method-suspendEvents" rel="Ext.mixin.Observable-method-suspendEvents" class="docClass">suspendEvents</a>.
   */
  function resumeEvents(discardQueue:Boolean = false):void;

  /**
   * An alias for <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>.  In
   * versions prior to 5.1, <a href="#!/api/Ext.mixin.Observable-cfg-listeners" rel="Ext.mixin.Observable-cfg-listeners" class="docClass">listeners</a> had a generated setter which could
   * be called to add listeners.  In 5.1 the listeners config is not processed
   * using the config system and has no generated setter, so this method is
   * provided for backward compatibility.  The preferred way of adding listeners
   * is to use the <a href="#!/api/Ext.mixin.Observable-method-on" rel="Ext.mixin.Observable-method-on" class="docClass">on</a> method.
   * @param listeners The listeners
   */
  function setListeners(listeners:Object):void;

  /**
   * Suspends firing of the named event(s).
   *
   * <p>After calling this method to suspend events, the events will no longer fire when requested to fire.</p>
   *
   * <p><strong>Note that if this is called multiple times for a certain event, the converse method
   * <a href="#!/api/Ext.mixin.Observable-method-resumeEvent" rel="Ext.mixin.Observable-method-resumeEvent" class="docClass">resumeEvent</a> will have to be called the same number of times for it to resume firing.</strong></p>
   * @param eventName Multiple event names to suspend.
   */
  function suspendEvent(...eventName):void;

  /**
   * Suspends the firing of all events. (see <a href="#!/api/Ext.mixin.Observable-method-resumeEvents" rel="Ext.mixin.Observable-method-resumeEvents" class="docClass">resumeEvents</a>)
   * @param queueSuspended <code>true</code> to queue up suspended events to be fired
   * after the <a href="#!/api/Ext.mixin.Observable-method-resumeEvents" rel="Ext.mixin.Observable-method-resumeEvents" class="docClass">resumeEvents</a> call instead of discarding all suspended events.
   */
  function suspendEvents(queueSuspended:Boolean):void;

  /**
   * Shorthand for <a href="#!/api/Ext.mixin.Observable-method-removeListener" rel="Ext.mixin.Observable-method-removeListener" class="docClass">removeListener</a>.
   * @param eventName The type of event the handler was associated with.
   * @param fn The handler to remove. <strong>This must be a reference to the function
   * passed into the
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> call.</strong>
   * @param scope The scope originally specified for the handler. It
   * must be the same as the scope argument specified in the original call to
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">Ext.util.Observable.addListener</a> or the listener will not be removed.
   * <p><strong>Convenience Syntax</strong></p>
   * <p>You can use the <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>
   * <code>destroyable: true</code> config option in place of calling un().  For example:</p>
   * <pre><code>var listeners = cmp.on({
   *     scope: cmp,
   *     afterrender: cmp.onAfterrender,
   *     beforehide: cmp.onBeforeHide,
   *     destroyable: true
   * });
   * // Remove listeners
   * listeners.destroy();
   * // or
   * cmp.un(
   *     scope: cmp,
   *     afterrender: cmp.onAfterrender,
   *     beforehide: cmp.onBeforeHide
   * );
   * </code></pre>
   * <p><strong>Exception - DOM event handlers using the element config option</strong></p>
   * <p>You must go directly through the element to detach an event handler attached using
   * the <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> <em>element</em> option.</p>
   * <pre><code>panel.on({
   *     element: 'body',
   *     click: 'onBodyCLick'
   * });
   * panel.body.un({
   *     click: 'onBodyCLick'
   * });
   * </code></pre>
   */
  function un(eventName:String, fn:Function, scope:Object = null):void;

  /**
   * Removes a before-event handler.
   *
   * <p>Same as <a href="#!/api/Ext.mixin.Observable-method-removeListener" rel="Ext.mixin.Observable-method-removeListener" class="docClass">removeListener</a> with <code>order</code> set to <code>'after'</code>.</p>
   * @param eventName The name of the event the handler was associated with.
   * @param fn The handler to remove.
   * @param scope The scope originally specified for <code>fn</code>.
   * @param options Extra options object.
   */
  function unAfter(eventName:*, fn:*, scope:Object = null, options:Object = null):void;

  /**
   * Removes a before-event handler.
   *
   * <p>Same as <a href="#!/api/Ext.mixin.Observable-method-removeListener" rel="Ext.mixin.Observable-method-removeListener" class="docClass">removeListener</a> with <code>order</code> set to <code>'before'</code>.</p>
   * @param eventName The name of the event the handler was associated with.
   * @param fn The handler to remove.
   * @param scope The scope originally specified for <code>fn</code>.
   * @param options Extra options object.
   */
  function unBefore(eventName:*, fn:*, scope:Object = null, options:Object = null):void;
}
}