package ext.picker {
import ext.Component;

[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Native("Ext.picker.Color", require)]
/**
 * Fires when a color is selected
 * @eventType ext.picker.events.ColorPicker_colorEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.picker.events.ColorPicker_colorEvent')]
/**
 * Color picker provides a simple color palette for choosing colors. The picker can be rendered to any container. The
 * available default to a standard 40-color palette; this can be customized with the <a href="#!/api/Ext.picker.Color-property-colors" rel="Ext.picker.Color-property-colors" class="docClass">colors</a> config.
 *
 * <p>Typically you will need to implement a handler function to be notified when the user chooses a color from the picker;
 * you can register the handler using the <a href="#!/api/Ext.picker.Color-event-select" rel="Ext.picker.Color-event-select" class="docClass">select</a> event, or by implementing the <a href="#!/api/Ext.picker.Color-cfg-handler" rel="Ext.picker.Color-cfg-handler" class="docClass">handler</a> method.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">Ext.picker.Color</a>', {
 *     value: '993300',  // initial selected color
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     listeners: {
 *         select: function(picker, selColor) {
 *             alert(selColor);
 *         }
 *     }
 * });
 * </code></pre>
 */
public class ColorPicker extends ext.Component {
  /**
   * @param config @inheritDoc
   */
  public function ColorPicker(config:ext.picker.ColorPicker = null) {
    super();
  }

  [ExtConfig]
  /**
   * The CSS class to apply to the containing element.
   * @default 'x-color-picker'
   */
  override public native function get componentCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set componentCls(value:String):void;

  /**
   * An array of 6-digit color hex code strings (without the # symbol). This array can contain any number of colors,
   * and each hex code should be unique. The width of the picker is controlled via CSS by adjusting the width property
   * of the 'x-color-picker' class (or assigning a custom class), so you can balance the number of colors with the
   * width setting until the box is symmetrical.
   *
   * <p>You can override individual colors if needed:</p>
   *
   * <pre><code>var cp = new <a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">Ext.picker.Color</a>();
   * cp.colors[0] = 'FF0000';  // change the first box to red
   * </code></pre>
   *
   * <p>Or you can provide a custom array of your own for complete control:</p>
   *
   * <pre><code>var cp = new <a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">Ext.picker.Color</a>();
   * cp.colors = ['000000', '993300', '333300'];
   * </code></pre>
   * @default ['000000', '993300', '333300', '003300', '003366', '000080', '333399', '333333', '800000', 'FF6600', '808000', '008000', '008080', '0000FF', '666699', '808080', 'FF0000', 'FF9900', '99CC00', '339966', '33CCCC', '3366FF', '800080', '969696', 'FF00FF', 'FFCC00', 'FFFF00', '00FF00', '00FFFF', '00CCFF', '993366', 'C0C0C0', 'FF99CC', 'FFCC99', 'FFFF99', 'CCFFCC', 'CCFFFF', '99CCFF', 'CC99FF', 'FFFFFF']
   */
  public native function get colors():Array;

  /**
   * @private
   */
  public native function set colors(value:Array):void;

  [ExtConfig]
  /**
   * If set to true then reselecting a color that is already selected fires the <a href="#!/api/Ext.picker.Color-event-select" rel="Ext.picker.Color-event-select" class="docClass">select</a> event
   * @default false
   */
  public native function get allowReselect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowReselect(value:Boolean):void;

  [ExtConfig]
  /**
   * The DOM event that will cause a color to be selected. This can be any valid event name (dblclick, contextmenu).
   * @default 'click'
   */
  public native function get clickEvent():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clickEvent(value:String):void;

  [ExtConfig]
  /**
   * A function that will handle the select event of this picker. The handler is passed the following parameters:
   *
   * <ul>
   * <li><p><code>picker</code> : ColorPicker</p>
   *
   * <p>The <a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">picker</a>.</p></li>
   * <li><p><code>color</code> : String</p>
   *
   * <p>The 6-digit color hex code (without the # symbol).</p></li>
   * </ul>
   *
   *
   * <p>&#64;controllable</p>
   */
  public native function get handler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handler(value:*):void;

  [ExtConfig]
  /**
   * The CSS class to apply to the color picker's items
   * @default Ext.baseCSSPrefix + 'color-picker-item'
   */
  public native function get itemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemCls(value:String):void;

  [ExtConfig]
  /**
   * The scope (<code>this</code> reference) in which the <code><a href="#!/api/Ext.picker.Color-cfg-handler" rel="Ext.picker.Color-cfg-handler" class="docClass">handler</a></code> function will be called.
   *
   * <p>Defaults to this Color picker instance.</p>
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * The CSS class to apply to the selected element
   * @default 'x-color-picker-selected'
   */
  public native function get selectedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selectedCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The initial color to highlight (should be a valid 6-digit color hex code without the # symbol). Note that the hex
   * codes are case-sensitive.
   * @see #getValue()
   */
  public native function get value():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set value(value:String):void;

  /**
   * Clears any selection and sets the value to <code>null</code>.
   */
  public native function clear():void;

  /**
   * Get the currently selected color value.
   * @return value The selected value. Null if nothing is selected.
   */
  public native function getValue():String;

  /**
   * Selects the specified color in the picker (fires the <a href="#!/api/Ext.picker.Color-event-select" rel="Ext.picker.Color-event-select" class="docClass">select</a> event)
   * @param color A valid 6-digit color hex code (# will be stripped if included)
   * @param suppressEvent True to stop the select event from firing.
   * @default false
   */
  public native function select(color:String, suppressEvent:Boolean = false):void;
}
}