package ext.resizer {
import ext.dd.DragTracker;
import ext.dom.Element;
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.resizer.Resizer", require)]
/**
 * Fired before resize is allowed. Return false to cancel resize.
 * @eventType ext.resizer.events.Resizer_width_height_eEvent.BEFORE_RESIZE
 */
[Event(name = 'onBeforeResize', type = 'ext.resizer.events.Resizer_width_height_eEvent')]
/**
 * Fired after a resize.
 * @eventType ext.resizer.events.Resizer_width_height_eEvent.RESIZE
 */
[Event(name = 'onResize', type = 'ext.resizer.events.Resizer_width_height_eEvent')]
/**
 * Fires during resizing.
 * @eventType ext.resizer.events.Resizer_width_height_eEvent.RESIZE_DRAG
 */
[Event(name = 'onResizeDrag', type = 'ext.resizer.events.Resizer_width_height_eEvent')]
/**
 * Applies drag handles to an element or component to make it resizable. The drag handles are inserted into the element
 * (or component's element) and positioned absolute.
 *
 * <p>Textarea and img elements will be wrapped with an additional div because these elements do not support child nodes.
 * The original element can be accessed through the originalTarget property.</p>
 *
 * <p>Here is the list of valid resize handles:</p>
 *
 * <pre><code>Value   Description
 * ------  -------------------
 *  'n'     north
 *  's'     south
 *  'e'     east
 *  'w'     west
 *  'nw'    northwest
 *  'sw'    southwest
 *  'se'    southeast
 *  'ne'    northeast
 *  'all'   all
 * </code></pre>
 *
 * <p><p><img src="" alt="Ext.resizer.Resizer component" width="" height=""/></p></p>
 *
 * <p>Here's an example showing the creation of a typical Resizer:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.resizer.Resizer" rel="Ext.resizer.Resizer" class="docClass">Ext.resizer.Resizer</a>', {
 *     target: 'elToResize',
 *     handles: 'all',
 *     minWidth: 200,
 *     minHeight: 100,
 *     maxWidth: 500,
 *     maxHeight: 400,
 *     pinned: true
 * });
 * </code></pre>
 */
public class Resizer extends ext.mixin.Observable {
  /**
   * @param config @inheritDoc
   */
  public function Resizer(config:ext.resizer.Resizer = null) {
    super();
  }

  /**
   * Outer element for resizing behavior.
   */
  public native function get el():ext.dom.Element;

  /**
   * @private
   */
  public native function set el(value:ext.dom.Element):void;

  /**
   * Reference to the original resize target if the element of the original resize target was a
   * <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Field</a>, or an IMG or a TEXTAREA which must be wrapped in a DIV.
   */
  public native function get originalTarget():*;

  /**
   * @private
   */
  public native function set originalTarget(value:*):void;

  public native function get resizeTracker():ext.dd.DragTracker;

  /**
   * @private
   */
  public native function set resizeTracker(value:ext.dd.DragTracker):void;

  [ExtConfig]
  /**
   * An element, or a <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a> into which the resize operation must be constrained.
   */
  public native function get constrainTo():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrainTo(value:*):void;

  [ExtConfig]
  /**
   * Specify as true to update the <a href="#!/api/Ext.resizer.Resizer-cfg-target" rel="Ext.resizer.Resizer-cfg-target" class="docClass">target</a> (Element or <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>) dynamically during
   * dragging. This is <code>true</code> by default, but the <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a> class passes <code>false</code> when it is
   * configured as <a href="#!/api/Ext.Component-cfg-resizable" rel="Ext.Component-cfg-resizable" class="docClass">Ext.Component.resizable</a>.
   *
   * <p>If specified as <code>false</code>, a proxy element is displayed during the resize operation, and the <a href="#!/api/Ext.resizer.Resizer-cfg-target" rel="Ext.resizer.Resizer-cfg-target" class="docClass">target</a> is
   * updated on mouseup.</p>
   * @default true
   */
  public native function get dynamic():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dynamic(value:Boolean):void;

  [ExtConfig]
  /**
   * String consisting of the resize handles to display. Defaults to 's e se' for Elements and fixed position
   * Components. Defaults to 8 point resizing for floating Components (such as Windows). Specify either <code>'all'</code> or any
   * of <code>'n s e w ne nw se sw'</code>.
   * @default 's e se'
   */
  public native function get handles():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set handles(value:String):void;

  [ExtConfig]
  /**
   * Optional. The height to set target to in pixels
   */
  public native function get height():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set height(value:Number):void;

  [ExtConfig]
  /**
   * The increment to snap the height resize in pixels.
   * @default 0
   */
  public native function get heightIncrement():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set heightIncrement(value:Number):void;

  [ExtConfig]
  /**
   * The maximum height for the element
   * @default 10000
   */
  public native function get maxHeight():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxHeight(value:Number):void;

  [ExtConfig]
  /**
   * The maximum width for the element
   * @default 10000
   */
  public native function get maxWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxWidth(value:Number):void;

  [ExtConfig]
  /**
   * The minimum height for the element
   * @default 20
   */
  public native function get minHeight():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minHeight(value:Number):void;

  [ExtConfig]
  /**
   * The minimum width for the element
   * @default 20
   */
  public native function get minWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minWidth(value:Number):void;

  [ExtConfig]
  /**
   * True to ensure that the resize handles are always visible, false indicates resizing by cursor changes only
   * @default false
   */
  public native function get pinned():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pinned(value:Boolean):void;

  [ExtConfig]
  /**
   * True to preserve the original ratio between height and width during resize
   * @default false
   */
  public native function get preserveRatio():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preserveRatio(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The Element or Component to resize.
   * @see #getTarget()
   */
  public native function get target():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set target(value:*):void;

  [ExtConfig]
  /**
   * True for transparent handles. This is only applied at config time.
   * @default false
   */
  public native function get transparent():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set transparent(value:Boolean):void;

  [ExtConfig]
  /**
   * Optional. The width to set the target to in pixels
   */
  public native function get width():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set width(value:Number):void;

  [ExtConfig]
  /**
   * The increment to snap the width resize in pixels.
   * @default 0
   */
  public native function get widthIncrement():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set widthIncrement(value:Number):void;

  /**
   * Returns the element that was configured with the el or target config property. If a component was configured with
   * the target property then this will return the element of this component.
   *
   * <p>Textarea and img elements will be wrapped with an additional div because these elements do not support child
   * nodes. The original element can be accessed through the originalTarget property.</p>
   * @return element
   */
  public native function getEl():ext.dom.Element;

  /**
   * Returns the element or component that was configured with the target config property.
   *
   * <p>Textarea and img elements will be wrapped with an additional div because these elements do not support child
   * nodes. The original element can be accessed through the originalTarget property.</p>
   */
  public native function getTarget():*;

  /**
   * Perform a manual resize and fires the 'resize' event.
   */
  public native function resizeTo(width:Number, height:Number):void;
}
}