package ext.selection {
import ext.grid.column.Column;

[Uses("ext.selection.RowSelectionModel")]
[Uses("ext.selection.DataViewSelectionModel")]
[Uses("ext.selection.SelectionModel")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.util.StoreHolder")]
[Native("Ext.selection.CheckboxModel", require)]
/**
 * A selection model that renders a column of checkboxes that can be toggled to
 * select or deselect rows. The default mode for this selection model is MULTI.
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *       fields: ['name', 'email', 'phone'],
 *       data: [{
 *           name: 'Lisa',
 *           email: 'lisa&#64;simpsons.com',
 *           phone: '555-111-1224'
 *       }, {
 *           name: 'Bart',
 *           email: 'bart&#64;simpsons.com',
 *           phone: '555-222-1234'
 *       }, {
 *           name: 'Homer',
 *           email: 'homer&#64;simpsons.com',
 *           phone: '555-222-1244'
 *       }, {
 *           name: 'Marge',
 *           email: 'marge&#64;simpsons.com',
 *           phone: '555-222-1254'
 *       }]
 *   });
 *
 *   <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *       title: 'Simpsons',
 *       store: store,
 *       columns: [{
 *           text: 'Name',
 *           dataIndex: 'name'
 *       }, {
 *           text: 'Email',
 *           dataIndex: 'email',
 *           flex: 1
 *       }, {
 *           text: 'Phone',
 *           dataIndex: 'phone'
 *       }],
 *       height: 200,
 *       width: 400,
 *       renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *       selModel: {
 *           selType: 'checkboxmodel'
 *       }
 *   });
 * </code></pre>
 *
 * <p>The selection model will inject a header for the checkboxes in the first view
 * and according to the <a href="#!/api/Ext.selection.CheckboxModel-cfg-injectCheckbox" rel="Ext.selection.CheckboxModel-cfg-injectCheckbox" class="docClass">injectCheckbox</a> configuration.</p>
 */
public class CheckboxSelectionModel extends ext.selection.RowSelectionModel {
  /**
   * @param config @inheritDoc
   */
  public function CheckboxSelectionModel(config:ext.selection.CheckboxSelectionModel = null) {
    super();
  }

  [ExtConfig]
  /**
   * Modes of selection.
   * Valid values are <code>"SINGLE"</code>, <code>"SIMPLE"</code>, and <code>"MULTI"</code>.
   * @default 'MULTI'
   */
  override public native function get mode():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set mode(value:String):void;

  [ExtConfig]
  /**
   * True if rows can only be selected by clicking on the checkbox column, not by clicking
   * on the row itself. Note that this only refers to selection via the UI, programmatic
   * selection will still occur regardless.
   * @default false
   */
  public native function get checkOnly():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkOnly(value:Boolean):void;

  [ExtConfig]
  /**
   * ARIA label for screen readers to announce for the check column's header when it is focused.
   * Note that this label will not be visible on screen.
   * @default "Row selector"
   * @since 6.2.0
   */
  public native function get headerAriaLabel():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set headerAriaLabel(value:String):void;

  [ExtConfig]
  /**
   * ARIA description text to announce for the check column's header when it is focused,
   * <a href="#!/api/Ext.selection.CheckboxModel-cfg-showHeaderCheckbox" rel="Ext.selection.CheckboxModel-cfg-showHeaderCheckbox" class="docClass">showHeaderCheckbox</a> is shown, and all rows are selected.
   * @default "Press Space to deselect all rows"
   */
  public native function get headerDeselectText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set headerDeselectText(value:String):void;

  [ExtConfig]
  /**
   * ARIA description text to announce for the check column's header when it is focused,
   * <a href="#!/api/Ext.selection.CheckboxModel-cfg-showHeaderCheckbox" rel="Ext.selection.CheckboxModel-cfg-showHeaderCheckbox" class="docClass">showHeaderCheckbox</a> is shown, and not all rows are selected.
   * @default "Press Space to select all rows"
   * @since 6.2.0
   */
  public native function get headerSelectText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set headerSelectText(value:String):void;

  [ExtConfig]
  /**
   * Displays the configured text in the check column's header.
   *
   * <p>if <a href="#!/api/Ext.selection.CheckboxModel-cfg-showHeaderCheckbox" rel="Ext.selection.CheckboxModel-cfg-showHeaderCheckbox" class="docClass">showHeaderCheckbox</a> is <code>true</code>, the text is shown <em>above</em> the checkbox.</p>
   * @since 6.0.1
   */
  public native function get headerText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set headerText(value:String):void;

  [ExtConfig]
  /**
   * The index at which to insert the checkbox column.
   * Supported values are a numeric index, and the strings 'first' and 'last'.
   * @default 0
   */
  public native function get injectCheckbox():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set injectCheckbox(value:*):void;

  [ExtConfig]
  /**
   * ARIA description text to announce when check column cell is focused and the row
   * is selected.
   * @default "Press Space to deselect this row"
   */
  public native function get rowDeselectText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rowDeselectText(value:String):void;

  [ExtConfig]
  /**
   * ARIA description text to announce when check column cell is focused and the row
   * is not selected.
   * @default "Press Space to select this row"
   */
  public native function get rowSelectText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rowSelectText(value:String):void;

  [ExtConfig]
  /**
   * Configure as <code>false</code> to not display the header checkbox at the top of the column.
   * When the store is a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">BufferedStore</a>, this configuration will
   * not be available because the buffered data set does not always contain all data.
   * @default false
   */
  public native function get showHeaderCheckbox():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showHeaderCheckbox(value:Boolean):void;

  /**
   * Toggle between selecting all and deselecting all when clicking on
   * a checkbox header.
   */
  override protected native function onHeaderClick():void;

  /**
   * Retrieve a configuration to be used in a HeaderContainer.
   * This is called when injectCheckbox is not <code>false</code>.
   */
  public native function getHeaderConfig():ext.grid.column.Column;
}
}