package ext.sparkline {
import ext.Widget;

[Uses("ext.Widget")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.sparkline.Base", require)]
/**
 * The base class for ExtJS SparkLines. SparkLines are small, inline graphs used to visually
 * display small amounts of data. For large datasets, use the chart package.
 *
 * <p>The SparkLine subclasses accept an <a href="#!/api/Ext.sparkline.Base-cfg-values" rel="Ext.sparkline.Base-cfg-values" class="docClass">array of values</a>, and present the data in different visualizations.</p>
 *
 * <pre class='inline-example '><code>new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.Panel</a>({
 *     height: 300,
 *     width: 600,
 *     frame: true,
 *     title: 'Test Sparklines',
 *     renderTo:document.body,
 *     bodyPadding: 10,
 *
 *     // Named listeners will resolve to methods in this Panel
 *     defaultListenerScope: true,
 *
 *     // Named references will be collected, and can be access from this Panel
 *     referenceHolder: true,
 *
 *     items: [{
 *         reference: 'values',
 *         xtype: 'textfield',
 *         fieldLabel: 'Values',
 *         validator: function(v) {
 *             var result = [];
 *
 *             v = v.replace(/\s/g, '');
 *             v = v.replace(/,$/, '');
 *             v = v.split(',');
 *             for (var i = 0; i &lt; v.length; i++) {
 *                 if (!<a href="#!/api/Ext-method-isNumeric" rel="Ext-method-isNumeric" class="docClass">Ext.isNumeric</a>(v[i])) {
 *                     return 'Value must be a comma separated array of numbers';
 *                 }
 *                 result.push(parseInt(v[i], 10));
 *             }
 *             this.values = result;
 *             return true;
 *         },
 *         listeners: {
 *             change: 'onTypeChange',
 *             buffer: 500,
 *             afterrender: {
 *                 fn: 'afterTypeRender',
 *                 single: true
 *             }
 *         }
 *     }, {
 *         reference: 'type',
 *         xtype: 'combobox',
 *         fieldLabel: 'Type',
 *         store: [
 *             ['sparklineline',     'Line'],
 *             ['sparklinebox',      'Box'],
 *             ['sparklinebullet',   'Bullet'],
 *             ['sparklinediscrete', 'Discrete'],
 *             ['sparklinepie',      'Pie'],
 *             ['sparklinetristate', 'TriState']
 *         ],
 *         value: 'sparklineline',
 *         listeners: {
 *             change: 'onTypeChange',
 *             buffer: 500
 *         }
 *     }],
 *
 *     // Start with a line plot.
 *     afterTypeRender: function(typeField) {
 *         typeField.setValue('6,10,4,-3,7,2');
 *     },
 *
 *     onTypeChange: function() {
 *         var me = this,
 *             refs = me.getReferences(),
 *             config;
 *
 *         if (me.sparkLine) {
 *             me.remove(me.sparkLine, true);
 *         }
 *         config = {
 *             xtype: refs.type.getValue(),
 *             values: refs.values.values,
 *             height: 25,
 *             width: 100
 *         };
 *        me.sparkLine = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>(config);
 *         me.add(me.sparkLine);
 *
 *         // Put under fields
 *         me.sparkLine.el.dom.style.marginLeft = refs.type.labelEl.getWidth() + 'px';
 *     }
 * });
 * </code></pre>
 */
public class SparklineBase extends ext.Widget {
  /**
   * @param config @inheritDoc
   */
  public function SparklineBase(config:ext.sparkline.SparklineBase = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Set to <code>true</code> to disable mouseover tooltips.
   * @default false
   * @see #getDisableTooltips()
   * @see #setDisableTooltips()
   */
  public native function get disableTooltips():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disableTooltips(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The hex value for the highlight color to use when mouseing over a graph segment.
   * @default null
   * @see #getHighlightColor()
   * @see #setHighlightColor()
   */
  public native function get highlightColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set highlightColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * How much to lighten the highlight color by when mouseing over a graph segment.
   * @default 0.1
   * @see #getHighlightLighten()
   * @see #setHighlightLighten()
   */
  public native function get highlightLighten():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set highlightLighten(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The hex value for line colors in graphs which display lines (<a href="#!/api/Ext.sparkline.Box" rel="Ext.sparkline.Box" class="docClass">Box</a>, <a href="#!/api/Ext.sparkline.Discrete" rel="Ext.sparkline.Discrete" class="docClass">Discrete and {&#64;link Ext.sparkline.Line Line</a>).
   * @default #157fcc
   * @see #getLineColor()
   * @see #setLineColor()
   */
  public native function get lineColor():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set lineColor(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An XTemplate used to display the value or values in a tooltip when hovering over a Sparkline.
   *
   * <p>The implemented subclases all define their own <code>tipTpl</code>, but it can be overridden.</p>
   * @see #getTipTpl()
   * @see #setTipTpl()
   */
  public native function get tipTpl():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tipTpl(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A string to prepend to each field displayed in a tooltip.
   * @default ''
   * @see #getTooltipPrefix()
   * @see #setTooltipPrefix()
   */
  public native function get tooltipPrefix():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tooltipPrefix(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Null values will not have a tooltip displayed.
   * @default true
   * @see #getTooltipSkipNull()
   * @see #setTooltipSkipNull()
   */
  public native function get tooltipSkipNull():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tooltipSkipNull(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A string to append to each field displayed in a tooltip.
   * @default ''
   * @see #getTooltipSuffix()
   * @see #setTooltipSuffix()
   */
  public native function get tooltipSuffix():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tooltipSuffix(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An array of numbers which define the chart.
   * @see #getValues()
   * @see #setValues()
   */
  public native function get values():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set values(value:Array):void;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-disableTooltips" rel="Ext.sparkline.Base-cfg-disableTooltips" class="docClass">disableTooltips</a>.
   */
  public native function getDisableTooltips():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-highlightColor" rel="Ext.sparkline.Base-cfg-highlightColor" class="docClass">highlightColor</a>.
   */
  public native function getHighlightColor():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-highlightLighten" rel="Ext.sparkline.Base-cfg-highlightLighten" class="docClass">highlightLighten</a>.
   */
  public native function getHighlightLighten():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-lineColor" rel="Ext.sparkline.Base-cfg-lineColor" class="docClass">lineColor</a>.
   */
  public native function getLineColor():String;

  /**
   * Return a region id for a given x/y co-ordinate
   */
  public native function getRegion():void;

  /**
   * Fetch the HTML to display as a tooltip
   */
  public native function getRegionTooltip(region:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-tipTpl" rel="Ext.sparkline.Base-cfg-tipTpl" class="docClass">tipTpl</a>.
   */
  public native function getTipTpl():*;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-tooltipPrefix" rel="Ext.sparkline.Base-cfg-tooltipPrefix" class="docClass">tooltipPrefix</a>.
   */
  public native function getTooltipPrefix():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-tooltipSkipNull" rel="Ext.sparkline.Base-cfg-tooltipSkipNull" class="docClass">tooltipSkipNull</a>.
   */
  public native function getTooltipSkipNull():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-tooltipSuffix" rel="Ext.sparkline.Base-cfg-tooltipSuffix" class="docClass">tooltipSuffix</a>.
   */
  public native function getTooltipSuffix():String;

  /**
   * Returns the value of <a href="#!/api/Ext.sparkline.Base-cfg-values" rel="Ext.sparkline.Base-cfg-values" class="docClass">values</a>.
   */
  public native function getValues():Array;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-disableTooltips" rel="Ext.sparkline.Base-cfg-disableTooltips" class="docClass">disableTooltips</a>.
   * @param disableTooltips The new value.
   */
  public native function setDisableTooltips(disableTooltips:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-highlightColor" rel="Ext.sparkline.Base-cfg-highlightColor" class="docClass">highlightColor</a>.
   * @param highlightColor The new value.
   */
  public native function setHighlightColor(highlightColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-highlightLighten" rel="Ext.sparkline.Base-cfg-highlightLighten" class="docClass">highlightLighten</a>.
   * @param highlightLighten The new value.
   */
  public native function setHighlightLighten(highlightLighten:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-lineColor" rel="Ext.sparkline.Base-cfg-lineColor" class="docClass">lineColor</a>.
   * @param lineColor The new value.
   */
  public native function setLineColor(lineColor:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-tipTpl" rel="Ext.sparkline.Base-cfg-tipTpl" class="docClass">tipTpl</a>.
   * @param tipTpl The new value.
   */
  public native function setTipTpl(tipTpl:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-tooltipPrefix" rel="Ext.sparkline.Base-cfg-tooltipPrefix" class="docClass">tooltipPrefix</a>.
   * @param tooltipPrefix The new value.
   */
  public native function setTooltipPrefix(tooltipPrefix:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-tooltipSkipNull" rel="Ext.sparkline.Base-cfg-tooltipSkipNull" class="docClass">tooltipSkipNull</a>.
   * @param tooltipSkipNull The new value.
   */
  public native function setTooltipSkipNull(tooltipSkipNull:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-tooltipSuffix" rel="Ext.sparkline.Base-cfg-tooltipSuffix" class="docClass">tooltipSuffix</a>.
   * @param tooltipSuffix The new value.
   */
  public native function setTooltipSuffix(tooltipSuffix:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.sparkline.Base-cfg-values" rel="Ext.sparkline.Base-cfg-values" class="docClass">values</a>.
   * @param values The new value.
   */
  public native function setValues(values:Array):void;
}
}