package ext.tip {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * Provides attractive and customizable tooltips for any element. The QuickTips
 * singleton is used to configure and manage tooltips globally for multiple elements
 * in a generic manner.  To create individual tooltips with maximum customizability,
 * you should consider either <a href="#!/api/Ext.tip.Tip" rel="Ext.tip.Tip" class="docClass">Ext.tip.Tip</a> or <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">Ext.tip.ToolTip</a>.
 *
 * <p>Quicktips can be configured via tag attributes directly in markup, or by
 * registering quick tips programmatically via the <a href="#!/api/Ext.tip.QuickTipManager-method-register" rel="Ext.tip.QuickTipManager-method-register" class="docClass">register</a> method.</p>
 *
 * <p>The singleton's instance of <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">Ext.tip.QuickTip</a> is available via
 * <a href="#!/api/Ext.tip.QuickTipManager-method-getQuickTip" rel="Ext.tip.QuickTipManager-method-getQuickTip" class="docClass">getQuickTip</a>, and supports all the methods and configuration properties
 * of <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">Ext.tip.QuickTip</a>. These settings will apply to all tooltips shown by the
 * singleton.</p>
 *
 * <p>Below is the summary of the configuration properties which can be used.
 * For detailed descriptions see the config options for the
 * <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">QuickTip</a> class</p>
 *
 * <h2>QuickTips singleton configs (all are optional)</h2>
 *
 * <ul>
 * <li><code>dismissDelay</code></li>
 * <li><code>hideDelay</code></li>
 * <li><code>maxWidth</code></li>
 * <li><code>minWidth</code></li>
 * <li><code>showDelay</code></li>
 * <li><code>trackMouse</code></li>
 * </ul>
 *
 *
 * <h2>Target element configs (optional unless otherwise noted)</h2>
 *
 * <ul>
 * <li><code>autoHide</code></li>
 * <li><code>cls</code></li>
 * <li><code>dismissDelay</code> (overrides singleton value)</li>
 * <li><code>target</code> (required)</li>
 * <li><code>text</code> (required)</li>
 * <li><code>title</code></li>
 * <li><code>width</code></li>
 * </ul>
 *
 *
 * <p>Here is an example showing how some of these config options could be used:</p>
 *
 * <pre class='inline-example '><code>// Init the singleton.  Any tag-based quick tips will start working.
 * <a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a>();
 *
 * // Apply a set of config properties to the singleton
 * <a href="#!/api/Ext-method-apply" rel="Ext-method-apply" class="docClass">Ext.apply</a>(<a href="#!/api/Ext.tip.QuickTipManager-method-getQuickTip" rel="Ext.tip.QuickTipManager-method-getQuickTip" class="docClass">Ext.tip.QuickTipManager.getQuickTip</a>(), {
 *     maxWidth: 200,
 *     minWidth: 100,
 *     showDelay: 50      // Show 50ms after entering target
 * });
 *
 * // Create a small panel to add a quick tip to
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a>', {
 *     id: 'quickTipContainer',
 *     width: 200,
 *     height: 150,
 *     style: {
 *         backgroundColor:'#000000'
 *     },
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 *
 *
 * // Manually register a quick tip for a specific element
 * <a href="#!/api/Ext.tip.QuickTipManager-method-register" rel="Ext.tip.QuickTipManager-method-register" class="docClass">Ext.tip.QuickTipManager.register</a>({
 *     target: 'quickTipContainer',
 *     title: 'My Tooltip',
 *     text: 'This tooltip was added in code',
 *     width: 100,
 *     dismissDelay: 10000 // Hide after 10 seconds hover
 * });
 * </code></pre>
 *
 * <p>To register a quick tip in markup, you simply add one or more of the valid QuickTip
 * attributes prefixed with the <strong>data-</strong> namespace.  The HTML element itself is
 * automatically set as the quick tip target. Here is the summary of supported attributes
 * (optional unless otherwise noted):</p>
 *
 * <ul>
 * <li><code>hide</code>: Specifying "user" is equivalent to setting autoHide = false.
 *  Any other value will be the same as autoHide = true.</li>
 * <li><code>qclass</code>: A CSS class to be applied to the quick tip
 *  (equivalent to the 'cls' target element config).</li>
 * <li><code>qtip (required)</code>: The quick tip text (equivalent to the 'text' target element config).</li>
 * <li><code>qtitle</code>: The quick tip title (equivalent to the 'title' target element config).</li>
 * <li><code>qwidth</code>: The quick tip width (equivalent to the 'width' target element config).</li>
 * </ul>
 *
 *
 * <p>Here is an example of configuring an HTML element to display a tooltip from markup:</p>
 *
 * <pre><code>// Add a quick tip to an HTML button
 * &lt;input type="button" value="OK" data-qtitle="OK Button" data-qwidth="100"
 *      data-qtip="This is a quick tip from markup!"&gt;&lt;/input&gt;
 * </code></pre>
 * <p>Type of singleton QuickTipManager.</p>
 * @see ext.tip.#QuickTipManager ext.tip.QuickTipManager
 */
public class SQuickTipManager extends ext.Base {
  /**
   * Destroys the QuickTips instance.
   */
  override public native function destroy(...params):void;

  /**
   * Disables quick tips globally.
   */
  public native function disable():void;

  /**
   * Enables quick tips globally.
   */
  public native function enable():void;

  /**
   * Gets the single <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">QuickTip</a> instance used to show tips
   * from all registered elements.
   */
  public native function getQuickTip():ext.tip.QuickTip;

  /**
   * Initializes the global QuickTips instance and prepare any quick tips.
   * @param autoRender True to render the QuickTips container
   * immediately to preload images.
   * @default true
   * @param config config object for the created QuickTip. By
   * default, the <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">QuickTip</a> class is instantiated, but this can
   * be changed by supplying an xtype property or a className property in this object.
   * All other properties on this object are configuration for the created component.
   */
  public native function init(autoRender:Boolean = true, config:Object = null):void;

  /**
   * Returns true if quick tips are enabled, else false.
   */
  public native function isEnabled():Boolean;

  /**
   * Configures a new quick tip instance and assigns it to a target element.
   *
   * <p>For example usage, see the <a href="#!/api/Ext.tip.QuickTipManager" rel="Ext.tip.QuickTipManager" class="docClass">Ext.tip.QuickTipManager</a> class header.</p>
   * @param config The config object with the following properties:
   * @param config.target (required) The target HTMLElement, <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or
   * id to associate with this Quicktip.  See <a href="#!/api/Ext.tip.QuickTip-cfg-target" rel="Ext.tip.QuickTip-cfg-target" class="docClass">target</a>.
   * @param config.text Tip body content.  See <a href="#!/api/Ext.tip.QuickTip-cfg-text" rel="Ext.tip.QuickTip-cfg-text" class="docClass">text</a>.
   * @param config.title Tip header.  See <a href="#!/api/Ext.tip.QuickTip-cfg-title" rel="Ext.tip.QuickTip-cfg-title" class="docClass">title</a>.
   * @param config.autoHide False to prevent the tip from automatically hiding on
   * mouseleave.  See <a href="#!/api/Ext.tip.QuickTip-cfg-autoHide" rel="Ext.tip.QuickTip-cfg-autoHide" class="docClass">autoHide</a>.
   * @param config.cls An optional extra CSS class that will be added to the tip.  See
   * <a href="#!/api/Ext.tip.QuickTip-cfg-cls" rel="Ext.tip.QuickTip-cfg-cls" class="docClass">cls</a>.
   * @param config.dismissDelay Delay in milliseconds before the tooltip automatically
   * hides (overrides singleton value).  See <a href="#!/api/Ext.tip.QuickTip-cfg-dismissDelay" rel="Ext.tip.QuickTip-cfg-dismissDelay" class="docClass">dismissDelay</a>.
   * @param config.width Tip width in pixels.  See <a href="#!/api/Ext.tip.QuickTip-cfg-width" rel="Ext.tip.QuickTip-cfg-width" class="docClass">width</a>.
   */
  public native function register(config:Object):void;

  /**
   * Alias of <a href="#!/api/Ext.tip.QuickTipManager-method-register" rel="Ext.tip.QuickTipManager-method-register" class="docClass">register</a>.
   */
  public native function tips():void;

  /**
   * Removes any registered quick tip from the target element and destroys it.
   * @param el The element from which the quick tip
   * is to be removed or ID of the element.
   */
  public native function unregister(el:*):void;
}
}