package ext.tip {
import js.HTMLElement;

[Uses("ext.tip.Tip")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.Container")]
[Uses("ext.Component")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.util.Floating")]
[Uses("ext.util.Positionable")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.Animate")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.container.DockingContainer")]
[Native("Ext.tip.ToolTip", require)]
/**
 * ToolTip is a <a href="#!/api/Ext.tip.Tip" rel="Ext.tip.Tip" class="docClass">Ext.tip.Tip</a> implementation that handles the common case of displaying a
 * tooltip when hovering over a certain element or elements on the page. It allows fine-grained
 * control over the tooltip's alignment relative to the target element or mouse, and the timing
 * of when it is automatically shown and hidden.
 *
 * <p>This implementation does <strong>not</strong> have a built-in method of automatically populating the tooltip's
 * text based on the target element; you must either configure a fixed <a href="#!/api/Ext.tip.ToolTip-cfg-html" rel="Ext.tip.ToolTip-cfg-html" class="docClass">html</a> value for each
 * ToolTip instance, or implement custom logic (e.g. in a <a href="#!/api/Ext.tip.ToolTip-event-beforeshow" rel="Ext.tip.ToolTip-event-beforeshow" class="docClass">beforeshow</a> event listener) to
 * generate the appropriate tooltip content on the fly. See <a href="#!/api/Ext.tip.QuickTip" rel="Ext.tip.QuickTip" class="docClass">Ext.tip.QuickTip</a> for a more
 * convenient way of automatically populating and configuring a tooltip based on specific DOM
 * attributes of each target element.</p>
 *
 * <h1>Basic Example</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().appendChild({
 *     id: 'clearButton',
 *     html: 'Clear Button',
 *     style: 'display:inline-block;background:#A2C841;padding:7px;cursor:pointer;'
 * });
 *
 * var tip = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">Ext.tip.ToolTip</a>', {
 *     target: 'clearButton',
 *     html: 'Press this button to clear the form'
 * });
 * </code></pre>
 *
 * <h1>Delegation</h1>
 *
 * <p>In addition to attaching a ToolTip to a single element, you can also use delegation to attach
 * one ToolTip to many elements under a common parent. This is more efficient than creating many
 * ToolTip instances. To do this, point the <a href="#!/api/Ext.tip.ToolTip-cfg-target" rel="Ext.tip.ToolTip-cfg-target" class="docClass">target</a> config to a common ancestor of all the
 * elements, and then set the <a href="#!/api/Ext.tip.ToolTip-cfg-delegate" rel="Ext.tip.ToolTip-cfg-delegate" class="docClass">delegate</a> config to a CSS selector that will select all the
 * appropriate sub-elements.</p>
 *
 * <p>When using delegation, it is likely that you will want to programmatically change the content
 * of the ToolTip based on each delegate element; you can do this by implementing a custom
 * listener for the <a href="#!/api/Ext.tip.ToolTip-event-beforeshow" rel="Ext.tip.ToolTip-event-beforeshow" class="docClass">beforeshow</a> event. Example:</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.ArrayStore" rel="Ext.data.ArrayStore" class="docClass">Ext.data.ArrayStore</a>', {
 *     fields: ['company', 'price', 'change'],
 *     data: [
 *         ['3m Co',                               71.72, 0.02],
 *         ['Alcoa Inc',                           29.01, 0.42],
 *         ['Altria Group Inc',                    83.81, 0.28],
 *         ['American Express Company',            52.55, 0.01],
 *         ['American International Group, Inc.',  64.13, 0.31],
 *         ['AT&amp;T Inc.',                           31.61, -0.48]
 *     ]
 * });
 *
 * var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Array Grid',
 *     store: store,
 *     columns: [
 *         {text: 'Company', flex: 1, dataIndex: 'company'},
 *         {text: 'Price', width: 75, dataIndex: 'price'},
 *         {text: 'Change', width: 75, dataIndex: 'change'}
 *     ],
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 *
 * var view = grid.getView();
 * var tip = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">Ext.tip.ToolTip</a>', {
 *     // The overall target element.
 *     target: view.el,
 *     // Each grid row causes its own separate show and hide.
 *     delegate: view.itemSelector,
 *     // Moving within the row should not hide the tip.
 *     trackMouse: true,
 *     // Render immediately so that tip.body can be referenced prior to the first show.
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     listeners: {
 *         // Change content dynamically depending on which element triggered the show.
 *         beforeshow: function updateTipBody(tip) {
 *             tip.update('Over company "' + view.getRecord(tip.triggerElement).get('company') + '"');
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <h1>Alignment</h1>
 *
 * <p>The following configuration properties allow control over how the ToolTip is aligned relative to
 * the target element and/or mouse pointer:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-anchor" rel="Ext.tip.ToolTip-cfg-anchor" class="docClass">anchor</a></li>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-anchorToTarget" rel="Ext.tip.ToolTip-cfg-anchorToTarget" class="docClass">anchorToTarget</a></li>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-trackMouse" rel="Ext.tip.ToolTip-cfg-trackMouse" class="docClass">trackMouse</a></li>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-mouseOffset" rel="Ext.tip.ToolTip-cfg-mouseOffset" class="docClass">mouseOffset</a></li>
 * </ul>
 *
 *
 * <h1>Showing/Hiding</h1>
 *
 * <p>The following configuration properties allow control over how and when the ToolTip is automatically
 * shown and hidden:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-autoHide" rel="Ext.tip.ToolTip-cfg-autoHide" class="docClass">autoHide</a></li>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-showDelay" rel="Ext.tip.ToolTip-cfg-showDelay" class="docClass">showDelay</a></li>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-hideDelay" rel="Ext.tip.ToolTip-cfg-hideDelay" class="docClass">hideDelay</a></li>
 * <li><a href="#!/api/Ext.tip.ToolTip-cfg-dismissDelay" rel="Ext.tip.ToolTip-cfg-dismissDelay" class="docClass">dismissDelay</a></li>
 * </ul>
 */
public class ToolTip extends ext.tip.Tip {
  /**
   * @param config @inheritDoc
   */
  public function ToolTip(config:ext.tip.ToolTip = null) {
    super();
  }

  [ExtConfig]
  /**
   * If specified, indicates that the tip should be anchored to a
   * particular side of the target element or mouse pointer ("top", "right", "bottom",
   * or "left"), with an arrow pointing back at the target or mouse pointer. If
   * <a href="#!/api/Ext.tip.ToolTip-cfg-constrainPosition" rel="Ext.tip.ToolTip-cfg-constrainPosition" class="docClass">constrainPosition</a> is enabled, this will be used as a preferred value
   * only and may be flipped as needed.
   */
  override public native function get anchor():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set anchor(value:String):void;

  [ExtConfig]
  /**
   * A string which specifies how this ToolTip is to align with regard to its
   * currentTarget by means of identifying the point of the tooltip to
   * join to the point of the target.
   *
   * <p>By default, the tooltip shows at <a href="#!/api/Ext.tip.ToolTip-cfg-mouseOffset" rel="Ext.tip.ToolTip-cfg-mouseOffset" class="docClass">mouseOffset</a> pixels from the
   * triggering pointer event. Using this config anchors the ToolTip to its target
   * instead.</p>
   *
   * <p>This may take the following forms:</p>
   *
   * <ul>
   * <li><strong>Blank</strong>: Defaults to aligning the element's top-left corner to the target's
   * bottom-left corner ("tl-bl").</li>
   * <li><strong>Two anchors</strong>: If two values from the table below are passed separated by a dash,
   * the first value is used as the element's anchor point, and the second value is
   * used as the target's anchor point.</li>
   * <li><strong>Two edge/offset descriptors:</strong> An edge/offset descriptor is an edge initial
   * (<code>t</code>/<code>r</code>/<code>b</code>/<code>l</code>) followed by a percentage along that side. This describes a
   * point to align with a similar point in the target. So <code>'t0-b0'</code> would be
   * the same as <code>'tl-bl'</code>, <code>'l0-r50'</code> would place the top left corner of this item
   * halfway down the right edge of the target item. This allows more flexibility
   * and also describes which two edges are considered adjacent when positioning a tip pointer.</li>
   * </ul>
   *
   *
   * <p>By default, tooltips are constrained to the viewport, but if <a href="#!/api/Ext.tip.ToolTip-cfg-constrain" rel="Ext.tip.ToolTip-cfg-constrain" class="docClass">constrain</a>
   * is configured as <code>false</code>, the position parameter also supports the "?"
   * character. If "?" is passed at the end of the position string, the element will
   * attempt to align as specified, but the position will be adjusted to constrain to
   * the viewport if necessary. Note that the element being aligned might be swapped to
   * align to a different position than that specified in order to enforce the viewport
   * constraints. Following are all of the supported anchor positions:</p>
   *
   * <pre><code> Value  Description
   *  -----  -----------------------------
   *  tl     The top left corner
   *  t      The center of the top edge
   *  tr     The top right corner
   *  l      The center of the left edge
   *  c      The center
   *  r      The center of the right edge
   *  bl     The bottom left corner
   *  b      The center of the bottom edge
   *  br     The bottom right corner
   * </code></pre>
   *
   * <p>Example Usage:</p>
   *
   * <pre><code>// align the top left corner of the tooltip with the top right corner of its target.
   * defaultAlign: 'tl-tr'
   *
   * // align the bottom right corner of the tooltip with the center left edge of its target.
   * defaultAlign: 'br-l'
   *
   * // align the top center of the tooltip with the bottom left corner of its target.
   * defaultAlign: 't-bl'
   *
   * // align the 25% point on the bottom edge of this tooltip
   * // with the 75% point on the top edge of its target.
   * defaultAlign: 'b25-c75'
   * </code></pre>
   * @default 'bl-tl'
   */
  override public native function get defaultAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set defaultAlign(value:String):void;

  /**
   * When a ToolTip is configured with the <code><a href="#!/api/Ext.tip.ToolTip-cfg-delegate" rel="Ext.tip.ToolTip-cfg-delegate" class="docClass">delegate</a></code>
   * option to cause selected child elements of the <code><a href="#!/api/Ext.tip.ToolTip-cfg-target" rel="Ext.tip.ToolTip-cfg-target" class="docClass">target</a></code>
   * Element to each trigger a separate show event, this property is set to
   * the DOM element which triggered the show.
   */
  public native function get triggerElement():js.HTMLElement;

  /**
   * @private
   */
  public native function set triggerElement(value:js.HTMLElement):void;

  [ExtConfig]
  /**
   * True to anchor the tooltip to the target element, false to anchor it relative to the mouse coordinates.
   * When <code>anchorToTarget</code> is true, use <code><a href="#!/api/Ext.tip.ToolTip-cfg-defaultAlign" rel="Ext.tip.ToolTip-cfg-defaultAlign" class="docClass">defaultAlign</a></code> to control tooltip alignment to the
   * target element.  When <code>anchorToTarget</code> is false, use <code><a href="#!/api/Ext.tip.ToolTip-cfg-anchor" rel="Ext.tip.ToolTip-cfg-anchor" class="docClass">anchor</a></code> instead to control alignment.
   * @default true
   */
  public native function get anchorToTarget():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set anchorToTarget(value:Boolean):void;

  [ExtConfig]
  /**
   * True to automatically hide the tooltip after the
   * mouse exits the target element or after the <code><a href="#!/api/Ext.tip.ToolTip-cfg-dismissDelay" rel="Ext.tip.ToolTip-cfg-dismissDelay" class="docClass">dismissDelay</a></code>
   * has expired if set.  If <code><a href="#!/api/Ext.tip.ToolTip-cfg-closable" rel="Ext.tip.ToolTip-cfg-closable" class="docClass">closable</a> = true</code>
   * a close tool button will be rendered into the tooltip header.
   * @default true
   */
  public native function get autoHide():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoHide(value:Boolean):void;

  [ExtConfig]
  /**
   * A <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">DomQuery</a> simple selector which allows selection of individual elements within the
   * <code><a href="#!/api/Ext.tip.ToolTip-cfg-target" rel="Ext.tip.ToolTip-cfg-target" class="docClass">target</a></code> element to trigger showing and hiding the ToolTip as the mouse moves within the
   * target. See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for information about simple selectors.
   *
   * <p>When specified, the child element of the target which caused a show event is placed into the
   * <code><a href="#!/api/Ext.tip.ToolTip-property-triggerElement" rel="Ext.tip.ToolTip-property-triggerElement" class="docClass">triggerElement</a></code> property before the ToolTip is shown.</p>
   *
   * <p>This may be useful when a Component has regular, repeating elements in it, each of which need a
   * ToolTip which contains information specific to that element.</p>
   *
   * <p>See the delegate example in class documentation of <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">Ext.tip.ToolTip</a>.</p>
   */
  public native function get delegate():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set delegate(value:String):void;

  [ExtConfig]
  /**
   * Delay in milliseconds before the tooltip automatically hides. To disable automatic hiding, set
   * dismissDelay = 0.
   * @default 5000
   */
  public native function get dismissDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dismissDelay(value:Number):void;

  [ExtConfig]
  /**
   * The number of milliseconds for the <code>fadeOut</code> animation. Only valid if <code>hideAction</code>
   * is set to <code>fadeOut</code>.
   * @default 1000
   */
  public native function get fadeOutDuration():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fadeOutDuration(value:Number):void;

  [ExtConfig]
  /**
   * The method to use to hide the tooltip. Another useful method for this is <code>fadeOut</code>.
   * @default "hide"
   */
  public native function get hideAction():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideAction(value:String):void;

  [ExtConfig]
  /**
   * Delay in milliseconds after the mouse exits the target element but before the tooltip actually hides.
   * Set to 0 for the tooltip to hide immediately.
   * @default 200
   */
  public native function get hideDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideDelay(value:Number):void;

  [ExtConfig]
  /**
   * An XY offset from the mouse position where the tooltip should be shown.
   * @default [15, 18]
   */
  public native function get mouseOffset():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set mouseOffset(value:Array):void;

  [ExtConfig]
  /**
   * Delay in milliseconds before the tooltip displays after the mouse enters the target element.
   * @default 500
   */
  public native function get showDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showDelay(value:Number):void;

  [ExtConfig]
  /**
   * On touch platforms, if <a href="#!/api/Ext.tip.ToolTip-cfg-showOnTap" rel="Ext.tip.ToolTip-cfg-showOnTap" class="docClass">showOnTap</a> is <code>true</code>, a tap on the target shows the tip.
   * In this case any <a href="#!/api/Ext.tip.ToolTip-cfg-showDelay" rel="Ext.tip.ToolTip-cfg-showDelay" class="docClass">showDelay</a> is ignored.
   *
   * <p>This is useful for adding tips on elements which do not have tap listeners. It would
   * not be appropriate for a ToolTip on a <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a>.</p>
   * @default false
   */
  public native function get showOnTap():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showOnTap(value:Object):void;

  [ExtConfig]
  /**
   * The target element or string id to monitor for mouseover events to trigger
   * showing this ToolTip.
   * @see #setTarget()
   */
  public native function get target():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set target(value:*):void;

  [ExtConfig]
  /**
   * When <a href="#!/api/Ext.tip.ToolTip-cfg-anchorToTarget" rel="Ext.tip.ToolTip-cfg-anchorToTarget" class="docClass">anchorToTarget</a> is being used to position this tip relative to its target element,
   * this may be used as an extra XY offset from the target element.
   * @default [0, 0]
   */
  public native function get targetOffset():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set targetOffset(value:Array):void;

  [ExtConfig]
  /**
   * True to have the tooltip follow the mouse as it moves over the target element.
   * @default false
   */
  public native function get trackMouse():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set trackMouse(value:Boolean):void;

  /**
   * Ensures this tooltip at the current event target XY position.
   */
  override protected native function afterShow(animateTarget:* = undefined, callback:Function = null, scope:Object = null):void;

  /**
   * Hides this tooltip if visible.
   * @param animateTarget <strong>only valid for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components
   * such as <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s or <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">ToolTip</a>s, or regular Components which have
   * been configured with <code>floating: true</code>.</strong>. The target to which the Component should animate while hiding.
   * @default null
   * @param callback A callback function to call after the Component is hidden.
   * @param scope The scope (<code>this</code> reference) in which the callback is executed.
   * Defaults to this Component.
   */
  override public native function hide(animateTarget:Object = null, callback:Function = null, scope:Object = null):void;

  /**
   * Shows this ToolTip aligned to the passed Component or element according to the <a href="#!/api/Ext.tip.ToolTip-cfg-anchor" rel="Ext.tip.ToolTip-cfg-anchor" class="docClass">anchor</a> config.
   * @param component The <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> to show the component by.
   * @param position Alignment position as used by <a href="#!/api/Ext.util.Positionable-method-getAlignToXY" rel="Ext.util.Positionable-method-getAlignToXY" class="docClass">Ext.util.Positionable.getAlignToXY</a>.
   * Defaults to <code><a href="#!/api/Ext.Component-cfg-defaultAlign" rel="Ext.Component-cfg-defaultAlign" class="docClass">defaultAlign</a></code>. See <a href="#!/api/Ext.Component-method-alignTo" rel="Ext.Component-method-alignTo" class="docClass">alignTo</a> for possible values.
   * @param offsets Alignment offsets as used by <a href="#!/api/Ext.util.Positionable-method-getAlignToXY" rel="Ext.util.Positionable-method-getAlignToXY" class="docClass">Ext.util.Positionable.getAlignToXY</a>. See <a href="#!/api/Ext.Component-method-alignTo" rel="Ext.Component-method-alignTo" class="docClass">alignTo</a> for possible values.
   */
  override public native function showBy(component:*, position:String = null, offsets:Array = null):void;

  /**
   * Binds this ToolTip to the specified element. The tooltip will be displayed when the mouse moves over the element.
   * @param target The Element, HTMLElement, or
   * ID of an element to bind to
   */
  public native function setTarget(target:*):void;
}
}