package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.Color", require)]
/**
 * Represents an RGB color and provides helper functions on it e.g. to get
 * color components in HSL color space.
 */
public class Color extends ext.Base {
  /**
   * Convenience method for creating a color.
   *
   * <p>Can be called with several different combinations of arguments:</p>
   *
   * <pre><code>// <a href="#!/api/Ext.util.Color" rel="Ext.util.Color" class="docClass">Ext.util.Color</a> is returned unchanged.
   * <a href="#!/api/Ext.util.Color-static-method-create" rel="Ext.util.Color-static-method-create" class="docClass">Ext.util.Color.create</a>(new Ext.util.color(255, 0, 0, 0));
   *
   * // CSS color string.
   * <a href="#!/api/Ext.util.Color-static-method-create" rel="Ext.util.Color-static-method-create" class="docClass">Ext.util.Color.create</a>("red");
   *
   * // Array of red, green, blue, alpha
   * <a href="#!/api/Ext.util.Color-static-method-create" rel="Ext.util.Color-static-method-create" class="docClass">Ext.util.Color.create</a>([255, 0, 0, 0]);
   *
   * // Separate arguments of red, green, blue, alpha
   * <a href="#!/api/Ext.util.Color-static-method-create" rel="Ext.util.Color-static-method-create" class="docClass">Ext.util.Color.create</a>(255, 0, 0, 0);
   *
   * // Returns black when no arguments given.
   * <a href="#!/api/Ext.util.Color-static-method-create" rel="Ext.util.Color-static-method-create" class="docClass">Ext.util.Color.create</a>();
   * </code></pre>
   * @param red Red component (0..255),
   * CSS color string or array of all components.
   * @param green Green component (0..255)
   * @param blue Blue component (0..255)
   * @param alpha Alpha component (0..1)
   * @default 1
   */
  public static native function create(red:* = undefined, green:Number = NaN, blue:Number = NaN, alpha:Number = 1):ext.util.Color;

  /**
   * Returns a flyweight instance of <a href="#!/api/Ext.util.Color" rel="Ext.util.Color" class="docClass">Ext.util.Color</a>.
   *
   * <p>Can be called with either a CSS color string or with separate
   * arguments for red, green, blue, alpha.</p>
   * @param red Red component (0..255) or CSS color string.
   * @param green Green component (0..255)
   * @param blue Blue component (0..255)
   * @param alpha Alpha component (0..1)
   * @default 1
   */
  public static native function fly(red:*, green:Number = NaN, blue:Number = NaN, alpha:Number = 1):ext.util.Color;

  /**
   * Create a new color based on the specified HSL values.
   * @param h Hue component [0..360)
   * @param s Saturation component [0..1]
   * @param l Lightness component [0..1]
   */
  public static native function fromHSL(h:Number, s:Number, l:Number):ext.util.Color;

  /**
   * Create a new color based on the specified HSV values.
   * @param h Hue component [0..360)
   * @param s Saturation component [0..1]
   * @param v Value component [0..1]
   */
  public static native function fromHSV(h:Number, s:Number, v:Number):ext.util.Color;

  /**
   * Parse the string and create a new color.
   *
   * <p>Supported formats:</p>
   *
   * <ul>
   * <li>'#rrggbb'</li>
   * <li>'#rgb', 'rgb(r,g,b)'</li>
   * <li>'rgba(r,g,b,a)'</li>
   * <li>supported CSS color names (e.g., 'black', 'white', etc).</li>
   * </ul>
   *
   *
   * <p>If the string is not recognized, fromString returns rgba(0,0,0,0).</p>
   * @param color Color as string.
   */
  public static native function fromString(color:String):ext.util.Color;

  /**
   * @param red Red component (0..255)
   * @param green Green component (0..255)
   * @param blue Blue component (0..255)
   * @param alpha Alpha component (0..1)
   * @default 1
   */
  public function Color(red:Number, green:Number, blue:Number, alpha:Number = 1) {
    super();
  }

  [ExtConfig]
  /**
   * The default factor to compute the lighter or darker color.
   * @default 0.2
   */
  public native function get lightnessFactor():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lightnessFactor(value:Number):void;

  /**
   * Returns a new color that is darker than this color in the HSL color space.
   * @param factor Darker factor (0..1).
   * @default 0.2
   */
  public native function createDarker(factor:Number = 0.2):ext.util.Color;

  /**
   * Returns a new color that is lighter than this color in the HSL color space.
   * @param factor Lighter factor (0..1).
   * @default 0.2
   */
  public native function createLighter(factor:Number = 0.2):ext.util.Color;

  /**
   * Darken this color in the HSL color space.
   * @param factor Darken factor (0..1).
   * @default 0.2
   */
  public native function darken(factor:Number = 0.2):void;

  /**
   * The the brightness of a color as defined by W3C:
   * https://www.w3.org/TR/AERT#color-contrast
   * @return The brightness, between <code>0</code> and <code>100</code>.
   */
  public native function getBrightness():Number;

  /**
   * Returns the gray value (0 to 255) of the color.
   *
   * <p>The gray value is calculated using the formula r<em>0.3 + g</em>0.59 + b*0.11.</p>
   */
  public native function getGrayscale():Number;

  /**
   * Get the equivalent HSL components of the color.
   */
  public native function getHSL():Array;

  /**
   * Get the equivalent HSV components of the color.
   */
  public native function getHSV():Array;

  /**
   * Lighten this color in the HSL color space.
   * @param factor Lighten factor (0..1).
   * @default 0.2
   */
  public native function lighten(factor:Number = 0.2):void;

  /**
   * Parse the string and set the current color.
   *
   * <p>Supported formats:</p>
   *
   * <ul>
   * <li>'#rrggbb'</li>
   * <li>'#rgb', 'rgb(r,g,b)'</li>
   * <li>'rgba(r,g,b,a)'</li>
   * <li>supported CSS color names (e.g., 'black', 'white', etc).</li>
   * </ul>
   *
   *
   * <p>If the string is not recognized, setFromString returns rgba(0,0,0,0).</p>
   * @param Color Color as string.
   * @return this
   */
  public native function setFromString(Color:String):Object;

  /**
   * Set current color based on the specified HSL values.
   * @param h Hue component [0..360)
   * @param s Saturation component [0..1]
   * @param l Lightness component [0..1]
   */
  public native function setHSL(h:Number, s:Number, l:Number):ext.util.Color;

  /**
   * Set current color based on the specified HSV values.
   * @param h Hue component [0..360)
   * @param s Saturation component [0..1]
   * @param v Value component [0..1]
   */
  public native function setHSV(h:Number, s:Number, v:Number):ext.util.Color;

  /**
   * Get this color in hexadecimal format.
   * @return The color in hexadecimal format.
   */
  public native function toHex(color:Object):String;

  /**
   * toString() returns a color in hex format ('#rrggbb') if the alpha is 1. If the
   * alpha is less than one, toString() returns the color in RGBA format ('rgba(255,0,0,0.3)').
   */
  public native function toString():String;
}
}