package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.DelimitedValue", require)]
/**
 * This base class contains utility methods for dealing with formats such as CSV (Comma
 * Separated Values) as specified in <a href="http://tools.ietf.org/html/rfc4180">RFC 4180</a>.
 *
 * <p>The base class implements the mechanics and is governed by these config options:</p>
 *
 * <ul>
 * <li><code><a href="#!/api/Ext.util.DelimitedValue-cfg-delimiter" rel="Ext.util.DelimitedValue-cfg-delimiter" class="docClass">delimiter</a></code></li>
 * <li><code><a href="#!/api/Ext.util.DelimitedValue-cfg-lineBreak" rel="Ext.util.DelimitedValue-cfg-lineBreak" class="docClass">lineBreak</a></code></li>
 * <li><code><a href="#!/api/Ext.util.DelimitedValue-cfg-quote" rel="Ext.util.DelimitedValue-cfg-quote" class="docClass">quote</a></code></li>
 * </ul>
 *
 *
 * <p>These options affect the <code><a href="#!/api/Ext.util.DelimitedValue-method-encode" rel="Ext.util.DelimitedValue-method-encode" class="docClass">encode</a></code> and <code><a href="#!/api/Ext.util.DelimitedValue-method-decode" rel="Ext.util.DelimitedValue-method-decode" class="docClass">decode</a></code> methods.
 * When <em>decoding</em>, however, <code><a href="#!/api/Ext.util.DelimitedValue-cfg-lineBreak" rel="Ext.util.DelimitedValue-cfg-lineBreak" class="docClass">lineBreak</a></code> is ignored and instead each line can
 * be separated by any standard line terminator character or character sequence:</p>
 *
 * <ul>
 * <li><code>\u000a</code></li>
 * <li><code>\u000d</code></li>
 * <li><code>\u000d\u000a</code></li>
 * </ul>
 *
 *
 * <p>Strings which contain the <a href="#!/api/Ext.util.DelimitedValue-cfg-delimiter" rel="Ext.util.DelimitedValue-cfg-delimiter" class="docClass">delimiter</a> character are quoted using the
 * <a href="#!/api/Ext.util.DelimitedValue-cfg-quote" rel="Ext.util.DelimitedValue-cfg-quote" class="docClass">quote</a> character, and any internal <a href="#!/api/Ext.util.DelimitedValue-cfg-quote" rel="Ext.util.DelimitedValue-cfg-quote" class="docClass">quote</a> characters are doubled.</p>
 *
 * <p><em>Important</em>
 * While the primary use case is to encode strings, other atomic data types can be encoded
 * as values within a line such as:</p>
 *
 * <ul>
 * <li>Number</li>
 * <li>Boolean</li>
 * <li>Date (encoded as an <a href="http://www.iso.org/iso/home/standards/iso8601.htm">ISO 8601</a> date string.)</li>
 * <li>null (encoded as an empty string.)</li>
 * <li>undefined (encoded as an empty string.)</li>
 * </ul>
 *
 *
 * <p>Not that when <em>decoding</em>, all data is read as strings. This class does not convert
 * incoming data. To do that, use an <a href="#!/api/Ext.data.reader.Array" rel="Ext.data.reader.Array" class="docClass">ArrayReader</a>.</p>
 *
 * <p>See <code><a href="#!/api/Ext.util.CSV" rel="Ext.util.CSV" class="docClass">Ext.util.CSV</a></code> and  <code><a href="#!/api/Ext.util.TSV" rel="Ext.util.TSV" class="docClass">Ext.util.TSV</a></code> for pre-configured instances.</p>
 */
public class DelimitedValue extends ext.Base {
  /**
   * @param config @inheritDoc
   */
  public function DelimitedValue(config:ext.util.DelimitedValue = null) {
    super();
  }

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">format</a> to use for dates
   * @default 'C'
   */
  public native function get dateFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dateFormat(value:String):void;

  [ExtConfig]
  /**
   * The string used to separate the values in a row. Common values for this config
   * are comma (",") and tab ("\t"). See <code><a href="#!/api/Ext.util.CSV" rel="Ext.util.CSV" class="docClass">Ext.util.CSV</a></code> and  <code><a href="#!/api/Ext.util.TSV" rel="Ext.util.TSV" class="docClass">Ext.util.TSV</a></code>
   * for pre-configured instances of these formats.
   * @default '\t'
   */
  public native function get delimiter():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set delimiter(value:String):void;

  [ExtConfig]
  /**
   * The string used by <code><a href="#!/api/Ext.util.DelimitedValue-method-encode" rel="Ext.util.DelimitedValue-method-encode" class="docClass">encode</a></code> to separate each row. The <code><a href="#!/api/Ext.util.DelimitedValue-method-decode" rel="Ext.util.DelimitedValue-method-decode" class="docClass">decode</a></code>
   * method accepts all forms of line break.
   * @default '\n'
   */
  public native function get lineBreak():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lineBreak(value:String):void;

  [ExtConfig]
  /**
   * The character to use as to quote values that contain the special <code>delimiter</code>
   * or <code><a href="#!/api/Ext.util.DelimitedValue-cfg-lineBreak" rel="Ext.util.DelimitedValue-cfg-lineBreak" class="docClass">lineBreak</a></code> characters.
   * @default '"'
   */
  public native function get quote():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set quote(value:String):void;

  /**
   * Decodes a string of encoded values into an array of rows. Each row is an array of
   * strings.
   *
   * <p>Note that this function does not convert the string values in each column into
   * other data types. To do that, use an <a href="#!/api/Ext.data.reader.Array" rel="Ext.data.reader.Array" class="docClass">ArrayReader</a>.</p>
   *
   * <p>For example:</p>
   *
   * <pre><code><a href="#!/api/Ext.util.CSV-method-decode" rel="Ext.util.CSV-method-decode" class="docClass">Ext.util.CSV.decode</a>('"foo ""bar"", bletch",Normal String,2010-01-01T21:45:32.004Z\u000a3.141592653589793,1,false');
   * </code></pre>
   *
   * <p>produces the following array of string arrays:</p>
   *
   * <pre><code>[
   *     ['foo "bar", bletch','Normal String', '2010-01-01T21:45:32.004Z'],
   *     ['3.141592653589793', '1', 'false']
   * ]
   * </code></pre>
   * @param input The string to parse.
   * @param delimiter The column delimiter to use if the default value
   * of <a href="#!/api/Ext.util.DelimitedValue-cfg-delimiter" rel="Ext.util.DelimitedValue-cfg-delimiter" class="docClass">delimiter</a> is not desired.
   * @return An array of rows where each row is an array of Strings.
   */
  public native function decode(input:String, delimiter:String = null):*;

  /**
   * Converts a two-dimensional array into an encoded string.
   *
   * <p>For example:</p>
   *
   * <pre><code><a href="#!/api/Ext.util.CSV-method-encode" rel="Ext.util.CSV-method-encode" class="docClass">Ext.util.CSV.encode</a>([
   *     ['foo "bar", bletch', 'Normal String', new Date()],
   *     [Math.PI, 1, false]
   * ]);
   * </code></pre>
   *
   * <p>The above produces the following string:</p>
   *
   * <pre><code>'"foo ""bar"", bletch",Normal String,2010-01-01T21:45:32.004Z\u000a3.141592653589793,1,false'
   * </code></pre>
   * @param input An array of row data arrays.
   * @param delimiter The column delimiter to use if the default value
   * of <a href="#!/api/Ext.util.DelimitedValue-cfg-delimiter" rel="Ext.util.DelimitedValue-cfg-delimiter" class="docClass">delimiter</a> is not desired.
   * @return A string in which data items are separated by <a href="#!/api/Ext.util.DelimitedValue-cfg-delimiter" rel="Ext.util.DelimitedValue-cfg-delimiter" class="docClass">delimiter</a>
   * characters, and rows are separated by <a href="#!/api/Ext.util.DelimitedValue-cfg-lineBreak" rel="Ext.util.DelimitedValue-cfg-lineBreak" class="docClass">lineBreak</a> characters.
   */
  public native function encode(input:*, delimiter:String = null):String;
}
}