package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.Filter", require)]
/**
 * Represents a filter that can be applied to a <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">MixedCollection</a>. Can either simply
 * filter on a property/value pair or pass in a filter function with custom logic. Filters are always used in the
 * context of MixedCollections, though <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a>s frequently create them when filtering and searching
 * on their records. Example usage:
 *
 * <pre><code>// Set up a fictional MixedCollection containing a few people to filter on
 * var allNames = new <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>();
 * allNames.addAll([
 *     { id: 1, name: 'Peter',  age: 25 },
 *     { id: 2, name: 'Egon',   age: 37 },
 *     { id: 3, name: 'Ray',    age: 32 },
 *     { id: 4, name: 'Winston',age: 26 }
 * ]);
 *
 * var ageFilter = new <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Ext.util.Filter</a>({
 *     property: 'age',
 *     value   : 32
 * });
 *
 * var longNameFilter = new <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Ext.util.Filter</a>({
 *     filterFn: function(item) {
 *         return item.name.length &gt; 4;
 *     }
 * });
 *
 * // a new MixedCollection with the 3 names longer than 4 characters
 * var longNames = allNames.filter(longNameFilter);
 *
 * // a new MixedCollection with the 2 people of age 32:
 * var youngFolk = allNames.filter(ageFilter);
 * </code></pre>
 */
public class Filter extends ext.Base {
  /**
   * Creates a single filter function which encapsulates the passed Filter array or
   * Collection.
   * @param filters The filters from which to
   * create a filter function.
   * @return A function, which when passed a candidate object returns <code>true</code>
   * if the candidate passes all the specified Filters.
   */
  public static native function createFilterFn(filters:*):Function;

  /**
   * Checks if two filters have the same properties (Property, Operator and Value).
   * @since 6.2.0
   * @param filter1 The first filter to be compared
   * @param filter2 The second filter to be compared
   * @return <code>true</code> if they have the same properties.
   */
  public static native function isEqual(filter1:ext.util.Filter, filter2:ext.util.Filter):Boolean;

  /**
   * Creates new Filter.
   * @param config Config object
   */
  public function Filter(config:ext.util.Filter = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to allow any match - no regex start/end line anchors will be added.
   * @default false
   * @see #getAnyMatch()
   * @see #setAnyMatch()
   */
  public native function get anyMatch():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set anyMatch(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to make the regex case sensitive (adds 'i' switch to regex).
   * @default false
   * @see #getCaseSensitive()
   * @see #setCaseSensitive()
   */
  public native function get caseSensitive():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set caseSensitive(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to not have this filter participate in the filtering process when the <a href="#!/api/Ext.util.Filter-cfg-value" rel="Ext.util.Filter-cfg-value" class="docClass">value</a> of
   * this the filter is empty according to <a href="#!/api/Ext-method-isEmpty" rel="Ext-method-isEmpty" class="docClass">Ext.isEmpty</a>.
   * @default false
   * @since 5.1.0
   * @see #getDisableOnEmpty()
   * @see #setDisableOnEmpty()
   */
  public native function get disableOnEmpty():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disableOnEmpty(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Setting this property to <code>true</code> disables this individual Filter so that it no longer contributes to a <a href="#!/api/Ext.data.Store-cfg-filters" rel="Ext.data.Store-cfg-filters" class="docClass">Store's filter set</a>
   *
   * <p>When disabled, the next time the store is filtered, the Filter plays no part in filtering and records eliminated by it may rejoin the dataset.</p>
   * @default false
   * @see #getDisabled()
   * @see #setDisabled()
   */
  public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disabled(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to force exact match (^ and $ characters added to the regex). Ignored if anyMatch is true.
   * @default false
   * @see #getExactMatch()
   * @see #setExactMatch()
   */
  public native function get exactMatch():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set exactMatch(value:Boolean):void;

  [ExtConfig]
  /**
   * A custom filter function which is passed each item in the <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a> in turn. Should return
   * <code>true</code> to accept each item or <code>false</code> to reject it.
   * @see #setFilterFn()
   */
  public native function get filterFn():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set filterFn(value:Function):void;

  [ExtConfig]
  /**
   * An identifier by which this Filter is indexed in a <a href="#!/api/Ext.data.Store-cfg-filters" rel="Ext.data.Store-cfg-filters" class="docClass">Store's filters collection</a>
   *
   * <p>Identified Filters may be individually removed from a Store's filter set by using <a href="#!/api/Ext.data.Store-method-removeFilter" rel="Ext.data.Store-method-removeFilter" class="docClass">Ext.data.Store.removeFilter</a>.</p>
   *
   * <p>Anonymous Filters may be removed en masse by passing <code>null</code> to <a href="#!/api/Ext.data.Store-method-removeFilter" rel="Ext.data.Store-method-removeFilter" class="docClass">Ext.data.Store.removeFilter</a>.</p>
   * @see #setId()
   */
  public native function get id():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set id(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The operator to use to compare the <a href="#!/api/Ext.util.Filter-cfg-property" rel="Ext.util.Filter-cfg-property" class="docClass">property</a> to this Filter's <a href="#!/api/Ext.util.Filter-cfg-value" rel="Ext.util.Filter-cfg-value" class="docClass">value</a>
   *
   * <p>Possible values are:</p>
   *
   * <ul>
   * <li> <code>&lt;</code></li>
   * <li> <code>&lt;=</code></li>
   * <li> <code>=</code></li>
   * <li> <code>&gt;=</code></li>
   * <li> <code>&gt;</code></li>
   * <li> <code>!=</code></li>
   * <li> <code>in</code></li>
   * <li> <code>notin</code></li>
   * <li> <code>like</code></li>
   * <li> /=</li>
   * </ul>
   *
   *
   * <p>The <code>in</code> and <code>notin</code> operator expects this filter's <a href="#!/api/Ext.util.Filter-cfg-value" rel="Ext.util.Filter-cfg-value" class="docClass">value</a> to be an array and matches
   * values that are present in that array.</p>
   *
   * <p>The <code>like</code> operator matches values that contain this filter's <a href="#!/api/Ext.util.Filter-cfg-value" rel="Ext.util.Filter-cfg-value" class="docClass">value</a> as a
   * substring.</p>
   *
   * <p>The <code>'*='</code> operator uses the <a href="#!/api/Ext.util.Filter-cfg-value" rel="Ext.util.Filter-cfg-value" class="docClass">value</a> as the source for a <code>RegExp</code> and tests whether the
   * candidate value matches the regular expression.</p>
   * @see #getOperator()
   * @see #setOperator()
   */
  public native function get operator():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set operator(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The property to filter on. Required unless a <a href="#!/api/Ext.util.Filter-cfg-filterFn" rel="Ext.util.Filter-cfg-filterFn" class="docClass">filterFn</a> is passed.
   * @default null
   * @see #getProperty()
   * @see #setProperty()
   */
  public native function get property():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set property(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional root property. This is mostly useful when filtering a Store, in which case we set the root to 'data' to
   * make the filter pull the <a href="#!/api/Ext.util.Filter-cfg-property" rel="Ext.util.Filter-cfg-property" class="docClass">property</a> out of the data object of each item
   * @default null
   * @see #getRoot()
   * @see #setRoot()
   */
  public native function get root():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set root(value:String):void;

  [ExtConfig]
  /**
   * The context (<code>this</code> property) in which the filtering function is called. Defaults
   * to this Filter object.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A function to post-process any serialization. Accepts a filter state object
   * containing <code>property</code>, <code>value</code> and <code>operator</code> properties, and may either
   * mutate it, or return a completely new representation. Returning a falsey
   * value does not modify the representation.
   * @since 6.2.0
   * @see #getSerializer()
   * @see #setSerializer()
   */
  public native function get serializer():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set serializer(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The value you want to match against. Required unless a <a href="#!/api/Ext.util.Filter-cfg-filterFn" rel="Ext.util.Filter-cfg-filterFn" class="docClass">filterFn</a> is passed.
   *
   * <p>Can be a regular expression which will be used as a matcher or any other value
   * such as an object or an array of objects. This value is compared using the configured
   * <a href="#!/api/Ext.util.Filter-cfg-operator" rel="Ext.util.Filter-cfg-operator" class="docClass">operator</a>.</p>
   * @default null
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:*):void;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-anyMatch" rel="Ext.util.Filter-cfg-anyMatch" class="docClass">anyMatch</a>.
   */
  public native function getAnyMatch():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-caseSensitive" rel="Ext.util.Filter-cfg-caseSensitive" class="docClass">caseSensitive</a>.
   */
  public native function getCaseSensitive():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-disableOnEmpty" rel="Ext.util.Filter-cfg-disableOnEmpty" class="docClass">disableOnEmpty</a>.
   * @since 5.1.0
   */
  public native function getDisableOnEmpty():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-disabled" rel="Ext.util.Filter-cfg-disabled" class="docClass">disabled</a>.
   */
  public native function getDisabled():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-exactMatch" rel="Ext.util.Filter-cfg-exactMatch" class="docClass">exactMatch</a>.
   */
  public native function getExactMatch():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-operator" rel="Ext.util.Filter-cfg-operator" class="docClass">operator</a>.
   */
  public native function getOperator():String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-property" rel="Ext.util.Filter-cfg-property" class="docClass">property</a>.
   */
  public native function getProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-root" rel="Ext.util.Filter-cfg-root" class="docClass">root</a>.
   */
  public native function getRoot():String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-serializer" rel="Ext.util.Filter-cfg-serializer" class="docClass">serializer</a>.
   * @since 6.2.0
   */
  public native function getSerializer():Function;

  /**
   * Returns this filter's state.
   */
  public native function getState():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Filter-cfg-value" rel="Ext.util.Filter-cfg-value" class="docClass">value</a>.
   */
  public native function getValue():*;

  /**
   * Returns this filter's serialized state. This is used when transmitting this filter
   * to a server.
   */
  public native function serialize():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-anyMatch" rel="Ext.util.Filter-cfg-anyMatch" class="docClass">anyMatch</a>.
   * @param anyMatch The new value.
   */
  public native function setAnyMatch(anyMatch:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-caseSensitive" rel="Ext.util.Filter-cfg-caseSensitive" class="docClass">caseSensitive</a>.
   * @param caseSensitive The new value.
   */
  public native function setCaseSensitive(caseSensitive:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-disableOnEmpty" rel="Ext.util.Filter-cfg-disableOnEmpty" class="docClass">disableOnEmpty</a>.
   * @since 5.1.0
   * @param disableOnEmpty The new value.
   */
  public native function setDisableOnEmpty(disableOnEmpty:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-disabled" rel="Ext.util.Filter-cfg-disabled" class="docClass">disabled</a>.
   * @param disabled The new value.
   */
  public native function setDisabled(disabled:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-exactMatch" rel="Ext.util.Filter-cfg-exactMatch" class="docClass">exactMatch</a>.
   * @param exactMatch The new value.
   */
  public native function setExactMatch(exactMatch:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-filterFn" rel="Ext.util.Filter-cfg-filterFn" class="docClass">filterFn</a>.
   * @param filterFn The new value.
   */
  public native function setFilterFn(filterFn:Function):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-id" rel="Ext.util.Filter-cfg-id" class="docClass">id</a>.
   * @param id The new value.
   */
  public native function setId(id:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-operator" rel="Ext.util.Filter-cfg-operator" class="docClass">operator</a>.
   * @param operator The new value.
   */
  public native function setOperator(operator:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-property" rel="Ext.util.Filter-cfg-property" class="docClass">property</a>.
   * @param property The new value.
   */
  public native function setProperty(property:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-root" rel="Ext.util.Filter-cfg-root" class="docClass">root</a>.
   * @param root The new value.
   */
  public native function setRoot(root:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-serializer" rel="Ext.util.Filter-cfg-serializer" class="docClass">serializer</a>.
   * @since 6.2.0
   * @param serializer The new value.
   */
  public native function setSerializer(serializer:Function):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Filter-cfg-value" rel="Ext.util.Filter-cfg-value" class="docClass">value</a>.
   * @param value The new value.
   */
  public native function setValue(value:*):void;
}
}