package ext.util {
import ext.Base;
import ext.Component;
import ext.dom.Element;
import ext.event.Event;

[Uses("ext.Base")]
[Native("Ext.util.Focusable", require)]
/**
 * Fires when this Component loses focus.
 * @eventType ext.util.events.Focusable_eventEvent.BLUR
 */
[Event(name = 'onBlur', type = 'ext.util.events.Focusable_eventEvent')]
/**
 * Fires when this Component receives focus.
 * @eventType ext.util.events.Focusable_eventEvent.FOCUS
 */
[Event(name = 'onFocus', type = 'ext.util.events.Focusable_eventEvent')]
/**
 * Fires when focus enters this Component's hierarchy.
 * @eventType ext.util.events.Focusable_eventEvent.FOCUS_ENTER
 */
[Event(name = 'onFocusEnter', type = 'ext.util.events.Focusable_eventEvent')]
/**
 * Fires when focus leaves this Component's hierarchy.
 * @eventType ext.util.events.Focusable_eventEvent.FOCUS_LEAVE
 */
[Event(name = 'onFocusLeave', type = 'ext.util.events.Focusable_eventEvent')]
/**
 * A mixin for individually focusable things (Components, Widgets, etc)
 */
public class Focusable extends ext.Base implements ext.util.IFocusable {
  /**
   * @param config @inheritDoc
   */
  public function Focusable(config:ext.util.Focusable = null) {
    super();
  }

  [ExtConfig]
  /**
   * <code>true</code> for interactive Components, <code>false</code> for static Components.
   * For Containers, this property reflects interactiveness of the
   * Container itself, not its children. See <a href="#!/api/Ext.util.Focusable-method-isFocusable" rel="Ext.util.Focusable-method-isFocusable" class="docClass">isFocusable</a>.
   *
   * <p><strong>Note:</strong> Plain components are static, so not focusable.</p>
   * @default false
   */
  public native function set focusable(value:Boolean):void;

  [ExtConfig]
  /**
   * @private
   */
  public native function get focusable():Boolean;

  [ExtConfig]
  /**
   * CSS class that will be added to focused
   * Component, and removed when Component blurs.
   * @default 'x-focus'
   */
  public native function get focusCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set focusCls(value:String):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute for this Focusable's
   * focusEl.
   * @see #getTabIndex()
   * @see #setTabIndex()
   */
  public native function get tabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tabIndex(value:Number):void;

  /**
   * Try to focus this component.
   *
   * <p>If this component is disabled, a close relation will be targeted for focus instead
   * to keep focus localized for keyboard users.</p>
   * @param selectText If applicable, <code>true</code> to also select all the text in this component, or an array consisting of start and end (defaults to start) position of selection.
   * @param delay Delay the focus this number of milliseconds (true for 10 milliseconds).
   * @param callback Only needed if the <code>delay</code> parameter is used. A function to call upon focus.
   * @param scope Only needed if the <code>delay</code> parameter is used. The scope (<code>this</code> reference) in which to execute the callback.
   * @return The focused Component. Usually <code>this</code> Component. Some Containers may
   * delegate focus to a descendant Component (<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Window</a>s can do this through their
   * <a href="#!/api/Ext.window.Window-cfg-defaultFocus" rel="Ext.window.Window-cfg-defaultFocus" class="docClass">defaultFocus</a> config option. If this component is disabled, a closely
   * related component will be focused and that will be returned.
   */
  public native function focus(selectText:* = undefined, delay:* = undefined, callback:Function = null, scope:Function = null):ext.Component;

  /**
   * Return the actual tabIndex for this Focusable.
   * @return tabIndex attribute value
   */
  public native function getTabIndex():Number;

  /**
   * Determine if this Focusable can receive focus at this time.
   *
   * <p>Note that Containers can be non-focusable themselves while delegating
   * focus treatment to a child Component; see
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">#defaultFocus</a> for more information.</p>
   * @param deep Optionally determine if the container itself
   * is focusable, or if container's focus is delegated to a child component
   * and that child is focusable.
   * @default false
   * @return True if component is focusable, false if not.
   */
  public native function isFocusable(deep:Boolean = false):Boolean;

  /**
   * Set the tabIndex property for this Focusable. If the focusEl
   * is available, set tabIndex attribute on it, too.
   * @param newTabIndex new tabIndex to set
   */
  public native function setTabIndex(newTabIndex:Number):void;

  /**
   * Template method to do any pre-blur processing.
   * @param e The event object
   */
  protected native function beforeBlur(e:ext.event.Event):void;

  /**
   * Template method to do any pre-focus processing.
   * @param e The event object
   */
  protected native function beforeFocus(e:ext.event.Event):void;

  /**
   * Cancel any deferred focus on this component
   */
  protected native function cancelFocus():void;

  /**
   * Returns the focus styling holder element associated with this Focusable.
   * By default it is the same element as <a href="#!/api/Ext.util.Focusable-method-getFocusEl" rel="Ext.util.Focusable-method-getFocusEl" class="docClass">getFocusEl</a>.
   * @return The focus styling element.
   */
  protected native function getFocusClsEl():ext.dom.Element;

  /**
   * Returns the focus holder element associated with this Focusable. At the
   * level of the Focusable base, this function returns <code>this.el</code> (or for Widgets,
   * <code>this.element</code>).
   *
   * <p>Subclasses with embedded focusable elements (such as Window, Field and Button)
   * should override this for use by <a href="#!/api/Ext.util.Focusable-method-focus" rel="Ext.util.Focusable-method-focus" class="docClass">focus</a>
   * method.</p>
   */
  protected native function getFocusEl():ext.dom.Element;

  /**
   * Template method to do any Focusable related initialization that
   * does not involve event listeners creation.
   */
  protected native function initFocusable():void;

  /**
   * Template method to do any event listener initialization for a Focusable.
   * This generally happens after the focusEl is available.
   */
  protected native function initFocusableEvents():void;

  /**
   * Called when focus enters this Component's hierarchy
   * @param e
   * @param {ext.event.Event} e.event The underlying DOM event.
   * @param {js.HTMLElement} e.target The element gaining focus.
   * @param {js.HTMLElement} e.relatedTarget The element losing focus.
   * @param {ext.Component} e.toComponent The Component gaining focus.
   * @param {ext.Component} e.fromComponent The Component losing focus.
   */
  protected native function onFocusEnter(e:Object):void;

  /**
   * Called when focus exits from this Component's hierarchy
   * @param e
   * @param {ext.event.Event} e.event The underlying DOM event.
   * @param {js.HTMLElement} e.target The element gaining focus.
   * @param {js.HTMLElement} e.relatedTarget The element losing focus.
   * @param {ext.Component} e.toComponent The Component gaining focus.
   * @param {ext.Component} e.fromComponent The Component losing focus.
   */
  protected native function onFocusLeave(e:ext.event.Event):void;

  /**
   * Template method to do any post-blur processing.
   * @param e The event object
   */
  protected native function postBlur(e:ext.event.Event):void;

  /**
   * Template method to do any post-focus processing.
   * @param e The event object
   */
  protected native function postFocus(e:ext.event.Event):void;
}
}