package ext.util {

[Uses("ext.util.ElementContainer")]
[Mixin("ext.util.ElementContainer")]
/**
 * This mixin enables classes to declare relationships to child elements and provides the
 * mechanics for acquiring the <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">elements</a> and storing them on an object
 * instance as properties.
 *
 * <p>This class is used by <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">components</a> and <a href="#!/api/Ext.layout.container.Container" rel="Ext.layout.container.Container" class="docClass">container layouts</a> to
 * manage their child elements.</p>
 *
 * <p>A typical component that uses these features might look something like this:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.ux.SomeComponent', {
 *      extend: '<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>',
 *
 *      childEls: [
 *          'bodyEl'
 *      ],
 *
 *      renderTpl: [
 *          '&lt;div id="{id}-bodyEl" data-ref="bodyEl"&gt;&lt;/div&gt;'
 *      ],
 *
 *      // ...
 *  });
 * </code></pre>
 *
 * <p>The <code><a href="#!/api/Ext.util.ElementContainer-cfg-childEls" rel="Ext.util.ElementContainer-cfg-childEls" class="docClass">childEls</a></code> config lists one or more relationships to child elements managed
 * by the component. The items in this array can be objects that more fully specify the
 * child. For example, the above could have used this instead to achieve the same result:</p>
 *
 * <pre><code> childEls: [
 *      { name: 'bodyEl', itemId: 'bodyEl' }
 *  ]
 * </code></pre>
 *
 * <p>Unlike a <code>renderTpl</code> where there is a single value for an instance, <code>childEls</code> are aggregated
 * up the class hierarchy so that they are effectively inherited. In other words, if a
 * class where to derive from <code>Ext.ux.SomeComponent</code> in the example above, it could also
 * have a <code>childEls</code> property in the same way as <code>Ext.ux.SomeComponent</code>.</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.ux.AnotherComponent', {
 *      extend: 'Ext.ux.SomeComponent',
 *
 *      childEls: [
 *          // 'bodyEl' is inherited
 *          'innerEl'
 *      ],
 *
 *      renderTpl: [
 *          '&lt;div id="{id}-bodyEl" data-ref="bodyEl"&gt;'
 *              '&lt;div id="{id}-innerEl" data-ref="innerEl"&gt;&lt;/div&gt;'
 *          '&lt;/div&gt;'
 *      ],
 *
 *      // ...
 *  });
 * </code></pre>
 *
 * <p><strong>IMPORTANT</strong>
 * The <code>renderTpl</code> contains both child elements and unites them in the desired markup, but
 * the <code>childEls</code> only contains the new child element. The <code>data-ref</code> attribute must be
 * rendered on to child elements that do not use <code>select</code> or <code>selectNode</code> options. This
 * is done for performance reasons on IE8 where element lookup (even by id) is not very
 * efficient.</p>
 * @see ext.util.ElementContainer
 */
public interface IElementContainer {
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The canonical form of <code>childEls</code> is an object keyed by child's property name
   * with values that are objects with the following properties.
   *
   * <ul>
   * <li><code>itemId</code> - The id to combine with the Component's id that is the id of the
   * child element.</li>
   * <li><code>id</code> - The id of the child element.</li>
   * <li><code>leaf</code> - Set to <code>true</code> to ignore content when scanning for childEls. This
   * should be set on things like the generated content for an <code><a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">Ext.view.View</a></code>.</li>
   * <li><code>select</code>: A selector that will be passed to <a href="#!/api/Ext.dom.Element-method-select" rel="Ext.dom.Element-method-select" class="docClass">Ext.dom.Element.select</a>.</li>
   * <li><code>selectNode</code>: A selector that will be passed to <a href="#!/api/Ext.dom.Element-method-selectNode" rel="Ext.dom.Element-method-selectNode" class="docClass">Ext.dom.Element.selectNode</a>.</li>
   * </ul>
   *
   *
   * <p>For example:</p>
   *
   * <pre><code> childEls: {
   *      button: true,
   *      buttonText: 'text',
   *      buttonImage: {
   *          itemId: 'image'
   *      }
   *  }
   * </code></pre>
   *
   * <p>The above is translated into the following complete form:</p>
   *
   * <pre><code> childEls: {
   *      button: {
   *          name: 'button',
   *          itemId: 'button'
   *      },
   *      buttonText: {
   *          name: 'buttonText',
   *          itemId: 'text'
   *      },
   *      buttonImage: {
   *          name: 'buttonImage',
   *          itemId: 'image'
   *      }
   *  }
   * </code></pre>
   *
   * <p>The above can be provided as an array like so:</p>
   *
   * <pre><code> childEls: [
   *      'button',
   *      { name: 'buttonText', itemId: 'text' },
   *      { name: 'buttonImage', itemId: 'image' }
   *  }
   * </code></pre>
   *
   * <p>For example, a Component which renders a title and body text:</p>
   *
   * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>', {
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     renderTpl: [
   *         '&lt;h1 id="{id}-title" data-ref="title"&gt;{title}&lt;/h1&gt;',
   *         '&lt;p&gt;{msg}&lt;/p&gt;',
   *     ],
   *     renderData: {
   *         title: "Error",
   *         msg: "Something went wrong"
   *     },
   *     childEls: ["title"],
   *     listeners: {
   *         afterrender: function(cmp){
   *             // After rendering the component will have a title property
   *             cmp.title.setStyle({color: "red"});
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <p><strong>Note:</strong> <code>childEl</code>s in the <a href="#!/api/Ext.Component-cfg-renderTpl" rel="Ext.Component-cfg-renderTpl" class="docClass">renderTpl</a>
   * must be referenced in a <strong>data-ref</strong> attribute.  Notice in the above example
   * that the "title" <code>childEl</code> is set in the <code>renderTpl</code> using
   * <strong>data-ref="title"</strong>.</p>
   *
   * <p>When using <code>select</code>, the property will be an instance of <a href="#!/api/Ext.dom.CompositeElement" rel="Ext.dom.CompositeElement" class="docClass">Ext.CompositeElement</a>.
   * In all other cases, the property will be an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> or <code>null</code>
   * if not found.</p>
   *
   * <p>Care should be taken when using <code>select</code> or <code>selectNode</code> to find child elements.
   * The following issues should be considered:</p>
   *
   * <ul>
   * <li>Performance: using selectors can be 10x slower than id lookup.</li>
   * <li>Over-selecting: selectors are applied after the DOM elements for all children
   * have been rendered, so selectors can match elements from child components
   * (including nested versions of the same component) accidentally.</li>
   * </ul>
   *
   *
   * <p>This above issues are most important when using <code>select</code> since it returns multiple
   * elements.</p>
   * @see #getChildEls()
   * @see #setChildEls()
   */
  function get childEls():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  function set childEls(value:*):void;

  /**
   * Returns the value of <a href="#!/api/Ext.util.ElementContainer-cfg-childEls" rel="Ext.util.ElementContainer-cfg-childEls" class="docClass">childEls</a>.
   */
  function getChildEls():*;

  /**
   * Sets the value of <a href="#!/api/Ext.util.ElementContainer-cfg-childEls" rel="Ext.util.ElementContainer-cfg-childEls" class="docClass">childEls</a>.
   * @param childEls The new value.
   */
  function setChildEls(childEls:*):void;
}
}