package ext.util {
import ext.Component;
import ext.ZIndexManager;
import ext.container.Container;

[Uses("ext.util.Floating")]
[Mixin("ext.util.Floating")]
/**
 * A mixin to add floating capability to a Component.
 * @see ext.util.Floating
 */
public interface IFloating {
  /**
   * Only present for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components after
   * they have been rendered.
   *
   * <p>A reference to the ZIndexManager which is managing this Component's z-index.</p>
   *
   * <p>The <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a> maintains a stack of floating Component z-indices, and also provides
   * a single modal mask which is insert just beneath the topmost visible modal floating Component.</p>
   *
   * <p>Floating Components may be <a href="#!/api/Ext.Component-method-toFront" rel="Ext.Component-method-toFront" class="docClass">brought to the front</a> or <a href="#!/api/Ext.Component-method-toBack" rel="Ext.Component-method-toBack" class="docClass">sent to the back</a> of the
   * z-index stack.</p>
   *
   * <p>This defaults to the global <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a> for floating Components that are
   * programatically <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">rendered</a>.</p>
   *
   * <p>For <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> Components that are added to a
   * Container, the ZIndexManager is acquired from the first ancestor Container found
   * that is floating. If no floating ancestor is found, the global
   * <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a> is used.</p>
   *
   * <p>See <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> and <a href="#!/api/Ext.util.Floating-property-zIndexParent" rel="Ext.util.Floating-property-zIndexParent" class="docClass">zIndexParent</a></p>
   */
  function get zIndexManager():ext.ZIndexManager;

  /**
   * Only present for <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> Components which were
   * inserted as child items of Containers, and which have a floating Container in
   * their containment ancestry.
   *
   * <p>For <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> Components which are child items of a
   * Container, the zIndexParent will be a floating ancestor Container which is
   * responsible for the base z-index value of all its floating descendants. It
   * provides a <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a> which provides z-indexing
   * services for all its descendant floating Components.</p>
   *
   * <p>Floating Components that are programmatically <a href="#!/api/Ext.Component-method-render" rel="Ext.Component-method-render" class="docClass">rendered</a> will not have a <code>zIndexParent</code>
   * property.</p>
   *
   * <p>For example, the dropdown <a href="#!/api/Ext.view.BoundList" rel="Ext.view.BoundList" class="docClass">BoundList</a> of a ComboBox which is in a Window will have the
   * Window as its <code>zIndexParent</code>, and will always show above that Window, wherever the Window is placed in the z-index stack.</p>
   *
   * <p>See <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">Ext.Component.floating</a> and <a href="#!/api/Ext.util.Floating-property-zIndexManager" rel="Ext.util.Floating-property-zIndexManager" class="docClass">zIndexManager</a></p>
   */
  function get zIndexParent():ext.container.Container;

  [ExtConfig]
  /**
   * By default, when the <a href="#!/api/Ext.Component-method-alignTo" rel="Ext.Component-method-alignTo" class="docClass">alignTo</a> method is called, a floating component will
   * scroll to keep aligned with the anchoring element if the anchoring element is part of the scroll.
   *
   * <p>If this is not necessary, and the <code>alignTo</code> is a one-off operation then set this config to <code>false</code>.</p>
   * @default true
   */
  function get alignOnScroll():Boolean;

  [ExtConfig]
  function set alignOnScroll(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A flag indicating that this component should be on the top of the z-index stack for use by the <a href="#!/api/Ext.util.Floating-property-zIndexManager" rel="Ext.util.Floating-property-zIndexManager" class="docClass">zIndexManager</a>
   * to sort its stack.
   *
   * <p>This may be a positive number to prioritize the ordering of multiple visible always on top components.</p>
   *
   * <p>This may be set to a <em>negative</em> number to prioritize a component to the <em>bottom</em> of the z-index stack.</p>
   * @default false
   * @see #getAlwaysOnTop()
   * @see #setAlwaysOnTop()
   */
  function get alwaysOnTop():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  function set alwaysOnTop(value:*):void;

  [ExtConfig]
  /**
   * <code>true</code> to animate the shadow along with the component while the component is animating.
   * By default the shadow is hidden while the component is animating
   * @default false
   */
  function get animateShadow():Boolean;

  [ExtConfig]
  function set animateShadow(value:Boolean):void;

  [ExtConfig]
  /**
   * True to constrain this Components within its containing element, false to allow it to fall outside of its containing
   * element. By default this Component will be rendered to <code>document.body</code>. To render and constrain this Component within
   * another element specify <a href="#!/api/Ext.Component-cfg-renderTo" rel="Ext.Component-cfg-renderTo" class="docClass">renderTo</a>.
   * @default false
   */
  function get constrain():Boolean;

  [ExtConfig]
  function set constrain(value:Boolean):void;

  [ExtConfig]
  /**
   * Configure as <code>true</code> to have this Component fixed at its <code>X, Y</code> coordinates in the browser viewport, immune
   * to scrolling the document.
   * @default false
   */
  function get fixed():Boolean;

  [ExtConfig]
  function set fixed(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether the floated component should be automatically <a href="#!/api/Ext.Component-method-focus" rel="Ext.Component-method-focus" class="docClass">focused</a> when
   * it is <a href="#!/api/Ext.util.Floating-method-toFront" rel="Ext.util.Floating-method-toFront" class="docClass">brought to the front</a>.
   * @default true
   */
  function get focusOnToFront():Boolean;

  [ExtConfig]
  function set focusOnToFront(value:Boolean):void;

  [ExtConfig]
  /**
   * True to make the floated component modal and mask everything behind it when displayed, false to display it without
   * restricting access to other UI elements.
   * @default false
   */
  function get modal():Boolean;

  [ExtConfig]
  function set modal(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether the floating component should be given a shadow. Set to true to automatically create an
   * <a href="#!/api/Ext.dom.Shadow" rel="Ext.dom.Shadow" class="docClass">Ext.Shadow</a>, or a string indicating the shadow's display <a href="#!/api/Ext.dom.Shadow-cfg-mode" rel="Ext.dom.Shadow-cfg-mode" class="docClass">Ext.Shadow.mode</a>. Set to false to
   * disable the shadow.
   * @default 'sides'
   */
  function get shadow():*;

  [ExtConfig]
  function set shadow(value:*):void;

  [ExtConfig]
  /**
   * Number of pixels to offset the shadow.
   */
  function get shadowOffset():Number;

  [ExtConfig]
  function set shadowOffset(value:Number):void;

  [ExtConfig]
  /**
   * <code>true</code> to enable an iframe shim for this Component to keep
   * windowed objects from showing through.
   */
  function get shim():Boolean;

  [ExtConfig]
  function set shim(value:Boolean):void;

  /**
   * Center this Component in its container.
   * @return this
   */
  function center():ext.Component;

  /**
   * Moves this floating Component into a constrain region.
   *
   * <p>By default, this Component is constrained to be within the container it was added to, or the element it was
   * rendered to.</p>
   *
   * <p>An alternative constraint may be passed.</p>
   * @param constrainTo The Element or <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a>
   * into which this Component is to be constrained. Defaults to the element into which this floating Component
   * was rendered.
   */
  function doConstrain(constrainTo:* = undefined):void;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Floating-cfg-alwaysOnTop" rel="Ext.util.Floating-cfg-alwaysOnTop" class="docClass">alwaysOnTop</a>.
   */
  function getAlwaysOnTop():*;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Floating-cfg-alwaysOnTop" rel="Ext.util.Floating-cfg-alwaysOnTop" class="docClass">alwaysOnTop</a>.
   * @param alwaysOnTop The new value.
   */
  function setAlwaysOnTop(alwaysOnTop:*):void;

  /**
   * Sends this Component to the back of (lower z-index than) any other visible windows
   * @return this
   */
  function toBack():ext.Component;

  /**
   * Brings this floating Component to the front of any other visible, floating Components managed by the same
   * <a href="#!/api/Ext.ZIndexManager" rel="Ext.ZIndexManager" class="docClass">ZIndexManager</a>
   *
   * <p>If this Component is modal, inserts the modal mask just below this Component in the z-index stack.</p>
   * @param preventFocus Specify <code>true</code> to prevent the Component from being focused.
   * @default false
   * @return this
   */
  function toFront(preventFocus:Boolean = false):ext.Component;
}
}