package ext.util {

[Uses("ext.util.Sortable")]
[Mixin("ext.util.Sortable")]
/**
 * A mixin which allows a data component to be sorted. This is used by e.g. <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> and <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a>.
 *
 * <p><strong>NOTE</strong>: This mixin is mainly for internal use and most users should not need to use it directly. It
 * is more likely you will want to use one of the component classes that import this mixin, such as
 * <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> or <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a>.</p>
 * @see ext.util.Sortable
 */
public interface ISortable {
  /**
   * <code>true</code> in this class to identify an object as an instantiated Sortable, or subclass thereof.
   * @default true
   */
  function get isSortable():Boolean;

  function set isSortable(value:Boolean):void;

  [ExtConfig]
  /**
   * The default sort direction to use if one is not specified.
   * @default "ASC"
   */
  function get defaultSortDirection():String;

  [ExtConfig]
  function set defaultSortDirection(value:String):void;

  [ExtConfig]
  /**
   * The maximum number of sorters which may be applied to this Sortable when using the "multi" insertion position
   * when adding sorters.
   *
   * <p>New sorters added using the "multi" insertion position are inserted at the top of the sorters list becoming the
   * new primary sort key.</p>
   *
   * <p>If the sorters collection has grown to longer then <strong><code>multiSortLimit</code></strong>, then the it is trimmed.</p>
   * @default 3
   */
  function get multiSortLimit():Number;

  [ExtConfig]
  function set multiSortLimit(value:Number):void;

  [ExtConfig]
  /**
   * The property in each item that contains the data to sort.
   */
  function get sortRoot():String;

  [ExtConfig]
  function set sortRoot(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The initial set of <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Sorters</a>.
   *
   * <pre><code>sorters: [{
   *     property: 'age',
   *     direction: 'DESC'
   * }, {
   *     property: 'firstName',
   *     direction: 'ASC'
   * }]
   * </code></pre>
   * @see #getSorters()
   * @see #setSorters()
   */
  function get sorters():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  [ArrayElementType("ext.util.Sorter")]
  function set sorters(value:Array):void;

  /**
   * Returns a comparator function which compares two items and returns -1, 0, or 1 depending
   * on the currently defined set of <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a>.
   *
   * <p>If there are no <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a> defined, it returns a function which returns <code>0</code> meaning
   * that no sorting will occur.</p>
   */
  function generateComparator():void;

  [ArrayElementType("ext.util.Sorter")]
  /**
   * Returns the value of <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a>.
   */
  function getSorters():Array;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sortable-cfg-sorters" rel="Ext.util.Sortable-cfg-sorters" class="docClass">sorters</a>.
   * @param sorters The new value.
   */
  function setSorters(sorters:Array):void;

  /**
   * Updates the sorters collection and triggers sorting of this Sortable. Example usage:
   *
   * <pre><code>//sort by a single field
   * myStore.sort('myField', 'DESC');
   *
   * //sorting by multiple fields
   * myStore.sort([{
   *     property : 'age',
   *     direction: 'ASC'
   * }, {
   *     property : 'name',
   *     direction: 'DESC'
   * }]);
   * </code></pre>
   *
   * <p>Classes which use this mixin must implement a <strong><code>soSort</code></strong> method which accepts a comparator function computed from
   * the full sorter set which performs the sort in an implementation-specific way.</p>
   *
   * <p>When passing a single string argument to sort, Store maintains a ASC/DESC toggler per field, so this code:</p>
   *
   * <pre><code>store.sort('myField');
   * store.sort('myField');
   * </code></pre>
   *
   * <p>Is equivalent to this code, because Store handles the toggling automatically:</p>
   *
   * <pre><code>store.sort('myField', 'ASC');
   * store.sort('myField', 'DESC');
   * </code></pre>
   * @param sorters Either a string name of one of the fields in this Store's configured <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>, or an array of sorter configurations.
   * @param direction The overall direction to sort the data by.
   * @default "ASC"
   * @param insertionPosition Where to put the new sorter in the collection of sorters.
   * This may take the following values:
   * <ul>
   * <li><p><code>replace</code> : This means that the new sorter(s) becomes the sole sorter set for this Sortable. This is the most useful call mode
   *         to programatically sort by multiple fields.</p></li>
   * <li><p><code>prepend</code> : This means that the new sorters are inserted as the primary sorters, unchanged, and the sorter list length must be controlled by the developer.</p></li>
   * <li><p><code>multi</code> :  This is mainly useful for implementing intuitive "Sort by this" user interfaces such as the <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">GridPanel</a>'s column sorting UI.</p>
   * <p>  This mode is only supported when passing a property name and a direction.</p>
   * <p>  This means that the new sorter is becomes the primary sorter. If the sorter was <strong>already</strong> the primary sorter, the direction
   *   of sort is toggled if no direction parameter is specified.</p>
   * <p>  The number of sorters maintained is limited by the <a href="#!/api/Ext.util.Sortable-cfg-multiSortLimit" rel="Ext.util.Sortable-cfg-multiSortLimit" class="docClass">multiSortLimit</a> configuration.</p></li>
   * <li><p><code>append</code> : This means that the new sorter becomes the last sorter.</p></li>
   * </ul>
   * @default "replace"
   * @return The new sorters.
   */
  function sort(sorters:* = undefined, direction:String = "ASC", insertionPosition:String = "replace"):Array;
}
}