package ext.util {
import ext.Base;
import ext.event.Event;

[Uses("ext.Base")]
[Native("Ext.util.KeyMap", require)]
/**
 * Handles mapping key events to handling functions for an element or a Component. One KeyMap can be used for multiple
 * actions.
 *
 * <p>A KeyMap must be configured with a <a href="#!/api/Ext.util.KeyMap-cfg-target" rel="Ext.util.KeyMap-cfg-target" class="docClass">target</a> as an event source which may be an Element or a Component.</p>
 *
 * <p>If the target is an element, then the <code>keydown</code> event will trigger the invocation of <a href="#!/api/Ext.util.KeyMap-cfg-binding" rel="Ext.util.KeyMap-cfg-binding" class="docClass">binding</a>s.</p>
 *
 * <p>It is possible to configure the KeyMap with a custom <a href="#!/api/Ext.util.KeyMap-cfg-eventName" rel="Ext.util.KeyMap-cfg-eventName" class="docClass">eventName</a> to listen for. This may be useful when the
 * <a href="#!/api/Ext.util.KeyMap-cfg-target" rel="Ext.util.KeyMap-cfg-target" class="docClass">target</a> is a Component.</p>
 *
 * <p>The KeyMap's event handling requires that the first parameter passed is a key event. So if the Component's event
 * signature is different, specify a <a href="#!/api/Ext.util.KeyMap-cfg-processEvent" rel="Ext.util.KeyMap-cfg-processEvent" class="docClass">processEvent</a> configuration which accepts the event's parameters and
 * returns a key event.</p>
 *
 * <p>Functions specified in <a href="#!/api/Ext.util.KeyMap-cfg-binding" rel="Ext.util.KeyMap-cfg-binding" class="docClass">binding</a>s are called with this signature : <code>(String key, <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">Ext.event.Event</a> e)</code> (if the
 * match is a multi-key combination the callback will still be called only once). A KeyMap can also handle a string
 * representation of keys. By default KeyMap starts enabled.</p>
 *
 * <p>Usage:</p>
 *
 * <pre><code>// map one key by key code
 * var map = new <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">Ext.util.KeyMap</a>({
 *     target: "my-element",
 *     key: 13, // or <a href="#!/api/Ext.event.Event-property-ENTER" rel="Ext.event.Event-property-ENTER" class="docClass">Ext.event.Event.ENTER</a>
 *     fn: myHandler,
 *     scope: myObject
 * });
 *
 * // map multiple keys to one action by string
 * var map = new <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">Ext.util.KeyMap</a>({
 *     target: "my-element",
 *     key: "a\r\n\t",
 *     fn: myHandler,
 *     scope: myObject
 * });
 *
 * // map multiple keys to multiple actions by strings and array of codes
 * var map = new <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">Ext.util.KeyMap</a>({
 *     target: "my-element",
 *     binding: [{
 *         key: [10,13],
 *         fn: function(){ alert("Return was pressed"); }
 *     }, {
 *         key: "abc",
 *         fn: function(){ alert('a, b or c was pressed'); }
 *     }, {
 *         key: "\t",
 *         ctrl:true,
 *         shift:true,
 *         fn: function(){ alert('Control + shift + tab was pressed.'); }
 *     }]
 * });
 * </code></pre>
 *
 * <p>Since 4.1.0, KeyMaps can bind to Components and process key-based events fired by Components.</p>
 *
 * <p>To bind to a Component, use the single parameter form of constructor and include the Component event name
 * to listen for, and a <code>processEvent</code> implementation which returns the key event for further processing by
 * the KeyMap:</p>
 *
 * <pre><code>var map = new <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">Ext.util.KeyMap</a>({
 *     target: myGridView,
 *     eventName: 'itemkeydown',
 *     processEvent: function(view, record, node, index, event) {
 *
 *         // Load the event with the extra information needed by the mappings
 *         event.view = view;
 *         event.store = view.getStore();
 *         event.record = record;
 *         event.index = index;
 *         return event;
 *     },
 *     binding: {
 *         key: <a href="#!/api/Ext.event.Event-property-DELETE" rel="Ext.event.Event-property-DELETE" class="docClass">Ext.event.Event.DELETE</a>,
 *         fn: function(keyCode, e) {
 *             e.store.remove(e.record);
 *
 *             // Attempt to select the record that's now in its place
 *             e.view.getSelectionModel().select(e.index);
 *             e.view.el.focus();
 *         }
 *     }
 * });
 * </code></pre>
 */
public class KeyMap extends ext.Base {
  /**
   * @param config @inheritDoc
   */
  public function KeyMap(config:ext.util.KeyMap = null) {
    super();
  }

  /**
   * The last key event that this KeyMap handled.
   */
  public native function get lastKeyEvent():ext.event.Event;

  /**
   * @private
   */
  public native function set lastKeyEvent(value:ext.event.Event):void;

  [ExtConfig]
  /**
   * Either a single object describing a handling function for s specified key (or set of keys), or
   * an array of such objects.
   * <ul>
   *   <li><code>key</code>
   * A single keycode or an array of keycodes to handle, or a RegExp
   * which specifies characters to handle, eg <code>/[a-z]/</code>.
   *   </li>
   *   <li><code>shift</code> : Boolean
   * True to handle key only when shift is pressed, False to handle the
   *  key only when shift is not pressed (defaults to undefined)
   *   </li>
   *   <li><code>ctrl</code> : Boolean
   * True to handle key only when ctrl is pressed, False to handle the
   *  key only when ctrl is not pressed (defaults to undefined)
   *   </li>
   *   <li><code>alt</code> : Boolean
   * True to handle key only when alt is pressed, False to handle the key
   *  only when alt is not pressed (defaults to undefined)
   *   </li>
   *   <li><code>handler</code> : Function
   * The function to call when KeyMap finds the expected key combination
   *   </li>
   *   <li><code>fn</code> : Function
   * Alias of handler (for backwards-compatibility)
   *   </li>
   *   <li><code>scope</code> : Object
   * The scope (<code>this</code> context) in which the handler function is executed.
   *   </li>
   *   <li><code>defaultEventAction</code> : String
   * A default action to apply to the event <em>when the handler returns <code>true</code></em>. Possible values
   *  are: stopEvent, stopPropagation, preventDefault. If no value is set no action is performed.
   *   </li>
   * </ul>
   */
  public native function get binding():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set binding(value:*):void;

  [ExtConfig]
  /**
   * The event to listen for to pick up key events.
   * @default 'keydown'
   */
  public native function get eventName():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set eventName(value:String):void;

  [ExtConfig]
  /**
   * Configure this as <code>true</code> if there are any input fields within the <a href="#!/api/Ext.util.KeyMap-cfg-target" rel="Ext.util.KeyMap-cfg-target" class="docClass">target</a>, and this KeyNav
   * should not process events from input fields, (<code>&amp;lt;input&gt;, &amp;lt;textarea&gt; and elements with</code>contentEditable="true"`)
   * @default false
   */
  public native function get ignoreInputFields():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreInputFields(value:Boolean):void;

  [ExtConfig]
  /**
   * The priority to set on this KeyMap's listener. Listeners with a higher priority are fired before those with
   * lower priority.
   */
  public native function get priority():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set priority(value:Number):void;

  [ExtConfig]
  /**
   * An optional event processor function which accepts the argument list provided by the
   * <a href="#!/api/Ext.util.KeyMap-cfg-eventName" rel="Ext.util.KeyMap-cfg-eventName" class="docClass">configured event</a> of the <a href="#!/api/Ext.util.KeyMap-cfg-target" rel="Ext.util.KeyMap-cfg-target" class="docClass">target</a>, and returns a keyEvent for processing by the KeyMap.
   *
   * <p>This may be useful when the <a href="#!/api/Ext.util.KeyMap-cfg-target" rel="Ext.util.KeyMap-cfg-target" class="docClass">target</a> is a Component with a complex event signature, where the event is not
   * the first parameter. Extra information from the event arguments may be injected into the event for use by the handler
   * functions before returning it.</p>
   *
   * <p>If <code>null</code> is returned the KeyMap stops processing the event.</p>
   */
  public native function get processEvent():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set processEvent(value:Function):void;

  [ExtConfig]
  /**
   * The scope (<code>this</code> context) in which the <a href="#!/api/Ext.util.KeyMap-cfg-processEvent" rel="Ext.util.KeyMap-cfg-processEvent" class="docClass">processEvent</a> method is executed.
   * @default this
   */
  public native function get processEventScope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set processEventScope(value:Object):void;

  [ExtConfig]
  /**
   * The object on which to listen for the event specified by the <a href="#!/api/Ext.util.KeyMap-cfg-eventName" rel="Ext.util.KeyMap-cfg-eventName" class="docClass">eventName</a> config option.
   */
  public native function get target():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set target(value:*):void;

  /**
   * Destroys the KeyMap instance and removes all handlers.
   * @param params true to also remove the <a href="#!/api/Ext.util.KeyMap-cfg-target" rel="Ext.util.KeyMap-cfg-target" class="docClass">target</a>
   */
  override public native function destroy(...params):void;

  /**
   * Add a new binding to this KeyMap.
   *
   * <p>Usage:</p>
   *
   * <pre><code>// Create a KeyMap
   * var map = new <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">Ext.util.KeyMap</a>(document, {
   *     key: <a href="#!/api/Ext.event.Event-property-ENTER" rel="Ext.event.Event-property-ENTER" class="docClass">Ext.event.Event.ENTER</a>,
   *     fn: handleKey,
   *     scope: this
   * });
   *
   * //Add a new binding to the existing KeyMap later
   * map.addBinding({
   *     key: 'abc',
   *     shift: true,
   *     fn: handleKey,
   *     scope: this
   * });
   * </code></pre>
   * @param binding A single KeyMap config or an array of configs.
   * The following config object properties are supported:
   * @param binding.key A single keycode or an array of keycodes to handle, or a RegExp
   * which specifies characters to handle, eg <code>/[a-z]/</code>.
   * @param {Boolean} binding.shift True to handle key only when shift is pressed,
   * False to handle the keyonly when shift is not pressed (defaults to undefined).
   * @param {Boolean} binding.ctrl True to handle key only when ctrl is pressed,
   * False to handle the key only when ctrl is not pressed (defaults to undefined).
   * @param {Boolean} binding.alt True to handle key only when alt is pressed,
   * False to handle the key only when alt is not pressed (defaults to undefined).
   * @param {Function} binding.handler The function to call when KeyMap finds the
   * expected key combination.
   * @param {Function} binding.fn Alias of handler (for backwards-compatibility).
   * @param {Object} binding.scope The scope (<code>this</code> context) in which the handler function is executed.
   * @param {String} binding.defaultEventAction A default action to apply to the event <em>when the handler returns <code>true</code></em>.
   * Possible values are: stopEvent, stopPropagation, preventDefault. If no value is
   * set no action is performed..
   */
  public native function addBinding(binding:*):void;

  /**
   * Disable this KeyMap
   */
  public native function disable():void;

  /**
   * Enables this KeyMap
   */
  public native function enable():void;

  /**
   * Returns true if this KeyMap is enabled
   */
  public native function isEnabled():Boolean;

  /**
   * Shorthand for adding a single key listener.
   * @param key Either the numeric key code, array of key codes or an object with the
   * following options: <code>{key: (number or array), shift: (true/false), ctrl: (true/false), alt: (true/false)}</code>
   * @param fn The function to call
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * Defaults to the browser window.
   */
  public native function on(key:*, fn:Function, scope:Object = null):void;

  /**
   * Remove a binding from this KeyMap.
   * @param binding See <a href="#!/api/Ext.util.KeyMap-method-addBinding" rel="Ext.util.KeyMap-method-addBinding" class="docClass">for options</a>
   */
  public native function removeBinding(binding:Object):void;

  /**
   * Convenience function for setting disabled/enabled by boolean.
   */
  public native function setDisabled(disabled:Boolean):void;

  /**
   * Shorthand for removing a single key listener.
   * @param key Either the numeric key code, array of key codes or an object with the
   * following options: <code>{key: (number or array), shift: (true/false), ctrl: (true/false), alt: (true/false)}</code>
   * @param fn The function to call
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * Defaults to the browser window.
   */
  public native function un(key:*, fn:Function, scope:Object = null):void;
}
}