package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.Region", require)]
/**
 * This class represents a rectangular region in X,Y space, and performs geometric
 * transformations or tests upon the region.
 *
 * <p>This class may be used to compare the document regions occupied by elements.</p>
 */
public class Region extends ext.Base {
  /**
   * Creates a Region from a "box" Object which contains four numeric properties <code>top</code>, <code>right</code>, <code>bottom</code> and <code>left</code>.
   * @param o An object with <code>top</code>, <code>right</code>, <code>bottom</code> and <code>left</code> properties.
   * @return region The Region constructed based on the passed object
   */
  public static native function from(o:Object):ext.util.Region;

  /**
   * Retrieves an <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Ext.util.Region</a> for a particular element.
   * @param el An element ID, htmlElement or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> representing an element in the document.
   * @return region
   */
  public static native function getRegion(el:*):ext.util.Region;

  /**
   * Creates a region from the bounding sides.
   * @param top The topmost pixel of the Region.
   * @param right The rightmost pixel of the Region.
   * @param bottom The bottom pixel of the Region.
   * @param left The leftmost pixel of the Region.
   */
  public function Region(top:Number, right:Number, bottom:Number, left:Number) {
    super();
  }

  /**
   * Modifies the current region to be adjusted by offsets.
   * @param top Top offset
   * @param right Right offset
   * @param bottom Bottom offset
   * @param left Left offset
   * @return this
   */
  public native function adjust(top:Number, right:Number, bottom:Number, left:Number):ext.util.Region;

  /**
   * Returns the Region to which this rectangle should be moved in order to
   * have the desired alignment with the specified target while remaining within the
   * constraint.
   *
   * <p>The <code>align</code> option can be one of these forms:</p>
   *
   * <ul>
   * <li><strong>Blank</strong>: Defaults to aligning the region's top-left corner to the target's
   * bottom-left corner ("tl-bl").</li>
   * <li><strong>Two anchors</strong>: If two values from the table below are passed separated by a dash,
   * the first value is used as this region's anchor point, and the second value is
   * used as the target's anchor point.</li>
   * <li><strong>One anchor</strong>: The passed anchor position is used as the target's anchor point.
   * This region will position its top-left corner (tl) to that point.</li>
   * <li><strong>Two edge/offset descriptors:</strong> An edge/offset descriptor is an edge initial
   * (<code>t</code>/<code>r</code>/<code>b</code>/<code>l</code>) followed by a percentage along that side. This describes a
   * point to align with a similar point in the target. So <code>'t0-b0'</code> would be
   * the same as <code>'tl-bl'</code>, <code>'l0-r50'</code> would place the top left corner of this item
   * halfway down the right edge of the target item. This allows more flexibility
   * and also describes which two edges are considered adjacent when positioning an anchor.</li>
   * </ul>
   *
   *
   * <p>If the <code>inside</code> option is passed, the Region will attempt to align as specified,
   * but the position will be adjusted to constrain to the <code>inside</code> Region if necessary.
   * Note that the Region being aligned might be swapped to align to a different position
   * than that specified in order to enforce the constraints. Following are all of the
   * supported anchor positions:</p>
   *
   * <pre><code> Value  Description
   *  -----  -----------------------------
   *  tl     The top left corner
   *  t      The center of the top edge
   *  tr     The top right corner
   *  l      The center of the left edge
   *  c      The center
   *  r      The center of the right edge
   *  bl     The bottom left corner
   *  b      The center of the bottom edge
   *  br     The bottom right corner
   * </code></pre>
   *
   * <p>Example Usage:</p>
   *
   * <pre><code> var xy = comp.getRegion().alignTo({
   *      align: 't-b',  // align comp's top/center to el's bottom/center
   *      target: el.getRegion(),
   *      anchorSize: new <a href="#!/api/Ext.util.Point" rel="Ext.util.Point" class="docClass">Ext.util.Point</a>(10, 10),
   *      inside: new <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Ext.util.Region</a>(0, <a href="#!/api/Ext.dom.Element-static-method-getViewportWidth" rel="Ext.dom.Element-static-method-getViewportWidth" class="docClass">Ext.Element.getViewportWidth</a>(), <a href="#!/api/Ext.dom.Element-static-method-getViewportHeight" rel="Ext.dom.Element-static-method-getViewportHeight" class="docClass">Ext.Element.getViewportHeight</a>(), 0)
   *  });
   * </code></pre>
   * @param options The alignment options.
   * @param {ext.util.Region} options.target The rectangle to which this rectangle
   * should align.
   * @param {String} [options.align] The alignment descriptor for positioning this
   * rectangle with respect to the <code>target</code>. See <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">Ext.util.Positionable.alignTo</a>.
   * Note that if the requested alignment results in violation of the <code>inside</code> constraint,
   * the result will be flipped align to the closest edge which conforms to the constraint.
   * @default tl-bl
   * @param [options.position] The position at which to place the
   * resulting region before being excluded from the target area and aligned to the closest
   * edge which allows conformity with any passed <code>inside</code> option. Used instead of the <code>align</code> option.
   * @param [options.offset] An extra exclusion zone round the target.
   * @param [options.anchorSize] The width and height of any external anchor
   * element. This is used to calculate the true bounds of the Region inclusive of the anchor.
   * The <code>x</code> dimension is the height of the arrow in all orientations, and the <code>y</code> dimension
   * is the width of the baseline of the arrow in all dimensions.
   * If this option is used, and the returned region successfully clears the
   * bounds of the target, then the anchor region will be returned in the return value
   * as the <code>anchor</code> property. This will in turn have a <code>position</code> property which will
   * be <code>'top'</code>, <code>'left</code>, <code>'right'</code>, or <code>'bottom'</code>.
   * @param {Boolean} [options.overlap] Pass <code>true</code> to allow this rectangle to overlap
   * the target.
   * @param {Boolean} [options.rtl] Pass <code>true</code> to swap left/right alignment.
   * @param {ext.util.Region} [options.inside] The rectangle to which this rectangle is
   * constrained.
   * @param {Number} [options.minHeight] Used when this Region is to be aligned directly
   * below or above  the target. Gives the option to reduce the height to fit in the
   * available space.
   * @param {Boolean} [options.axisLock] If <code>true</code>, then fallback on constraint violation will
   * only take place along the major align axis. That is, if <code>align: "l-r"</code> is being used, and
   * <code>axisLock: true</code> is used, then if constraints fail, only fallback to <code>r-l</code> is considered.
   * @return The Region that will align this rectangle. Note that if
   * a <code>minHeight</code> option was passed, and aligment is either above or below the target,
   * the Region might be reduced to fit within the space.
   */
  public native function alignTo(options:Object):ext.util.Region;

  /**
   * Modifies the current region to be constrained to the targetRegion.
   * @return this
   */
  public native function constrainTo(targetRegion:ext.util.Region):ext.util.Region;

  /**
   * Checks if this region completely contains the region or point that is passed in.
   */
  public native function contains(region:*):Boolean;

  /**
   * Create a copy of this Region.
   */
  public native function copy():ext.util.Region;

  /**
   * Copy the values of another Region to this Region
   * @param p The region to copy from.
   * @return This Region
   */
  public native function copyFrom(p:ext.util.Region):ext.util.Region;

  /**
   * Check whether this region is equivalent to the given region
   * @param region The region to compare with
   */
  public native function equals(region:ext.util.Region):Boolean;

  /**
   * This method pushes the "other" Region out of this region via the shortest
   * translation. If an "inside" Region is passed, the exclusion also honours
   * that constraint.
   * @param other The Region to move so that it does not intersect this Region.
   * @param inside A Region into which the other Region must be constrained.
   * @param minHeight If passed, indicates that the height may be reduced up
   * to a point to fit the "other" region below or above the target but within the "inside" Region.
   * @param allowX Pass <code>false</code> to disallow translation along the X axis.
   * @default true
   * @param allowY Pass <code>false</code> to disallow translation along the Y axis.
   * @default true
   * @return The edge it is now aligned to, 0=top, 1=right, 2=bottom, 3=left.
   */
  public native function exclude(other:Object, inside:Object, minHeight:Number = NaN, allowX:Boolean = true, allowY:Boolean = true):Number;

  /**
   * This function converts a legacy alignment string such as 't-b' into a
   * pair of edge, offset objects which describe the alignment points of
   * the two regions.
   *
   * <p>So tl-br becomes {myEdge:'t', offset:0}, {otherEdge:'b', offset:100}</p>
   *
   * <p>This not only allows more flexibility in the alignment possibilities,
   * but it also resolves any ambiguity as to chich two edges are desired
   * to be adjacent if an anchor pointer is required.</p>
   * @param align The align spec, eg <code>"tl-br"</code>
   * @param rtl Pass <code>true</code> to use RTL calculations.
   */
  public native function getAlignInfo(align:String, rtl:Boolean = false):void;

  /**
   * Returns the offsets of this region from the passed region or point.
   * @param offsetsTo The region or point to get get
   * the offsets from.
   * @return The XY page offsets
   * <ul>
   *   <li><code>x</code> : Number
   * The x offset
   *   </li>
   *   <li><code>y</code> : Number
   * The y offset
   *   </li>
   * </ul>
   */
  public native function getOffsetsTo(offsetsTo:*):Object;

  /**
   * Get the offset amount of a point outside the region
   * @param p the point
   */
  public native function getOutOfBoundOffset(axis:String = null, p:ext.util.Point = null):ext.Base;

  /**
   * Get the offset amount on the x-axis
   * @param p the offset
   */
  public native function getOutOfBoundOffsetX(p:Number):Number;

  /**
   * Get the offset amount on the y-axis
   * @param p the offset
   */
  public native function getOutOfBoundOffsetY(p:Number):Number;

  /**
   * Checks if this region intersects the region passed in.
   * @return Returns the intersected region or false if there is no intersection.
   */
  public native function intersect(region:ext.util.Region):*;

  /**
   * Check whether the point / offset is out of bound
   * @param p the point / offset
   */
  public native function isOutOfBound(axis:String = null, p:* = undefined):Boolean;

  /**
   * Check whether the offset is out of bound in the x-axis
   * @param p the offset
   */
  public native function isOutOfBoundX(p:Number):Boolean;

  /**
   * Check whether the offset is out of bound in the y-axis
   * @param p the offset
   */
  public native function isOutOfBoundY(p:Number):Boolean;

  /**
   * Round all the properties of this region
   * @return this This Region
   */
  public native function round():ext.util.Region;

  /**
   * Translates this Region to the specified position
   * @param x The new X position.
   * @param y The new Y position.
   * @return This region after translation.
   */
  public native function setPosition(x:Object, y:Object):ext.util.Region;

  /**
   * Translate this Region by the given offset amount
   * @param x Object containing the <code>x</code> and <code>y</code> properties.
   * Or the x value is using the two argument form.
   * @param y The y value unless using an Offset object.
   * @return this This Region
   */
  public native function translateBy(x:*, y:Number):ext.util.Region;

  /**
   * Returns the smallest region that contains the current AND targetRegion.
   * @return a new region
   */
  public native function union(region:ext.util.Region):ext.util.Region;
}
}