package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.Sorter", require)]
/**
 * Represents a single sorter that can be used as part of the sorters configuration in Ext.mixin.Sortable.
 *
 * <p>A common place for Sorters to be used are <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Stores</a>. For example:</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *       fields: ['firstName', 'level'],
 *       sorters: 'level',
 *
 *       data: [
 *           { firstName: 'Mitch',  level: 9000},
 *           { firstName: 'Seth',   level: 42},
 *           { firstName: 'Fred',   level: 510},
 *           { firstName: 'Israel', level: 690},
 *           { firstName: 'Greg',   level: 101},
 *           { firstName: 'Pat',    level: 0},
 *           { firstName: 'Kevin',  level: 17},
 *           { firstName: 'Brandon',level: 690},
 *           { firstName: 'Gary',   level: 409},
 *           { firstName: 'Scott',  level: 789}
 *       ]
 *    });
 *
 *    <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *        title: 'Support',
 *        store: store,
 *        columns: [
 *            { text: 'Name',  dataIndex: 'firstName' },
 *            { text: 'Level', dataIndex: 'level' }
 *        ],
 *        height: 300,
 *        width: 200,
 *        renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 *    });
 * </code></pre>
 *
 * <p>In the next example, we specify a custom sorter function:</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *       fields: ['firstName', 'spiritAnimal'],
 *        sorters: [
 *            {
 *                // Sort by first letter of second word of spirit animal, in descending order
 *                sorterFn: function(record1, record2) {
 *                    var name1 = record1.data.spiritAnimal.split(' ')[1].substr(0,1),
 *                        name2 = record2.data.spiritAnimal.split(' ')[1].substr(0,1);
 *
 *                    return name1 &gt; name2 ? 1 : (name1 === name2) ? 0 : -1;
 *                },
 *                direction: 'DESC'
 *            }
 *        ],
 *
 *       data: [
 *           { firstName: 'Mitch',  spiritAnimal: "Panda Bear"},
 *           { firstName: 'Seth',   spiritAnimal: "Tina Belcher"},
 *           { firstName: 'Fred',   spiritAnimal: "Honey Badger"},
 *           { firstName: 'Israel', spiritAnimal: "Mysterious Capybara"},
 *           { firstName: 'Greg',   spiritAnimal: "Majestic Platypus"},
 *           { firstName: 'Kevin',  spiritAnimal: "Sparkling Unicorn"},
 *           { firstName: 'Brandon',spiritAnimal: "Pygmy Goat"},
 *           { firstName: 'Gary',   spiritAnimal: "Suri Alpaca"},
 *           { firstName: 'Scott',  spiritAnimal: "Ripe Armadillo"},
 *           { firstName: 'Pat',    spiritAnimal: "The Cougar"}
 *       ]
 *    });
 *
 *    <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *        title: 'Support',
 *        store: store,
 *        columns: [
 *            { text: 'Name',          dataIndex: 'firstName' },
 *            { text: 'Spirit Animal', dataIndex: 'spiritAnimal', flex: 1 }
 *        ],
 *        height: 310,
 *        renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 *    });
 * </code></pre>
 */
public class Sorter extends ext.Base {
  /**
   * Creates a comparator function (a function that can be passed to <code>Array.sort</code>)
   * given one or more <code>Sorter</code> instances.
   *
   * <p>The returned function retains a reference to the collection or array of sorters
   * passed. This means the function will produce a comparison based on the current
   * content of the collection or array, and not based on the content at the time of
   * this call.</p>
   * @param sorters The <code>Sorter</code> instances.
   * @param nextFn The next comparator function to call if all the <code>sorters</code> end
   * with equality.
   * @return The comparator function.
   */
  public static native function createComparator(sorters:*, nextFn:Object = null):Function;

  /**
   * @param config @inheritDoc
   */
  public function Sorter(config:ext.util.Sorter = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The direction to sort by. Valid values are "ASC", and "DESC".
   * @default "ASC"
   * @see #getDirection()
   * @see #setDirection()
   */
  public native function get direction():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set direction(value:String):void;

  [ExtConfig]
  /**
   * An optional id this sorter can be keyed by in Collections. If
   * no id is specified it will use the property name used in this Sorter. If no
   * property is specified, e.g. when adding a custom sorter function we will generate
   * a random id.
   * @see #setId()
   */
  public native function get id():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set id(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The property to sort by. Required unless <code>sorterFn</code> is provided
   * @see #getProperty()
   * @see #setProperty()
   */
  public native function get property():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set property(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Optional root property. This is mostly useful when sorting a Store, in which case we set the
   * root to 'data' to make the filter pull the <a href="#!/api/Ext.util.Sorter-cfg-property" rel="Ext.util.Sorter-cfg-property" class="docClass">property</a> out of the data object of each item
   * @see #getRoot()
   * @see #setRoot()
   */
  public native function get root():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set root(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A specific sorter function to execute. Can be passed instead of <a href="#!/api/Ext.util.Sorter-cfg-property" rel="Ext.util.Sorter-cfg-property" class="docClass">property</a>.
   * This function should compare the two passed arguments, returning -1, 0 or 1 depending on if item 1 should be
   * sorted before, at the same level, or after item 2.
   *
   * <pre><code>sorterFn: function(person1, person2) {
   *     return (person1.age &gt; person2.age) ? 1 : (person1.age === person2.age ? 0 : -1);
   * }
   * </code></pre>
   * @see #getSorterFn()
   * @see #setSorterFn()
   */
  public native function get sorterFn():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set sorterFn(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A function that will be run on each value before
   * it is compared in the sorter. The function will receive a single argument,
   * the value.
   * @see #getTransform()
   * @see #setTransform()
   */
  public native function get transform():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set transform(value:Function):void;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Sorter-cfg-direction" rel="Ext.util.Sorter-cfg-direction" class="docClass">direction</a>.
   */
  public native function getDirection():String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Sorter-cfg-property" rel="Ext.util.Sorter-cfg-property" class="docClass">property</a>.
   */
  public native function getProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Sorter-cfg-root" rel="Ext.util.Sorter-cfg-root" class="docClass">root</a>.
   */
  public native function getRoot():String;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Sorter-cfg-sorterFn" rel="Ext.util.Sorter-cfg-sorterFn" class="docClass">sorterFn</a>.
   */
  public native function getSorterFn():Function;

  /**
   * Returns this sorter's state.
   */
  public native function getState():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.util.Sorter-cfg-transform" rel="Ext.util.Sorter-cfg-transform" class="docClass">transform</a>.
   */
  public native function getTransform():Function;

  /**
   * Returns this sorter's serialized state. This is used when transmitting this sorter
   * to a server.
   */
  public native function serialize():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sorter-cfg-direction" rel="Ext.util.Sorter-cfg-direction" class="docClass">direction</a>.
   * @param direction The new value.
   */
  public native function setDirection(direction:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sorter-cfg-id" rel="Ext.util.Sorter-cfg-id" class="docClass">id</a>.
   * @param id The new value.
   */
  public native function setId(id:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sorter-cfg-property" rel="Ext.util.Sorter-cfg-property" class="docClass">property</a>.
   * @param property The new value.
   */
  public native function setProperty(property:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sorter-cfg-root" rel="Ext.util.Sorter-cfg-root" class="docClass">root</a>.
   * @param root The new value.
   */
  public native function setRoot(root:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sorter-cfg-sorterFn" rel="Ext.util.Sorter-cfg-sorterFn" class="docClass">sorterFn</a>.
   * @param sorterFn The new value.
   */
  public native function setSorterFn(sorterFn:Function):void;

  /**
   * Sets the value of <a href="#!/api/Ext.util.Sorter-cfg-transform" rel="Ext.util.Sorter-cfg-transform" class="docClass">transform</a>.
   * @param transform The new value.
   */
  public native function setTransform(transform:Function):void;

  /**
   * Toggles the direction of this Sorter. Note that when you call this function,
   * the Collection this Sorter is part of does not get refreshed automatically.
   */
  public native function toggle():void;
}
}