package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.TextMetrics", require)]
/**
 * Provides precise pixel measurements for blocks of text so that you can determine
 * the exact pixel height and width of a block of text.
 *
 * <p><strong>Note:</strong> The TextMetrics tool should only be utilized to measure plain text. Attempting to
 * measure text that includes HTML may return inaccurate results.</p>
 *
 * <p>This measurement works by copying the relevant font-related CSS styles from the element<br/>
 * param to the TextMetrics' cached measurement element.  This returns the dimensions of the cached
 * element wrapping the text.  By default, the wrapping element is auto-sized.<br/>
 * You must provide a <strong>fixed width</strong> if the passed text is multi-lined.</p>
 *
 * <p>When multiple measurements are being done with the same element styling, you should
 * create a single, reusable TextMetrics instance.  This is more efficient than using the
 * static <a href="#!/api/Ext.util.TextMetrics-static-method-measure" rel="Ext.util.TextMetrics-static-method-measure" class="docClass">measure</a> method.  The element styles are copied to the cached
 * TextMetrics element once during instantiation versus repeated copying using
 * <em>measure()</em>.</p>
 *
 * <p>The following example demonstrates the recommended use of TextMetrics where the custom
 * textfield class sets up a reusable TextMetrics instance used to measure the label
 * width. This example assumes that all instances of <em>mytextfield</em> have the same
 * <a href="#!/api/Ext.form.Labelable-cfg-labelClsExtra" rel="Ext.form.Labelable-cfg-labelClsExtra" class="docClass">labelClsExtra</a> and
 * <a href="#!/api/Ext.form.Labelable-cfg-labelStyle" rel="Ext.form.Labelable-cfg-labelStyle" class="docClass">labelStyle</a> configs.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.MyTextField', {
 *     extend: '<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>',
 *     xtype: 'mytextfield',
 *
 *     initComponent: function () {
 *         var me = this,
 *             tm = me.getTextMetrics();
 *
 *         me.labelWidth = tm.getWidth(me.fieldLabel + me.labelSeparator);
 *         me.callParent();
 *     },
 *
 *     getTextMetrics: function () {
 *         var me = this,
 *             // Using me.self allows labelCls etc. to vary by derived
 *             // class, but not by instance.
 *             cls = me.self,
 *             tm = cls.measurer,
 *             el;
 *
 *         if (!tm) {
 *             el = <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().createChild();
 *             el.addCls(me.labelCls + ' ' + me.labelClsExtra).
 *                 applyStyles(me.labelStyle);
 *
 *             cls.measurer = tm = new <a href="#!/api/Ext.util.TextMetrics" rel="Ext.util.TextMetrics" class="docClass">Ext.util.TextMetrics</a>(el);
 *         }
 *
 *         return tm;
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Contact Info',
 *     width: 600,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'mytextfield',
 *         fieldLabel: 'Name',
 *         labelStyle: 'font-size: 10px;'
 *     }, {
 *         xtype: 'mytextfield',
 *         fieldLabel: 'Email Address',
 *         labelStyle: 'font-size: 10px;'
 *     }]
 * });
 * </code></pre>
 *
 * <p>While less efficient than the preceding example, this example allows each instance of
 * <em>mytextfield2</em> to have unique labelClsExtra and labelStyle configs.  Each custom
 * textfield instance uses the static TextMetrics measure method which will copy the
 * label styles repeatedly, thus being less efficient but more versatile.</p>
 *
 * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.view.MyTextField2', {
 *     extend: '<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>',
 *     xtype: 'mytextfield2',
 *
 *     initComponent: function () {
 *         var me = this,
 *             el = me.getMeasurementEl(),
 *             tm = <a href="#!/api/Ext.util.TextMetrics" rel="Ext.util.TextMetrics" class="docClass">Ext.util.TextMetrics</a>;
 *
 *         me.labelWidth = tm.measure(el, me.fieldLabel + me.labelSeparator).width;
 *         me.callParent();
 *     },
 *
 *     getMeasurementEl: function () {
 *         var me = this,
 *             cls = MyApp.view.MyTextField2,
 *             el = cls.measureEl;
 *
 *         if (!el) {
 *             cls.measureEl = el = <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().createChild();
 *         }
 *
 *         el.dom.removeAttribute('style');
 *         el.removeCls(el.dom.className).
 *             addCls(me.labelCls + ' ' + me.labelClsExtra).
 *             applyStyles(me.labelStyle);
 *
 *         return el;
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Contact Info',
 *     width: 600,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'mytextfield2',
 *         fieldLabel: 'Name',
 *         labelStyle: 'font-size: 14px;font-weight: bold;',
 *         labelClsExtra: 'nameLabel'
 *     }, {
 *         xtype: 'mytextfield2',
 *         fieldLabel: 'Email Address',
 *         labelStyle: 'font-size: 10px;',
 *         labelClsExtra: 'emailLabel'
 *     }]
 * });
 * </code></pre>
 */
public class TextMetrics extends ext.Base {
  /**
   * Destroy the TextMetrics instance created by <a href="#!/api/Ext.util.TextMetrics-static-method-measure" rel="Ext.util.TextMetrics-static-method-measure" class="docClass">measure</a>.
   */
  public static native function destroy():void;

  /**
   * Measures the size of the specified text
   * @param el The element, dom node or id from which to copy existing CSS styles
   * that can affect the size of the rendered text
   * @param text The text to measure
   * @param fixedWidth If the text will be multiline, you have to set a fixed width
   * in order to accurately measure the text height
   * @return An object containing the text's size <code>{width: (width), height: (height)}</code>
   */
  public static native function measure(el:*, text:String, fixedWidth:Number = NaN):Object;

  /**
   * Creates new TextMetrics.
   * @param bindTo The element or its ID to bind to.
   * @param fixedWidth A fixed width to apply to the measuring element.
   */
  public function TextMetrics(bindTo:*, fixedWidth:Number = NaN) {
    super();
  }

  /**
   * Destroy this instance
   */
  override public native function destroy(...params):void;

  /**
   * Binds this TextMetrics instance to a new element
   * @param el The element or its ID.
   */
  public native function bind(el:*):void;

  /**
   * Returns the measured height of the specified text
   * @param text The text to measure
   * @return height The height in pixels
   */
  public native function getHeight(text:String):Number;

  /**
   * Returns the size of the specified text based on the internal element's style and width properties
   * @param text The text to measure
   * @return An object containing the text's size <code>{width: (width), height: (height)}</code>
   */
  public native function getSize(text:String):Object;

  /**
   * Returns the measured width of the specified text
   * @param text The text to measure
   * @return width The width in pixels
   */
  public native function getWidth(text:String):Number;

  /**
   * Sets a fixed width on the internal measurement element.  If the text will be multiline, you have
   * to set a fixed width in order to accurately measure the text height.
   * @param width The width to set on the element
   */
  public native function setFixedWidth(width:Number):void;
}
}